package com.baijiayun.groupclassui.demo;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.Toast;

import com.baijiahulian.common.cache.sp.SharePreferenceUtil;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.InteractiveClassUI;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPJoinCodeEnterRoomModel;
import com.baijiayun.livecore.LiveSDK;


import java.util.concurrent.TimeUnit;

import io.reactivex.disposables.Disposable;


/**
 * A placeholder fragment containing a simple view.
 */
public class MainActivityFragment extends Fragment {
    private static final String BRTC = "brtc";
    private static final String KDEPLOYTYPE = "kDeployType";
    private EditText joinCode;
    private EditText password;
    private EditText customCodeEt;
    private SharePreferenceUtil mSharePreferenceUtil;
    private SharedPreferences sp;

    private Disposable disposableOfLogin;

    private String JOIN_CODE = "join_code";
    private String USER_NAME = "user_name";
    private String CUSTOM_CODE = "custom_code";

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mSharePreferenceUtil = new SharePreferenceUtil(getActivity(), "groupclassUI_demo_sharepreference_file");
    }

    @Override
    public void onAttach(@NonNull  Context context) {
        super.onAttach(context);
        sp = context.getSharedPreferences(BRTC, Context.MODE_PRIVATE);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View view = inflater.inflate(R.layout.fragment_main, container, false);
        joinCode = (EditText) view.findViewById(R.id.join_Code);
        password = (EditText) view.findViewById(R.id.pw_join_code);
        customCodeEt = view.findViewById(R.id.custom_code);
        Button btnLogin = (Button) view.findViewById(R.id.btnLogin);
        initData();
        disposableOfLogin = RxUtils.clicks(btnLogin)
                .throttleFirst(500, TimeUnit.MILLISECONDS)
                .subscribe(aVoid -> {
                    String code = joinCode.getText().toString();
                    String name = password.getText().toString();
                    if (TextUtils.isEmpty(code)) {
                        Toast.makeText(getActivity(), "请输入参加码", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    if (TextUtils.isEmpty(name)) {
                        Toast.makeText(getActivity(), "请输入昵称", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    if(!TextUtils.isEmpty(customCodeEt.getText().toString())){
                        LiveSDK.customEnvironmentPrefix = customCodeEt.getText().toString();
                        Toast.makeText(getActivity(), "设置个性域名 " + LiveSDK.customEnvironmentPrefix + "成功", Toast.LENGTH_SHORT).show();
                    }
                    mSharePreferenceUtil.putString(JOIN_CODE, code);
                    mSharePreferenceUtil.putString(USER_NAME, name);
                    mSharePreferenceUtil.putString(CUSTOM_CODE, customCodeEt.getText().toString());
                    InteractiveClassUI.enterRoom(getActivity(), new LPJoinCodeEnterRoomModel(code, name));
                });

        RadioGroup radioGroup = view.findViewById(R.id.radio);
        radioGroup.setOnCheckedChangeListener((group, checkedId) -> {
            switch (checkedId) {
                case R.id.radio_1:
                    InteractiveClassUI.setDefaultResolution(LPConstants.LPResolutionType.LOW);
                    break;
                case R.id.radio_2:
                    InteractiveClassUI.setDefaultResolution(LPConstants.LPResolutionType.HIGH);
                    break;
                case R.id.radio_3:
                    InteractiveClassUI.setDefaultResolution(LPConstants.LPResolutionType._720);
                    break;
                case R.id.radio_4:
                    InteractiveClassUI.setDefaultResolution(LPConstants.LPResolutionType._1080);
                    break;
            }
        });

        RadioGroup radioEnv = view.findViewById(R.id.radio_env);
        radioEnv.setOnCheckedChangeListener((group, checkedId) -> {
            switch (checkedId) {
                case R.id.radio_test:
                    LiveSDK.deployType = LPConstants.LPDeployType.Test;
                    sp.edit().putInt(KDEPLOYTYPE, 2).apply();
                    break;
                case R.id.radio_beta:
                    LiveSDK.deployType = LPConstants.LPDeployType.Beta;
                    sp.edit().putInt(KDEPLOYTYPE, 1).apply();
                    break;
                case R.id.radio_pro:
                    LiveSDK.deployType = LPConstants.LPDeployType.Product;
                    sp.edit().putInt(KDEPLOYTYPE, 0).apply();
                    break;
                default:
                    break;
            }
        });
        return view;
    }

    private void initData() {
        String code = mSharePreferenceUtil.getStringValue(JOIN_CODE, "");
        if (!TextUtils.isEmpty(code)) {
            joinCode.setText(code);
        }
        String name = mSharePreferenceUtil.getStringValue(USER_NAME, "");
        if (!TextUtils.isEmpty(name)) {
            password.setText(name);
        }
        String customCode = mSharePreferenceUtil.getStringValue(CUSTOM_CODE, "");
        if (!TextUtils.isEmpty(name)) {
            customCodeEt.setText(customCode);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (disposableOfLogin != null) {
            disposableOfLogin.dispose();
        }
    }
}