package com.baijiayun.groupclassui.base;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import androidx.annotation.DrawableRes;
import androidx.annotation.IdRes;
import androidx.annotation.StringRes;
import androidx.recyclerview.widget.RecyclerView;
import android.util.SparseArray;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * 通用ViewHolder
 * Created by Dujuncan on 2017/1/3.
 */
public class BaseViewHolder extends RecyclerView.ViewHolder {
    private Context mContext;
    private View mContentView;
    private SparseArray<View> mViews;//缓存控件实例

    public BaseViewHolder(Context context, View itemView) {
        super(itemView);
        mContext = context;
        mContentView = itemView;
        mViews = new SparseArray<>();
    }

    /**
     * 根据控件id获取控件实例，如果在缓存中没有，通过findViewById查询，再缓存起来。
     *
     * @param viewId 控件id
     * @return
     */
    @SuppressWarnings("unchecked")
    public <T extends View> T getView(@IdRes int viewId) {
        View view = mViews.get(viewId);
        if (null == view) {
            view = mContentView.findViewById(viewId);
            mViews.put(viewId, view);
        }
        return (T) view;
    }

    public View getRootView() {
        return mContentView;
    }

    public void setVisibility(@IdRes int id, boolean isShow) {
        View view = mContentView.findViewById(id);
        view.setVisibility(isShow ? View.VISIBLE : View.GONE);
    }

    public void setText(@IdRes int id, @StringRes int resId) {
        TextView tv = mContentView.findViewById(id);
        tv.setText(resId);
    }

    public void setText(@IdRes int id, String content) {
        TextView tv = mContentView.findViewById(id);
        tv.setText(content);
    }

    public void setText(@IdRes int id, CharSequence content) {
        TextView tv = mContentView.findViewById(id);
        tv.setText(content);
    }

    public void setText(@IdRes int id, @StringRes int resId, String color) {
        TextView tv = mContentView.findViewById(id);
        tv.setTextColor(Color.parseColor(color));
        tv.setText(resId);
    }

    public void setText(@IdRes int id, String content, String color) {
        TextView tv = mContentView.findViewById(id);
        tv.setTextColor(Color.parseColor(color));
        tv.setText(content);
    }

    public void setText(@IdRes int id, CharSequence content, String color) {
        TextView tv = mContentView.findViewById(id);
        tv.setTextColor(Color.parseColor(color));
        tv.setText(content);
    }

    public String getText(@IdRes int id) {
        TextView tv = mContentView.findViewById(id);
        return tv.getText().toString();
    }

    public void setSelected(@IdRes int id, boolean isSelected) {
        View view = mContentView.findViewById(id);
        view.setSelected(isSelected);
    }

    public void setTvDrawable(@IdRes int id, int resId) {
        TextView tv = mContentView.findViewById(id);
        Drawable drawable = mContext.getResources().getDrawable(resId);
        drawable.setBounds(0, 0, drawable.getMinimumWidth(), drawable.getMinimumHeight());
        tv.setCompoundDrawables(drawable, null, null, null);
    }

    public void setBitmap(@IdRes int id, Bitmap bitmap) {
        ImageView iv = mContentView.findViewById(id);
        iv.setImageBitmap(bitmap);
    }

    public void setDrawable(@IdRes int id, Drawable drawable) {
        ImageView iv = mContentView.findViewById(id);
        iv.setImageDrawable(drawable);
    }

    public void click(@IdRes int id, View.OnClickListener listener) {
        mContentView.findViewById(id).setOnClickListener(listener);
    }

    public BaseViewHolder setImageResource(@IdRes int id, @DrawableRes int resId) {
        ImageView iv = mContentView.findViewById(id);
        iv.setImageResource(resId);
        return this;
    }

    public void setImageUrl(@IdRes int id, String url) {
        ImageView iv = mContentView.findViewById(id);
    }

    public void setGifUrl(@IdRes int id, String url) {
        ImageView iv = mContentView.findViewById(id);
    }

    public void setCircleImageUrl(@IdRes int id, String url) {
        ImageView iv = mContentView.findViewById(id);
    }
}
