package com.baijiayun.groupclassui.chat;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;
import static com.baijiayun.groupclassui.chat.ChatWindow.MESSAGE_USER_IN;
import static com.baijiayun.groupclassui.chat.ChatWindow.MESSAGE_USER_OUT;

import android.content.Context;
import android.text.TextUtils;

import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPExpressionModel;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.LPChatMessageParser;
import com.baijiayun.livecore.models.LPMessageModel;
import com.baijiayun.livecore.models.LPMessageTranslateModel;
import com.baijiayun.livecore.models.imodels.IMessageModel;
import com.baijiayun.liveuibase.widgets.chat.ChatMessageView;
import com.baijiayun.liveuibase.widgets.courseware.models.UploadingImageModel;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.LinkedBlockingQueue;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;

/**
 * Created by yongjiaming on 2018/10/15
 */
public class ChatPresenter implements IChatContract.Presenter {

    private final static int DEFAULT_MESSAGE_POOL_SIZE = 500;
    private IRouter router;
    private CompositeDisposable compositeDisposable = new CompositeDisposable();
    private IChatContract.View listener;
    private LinkedBlockingQueue<UploadingImageModel> imageMessageUploadingQueue;
    private List<IMessageModel> messageModelList = new CopyOnWriteArrayList<>();
    //聊天图片list
    private List<IMessageModel> imageMessageModelList = new CopyOnWriteArrayList<>();

    private OnChatFilterListener mOnChatFilterListener;

    private long lastUserInUserOutTimestamp = 0;
    //表情 字典
    private Map<String, String> expressions;
    private boolean isDestroy = false;
    private Context context;

    ChatPresenter(IChatContract.View listener, Context context) {
        this.context = context;
        this.listener = listener;
        imageMessageUploadingQueue = new LinkedBlockingQueue<>();
        expressions = new HashMap<>();
    }

    @Override
    public void setRouter(IRouter router) {
        this.router = router;
    }

    @Override
    public void subscribe() {
        unSubscribe();
        isDestroy = false;
        for (LPExpressionModel lpExpressionModel : router.getLiveRoom().getExpressions()) {
            expressions.put("[" + lpExpressionModel.name + "]", lpExpressionModel.url);
            expressions.put("[" + lpExpressionModel.key + "]", lpExpressionModel.url);
            expressions.put("[" + lpExpressionModel.name + "]", lpExpressionModel.url);
        }
        compositeDisposable.add(router.getLiveRoom()
                .getChatVM()
                .getObservableOfNotifyDataChange()
                .filter(iMessageModels -> iMessageModels.size() > 0)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMessageModels -> {
                    messageModelList.clear();
                    imageMessageModelList.clear();
                    messageModelList.addAll(iMessageModels);
                    for (IMessageModel msg : iMessageModels) {
                        if (msg.getMessageType() == LPConstants.MessageType.Image) {
                            imageMessageModelList.add(msg);
                        }
                    }
                    listener.notifyMessageDataSetChange();
                }));

        compositeDisposable.add(router.getLiveRoom()
                .getChatVM()
                .getObservableOfReceiveMessage()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMessageModel -> {
                    trimMessagePool();
                    messageModelList.add(iMessageModel);
                    if (iMessageModel.getMessageType() == LPConstants.MessageType.Image) {
                        imageMessageModelList.add(iMessageModel);
                    }
                    if (iMessageModel.getMessageType() == LPConstants.MessageType.Image && iMessageModel.getFrom().equals(router.getLiveRoom().getCurrentUser())) {
                        UploadingImageModel toRemoveModel = null;
                        for (IMessageModel messageModel : imageMessageUploadingQueue) {
                            if (iMessageModel.getUrl().equals(((UploadingImageModel) messageModel).getUploadUrl())) {
                                toRemoveModel = (UploadingImageModel) messageModel;
                                break;
                            }
                        }
                        if (toRemoveModel != null) {
                            imageMessageUploadingQueue.remove(toRemoveModel);
                            messageModelList.remove(toRemoveModel);
                            imageMessageModelList.remove(toRemoveModel);
                        }
                    }
                    if (!iMessageModel.getFrom().equals(router.getLiveRoom().getCurrentUser())) {
                        final boolean displayChat = router.getSubjectValueByKey(EventKey.DisplayChat, Boolean.class, false);
                        if (displayChat) {
                            router.getSubjectByKey(EventKey.NewMsgCount).onNext(0);
                        } else {
                            final int newMsgCount = router.getSubjectValueByKey(EventKey.NewMsgCount, Integer.class, 0);
                            router.getSubjectByKey(EventKey.NewMsgCount).onNext(newMsgCount + 1);
                        }
                    }
                    listener.notifyMessageItemInsert(getMessageCount() - 1, iMessageModel.getFrom().equals(router.getLiveRoom().getCurrentUser()));
                }));

        compositeDisposable.add(router.getLiveRoom()
                .getChatVM()
                .getObservableOfReceiveTranslateMessage()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpMessageTranslateModel -> {
                    listener.notifyTranslateMessage(lpMessageTranslateModel);
                }));

        compositeDisposable.add(router.getLiveRoom()
                .getOnlineUserVM()
                .getObservableOfOnlineUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserModels -> {
                    listener.notifyOnlineUserCountChange();
                }));

        compositeDisposable.add(router.getLiveRoom()
                .getChatVM().getObservableOfForbidAllChatStatus()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lPRoomForbidChatResult -> {
                    listener.notifyForbidStatus(lPRoomForbidChatResult.isForbid);
                }));

        compositeDisposable.add(router.getLiveRoom().getOnlineUserVM().getObservableOfUserIn()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(userInModel -> !router.getLiveRoom().getCurrentUser().equals(userInModel.getUser()))
                .filter(userInModel ->
                        // 进来的是助教且关闭了配置项则不显示助教进记录
                        userInModel.getUser().getType() != LPConstants.LPUserType.Assistant
                                || router.getLiveRoom().getPartnerConfig().enableTipAssistantInOutRoom)
                .filter(userInModel -> userInModel.getUser().getGroup() == router.getLiveRoom().getCurrentUser().getGroup())
                .subscribe(userInModel -> {
                    messageModelList.add(constructFakeUserInAndOutMsg(userInModel.getUser(), true));
                    listener.notifyMessageItemInsert(getMessageCount() - 1, true);
                }));

        compositeDisposable.add(router.getLiveRoom().getOnlineUserVM().getObservableOfUserOut()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(userModel -> !userModel.equals(router.getLiveRoom().getCurrentUser()))
                .filter(userModel ->
                        // 出去的是助教且关闭了配置项则不显示助教出记录
                        userModel.getType() != LPConstants.LPUserType.Assistant
                                || router.getLiveRoom().getPartnerConfig().enableTipAssistantInOutRoom)
                .filter(userModel -> userModel.getGroup() == router.getLiveRoom().getCurrentUser().getGroup())
                .subscribe(userModel -> {
                    messageModelList.add(constructFakeUserInAndOutMsg(userModel, false));
                    listener.notifyMessageItemInsert(getMessageCount() - 1, true);
                }));

        compositeDisposable.add(router.getLiveRoom().getOnlineUserVM().getObservableOfOnLineUserCount()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(onUserCount -> {
                    if (listener != null) {
                        listener.notifyOnlineUserCountChange();
                    }
                }));
        compositeDisposable.add(router.getLiveRoom().getChatVM().getObservableOfMsgRevoke()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpMessageRevoke -> {
                    if (lpMessageRevoke.messageId == null) {
                        return;
                    }
                    for (IMessageModel iMessageModel : imageMessageModelList) {
                        if (lpMessageRevoke.messageId.equals(iMessageModel.getId())) {
                            imageMessageModelList.remove(iMessageModel);
                            break;
                        }
                    }
                    int index = -1;
                    for (int i = 0; i < messageModelList.size(); i++) {
                        final IMessageModel iMessageModel = messageModelList.get(i);
                        if (lpMessageRevoke.messageId.equals(iMessageModel.getId())) {
                            index = i;
                            messageModelList.remove(iMessageModel);
                            break;
                        }
                    }
                    listener.notifyMessageItemRemove(index);
                }));
    }


    IMessageModel constructFakeUserInAndOutMsg(IUserModel userModel, boolean isIn) {
        LPMessageModel messageModel = new LPMessageModel();
        StringBuilder sb = new StringBuilder();
        sb.append(getEncodePhoneNumber(userModel.getName()));
        if (userModel.getType() == LPConstants.LPUserType.Teacher) {
            sb.append(" [ ").append(context.getString(R.string.lp_override_role_teacher)).append(" ] ");
        } else if (userModel.getType() == LPConstants.LPUserType.Assistant) {
            sb.append(" [ ").append(context.getString(R.string.bjysc_role_assistant)).append(" ] ");
        }
        if (isIn) {
            sb.append(String.format(" %s", context.getString(R.string.base_live_enter_room)));
            messageModel.id = MESSAGE_USER_IN;
        } else {
            sb.append(String.format(" %s", context.getString(R.string.base_live_leave_room)));
            messageModel.id = MESSAGE_USER_OUT;
        }
        if (System.currentTimeMillis() - lastUserInUserOutTimestamp > 60 * 1000) {
            messageModel.newTime = new Date();
            lastUserInUserOutTimestamp = System.currentTimeMillis();
        }
        messageModel.content = sb.toString();
        return messageModel;
    }

    void sendTextMessage(String message) {
        if (mOnChatFilterListener != null) {
            boolean isFilter = mOnChatFilterListener.onChatFilterListener(message);
            if (isFilter) {
                return;
            }
        }
        router.getLiveRoom().getChatVM().sendMessage(message);
    }


    void sendEmojiMessage(String emoji) {
        router.getLiveRoom().getChatVM().sendEmojiMessage(emoji);
    }

    @Override
    public void sendTranslateMessage(String message, String messageId, String classId, String userId, String fromLanguage, String toLanguage) {
        router.getLiveRoom().getChatVM().sendTranslateMessage(message, messageId, classId, userId, fromLanguage, toLanguage);
    }

    int getMessageCount() {
        return messageModelList.size();
    }

    IMessageModel getMessage(int pos) {
        if (pos < 0 || pos >= messageModelList.size()) {
            return null;
        }
        return messageModelList.get(pos);
    }

    int translateMessage(LPMessageTranslateModel translateModel) {
        int pos = -1;
        for (int i = 0; i < messageModelList.size(); i++) {
            //messageId不唯一，用消息发送人的id加上timestamp识别消息
            if (messageModelList.get(i) != null
                    && messageModelList.get(i).getFrom() != null
                    && (messageModelList.get(i).getFrom().getUserId()
                    + messageModelList.get(i).getTimestamp().getTime()).equals(translateModel.messageId)
                    && translateModel.content != null) {
                pos = i;
                break;
            }
        }
        if (pos < 0 || pos >= messageModelList.size()) {
            return -1;
        }

        LPMessageModel messageModel = (LPMessageModel) messageModelList.get(pos);
        if (translateModel.code == 0) {
            messageModel.setContent(messageModel.getContent().split(ChatMessageView.MARK)[0] + ChatMessageView.MARK + translateModel.result);
        } else if (translateModel.code == 1) {
            String tips = Locale.getDefault().getCountry().equalsIgnoreCase("cn") ? "翻译失败" : "Translate Fail!";
            messageModel.setContent(messageModel.getContent() + ChatMessageView.MARK + tips);
        }
        messageModelList.set(pos, messageModel);
        return pos;
    }

    public IUserModel getCurrentUser() {
        //todo 暂时规避 https://bugly.qq.com/v2/crash-reporting/crashes/d80df85494/279273?pid=1
        if (router.getLiveRoom() == null) {
            return new LPUserModel();
        }
        return router.getLiveRoom().getCurrentUser();
    }

    int getOnlineUserCount() {
        return router.getLiveRoom().getOnlineUserVM().getAllCount();
    }

    boolean enableFixChatPanel1v1() {
        return router.getLiveRoom().enableFixChatPanel1v1();
    }

    /**
     * 禁止聊天
     *
     * @param isForbid 是否禁止
     */
    void forbidChat(boolean isForbid) {
        router.getLiveRoom().getChatVM().requestForbidAllChat(isForbid);
    }

    void reUploadImage() {
        continueUploadQueue();
    }


    void sendImageMessage(String path) {
        if (isDestroy) {
            return;
        }
        UploadingImageModel model = new UploadingImageModel(path, getCurrentUser(), null);
        imageMessageUploadingQueue.offer(model);
        messageModelList.add(model);
        continueUploadQueue();
    }

    void showMessageSendFragment(String message) {
        router.getSubjectByKey(EventKey.ShowMessageSendFragment).onNext(message);
    }

    private void continueUploadQueue() {
        final UploadingImageModel model = imageMessageUploadingQueue.peek();
        if (model == null || !TextUtils.isEmpty(model.getUploadUrl())) {
            return;
        }
        listener.notifyMessageItemInsert(getMessageCount() - 1, true);
        compositeDisposable.add(router.getLiveRoom().getChatVM().getObservableOfUploadImage(model.getUrl())
                .subscribe(uploadModel -> {
                    String imageContent = LPChatMessageParser.toImageMessage(uploadModel.url);
                    if (router != null) {
                        router.getLiveRoom().getChatVM().sendImageMessageToUser(null, imageContent, uploadModel.width, uploadModel.height);
                        model.setUploadUrl(uploadModel.url);
                    }
                    continueUploadQueue();
                }, throwable -> {
                    if (isDestroy) {
                        return;
                    }
                    model.setStatus(UploadingImageModel.STATUS_UPLOAD_FAILED);
                    listener.notifyMessageDataSetChange();
                }));
    }

    boolean getForbidAllChatStatus() {
        return router.getLiveRoom().getChatVM().isChatForbidden();
    }

    void closeChatWindow() {
        router.getSubjectByKey(EventKey.DisplayChat).onNext(false);
    }

    private void trimMessagePool() {
        if (messageModelList.size() <= DEFAULT_MESSAGE_POOL_SIZE) {
            return;
        }
        for (int i = messageModelList.size() - DEFAULT_MESSAGE_POOL_SIZE; i > 0; --i) {
            if (!MESSAGE_USER_IN.equals(messageModelList.get(i).getId()) && !MESSAGE_USER_OUT.equals(messageModelList.get(i).getId())) {
                IMessageModel removedMsg = messageModelList.remove(0);
                if (removedMsg.getMessageType() == LPConstants.MessageType.Image) {
                    imageMessageModelList.remove(removedMsg);
                }
            }
        }
    }

    public void unSubscribe() {
        compositeDisposable.clear();
        isDestroy = true;
    }

    @Override
    public void destroy() {
        unSubscribe();
        listener = null;
        router = null;
        if (imageMessageUploadingQueue != null) {
            imageMessageUploadingQueue.clear();
        }
        imageMessageUploadingQueue = null;
        messageModelList.clear();
        imageMessageModelList.clear();
        mOnChatFilterListener = null;
    }

    void setOnChatFilterListener(OnChatFilterListener listener) {
        this.mOnChatFilterListener = listener;
    }

    List<String> getChatImageUrls() {
        List<String> imageList = new ArrayList<>();
        for (IMessageModel messageModel : imageMessageModelList) {
            imageList.add(messageModel.getUrl());
        }
        return imageList;
    }

    int getIndexOfChatImage(String url) {
        int index = 0;
        for (IMessageModel messageModel : imageMessageModelList) {
            if (url.equals(messageModel.getUrl())) {
                break;
            }
            index++;
        }
        return index;
    }

    @Override
    public Map<String, String> getExpressions() {
        return expressions;
    }

    @Override
    public int getRecallStatus(IMessageModel message) {
        if (router.getLiveRoom().getCurrentUser().getNumber().equals(message.getFrom().getNumber())) {
            return ChatMessageView.RECALL;
        }
        if (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant ||
                router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            return ChatMessageView.DELETE;
        }
        return ChatMessageView.NONE;
    }

    @Override
    public void getQuickReplyList() {
        compositeDisposable.add(router.getLiveRoom().getChatVM().getObservableOfChatQuickReplyList()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(result -> listener.notifyQuickReplyList(result), throwable -> {
                    // do nothing
                }));
    }

    @Override
    public void reCallMessage(IMessageModel message) {
        router.getLiveRoom().getChatVM().requestMsgRevoke(message.getId(), message.getFrom().getUserId());
    }
}
