package com.baijiayun.groupclassui.chat;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;
import static com.baijiayun.groupclassui.util.KUtilsKt.atMostViewSize;

import android.annotation.SuppressLint;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.graphics.BitmapFactory;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Looper;
import android.text.Spannable;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.TextUtils;
import android.text.style.ForegroundColorSpan;
import android.text.style.ImageSpan;
import android.text.util.Linkify;
import android.view.GestureDetector;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.BaseAdapter;
import android.widget.CheckBox;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.PopupWindow;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.core.content.res.ResourcesCompat;
import androidx.core.view.GestureDetectorCompat;
import androidx.fragment.app.FragmentTransaction;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.baijiahulian.common.cropperv2.BJCommonImageCropHelper;
import com.baijiahulian.common.cropperv2.ThemeConfig;
import com.baijiahulian.common.cropperv2.model.PhotoInfo;
import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.drawable.StateListDrawableBuilder;
import com.baijiayun.bjyutils.keyboard.KeyboardUtils;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.bjyutils.toast.ToastCompat;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.load.engine.DiskCacheStrategy;
import com.baijiayun.glide.load.resource.bitmap.RoundedCorners;
import com.baijiayun.glide.request.RequestOptions;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.chat.preview.IChatMessageCallback;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.util.LinearLayoutWrapManager;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.LPExpressionModel;
import com.baijiayun.livebase.utils.AliCloudImageUtil;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.utils.URLImageParser;
import com.baijiayun.livebase.widgets.emoji.EmojiFragment;
import com.baijiayun.livebase.widgets.emoji.EmojiPresenter;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.livecore.models.LPMessageModel;
import com.baijiayun.livecore.models.LPMessageTranslateModel;
import com.baijiayun.livecore.models.imodels.IMessageModel;
import com.baijiayun.liveuibase.utils.ShadowUtil;
import com.baijiayun.liveuibase.widgets.chat.BitmapCropTarget;
import com.baijiayun.liveuibase.widgets.chat.CenterImageSpan;
import com.baijiayun.liveuibase.widgets.chat.ChatImageUtil;
import com.baijiayun.liveuibase.widgets.chat.ChatMessageView;
import com.baijiayun.liveuibase.widgets.chat.GifCropTarget;
import com.baijiayun.liveuibase.widgets.common.CheckImageView;
import com.baijiayun.liveuibase.widgets.courseware.models.UploadingImageModel;

import org.jetbrains.annotations.NotNull;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Created by yongjiaming on 2018/10/15
 * 聊天弹窗
 */
public class ChatWindow extends InteractiveBaseWindow implements IChatContract.View, ChatMessageView.IChatMessageListener {

    public static final String MESSAGE_USER_IN = "fake_message_user_in";
    public static final String MESSAGE_USER_OUT = "fake_message_user_out";

    private static final int INPUT_TYPE_IMAGE = 0;
    private static final int INPUT_TYPE_TEXT = 1;
    private static final int MAX_MESSAGE_LENGTH = 400;

    private RecyclerView chatRv;
    private CheckBox forbiddenCheckBox;
    private TextView msgSendTv;
    private ImageView msgSendIv, emojiSendIv, closeIv;
    private TextView chatUserCountTv, newMessageTv;
    private FrameLayout emojiContainer;
    private ImageView floatWindowIv;
    private ImageView languageSelectBtn;

    private ChatMessageAdapter messageAdapter;
    private ChatPresenter chatPresenter;
    private boolean isHideBubbleBg = false;

    EmojiFragment emojiFragment;
    Context context;
    private boolean isShowEmoji;
    private IChatMessageCallback callback;
    private int inputType = INPUT_TYPE_IMAGE;
    private Map<String, LPExpressionModel> emojiMap;
    private String languageTranslateTo;
    private boolean enableChatTranslation;
    private final SimpleDateFormat simpleDateFormat = new SimpleDateFormat("HH:mm", Locale.getDefault());
    private final Pattern pattern = Pattern.compile("\\[[a-zA-Z0-9\u4e00-\u9fa5]+]");
    private String messageText = "";
    private List<String> quickReplyList = null;

    public ChatWindow(Context context) {
        super(context);
        this.callback = (IChatMessageCallback) context;
        initEmoji();
    }

    private void initEmoji() {
        emojiMap = new HashMap<>();
        final List<LPExpressionModel> expressions = iRouter.getLiveRoom().getExpressions();
        for (LPExpressionModel lpExpressionModel : expressions) {
            emojiMap.put(lpExpressionModel.getBoxName(), lpExpressionModel);
        }
        $.id(R.id.dialog_back_rl).clicked(v -> chatPresenter.closeChatWindow());
    }

    @Override
    protected View onCreateView(Context context) {
        this.context = context;
        View view = LayoutInflater.from(context).inflate(R.layout.bjy_group_dialog_chat_layout, null);
        setNeedShowShadow(!(routerListener.getLiveRoom() != null && routerListener.getLiveRoom().enableFixChatPanel1v1() && DisplayUtils.isPad(context)));
        initView(context, view);
        return view;
    }

    private void initView(Context context, View view) {
        chatUserCountTv = view.findViewById(R.id.chat_user_count_tv);
        chatRv = view.findViewById(R.id.chat_rv);
        forbiddenCheckBox = view.findViewById(R.id.chat_forbidden_cb);
        msgSendTv = view.findViewById(R.id.chat_messageText);
        msgSendIv = view.findViewById(R.id.text_send_iv);
        emojiSendIv = view.findViewById(R.id.dialog_message_emoji);
        emojiContainer = view.findViewById(R.id.dialog_message_send_emoji);
        newMessageTv = view.findViewById(R.id.new_message_tips_tv);
        floatWindowIv = view.findViewById(R.id.chat_float_window_iv);
        languageSelectBtn = view.findViewById(R.id.language_select_button);

        msgSendTv.setHint(context.getString(R.string.chat_enter_editText_hint, MAX_MESSAGE_LENGTH));

        messageAdapter = new ChatMessageAdapter();
        chatRv.getRecycledViewPool().setMaxRecycledViews(0, 0);
        chatRv.getRecycledViewPool().setMaxRecycledViews(1, 0);
        chatRv.setLayoutManager(new LinearLayoutWrapManager(context));
        chatRv.setAdapter(messageAdapter);

        closeIv = view.findViewById(R.id.chat_close_iv);

        chatPresenter = new ChatPresenter(this, context);
        languageTranslateTo = isAdmin() ? iRouter.getLiveRoom().getPartnerConfig().defaultTranslateLanguage
                : iRouter.getLiveRoom().getPartnerConfig().defaultTranslateLanguageStu;
        initListener();
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if (isWindowInit) {
            if (isActive) {
                chatPresenter.setRouter(iRouter);
                chatPresenter.subscribe();
                updateView();
            } else {
                chatPresenter.unSubscribe();
            }
        }
        return true;
    }

    /**
     * 进房间成功后初始化好presenter，更新视图
     */
    private void updateView() {
        enableChatTranslation = iRouter.getLiveRoom().getPartnerConfig().isEnableChatTranslation();
        if (enableChatTranslation) {
            languageSelectBtn.setVisibility(View.VISIBLE);
        } else {
            languageSelectBtn.setVisibility(View.GONE);
        }
        chatUserCountTv.setText(context.getString(R.string.chat_user_count, chatPresenter.getOnlineUserCount()));
        chatRv.smoothScrollToPosition(messageAdapter.getItemCount());
        notifyForbidStatus(chatPresenter.getForbidAllChatStatus());
        if (chatPresenter.enableFixChatPanel1v1()) {
            $.id(R.id.chat_window_top_ll).gone();
            $.id(R.id.chat_forbidden_cb).gone();
            if (!DisplayUtils.isPad(context)) {
                $.id(R.id.dialog_back_rl).visible();
            }
            msgSendTv.setHint(getString(R.string.bjysc_input_chat_hint));
            msgSendTv.setMaxLines(1);
            msgSendTv.setEllipsize(TextUtils.TruncateAt.END);
            view.setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_bg_color));
        }
        if (!isAdmin()) {
            forbiddenCheckBox.setVisibility(View.GONE);
        } else {
            forbiddenCheckBox.setChecked(chatPresenter.getForbidAllChatStatus());
        }
        chatPresenter.getQuickReplyList();
    }


    private void initListener() {
        forbiddenCheckBox.setOnCheckedChangeListener((buttonView, isChecked) -> {
            if (!forbiddenCheckBox.isPressed()) {
                return;
            }
            chatPresenter.forbidChat(isChecked);
        });

        msgSendTv.setOnClickListener(v -> {
            chatPresenter.showMessageSendFragment(messageText);
            removeEmojiFragment();
        });

        msgSendIv.setOnClickListener(v -> {
            if (inputType == INPUT_TYPE_TEXT) {
                String toSendMsg = msgSendTv.getText().toString().trim();
                if (!TextUtils.isEmpty(toSendMsg)) {
                    sendTextMessage(messageText);
                }
                messageText = "";
                msgSendTv.setText(messageText);
                msgSendIv.setImageResource(R.drawable.bg_chat_send_img_selector);
                int _2dp = UtilsKt.getDp(2);
                msgSendIv.setPadding(_2dp, _2dp, _2dp, _2dp);
                inputType = INPUT_TYPE_IMAGE;
                removeEmojiFragment();
                KeyboardUtils.hideSoftInput(msgSendTv);
            } else {
                showPictureSelect();
            }
        });

        emojiSendIv.setOnClickListener(v -> {
            KeyboardUtils.hideSoftInput(msgSendTv);
            isShowEmoji = !isShowEmoji;
            if (!isShowEmoji) {
                removeEmojiFragment();
                return;
            }
            showEmojiMenu();
        });

        closeIv.setOnClickListener(v -> chatPresenter.closeChatWindow());

        newMessageTv.setOnClickListener(v -> {
            chatRv.smoothScrollToPosition(messageAdapter.getItemCount());
        });

        chatRv.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NotNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                if (Math.abs(dy) > 0 && newMessageTv.getVisibility() == View.VISIBLE) {
                    newMessageTv.setVisibility(View.GONE);
                }
            }
        });

        floatWindowIv.setSelected(routerListener.getLiveRoom().getPartnerConfig().enableFloatChatPanel);
        floatWindowIv.setImageDrawable(ResourcesCompat.getDrawable(context.getResources(), floatWindowIv.isSelected() ? R.drawable.bjy_group_float_chat_window : R.drawable.bjy_group_float_chat_window_hide, null));
        floatWindowIv.setOnClickListener(v -> {
            floatWindowIv.setSelected(!floatWindowIv.isSelected());
            floatWindowIv.setImageDrawable(ResourcesCompat.getDrawable(context.getResources(), floatWindowIv.isSelected() ? R.drawable.bjy_group_float_chat_window : R.drawable.bjy_group_float_chat_window_hide, null));
            iRouter.getSubjectByKey(EventKey.DisplayFloatChat).onNext(floatWindowIv.isSelected());
        });
        languageSelectBtn.setOnClickListener(v -> {
            PopupWindow popupWindow = new PopupWindow(context);
            popupWindow.setFocusable(true);
            popupWindow.setWidth(UtilsKt.getDp(100));
            popupWindow.setBackgroundDrawable(new ColorDrawable(0));
            List<String> items = new ArrayList<>();
            items.add(context.getString(R.string.bjy_live_translate_language_cambodian));
            items.add(context.getString(R.string.bjysc_translate_language_english));
            items.add(context.getString(R.string.bjysc_translate_language_japanese));
            items.add(context.getString(R.string.bjysc_translate_language_vietnamese));
            items.add(context.getString(R.string.bjysc_translate_language_indonesian));
            items.add(context.getString(R.string.bjysc_translate_language_simplified_chinese));
            items.add(context.getString(R.string.bjysc_translate_language_traditional_chinese));
            popupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);
            int position;
            switch (languageTranslateTo) {
                case "hkm":
                    position = 0;
                    break;
                case "jp":
                    position = 2;
                    break;
                case "vie":
                    position = 3;
                    break;
                case "id":
                    position = 4;
                    break;
                case "zh":
                    position = 5;
                    break;
                case "cht":
                    position = 6;
                    break;
                default:
                    position = 1;
                    break;
            }
            LanguageSelectMenuAdapter adapter = new LanguageSelectMenuAdapter(items, position);
            ListView listView = new ListView(context);
            ShadowUtil.ShadowGroup shadowGroup = ShadowUtil.initShadowView(context, listView);
            if (shadowGroup == null) {
                return;
            }
            if (shadowGroup.getContentView() != null) {
                ShadowUtil.setViewBoundShadow(shadowGroup.getShadowContainer());
            }
            GradientDrawable bgDrawable = new GradientDrawable();
            bgDrawable.setColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color));
            bgDrawable.setCornerRadius(UtilsKt.getDp(4));
            listView.setBackground(bgDrawable);
            listView.setAdapter(adapter);
            listView.setDividerHeight(0);
            listView.setPadding(0, UtilsKt.getDp(2), 0, UtilsKt.getDp(2));
            listView.setOnItemClickListener((parent, view, positions, id) -> {
                setLanguageSelect(listView, positions, items.size());
                popupWindow.dismiss();
            });
            popupWindow.setContentView(shadowGroup.getContentView());
            int[] location = new int[2];
            languageSelectBtn.getLocationOnScreen(location);
            int measuredHeight = atMostViewSize(shadowGroup.getContentView())[1];
            popupWindow.showAtLocation(languageSelectBtn, Gravity.NO_GRAVITY, location[0] - UtilsKt.getDp(100), location[1] - measuredHeight);
        });
    }

    private void setLanguageSelect(ListView listView, int position, int size) {
        for (int i = 0; i < size; i++) {
            TextView languageTv = listView.getChildAt(i).findViewById(R.id.tv_language);
            if (i == position) {
                listView.getChildAt(i).findViewById(R.id.ic_language_is_selected).setVisibility(View.VISIBLE);
                languageTv.setTextColor(ContextCompat.getColor(context, R.color.base_theme_live_product));
                switch (i) {
                    case 0:
                        languageTranslateTo = "hkm";
                        break;
                    case 2:
                        languageTranslateTo = "jp";
                        break;
                    case 3:
                        languageTranslateTo = "vie";
                        break;
                    case 4:
                        languageTranslateTo = "id";
                        break;
                    case 5:
                        languageTranslateTo = "zh";
                        break;
                    case 6:
                        languageTranslateTo = "cht";
                        break;
                    default:
                        languageTranslateTo = "en";
                        break;
                }
            } else {
                listView.getChildAt(i).findViewById(R.id.ic_language_is_selected).setVisibility(View.INVISIBLE);
                languageTv.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));
            }
        }
    }

    public void sendTextMessage(String messageText) {
        final LPExpressionModel singleEmoji = emojiMap.get(messageText);
        if (singleEmoji != null) {
            chatPresenter.sendEmojiMessage("[" + singleEmoji.getKey() + "]");
        } else {
            chatPresenter.sendTextMessage(messageText);
        }
    }

    private void removeEmojiFragment() {
        if (emojiFragment != null) {
            emojiContainer.setVisibility(View.GONE);
            emojiSendIv.setSelected(false);
            FragmentTransaction transaction = ((AppCompatActivity) context).getSupportFragmentManager().beginTransaction();
            transaction.remove(emojiFragment);
            transaction.commitAllowingStateLoss();
        }
    }

    @Override
    public void notifyOnlineUserCountChange() {
        chatUserCountTv.setText(context.getString(R.string.chat_user_count, chatPresenter.getOnlineUserCount()));
    }

    @Override
    public void notifyMessageDataSetChange() {
        if (Looper.myLooper() != Looper.getMainLooper()) {
            if (context instanceof AppCompatActivity) {
                ((AppCompatActivity) context).runOnUiThread(() -> {
                    messageAdapter.notifyDataSetChanged();
                    chatRv.smoothScrollToPosition(messageAdapter.getItemCount());
                });
            }
        } else {
            messageAdapter.notifyDataSetChanged();
            chatRv.smoothScrollToPosition(messageAdapter.getItemCount());
        }
    }

    @Override
    public void notifyMessageItemInsert(int position, boolean isSendByMyself) {
        if (Looper.myLooper() != Looper.getMainLooper()) {
            if (context instanceof AppCompatActivity) {
                ((AppCompatActivity) context).runOnUiThread(() -> {
                    onMessageArrived(position, isSendByMyself);
                });
            }
        } else {
            onMessageArrived(position, isSendByMyself);
        }
    }

    @Override
    public void notifyMessageItemRemove(int index) {
        messageAdapter.notifyItemRemoved(index);
    }

    @Override
    public void notifyForbidStatus(boolean isForbid) {
        if (!isAdmin()) {
            msgSendTv.setHint(isForbid ? context.getString(R.string.chat_forbidden) : context.getString(R.string.chat_enter_editText_hint, MAX_MESSAGE_LENGTH));
            emojiSendIv.setClickable(!isForbid);
            msgSendIv.setClickable(!isForbid);
        }
        forbiddenCheckBox.setChecked(chatPresenter.getForbidAllChatStatus());
    }

    @Override
    public void notifyTranslateMessage(LPMessageTranslateModel translateModel) {
        int pos = chatPresenter.translateMessage(translateModel);
        if (Looper.myLooper() != Looper.getMainLooper()) {
            if (context instanceof AppCompatActivity) {
                ((AppCompatActivity) context).runOnUiThread(() -> {
                    onTranslateMessageArrived(pos);
                });
            }
        } else {
            onTranslateMessageArrived(pos);
        }
    }

    @Override
    public void notifyQuickReplyList(List<String> quickReplyList) {
        this.quickReplyList = quickReplyList;
    }

    public List<String> getQuickReplyList() {
        return quickReplyList;
    }

    private void onMessageArrived(int position, boolean isSendByMyself) {
        messageAdapter.notifyItemInserted(position);
        if (isChatRVAtBottom() || isSendByMyself) {
            newMessageTv.setVisibility(View.GONE);
            chatRv.smoothScrollToPosition(messageAdapter.getItemCount());
        } else {
            newMessageTv.setVisibility(View.VISIBLE);
        }
    }

    private void onTranslateMessageArrived(int position) {
        messageAdapter.notifyItemChanged(position);
    }

    public void setMessageText(String message) {
        messageText = message;
        msgSendTv.setText(message);
        if (!TextUtils.isEmpty(messageText)) {
            msgSendIv.setImageResource(R.drawable.ic_chat_send);
            msgSendIv.setPadding(0, 0, 0, 0);
            inputType = INPUT_TYPE_TEXT;
        } else {
            msgSendIv.setImageResource(R.drawable.bg_chat_send_img_selector);
            int _2dp = UtilsKt.getDp(2);
            msgSendIv.setPadding(_2dp, _2dp, _2dp, _2dp);
            inputType = INPUT_TYPE_IMAGE;
        }
    }

    public void showEmojiMenu() {
        isShowEmoji = true;
        emojiContainer.postDelayed(() -> {
            emojiContainer.setVisibility(View.VISIBLE);
            emojiSendIv.setSelected(true);
            emojiFragment = EmojiFragment.newInstance();
            if (routerListener.getLiveRoom() != null && routerListener.getLiveRoom().enableFixChatPanel1v1()) {
                emojiFragment.setItemSize(UtilsKt.getDp(48));
                emojiFragment.setMaxWidth(view.getMeasuredWidth() == 0 ? UtilsKt.getDp(330) : view.getMeasuredWidth());
                emojiFragment.setCount(7, 5);
                if (DisplayUtils.isPad(context)) {
                    //在updateView中view.post之后测量还是0，放到这里变化
                    final ViewGroup.LayoutParams layoutParams = emojiContainer.getLayoutParams();
                    layoutParams.height = view.getMeasuredHeight() - $.id(R.id.chat_enter_rl).view().getMeasuredHeight() - 20;
                    emojiFragment.setMaxHeight(layoutParams.height);
                }
            } else {
                emojiFragment.setCount(6, 5);
                emojiFragment.setMaxWidth(view.getMeasuredWidth() == 0 ? UtilsKt.getDp(330) : view.getMeasuredWidth());
            }
            EmojiPresenter presenter = new EmojiPresenter(emojiFragment, iRouter.getLiveRoom().getChatVM().getExpressions());
            emojiFragment.setPresenter(presenter);
            emojiFragment.setCallBack(emoji -> {
                if (messageText.length() > 400) {
                    ToastCompat.showToast(context, context.getString(R.string.chat_input_text_beyond_tips, MAX_MESSAGE_LENGTH), Toast.LENGTH_SHORT);
                    return;
                }
                messageText += emoji.getBoxName();
                msgSendTv.setText(messageText);
                if (!TextUtils.isEmpty(messageText)) {
                    msgSendIv.setImageResource(R.drawable.ic_chat_send);
                    msgSendIv.setPadding(0, 0, 0, 0);
                    inputType = INPUT_TYPE_TEXT;
                } else {
                    msgSendIv.setImageResource(R.drawable.bg_chat_send_img_selector);
                    int _2dp = UtilsKt.getDp(2);
                    msgSendIv.setPadding(_2dp, _2dp, _2dp, _2dp);
                    inputType = INPUT_TYPE_IMAGE;
                }
            });
            FragmentTransaction transaction = ((AppCompatActivity) context).getSupportFragmentManager().beginTransaction();
            transaction.add(R.id.dialog_message_send_emoji, emojiFragment);
            transaction.commitAllowingStateLoss();
        }, 100);
    }

    public void showPictureSelect() {
        if (context == null) {
            return;
        }
        ThemeConfig.Builder builder = new ThemeConfig.Builder();
        builder.setScreenOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE)
                .setMainElementsColor(ContextCompat.getColor(context, R.color.bjysc_live_blue));
        BJCommonImageCropHelper.openImageSingleAblum(context, BJCommonImageCropHelper.PhotoCropType.None, builder.build(), new BJCommonImageCropHelper.OnHandlerResultCallback() {
            @Override
            public void onHandlerSuccess(List<PhotoInfo> list) {
                if (list.size() == 1) {
                    chatPresenter.sendImageMessage(list.get(0).getPhotoPath());
                }
            }

            @Override
            public void onHandlerFailure(String s) {
                showToastMessage(s);
            }
        });
        removeEmojiFragment();
        KeyboardUtils.hideSoftInput(msgSendTv);
    }

    class ChatMessageAdapter extends RecyclerView.Adapter<ChatMessageAdapter.BaseViewHolder> {

        private static final int MESSAGE_TYPE_TEXT_MINE = 0;
        private static final int MESSAGE_TYPE_TEXT_OTHERS = 1;
        private static final int MESSAGE_TYPE_IMAGE_MINE = 4;
        private static final int MESSAGE_TYPE_IMAGE_OTHERS = 5;
        private static final int MESSAGE_TYPE_USER_IN_USER_OUT = 6;

        @Override
        public BaseViewHolder onCreateViewHolder(@NonNull ViewGroup viewGroup, int viewType) {

            if (viewType == MESSAGE_TYPE_TEXT_MINE) {
                View view = LayoutInflater.from(viewGroup.getContext())
                        .inflate(R.layout.item_chat_text_msg_myself, viewGroup, false);
                return new TextViewHolder(view);
            } else if (viewType == MESSAGE_TYPE_TEXT_OTHERS) {
                View view = LayoutInflater.from(viewGroup.getContext())
                        .inflate(R.layout.item_chat_text_msg_others, viewGroup, false);
                return new TextViewHolder(view);
            } else if (viewType == MESSAGE_TYPE_IMAGE_MINE) {
                View view = LayoutInflater.from(viewGroup.getContext())
                        .inflate(R.layout.item_chat_img_msg_myself, viewGroup, false);
                return new ImageViewHolder(view);
            } else if (viewType == MESSAGE_TYPE_IMAGE_OTHERS) {
                View view = LayoutInflater.from(viewGroup.getContext())
                        .inflate(R.layout.item_chat_img_msg_others, viewGroup, false);
                return new ImageViewHolder(view);
            } else if (viewType == MESSAGE_TYPE_USER_IN_USER_OUT) {
                View view = LayoutInflater.from(viewGroup.getContext()).inflate(R.layout.item_chat_user_in_user_out, viewGroup, false);
                return new UserInUserOutViewHolder(view);
            }
            return null;
        }

        @SuppressLint("ClickableViewAccessibility")
        @Override
        public void onBindViewHolder(@NonNull BaseViewHolder baseViewHolder, int position) {
            IMessageModel messageModel = chatPresenter.getMessage(position);
            if (context == null || messageModel == null) {
                return;
            }

            if (baseViewHolder instanceof UserInUserOutViewHolder) {
                UserInUserOutViewHolder userInUserOutViewHolder = (UserInUserOutViewHolder) baseViewHolder;
                userInUserOutViewHolder.nameTv.setText(messageModel.getContent());
                if (messageModel.getTimestamp() != null) {
                    userInUserOutViewHolder.timeTv.setVisibility(View.VISIBLE);
                    userInUserOutViewHolder.timeTv.setText(simpleDateFormat.format(messageModel.getTimestamp()));
                } else {
                    userInUserOutViewHolder.timeTv.setVisibility(View.GONE);
                }
                return;
            }
            final LPGroupItem groupItem = iRouter.getLiveRoom().getOnlineUserVM().getGroupMap().get(messageModel.getFrom().getGroup());
            SpannableStringBuilder ssbName = new SpannableStringBuilder();
            boolean isGroupValid = messageModel.getFrom().getGroup() != 0 && groupItem != null && groupItem.groupItemModel != null;
            if (isGroupValid) {
                String str = "「" + groupItem.groupItemModel.name + "」";
                ForegroundColorSpan colorSpan = new ForegroundColorSpan(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_assistant_text_color));
                if (isMyself(messageModel)) {
                    ssbName.append(str);
                    ssbName.append(getEncodePhoneNumber(messageModel.getFrom().getName()));
                    ssbName.setSpan(colorSpan, 0, str.length(), Spannable.SPAN_INCLUSIVE_INCLUSIVE);

                } else {
                    ssbName.append(getEncodePhoneNumber(messageModel.getFrom().getName()));
                    ssbName.append(str);
                    int start = getEncodePhoneNumber(messageModel.getFrom().getName()).length();
                    ssbName.setSpan(colorSpan, start, str.length() + start, Spannable.SPAN_INCLUSIVE_INCLUSIVE);
                }
            } else {
                ssbName.append(getEncodePhoneNumber(messageModel.getFrom().getName()));
            }
            baseViewHolder.chatNameTv.setText(ssbName);
            baseViewHolder.chatTimeTv.setText(simpleDateFormat.format(messageModel.getTimestamp()));
            if (baseViewHolder instanceof TextViewHolder) {
                TextViewHolder textViewHolder = (TextViewHolder) baseViewHolder;
                textViewHolder.chatMessageView.setMessage(messageModel.getContent());
                textViewHolder.chatMessageView.setMessageModel(messageModel);
                textViewHolder.chatMessageView.setChatMessageListener(ChatWindow.this);
                if ((enableChatTranslation && messageModel.getMessageType() != LPConstants.MessageType.EmojiWithName
                        && messageModel.getMessageType() != LPConstants.MessageType.Emoji)) {
                    textViewHolder.chatTranslateBtn.setVisibility(View.VISIBLE);
                } else {
                    textViewHolder.chatTranslateBtn.setVisibility(View.GONE);
                }
                SpannableStringBuilder ssb = new SpannableStringBuilder();
                String contentMessage = messageModel.getContent();
                if (contentMessage.contains(ChatMessageView.MARK)) {
                    contentMessage = contentMessage.split(ChatMessageView.MARK)[0];
                }
                ssb.append(replaceWithEmoji(contentMessage, textViewHolder.chatMessageView.getTextView(), textViewHolder));
                ssb.setSpan(textViewHolder.chatMessageView.getLineHeightSpan(), 0, ssb.length() - 1, Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
                textViewHolder.chatMessageView.getTextView().setText(ssb);
                textViewHolder.chatMessageView.removeReference();
                if (messageModel.getReference() != null) {
                    textViewHolder.chatMessageView.addReferenceMessage();
                    String referenceMessage = messageModel.getReference().content;
                    SpannableStringBuilder referenceMessageSsb = new SpannableStringBuilder();
                    referenceMessageSsb.append(replaceWithEmoji(referenceMessage, textViewHolder.chatMessageView.getTvReference(), textViewHolder));
                    referenceMessageSsb.setSpan(textViewHolder.chatMessageView.getLineHeightSpan(), 0, referenceMessage.length() - 1, Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
                    textViewHolder.chatMessageView.getTvReference().setText(referenceMessageSsb);
                }

                textViewHolder.chatTranslateBtn.setOnClickListener(v -> {
                    String toLanguage = TextUtils.isEmpty(languageTranslateTo) ? "en" : languageTranslateTo;
                    if (TextUtils.equals(toLanguage, ((LPMessageModel) messageModel).translateLanguage) && textViewHolder.chatMessageView.isTranslate()) {
                        textViewHolder.chatMessageView.removeTranslate();
                    } else {
                        textViewHolder.chatMessageView.translateMessage();
                        String message = getTranslateText(messageModel.getContent());
                        //判断是否有中文，有就翻译成英文，没有就翻译成中文
                        String fromLanguage;
                        fromLanguage = "auto";
                        //messageId不唯一，用消息发送人的id加上timestamp识别消息
                        ((LPMessageModel) messageModel).translateLanguage = toLanguage;
                        chatPresenter.sendTranslateMessage(message, messageModel.getFrom().getUserId() + messageModel.getTimestamp().getTime(), String.valueOf(iRouter.getLiveRoom().getRoomInfo().roomId), iRouter.getLiveRoom().getCurrentUser().getUserId(), fromLanguage, toLanguage);
                    }
                });

                if (messageModel.getFrom().getType() == LPConstants.LPUserType.Teacher || messageModel.getFrom().getType() == LPConstants.LPUserType.Assistant) {
                    Linkify.addLinks(textViewHolder.chatMessageView.getTextView(), Linkify.WEB_URLS | Linkify.EMAIL_ADDRESSES);
                } else {
                    textViewHolder.chatMessageView.getTextView().setAutoLinkMask(0);
                }
                if (isMyself(messageModel)) {
                    textViewHolder.chatMessageView.setMessageColor(
                            ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_positive_text_color));
                    textViewHolder.chatMessageView.setDividerLineColor(
                            ContextCompat.getColor(context, R.color.bjysc_divider_line_white));
                    textViewHolder.chatMessageView.setLinkTextColor(
                            ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_positive_text_color));
                } else {
                    textViewHolder.chatMessageView.setMessageColor(
                            ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_text_color));
                    textViewHolder.chatMessageView.setDividerLineColor(
                            ContextCompat.getColor(context, R.color.bjysc_divider_line_black));
                    textViewHolder.chatMessageView.setLinkTextColor(ContextCompat.getColor(context, R.color.bjlive_link_text));
                }
                if (isHideBubbleBg) {
                    textViewHolder.chatMessageView.setBackgroundResource(0);
                } else {
                    setMultiState(textViewHolder.chatMessageView, messageModel);
                }
            } else if (baseViewHolder instanceof ImageViewHolder) {
                ImageViewHolder imageViewHolder = (ImageViewHolder) baseViewHolder;
                if (isHideBubbleBg) {
                    imageViewHolder.chatImageView.setBackgroundResource(0);
                } else {
                    setMultiCheckedState(imageViewHolder.chatImageView, messageModel);
                }
                if (messageModel instanceof UploadingImageModel) {
                    BitmapFactory.Options options = new BitmapFactory.Options();
                    options.inJustDecodeBounds = true;
                    BitmapFactory.decodeFile(messageModel.getUrl(), options);
                    int[] size = {options.outWidth, options.outHeight};
                    ChatImageUtil.calculateImageSize(size, UtilsKt.getDp(100), UtilsKt.getDp(50));
                    Glide.with(context)
                            .load(messageModel.getUrl())
                            .apply(new RequestOptions().override(size[0], size[1]).transform(new RoundedCorners(UtilsKt.getDp(10))))
                            .into(imageViewHolder.chatImageView);

                    if (((UploadingImageModel) messageModel).getStatus() == UploadingImageModel.STATUS_UPLOADING) {
                        imageViewHolder.exclamationTv.setVisibility(View.GONE);
                    } else if (((UploadingImageModel) messageModel).getStatus() == UploadingImageModel.STATUS_UPLOAD_FAILED) {
                        imageViewHolder.exclamationTv.setVisibility(View.VISIBLE);
                        imageViewHolder.chatImageView.setOnClickListener(v -> chatPresenter.reUploadImage());
                    }
                } else {
                    final RequestOptions requestOptions = new RequestOptions().transform(new RoundedCorners(UtilsKt.getDp(10)));
                    if (messageModel.getUrl().endsWith(URLImageParser.GIF_SUFFIX)) {
                        Glide.with(context).asGif()
                                .apply(requestOptions.diskCacheStrategy(DiskCacheStrategy.RESOURCE))
                                .load(messageModel.getUrl())
                                .into(new GifCropTarget(imageViewHolder.chatImageView, 180, 135));
                    } else {
                        Glide.with(context)
                                .asBitmap()
                                .apply(requestOptions)
                                .load(AliCloudImageUtil.getScaledUrl(messageModel.getUrl(), AliCloudImageUtil.SCALED_MFIT, 300, 300))
                                .into(new BitmapCropTarget(imageViewHolder.chatImageView, 180, 135));
                    }
                    GestureDetectorCompat gestureDetectorCompat = new GestureDetectorCompat(context, new PressListener(messageModel, imageViewHolder));
                    imageViewHolder.chatImageView.setOnTouchListener((v, event) -> {
                        gestureDetectorCompat.onTouchEvent(event);
                        return true;
                    });
                }
            }
        }

        @Override
        public void onViewRecycled(@NonNull BaseViewHolder holder) {
            super.onViewRecycled(holder);
            if (holder instanceof TextViewHolder) {
                ((TextViewHolder) holder).clearGifCache();
            }
        }

        @Override
        public int getItemViewType(int position) {
            IMessageModel messageModel = chatPresenter.getMessage(position);
            boolean isMineMessage = false;
            if (messageModel.getFrom() != null) {
                isMineMessage = messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber());
            }
            if (MESSAGE_USER_IN.equals(messageModel.getId()) || MESSAGE_USER_OUT.equals(messageModel.getId())) {
                return MESSAGE_TYPE_USER_IN_USER_OUT;
            }
            switch (messageModel.getMessageType()) {
                case Text:
                case Emoji:
                case EmojiWithName:
                    return isMineMessage ? MESSAGE_TYPE_TEXT_MINE : MESSAGE_TYPE_TEXT_OTHERS;
                case Image:
                    return isMineMessage ? MESSAGE_TYPE_IMAGE_MINE : MESSAGE_TYPE_IMAGE_OTHERS;
                default:
                    return MESSAGE_TYPE_TEXT_MINE;
            }
        }

        @Override
        public int getItemCount() {
            return chatPresenter.getMessageCount();
        }

        private void setMultiState(ChatMessageView view, IMessageModel messageModel) {
            if (messageModel.getFrom().getType() == LPConstants.LPUserType.Teacher) {
                view.setSelected(messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber()));
                view.setStateChecked(true);
            } else {
                if (messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber())) {
                    view.setSelected(true);
                    view.setStateChecked(true);
                } else {
                    view.setSelected(false);
                    view.setStateChecked(false);
                }
            }
        }

        private void setMultiCheckedState(CheckImageView view, IMessageModel messageModel) {
            if (messageModel.getFrom().getType() == LPConstants.LPUserType.Teacher) {
                view.setSelected(messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber()));
                view.setStateChecked(true);
            } else {
                if (messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber())) {
                    view.setSelected(true);
                    view.setStateChecked(true);
                } else {
                    view.setSelected(false);
                    view.setStateChecked(false);
                }
            }
        }

        /**
         * 是自己
         *
         * @param messageModel
         * @return
         */
        private boolean isMyself(IMessageModel messageModel) {
            return messageModel.getFrom().getNumber().equals(chatPresenter.getCurrentUser().getNumber());
        }

        private SpannableStringBuilder replaceWithEmoji(String content, TextView textView, TextViewHolder textViewHolder) {
            Matcher matcher = pattern.matcher(content);
            SpannableStringBuilder ssb = new SpannableStringBuilder(content);
            List<URLImageParser.GlideGifDrawable> cacheDrawables = null;
            List<URLImageParser> imageParserList = null;
            while (matcher.find()) {
                String group = matcher.group();
                if (chatPresenter.getExpressions().containsKey(group)) {
                    final URLImageParser urlImageParser = new URLImageParser(textView, textView.getTextSize());
                    Drawable drawable = urlImageParser.getDrawable(chatPresenter.getExpressions().get(group));
                    CenterImageSpan centerImageSpan = new CenterImageSpan(drawable, ImageSpan.ALIGN_BASELINE);
                    ssb.setSpan(centerImageSpan, matcher.start(), matcher.end(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
                    ssb.removeSpan(group);
                    if (drawable instanceof URLImageParser.GlideGifDrawable) {
                        if (cacheDrawables == null) {
                            cacheDrawables = new ArrayList<>();
                        }
                        cacheDrawables.add((URLImageParser.GlideGifDrawable) drawable);
                        if (imageParserList == null) {
                            imageParserList = new ArrayList<>();
                        }
                        imageParserList.add(urlImageParser);
                    }
                }
            }
            if (cacheDrawables != null) {
                textViewHolder.setGifCache(cacheDrawables, imageParserList);
            }
            return ssb;
        }

        /**
         * 翻译去除表情中的[大笑]
         */
        private String getTranslateText(String content) {
            content = content.split(ChatMessageView.MARK)[0];
            Matcher m = pattern.matcher(content);
            while (m.find()) {
                String group = m.group();
                if (chatPresenter.getExpressions().containsKey(group)) {
                    content = content.replace(group, "");
                }
            }
            return content;
        }

        class BaseViewHolder extends RecyclerView.ViewHolder {
            TextView chatTimeTv;
            TextView chatNameTv;

            BaseViewHolder(@NonNull View itemView) {
                super(itemView);
                chatTimeTv = itemView.findViewById(R.id.chat_time_tv);
                chatNameTv = itemView.findViewById(R.id.chat_user_tv);
            }
        }

        class TextViewHolder extends BaseViewHolder {
            ChatMessageView chatMessageView;
            ImageView chatTranslateBtn;
            private List<URLImageParser.GlideGifDrawable> gifDrawables;
            private List<URLImageParser> imageParserList;

            TextViewHolder(@NonNull View itemView) {
                super(itemView);
                chatMessageView = itemView.findViewById(R.id.chat_message_view);
                chatMessageView.setBackground(getMultiTextStateDrawable());
                chatTranslateBtn = itemView.findViewById(R.id.translate_btn);
                // 小班课翻译按钮在外部
                chatMessageView.enableTranslation(false);
            }

            public void setGifCache(List<URLImageParser.GlideGifDrawable> drawables, List<URLImageParser> imageParserList) {
                this.gifDrawables = drawables;
                this.imageParserList = imageParserList;
            }

            public void clearGifCache() {
                if (gifDrawables != null) {
                    for (URLImageParser.GlideGifDrawable drawable : gifDrawables) {
                        drawable.onDestroy();
                    }
                }
                if (imageParserList != null) {
                    for (URLImageParser urlImageParser : imageParserList) {
                        urlImageParser.onDestroy();
                    }
                }
            }
        }

        class ImageViewHolder extends BaseViewHolder {
            CheckImageView chatImageView;
            TextView exclamationTv;

            ImageViewHolder(@NonNull View itemView) {
                super(itemView);
                chatImageView = itemView.findViewById(R.id.img_msg_iv);
                exclamationTv = itemView.findViewById(R.id.item_chat_image_exclamation);
                chatImageView.setBackground(getMultiImgStateDrawable());
            }
        }

        class UserInUserOutViewHolder extends BaseViewHolder {

            TextView nameTv;
            TextView timeTv;

            UserInUserOutViewHolder(@NonNull View itemView) {
                super(itemView);
                nameTv = itemView.findViewById(R.id.tv_user_name);
                timeTv = itemView.findViewById(R.id.user_in_time);
            }
        }
    }

    /**
     * textViewHolder.chatMessageView使用
     *
     * @return
     */
    private Drawable getMultiTextStateDrawable() {
        final int dp_16 = UtilsKt.getDp(16);
        Drawable mySelfDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadii(dp_16, 0, dp_16, dp_16)
                .build();
        Drawable otherTeacherDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadii(0, dp_16, dp_16, dp_16)
                .build();
        Drawable otherDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadii(0, dp_16, dp_16, dp_16)
                .build();

        return new StateListDrawableBuilder()
                .multiState(new int[][]{
                        new int[]{
                                android.R.attr.state_selected,
                                R.attr.state_checked
                        },
                        new int[]{
                                -android.R.attr.state_selected,
                                R.attr.state_checked
                        },
                        new int[]{
                                -android.R.attr.state_selected,
                                -R.attr.state_checked
                        }
                }, new Drawable[]{
                        mySelfDrawable, otherTeacherDrawable, otherDrawable
                })
                .multiBuild();
    }

    private Drawable getMultiImgStateDrawable() {
        final int dp_16 = UtilsKt.getDp(16);
        Drawable mySelfDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadii(dp_16, 0, dp_16, dp_16)
                .build();
        Drawable otherTeacherDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadii(0, dp_16, dp_16, dp_16)
                .build();
        Drawable otherDrawable = new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_negative_bg_color))
                .cornerRadii(0, dp_16, dp_16, dp_16)
                .build();

        return new StateListDrawableBuilder()
                .multiState(new int[][]{
                        new int[]{
                                android.R.attr.state_selected,
                                R.attr.bjlive_state_checked_img
                        },
                        new int[]{
                                -android.R.attr.state_selected,
                                R.attr.bjlive_state_checked_img
                        },
                        new int[]{
                                -android.R.attr.state_selected,
                                -R.attr.bjlive_state_checked_img
                        }
                }, new Drawable[]{
                        mySelfDrawable, otherTeacherDrawable, otherDrawable
                })
                .multiBuild();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        chatPresenter.destroy();
        callback = null;
        context = null;
    }

    private boolean isChatRVAtBottom() {
        LinearLayoutManager linearLayoutManager = (LinearLayoutManager) chatRv.getLayoutManager();
        int lastVisibleItemPosition = linearLayoutManager.findLastVisibleItemPosition();
        int visibleItemCount = linearLayoutManager.getChildCount();
        int totalItemCount = linearLayoutManager.getItemCount();
        int state = chatRv.getScrollState();
        return visibleItemCount == 0 || visibleItemCount > 0 &&
                lastVisibleItemPosition == totalItemCount - 2 && state == RecyclerView.SCROLL_STATE_IDLE;
    }

    public void setOnChatFilterListener(OnChatFilterListener listener) {
        chatPresenter.setOnChatFilterListener(listener);
    }

    @Override
    public void showMenu(int x, int y, View parentView, IMessageModel messageModel, boolean isImage) {
        PopupWindow popupWindow = new PopupWindow(context);
        popupWindow.setFocusable(true);
        popupWindow.setWidth(UtilsKt.getDp(64));
        popupWindow.setBackgroundDrawable(new ColorDrawable(0));
        popupWindow.setHeight(ViewGroup.LayoutParams.WRAP_CONTENT);

        List<String> items = new ArrayList<>();
        final int recallStatus = chatPresenter.getRecallStatus(messageModel);
        if (recallStatus == ChatMessageView.RECALL) {
            items.add(getString(R.string.bjy_base_chat_revoke));
        }
        if (recallStatus == ChatMessageView.DELETE) {
            items.add(getString(R.string.bjy_base_chat_delete));
        }
        items.add(getString(R.string.bjy_base_chat_copy));
        if (messageModel.getFrom() != null
                && routerListener.getLiveRoom() != null
                && routerListener.getLiveRoom().getCurrentUser() != null
                && !messageModel.getFrom().getNumber().equals(routerListener.getLiveRoom().getCurrentUser().getNumber())) {
            items.add(getString(R.string.bjy_base_chat_report));
        }
        ArrayAdapter<String> adapter = new ArrayAdapter<>(context, R.layout.bjy_group_menu_chat_message, items.toArray(new String[0]));
        GradientDrawable bgDrawable = new GradientDrawable();
        bgDrawable.setColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color));
        bgDrawable.setCornerRadius(UtilsKt.getDp(4));
        ListView listView = new ListView(context);
        listView.setBackground(bgDrawable);
        listView.setAdapter(adapter);
        listView.setDividerHeight(0);
        listView.setPadding(0, UtilsKt.getDp(2), 0, UtilsKt.getDp(2));
        listView.setOnItemClickListener((parent, view, position, id) -> {
            final String item = items.get(position);
            if (getString(R.string.bjy_base_chat_copy).equals(item)) {
                if (isImage) {
                    copy("[img:" + messageModel.getUrl() + "]");
                } else {
                    copy(messageModel.getContent());
                }
            } else if (getString(R.string.bjy_base_chat_revoke).equals(item)) {
                chatPresenter.reCallMessage(messageModel);
            } else if (getString(R.string.bjy_base_chat_report).equals(item)) {
                iRouter.getSubjectByKey(EventKey.ReportDialog).onNext(messageModel);
            }
            popupWindow.dismiss();
        });
        popupWindow.setContentView(listView);
        popupWindow.showAtLocation(parentView, Gravity.NO_GRAVITY, x - popupWindow.getWidth() / 2, y - popupWindow.getHeight());
    }

    private void copy(String copyStr) {
        try {
            //获取剪贴板管理器
            ClipboardManager cm = (ClipboardManager) context.getSystemService(Context.CLIPBOARD_SERVICE);
            // 创建普通字符型ClipData
            ClipData mClipData = ClipData.newPlainText("Label", copyStr);
            // 将ClipData内容放到系统剪贴板里。
            cm.setPrimaryClip(mClipData);
        } catch (Exception e) {
            // do nothing
        }
    }

    private class LanguageSelectMenuAdapter extends BaseAdapter {
        private List<String> mData;
        private int selectPosition;

        LanguageSelectMenuAdapter(List<String> list, int position) {
            mData = list;
            selectPosition = position;
        }

        @Override
        public int getCount() {
            return mData.size();
        }

        @Override
        public Object getItem(int position) {
            return mData.get(position);
        }

        @Override
        public long getItemId(int position) {
            return position;
        }

        @Override
        public View getView(int position, View convertView, ViewGroup parent) {
            @SuppressLint("ViewHolder")
            View view = LayoutInflater.from(context).inflate(R.layout.bjysc_language_select_menu, null);
            TextView languageTv = view.findViewById(R.id.tv_language);
            languageTv.setText(mData.get(position));
            languageTv.setTextColor(Color.GRAY);
            if (position == selectPosition) {
                languageTv.setTextColor(ContextCompat.getColor(context, R.color.base_theme_live_product));
                view.findViewById(R.id.ic_language_is_selected).setVisibility(View.VISIBLE);
            }
            return view;
        }
    }

    class PressListener extends GestureDetector.SimpleOnGestureListener {
        private IMessageModel iMessageModel;
        private View parent;

        public PressListener(IMessageModel iMessageModel, RecyclerView.ViewHolder holder) {
            this.iMessageModel = iMessageModel;
            this.parent = holder.itemView;
        }

        @Override
        public void onLongPress(MotionEvent e) {
            super.onLongPress(e);
            showMenu((int) e.getRawX(), (int) e.getRawY(), parent, iMessageModel, true);
        }

        @Override
        public boolean onSingleTapUp(MotionEvent e) {
            callback.displayImage(chatPresenter.getChatImageUrls(), chatPresenter.getIndexOfChatImage(iMessageModel.getUrl()));
            return true;
        }
    }
}
