package com.baijiayun.groupclassui.chat

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.text.Editable
import android.text.InputFilter
import android.text.TextUtils
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.view.WindowManager
import android.view.inputmethod.EditorInfo
import android.view.inputmethod.InputMethodManager
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.keyboard.KeyboardUtils
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.widgets.SimpleTextWatcher
import com.baijiayun.groupclassui.R
import com.baijiayun.livebase.base.BaseDialogFragment
import com.baijiayun.liveuibase.chat.LengthFilter
import kotlinx.android.synthetic.main.bjy_group_layout_message_send_fragment.view.*

class GroupMessageSendFragment : BaseDialogFragment(){
    private val QUICK_REPLY_FILE_NAME = "group_class_quick_reply"
    private val MAX_MESSAGE_LENGTH = 400
    private var haveTextToSend = false
    private var messageText = ""
    private lateinit var callBack: MessageSendCallBack
    private var chatQuickReplyMap : MutableMap<String, Int>? = null
    private var chatQuickReplyList : List<String>? = null

    override fun getLayoutId(): Int {
        return R.layout.bjy_group_layout_message_send_fragment
    }

    override fun init(savedInstanceState: Bundle?, arguments: Bundle?) {
        super.hideTitleBar()
        contentView.bjy_group_chat_editText.hint = context?.getString(R.string.chat_enter_editText_hint, MAX_MESSAGE_LENGTH)
        contentView.bjy_group_chat_editText.postDelayed(Runnable {
            if (activity == null) return@Runnable
            val imm = activity!!.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            imm.showSoftInput(contentView.bjy_group_chat_editText, InputMethodManager.SHOW_IMPLICIT)
        }, 100)
        contentView.bjy_group_chat_editText.setOnClickListener {
            val imm = activity!!.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
            if (!imm.isActive) {
                imm.showSoftInput(contentView.bjy_group_chat_editText, InputMethodManager.SHOW_IMPLICIT)
            }
        }
        contentView.bjy_group_chat_editText.setOnEditorActionListener { v, actionId, event ->
            if (actionId == EditorInfo.IME_ACTION_SEND) {
                messageText = contentView.bjy_group_chat_editText.text.toString()
                if (!TextUtils.isEmpty(messageText)) {
                    callBack.onSendMessage(messageText)
                    contentView.bjy_group_chat_editText.text.clear()
                    KeyboardUtils.hideSoftInput(contentView.bjy_group_chat_editText)
                    true
                }
            }
            false
        }
        contentView.bjy_group_chat_editText.addTextChangedListener(object : SimpleTextWatcher() {

            override fun afterTextChanged(s: Editable?) {
                if (s != null) {
                    haveTextToSend = if (s.isNotEmpty()) {
                        contentView.bjy_group_text_send_iv.setImageResource(R.drawable.ic_chat_send)
                        contentView.bjy_group_text_send_iv.setPadding(0, 0, 0, 0)
                        true
                    } else {
                        contentView.bjy_group_text_send_iv.setImageResource(R.drawable.bg_chat_send_img_selector)
                        val _2dp = 2.dp
                        contentView.bjy_group_text_send_iv.setPadding(_2dp, _2dp, _2dp, _2dp)
                        false
                    }
                }
            }
        })
        contentView.bjy_group_chat_editText.filters = arrayOf<InputFilter>(
            LengthFilter(context, MAX_MESSAGE_LENGTH)
        )
        contentView.bjy_group_chat_editText.editableText.clear()
        contentView.bjy_group_chat_editText.text = contentView.bjy_group_chat_editText.editableText.append(messageText)
        contentView.bjy_group_chat_editText.requestFocus()

        contentView.bjy_group_dialog_message_emoji.setOnClickListener {
            callBack.onSendEmoji()
            KeyboardUtils.hideSoftInput(contentView.bjy_group_chat_editText)
            dismissAllowingStateLoss()
        }

        contentView.bjy_group_text_send_iv.setOnClickListener {
            messageText = contentView.bjy_group_chat_editText.text.toString()
            if (TextUtils.isEmpty(messageText)) {
                callBack.onSendPicture()
            } else {
                callBack.onSendMessage(messageText)
                contentView.bjy_group_chat_editText.text.clear()
                KeyboardUtils.hideSoftInput(contentView.bjy_group_chat_editText)
            }
            dismissAllowingStateLoss()
        }
        val sharedPreferences = context?.getSharedPreferences(QUICK_REPLY_FILE_NAME, Context.MODE_PRIVATE)
        if (sharedPreferences != null) {
            chatQuickReplyMap = sharedPreferences.all as MutableMap<String, Int>?
        }
        combineListAndMap()
        if (chatQuickReplyMap.isNullOrEmpty()) {
            contentView.bjy_group_dialog_quick_reply_sclv.visibility = View.GONE
        } else {
            contentView.bjy_group_dialog_quick_reply_sclv.visibility = View.VISIBLE
            updateChatQuickReplyArea()
        }
    }

    override fun setWindowParams(windowParams: WindowManager.LayoutParams) {
        windowParams.height = WindowManager.LayoutParams.WRAP_CONTENT
        windowParams.width = WindowManager.LayoutParams.MATCH_PARENT
        windowParams.gravity = Gravity.BOTTOM or Gravity.END
        windowParams.windowAnimations = R.style.BJYGroupLiveBaseSendMsgDialogAnim
    }

    fun setMessage(message: String) {
        this.messageText = message
    }

    fun setMessageCallBack(messageSendCallBack: MessageSendCallBack) {
        this.callBack = messageSendCallBack
    }

    fun setQuickReplyList(list: List<String>?) {
        this.chatQuickReplyList = list
        combineListAndMap()
    }

    private fun combineListAndMap() {
        if (chatQuickReplyMap == null) {
            chatQuickReplyMap = mutableMapOf()
        }
        if (chatQuickReplyList == null) {
            chatQuickReplyList = ArrayList()
        }
        for (reply in chatQuickReplyList!!) {
            if (chatQuickReplyMap!!.contains(reply)) {
                continue
            } else {
                chatQuickReplyMap!![reply] = 0
            }
        }
        val deleteReplyList = mutableListOf<String>()
        for (reply in chatQuickReplyMap!!.keys) {
            if (!chatQuickReplyList!!.contains(reply)) {
                deleteReplyList.add(reply)
            }
        }
        for (reply in deleteReplyList) {
            chatQuickReplyMap!!.remove(reply)
        }
    }

    override fun onResume() {
        super.onResume()
        contentView.bjy_group_chat_editText.editableText.clear()
        contentView.bjy_group_chat_editText.text = contentView.bjy_group_chat_editText.editableText.append(messageText)
        contentView.bjy_group_chat_editText.setSelection(messageText.length)
        updateChatQuickReplyArea()
    }

    override fun onDetach() {
        super.onDetach()
        activity?.run {
            if (!isFinishing && !isDestroyed) {
                messageText = contentView.bjy_group_chat_editText.text.toString()
                callBack.onMessageEdited(messageText)
            }
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        val sharedPreferences = context?.getSharedPreferences(QUICK_REPLY_FILE_NAME, Context.MODE_PRIVATE)
        if (sharedPreferences != null && !chatQuickReplyMap.isNullOrEmpty()) {
            sharedPreferences.edit().let {
                it.clear()
                for (reply in chatQuickReplyMap!!.entries) {
                    it.putInt(reply.key, reply.value)
                }
                it.apply()
            }
        }
    }

    private fun updateChatQuickReplyArea() {
        if (chatQuickReplyMap.isNullOrEmpty()) {
            return
        }
        val chatQuickReplyList = mutableListOf<MutableMap.MutableEntry<String, Int>>()
        for (entry in chatQuickReplyMap!!.entries) {
            chatQuickReplyList.add(entry)
        }
        chatQuickReplyList.sortWith { o1, o2 -> o2.value.compareTo(o1.value) }
        contentView.bjy_group_dialog_quick_reply_container.removeAllViews()
        for (quickReply in chatQuickReplyList) {
            val textView = TextView(context)
            textView.apply {
                maxLines = 1
                maxEms = 20
                ellipsize = TextUtils.TruncateAt.END
                setTextColor(Color.parseColor("#FF999999"))
                text = quickReply.key
                background = ContextCompat.getDrawable(context!!, R.drawable.bjy_group_bg_chat_quick_reply)
                setPadding(11, 4, 11, 4)
                setOnClickListener { v: View? ->
                    contentView.bjy_group_chat_editText.post {
                        contentView.bjy_group_chat_editText.setText(textView.text)
                        contentView.bjy_group_chat_editText.setSelection(textView.text.length)
                    }
                    chatQuickReplyMap!![textView.text as String] = chatQuickReplyMap!![textView.text as String]!! + 1
                }
            }
            val layoutParams = LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT)
            layoutParams.setMargins(20, 0, 20, 0)
            contentView.bjy_group_dialog_quick_reply_container.addView(textView, layoutParams)
        }
    }

    interface MessageSendCallBack {
        fun onMessageEdited(message: String)
        fun onSendEmoji()
        fun onSendPicture()
        fun onSendMessage(message: String)
    }
}
