package com.baijiayun.groupclassui.container;

import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.layer.BaseLayerOfProfessionalClass;
import com.baijiayun.groupclassui.model.VideoMenuControlModel;
import com.baijiayun.groupclassui.window.video.VideoMenuMoreWindow;
import com.baijiayun.groupclassui.window.video.VideoWindow;
import com.baijiayun.livebase.models.imodels.IUserModel;

/**
 * Created by Dujuncan on 2018/11/8
 * todo popupWindow替换
 */
public class VideoMenuContainer extends BaseLayerOfProfessionalClass {
    private VideoMenuControlModel videoMenuControlModel;
    private VideoMenuMoreWindow menuMoreWindow;

    public VideoMenuContainer(@NonNull Context context) {
        this(context, null);
    }

    public VideoMenuContainer(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public VideoMenuContainer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public VideoMenuContainer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init();
    }

    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if(isLayerInit && isRoomActive) {
            subscribe();
        }
        return true;
    }

    private void subscribe(){
        compositeDisposable.add(router.getSubjectByKey(EventKey.HideVideoMenu)
                .ofType(IUserModel.class)
                .subscribe(iUserModel -> {
                    if (getChildCount() > 0) {
                        if (iUserModel.getUserId().equals(videoMenuControlModel.getVideoWindow().getUserId())) {
                            removeMenu();
                        } else {
                            menuMoreWindow.repositionMenu();
                        }
                    }
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.RepositionVideoMenu)
                .subscribe(o -> {
                    if (getChildCount() == 0) return;
                    menuMoreWindow.repositionMenu();
                }));

        compositeDisposable.add(router.getSubjectByKey(EventKey.StudyRoomSwitch)
                .subscribe(o -> removeMenu()));
    }

    @Override
    public boolean onTouchEvent(MotionEvent event) {
        boolean isConsume;
        if (getChildCount() > 0) {
            isConsume = true;
            videoMenuControlModel.getVideoWindow().setMenuState(false);
            removeAllViews();
            videoMenuControlModel = null;
            menuMoreWindow = null;
        } else {
            isConsume = false;
        }
        return super.onTouchEvent(event) || isConsume;
    }

    private int menuWidthDp;

    public void addMenu(VideoMenuControlModel videoMenuControlModel) {
        this.videoMenuControlModel = videoMenuControlModel;
        VideoWindow videoWindow = videoMenuControlModel.getVideoWindow();
        menuWidthDp = 80;
        if (videoWindow.isPPTAuthOn() || videoWindow.isAssistCameraOn() || videoWindow.isScreenShareOn() || videoWindow.isDrawAuthOn()) {
            menuWidthDp = 105;
        }
        if (router.getLiveRoom().getSpeakQueueVM().enableAttachPhoneCamera()) {
            menuWidthDp = 120;
        }
        menuMoreWindow = new VideoMenuMoreWindow(getContext(), menuWidthDp, videoMenuControlModel);
        View view = menuMoreWindow.getView();
        int[] viewSize = DisplayUtils.getUnDisplayViewSize(view);
        LayoutParams params = new LayoutParams(UtilsKt.getDp(menuWidthDp), viewSize[1]);
        params.leftMargin = videoMenuControlModel.getCurrX() - UtilsKt.getDp(menuWidthDp);
        params.topMargin =  videoMenuControlModel.getCurrY();
        view.setLayoutParams(params);

        boolean adjustWindow = adjustWindow(videoMenuControlModel, view, menuWidthDp);
        if (adjustWindow) {
            videoWindow.currentX = videoMenuControlModel.getCurrX() - videoWindow.getWindowPosition()[0];
            videoWindow.currentY = videoMenuControlModel.getCurrY() - videoWindow.getWindowPosition()[1];
            params = new LayoutParams(UtilsKt.getDp(menuWidthDp), viewSize[1]);
            params.leftMargin = videoMenuControlModel.getCurrX() - UtilsKt.getDp(menuWidthDp);
            params.topMargin =  videoMenuControlModel.getCurrY();
            view.setLayoutParams(params);
        }
        addView(view);
    }

    private boolean adjustWindow(VideoMenuControlModel videoMenuControlModel,View view,int menuWidthDp) {
        boolean adjust = false;
        int[] viewSize = DisplayUtils.getUnDisplayViewSize(view);
        int top = videoMenuControlModel.currY;
        int bottom = videoMenuControlModel.currY + viewSize[1];
        int left = (int) (videoMenuControlModel.currX - viewSize[0] / 2 - UtilsKt.getDp(menuWidthDp * 0.5f));
        int right = (int) (videoMenuControlModel.currX + viewSize[0] / 2 - UtilsKt.getDp(menuWidthDp * 0.5f));
        int offsetBottom = bottom - DisplayUtils.getScreenHeightPixels(view.getContext());
        int offsetRight = right - DisplayUtils.getScreenWidthPixels(view.getContext());
        int margin = UtilsKt.getDp(8);
        if (offsetBottom >= 0) {
            videoMenuControlModel.currY -= offsetBottom + margin ;
            adjust = true;
        }
        if (offsetRight >= 0) {
            videoMenuControlModel.currX -= offsetRight + margin;
            adjust = true;
        }
        if (top <= 0) {
            videoMenuControlModel.currY += -top + margin;
            adjust = true;
        }
        if (left <= 0) {
            videoMenuControlModel.currX += -left + margin;
            adjust = true;
        }
        return adjust;
    }

    public void removeMenu() {
        if (getChildCount() > 0) {
            removeAllViews();
            videoMenuControlModel = null;
            menuMoreWindow = null;
        }
    }
}
