package com.baijiayun.groupclassui.dialog

import android.annotation.SuppressLint
import android.os.Bundle
import android.view.Gravity
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.reactivex.RxUtils
import com.baijiayun.bjyutils.screen.DisplayUtils.getScreenHeightPixels
import com.baijiayun.bjyutils.screen.DisplayUtils.setRoundCorner
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.base.BaseGroupDialogFragment
import com.baijiayun.groupclassui.global.EventCode
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.groupclassui.util.convertTime
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.disposables.Disposable
import kotlinx.android.synthetic.main.bjysc_dialog_study_hang.*
import java.util.concurrent.TimeUnit

/**
 * Created by pengwenjian on 2020/12/10
 * Describe: 学生挂机dialog
 */
class StudyHangDialog : BaseGroupDialogFragment() {
    private var standardTimer: Disposable? = null
    var endHangListener: View.OnClickListener? = null
    var router: IRouter? = null

    override fun getLayoutId() = R.layout.bjysc_dialog_study_hang

    override fun init(savedInstanceState: Bundle?, arguments: Bundle?) {
        // do nothing
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        tvEndHang.background = DrawableBuilder()
            .solidColor(ContextCompat.getColor(view.context, R.color.base_warning_color))
            .cornerRadius(view.context.resources.getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
            .build()
        startTimer()
        router?.run {
            if (liveRoom.recorder.isAudioAttached) {
                getPublishSubjectByKey<Any>(EventCode.AttachAudio).onNext(false)
            }
            if (liveRoom.recorder.isVideoAttached) {
                getPublishSubjectByKey<Any>(EventCode.AttachVideo).onNext(false)
            }
        }
        tvEndHang.setOnClickListener {
            endHangListener?.onClick(view)
        }
    }

    @SuppressLint("DefaultLocale")
    private fun startTimer() {
        RxUtils.dispose(standardTimer)
        standardTimer = Observable.interval(0, 1, TimeUnit.SECONDS)
            .observeOn(AndroidSchedulers.mainThread())
            .subscribe { aLong: Long ->
                val time = convertTime(aLong)
                tvHangDuration.text = time
            }
    }

    override fun onStart() {
        super.onStart()
        dialog?.setCanceledOnTouchOutside(false)
        dialog?.setCancelable(false)
        view?.run {
            setRoundCorner(this, 4f.dp)
        }
    }

    override fun setWindowParams(windowParams: WindowManager.LayoutParams) {
        super.setWindowParams(windowParams)
        windowParams.height =
            ((context?.let { getScreenHeightPixels(it) } ?: 0) * 300f / 376).toInt()
        windowParams.width = (windowParams.height * 440f / 300).toInt()
        windowParams.gravity = Gravity.CENTER
    }

    override fun onDestroyView() {
        super.onDestroyView()
        RxUtils.dispose(standardTimer)
    }
}