package com.baijiayun.groupclassui.global;


import com.baijiayun.livecore.context.LiveRoom;

import java.util.List;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.concurrent.ConcurrentLinkedQueue;

import io.reactivex.subjects.BehaviorSubject;
import io.reactivex.subjects.PublishSubject;

/**
 * Created by Shubo on 2018/9/12.
 */
public interface IRouter {

    /**
     * 通过key获取Subject
     *
     * @param key
     * @param <T>
     * @return
     */
    <T> BehaviorSubject<T> getSubjectByKey(EventKey key);

    /**
     * 通过key获取Subject
     *
     * @param key EventKey
     * @param <T>
     * @return
     */
    <T> PublishSubject<T> getPublishSubjectByKey(EventCode key);

    /**
     * 通过Key获取 指定类型 值
     * @Warning 若 key 未设置值，则会返回null，可能会空指针！可以使用{@link #getValueByKey(EventKey, Class, Object)} 设置默认值
     * @param key EventKey
     * @param targetClass 指定类型
     * @return 如未设置值则默认返回 null
     */
    <T> T getValueByKey(EventKey key, Class<T> targetClass);

    /**
     * 通过Key获取 Boolean 值
     * @param key EventKey
     * @param targetClass 指定类型
     * @param defaultValue 如未设置返回的默认值
     */
    <T> T getValueByKey(EventKey key, Class<T> targetClass, T defaultValue);

    /**
     * 通过 Key 获取指定泛型的 Map，如未设置值则默认返回新的 Map（不为 null）。
     * @Warning 若 Key 中的 Map 中，key value 若不正确，则不会 put 进结果 Map
     * @param key EventKey
     * @param keyClass key 目标泛型类
     * @param valueClass value 目标泛型类
     * @return 指定泛型的 Map
     */
    <K, V> Map<K, V> getMapValueByKey(EventKey key, Class<K> keyClass, Class<V> valueClass);

    /**
     * 通过Key获取指定泛型的List，如未设置值则默认返回新的List（不为 null）。
     * @Warning 若Key中的List原本包含有 null 元素，本方法会剔除，故可能和目标 List 不一致
     * @param key EventKey
     * @param targetClass 目标泛型类
     * @return 指定泛型的List
     */
    <T> List<T> getListByKey(EventKey key, Class<T> targetClass);

    <T> Set<T> getSetByKey(EventKey key, Class<T> targetClass);

    void setObjectByKey(EventKey key, Object object);

    <T> ConcurrentLinkedQueue<T> getQueueByKey(EventKey key, Class<T> targetClass);

    /**
     * 获取subject的值
     * @param key
     * @param targetClass
     * @param <T>
     * @return
     */
    <T> T getSubjectValueByKey(EventKey key, Class<T> targetClass);

    <T> T getSubjectValueByKey(EventKey key, Class<T> targetClass,T defaultValue);

    /**
     * 获得LiveRoom
     */
    LiveRoom getLiveRoom();

    void setLiveRoom(LiveRoom liveRoom);
}
