package com.baijiayun.groupclassui.layer;

import android.content.Context;
import android.util.AttributeSet;

import androidx.annotation.Nullable;

import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.global.RouterListener;
import com.baijiayun.groupclassui.window.IRoomStatusListener;
import com.baijiayun.liveuibase.base.BaseLayer;

import org.jetbrains.annotations.NotNull;

/**
 * 专业小班课基础视图容器，继承自 FrameLayout
 * @author Shubo
 * @date 2018/9/11
 */
public class BaseLayerOfProfessionalClass extends BaseLayer implements  IRoomStatusListener {

    protected IRouter router;
    private boolean isLayerInit;
    // 是否延时回调
    private boolean laterCallback;
    private boolean isRoomActive;

    public BaseLayerOfProfessionalClass(@NotNull Context context) {
        super(context);
    }

    public BaseLayerOfProfessionalClass(@NotNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public BaseLayerOfProfessionalClass(@NotNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    public BaseLayerOfProfessionalClass(@NotNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @Override
    protected void init() {
        super.init();
        Context context = getContext();
        if (context instanceof RouterListener) {
            RouterListener routerListener = (RouterListener) context;
            router = routerListener.getRouter();
            ((RouterListener)context).addRoomStatusListener(this);
        }
    }

    /**
     * 房间状态更新回调
     * @param isRoomActive  true,进房间成功  false,退出房间
     */
    public boolean onRoomStatusChange(boolean isRoomActive){
        if(isLayerInit){
            if(isRoomActive){
                onRoomActive();
            } else{
                onRoomInActive();
            }
        } else{
            laterCallback = true;
        }
        this.isRoomActive = isRoomActive;
        return isLayerInit;
    }

    protected void onRoomActive() {
        // do nothing
    }

    protected void onRoomInActive() {
        compositeDisposable.clear();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (isDestroyRouter()) {
            router = null;
        }
    }

    /**
     * 如果切换容器则会走onDestroy
     * @return
     */
    protected boolean isDestroyRouter() {
        return true;
    }

    @Override
    protected void onDetachedFromWindow() {
        super.onDetachedFromWindow();
        if(getContext() != null && getContext() instanceof RouterListener){
            ((RouterListener) getContext()).removeRoomStatusListener(this);
        }
    }

    @Override
    protected void onAttachedToWindow() {
        super.onAttachedToWindow();
        isLayerInit = true;
        if(laterCallback){
            onRoomStatusChange(isRoomActive);
            laterCallback = false;
        }
    }
}
