package com.baijiayun.groupclassui.layer;

import android.annotation.TargetApi;
import android.content.Context;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.FragmentTransaction;
import androidx.appcompat.app.AppCompatActivity;
import android.util.AttributeSet;
import android.view.View;

import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.coursewaremanage.CourseManageWindow;

public class CoursewareLayer extends BaseLayerOfProfessionalClass {

    private CourseManageWindow courseManageWindow;

    public CoursewareLayer(@NonNull Context context) {
        super(context);
    }

    public CoursewareLayer(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public CoursewareLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @TargetApi(21)
    public CoursewareLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean layerInit = super.onRoomStatusChange(isRoomActive);
        if (layerInit && isRoomActive) {
            subscribe();
        }
        return layerInit;
    }

    private void subscribe() {
        compositeDisposable.add(router.getSubjectByKey(EventKey.CoursewareManageEnable)
                .ofType(Boolean.class)
                .subscribe(aBoolean -> {
                    if (aBoolean) {
                        showCoursewareWindow();
                        setVisibility(View.VISIBLE);
                    } else {
                        hideCoursewareWindow();
                        setVisibility(View.GONE);
                    }
                }));
    }

    private void showCoursewareWindow() {
        if (courseManageWindow == null) {
            courseManageWindow = new CourseManageWindow();
        }
        FragmentTransaction transaction = ((AppCompatActivity) getContext()).getSupportFragmentManager().beginTransaction();
        if (!courseManageWindow.isAdded()) {
            if (getId() == View.NO_ID) {
                setId(View.generateViewId());
            }
            transaction.add(getId(), courseManageWindow);
        }
        if (courseManageWindow.isHidden()) {
            transaction.show(courseManageWindow);
        }
        transaction.commitAllowingStateLoss();
    }

    private void hideCoursewareWindow() {
        FragmentTransaction transaction = ((AppCompatActivity) getContext()).getSupportFragmentManager().beginTransaction();
        if (courseManageWindow != null && courseManageWindow.isVisible()) {
            transaction.hide(courseManageWindow);
        }
        transaction.commitAllowingStateLoss();
    }

    @Override
    public void onDestroy() {
        if (courseManageWindow != null) {
            courseManageWindow.onDestroy();
        }
        courseManageWindow = null;
        super.onDestroy();
    }
}
