package com.baijiayun.groupclassui.layer;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.text.TextUtils;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.utils.ShadowUtil;

import java.text.NumberFormat;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

public class ReminderLayer extends BaseLayerOfProfessionalClass {

    private Disposable disposableOfUserIn, disposableOfUserOut,
            disposableOfMessage, disposableOfErrorMessage, disposableOfUpdateBlackboardPage, BBPageShowTimer,
            disposableOfClassStart, disposableOfSpeakApplyResResult;

    private LinearLayout reminderContainer;

    // 黑板页数
    private FrameLayout bbPage;
    private TextView pageText;
    private LinearLayout containerClassStart;
    // 开始上课
    private LinearLayout interactiveClassStartClass;

    private boolean isShowWeakNetworkReminder = false;
    // 只初始化subscribe一次
    private boolean isInit;

    public ReminderLayer(@NonNull Context context) {
        super(context);
    }

    public ReminderLayer(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public ReminderLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @Override
    public void init(){
        super.init();
        LinearLayout constraintLayout = (LinearLayout) LayoutInflater.from(getContext()).inflate(R.layout.bjy_group_window_reminder, null);
        reminderContainer = constraintLayout.findViewById(R.id.window_reminder_content);
        addView(constraintLayout);
        addBlackboardPageView();
    }

    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if(isLayerInit && isRoomActive){
            if (!isInit) {
                initListener();
            }
            //router.clear()会移除subscribe()里面的key，需重新监听
            subscribe();
        }
        return true;
    }

    public void showStartClassButton() {
        if (router.getLiveRoom().getStudyRoomVM().isStudyRoom()) {
            return;
        }
        if (containerClassStart == null) {
            return;
        }
        UtilsKt.removeViewFromParent(containerClassStart);
        addView(containerClassStart);
    }

    private boolean isShowClassStart() {
        return !router.getLiveRoom().getStudyRoomVM().isStudyRoom() && (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher
                || (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant &&
                router.getLiveRoom().getPartnerConfig().enableAssistantStartClass == 1));
    }

    private void initListener() {
        isInit = true;
        if (isShowClassStart()) {
            containerClassStart = (LinearLayout) LayoutInflater.from(getContext()).inflate(R.layout.bjy_group_item_start_class, null);
            interactiveClassStartClass = containerClassStart.findViewById(R.id.bjysc_start_class);
            Context context = interactiveClassStartClass.getContext();
            interactiveClassStartClass.setBackground(new DrawableBuilder()
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                    .cornerRadius(UtilsKt.getDp(4))
                    .build());
            interactiveClassStartClass.setOnClickListener(v -> {
                if (router.getLiveRoom().getRecorder() == null || router.getLiveRoom().getPlayer() == null) {
                    return;
                }
                router.getLiveRoom().requestClassStart();
                router.getSubjectByKey(EventKey.CheckSyncPreviewDoc).onNext(true);
                if (containerClassStart.getParent() instanceof ViewGroup) {
                    ((ViewGroup) containerClassStart.getParent()).removeView(containerClassStart);
                }
            });
        }
        if (isShowClassStart() && !router.getLiveRoom().isClassStarted()) {
            showStartClassButton();
        }

        disposableOfMessage = router.getSubjectByKey(EventKey.ReminderMessage)
                .ofType(String.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(ReminderLayer.this::addMessage);

        disposableOfErrorMessage = router.getSubjectByKey(EventKey.ReminderErrorMessage)
                .ofType(String.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(ReminderLayer.this::addErrorMessage);

        disposableOfUpdateBlackboardPage = router.getSubjectByKey(EventKey.ReminderBlackboardPage)
                .ofType(Float.class)
                .subscribe(y -> {
                    NumberFormat format = NumberFormat.getNumberInstance();
                    format.setMaximumFractionDigits(1);
                    int totalPage = router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE ? router.getLiveRoom().getPartnerConfig().live1v1BlackboardPages : router.getLiveRoom().getPartnerConfig().liveBlackboardPages;
                    String s = format.format(y * (totalPage - 1) + 1f) + "/" + totalPage;
                    pageText.setText(s);
                    bbPage.setBackground(getContext().getResources().getDrawable(R.drawable.bg_view_blackboard_page));
                    bbPage.setVisibility(router.getLiveRoom().getSmallClassTemplateType() != LPConstants.SmallClassTemplateType.ONEONONE ? VISIBLE : GONE);
                    if(router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE){
                        bbPage.setVisibility(GONE);
                        if(router.getLiveRoom().getPartnerConfig().live1v1BlackboardPages > 1){
                            bbPage.setVisibility(VISIBLE);
                        }
                    } else{
                        bbPage.setVisibility(VISIBLE);
                    }
                    RxUtils.dispose(BBPageShowTimer);
                    BBPageShowTimer = Observable.timer(1, TimeUnit.SECONDS)
                            .observeOn(AndroidSchedulers.mainThread())
                            .subscribe(aLong -> bbPage.setVisibility(GONE));
                });
    }

    private void subscribe() {
        if (BaseUtilsKt.isAdmin(router.getLiveRoom())) {
            disposableOfSpeakApplyResResult = router.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakApplyResResult()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(iMediaControlModel -> {
                        if (!iMediaControlModel.isApplyAgreed()) {
                            addMessage(getContext().getString(R.string.bjysc_speak_apply_res_result));
                        }
                    });
        }

        disposableOfClassStart = router.getLiveRoom().getObservableOfClassStart()
                .filter(integer -> router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher ||
                        router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant)
                .subscribe(integer -> {
                    if (containerClassStart != null && containerClassStart.getParent() != null) {
                        ((ViewGroup) containerClassStart.getParent()).removeView(containerClassStart);
                    }
                });
        disposableOfUserIn = router.getLiveRoom().getOnlineUserVM().getObservableOfUserIn()
                .filter(iUserInModel -> router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher)
                .subscribe(iUserInModel -> {
                    String message = iUserInModel.getUser().getName() + "  " + getContext().getString(R.string.bjysc_enter_room);
                    addMessage(message);
                });

        disposableOfUserOut = router.getLiveRoom().getOnlineUserVM().getObservableOfUserOut()
                .filter(iUserInModel -> router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher)
                .subscribe(userModel -> {
                    addMessage(userModel.getName() + "  " + getContext().getString(R.string.bjysc_leave_room));
                });
    }


    private void addBlackboardPageView() {
        if (bbPage == null) {
            bbPage = new FrameLayout(getContext());
        }
        if (pageText == null) {
            pageText = new TextView(getContext());
        }
        FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(UtilsKt.getDp(60), UtilsKt.getDp(20));
        layoutParams.gravity = Gravity.CENTER_HORIZONTAL | Gravity.TOP;
        bbPage.setVisibility(GONE);
        layoutParams.topMargin = UtilsKt.getDp(24);
        bbPage.setLayoutParams(layoutParams);
        FrameLayout.LayoutParams textLayoutParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        textLayoutParams.gravity = Gravity.CENTER;
        pageText.setTextSize(12);
        pageText.setTextColor(Color.WHITE);
        bbPage.addView(pageText, textLayoutParams);
        addView(bbPage);
    }

    String preReconnectMessage = null;

    private void addMessage(String message) {
        if (TextUtils.isEmpty(message)) {
            return;
        }
        if (message.startsWith(getContext().getString(R.string.reconnecting))) {
            if (!TextUtils.isEmpty(preReconnectMessage) && reminderContainer.getChildCount() > 0) {
                reminderContainer.removeViewAt(0);
            }
            reminderContainer.addView(newLabel(message, true), 0);
            preReconnectMessage = message;
            return;
        }
        if (reminderContainer.getChildCount() >= 3) {
            reminderContainer.removeViewAt(reminderContainer.getChildCount() - 1);
        }
        if (isShowWeakNetworkReminder) {
            reminderContainer.addView(newLabel(message, false), 1);
        } else {
            reminderContainer.addView(newLabel(message, false), 0);
        }
    }

    private void addErrorMessage(String message) {
        if (TextUtils.isEmpty(message)) {
            return;
        }
        if (reminderContainer.getChildCount() >= 3) {
            reminderContainer.removeViewAt(reminderContainer.getChildCount() - 1);
        }
        reminderContainer.addView(newLabel(message, true), 0);
    }

    @SuppressLint("CheckResult")
    private RelativeLayout newLabel(String message, boolean isRedBg) {
        RelativeLayout label = new RelativeLayout(getContext());
        LinearLayout.LayoutParams lp = new LinearLayout.LayoutParams(
                getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_remind_label_width),
                getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_remind_label_height));
        lp.setMargins(0, 0, 0, 8);
        label.setLayoutParams(lp);

        if (isRedBg) {
            label.setBackground(getContext().getResources().getDrawable(R.drawable.ic_window_reminder_red_bg));
        } else {
            label.setBackground(new DrawableBuilder()
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_brand_container_color))
                    .cornerRadius(getContext().getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                    .build());
        }
        label.setOnClickListener(v -> {
            removeLabel(label, message);
        });

        TextView textView = new TextView(getContext());
        textView.setText(message);
        textView.setTextSize(12);
        textView.setTextColor(isRedBg ? Color.WHITE :
                ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_window_main_text_color));
        RelativeLayout.LayoutParams tp = new RelativeLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT);
        textView.setLayoutParams(tp);
        tp.setMarginStart(UtilsKt.getDp(10));
        tp.setMarginEnd(UtilsKt.getDp(10));
        textView.setSingleLine(true);
        textView.setEllipsize(TextUtils.TruncateAt.MIDDLE);
        textView.setGravity(Gravity.CENTER);
        textView.setTextAlignment(TEXT_ALIGNMENT_CENTER);

        label.addView(textView);
        // timer 内部会自动dispose
        Observable.timer(3, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    removeLabel(label, message);
                });
        ShadowUtil.setViewBoundShadow(label);
        return label;
    }

    private void removeLabel(View label, String message) {
        if (label.getParent() instanceof ViewGroup && !getContext().getString(R.string.bjysc_network_reminder).equals(message)) {
            ViewGroup viewGroup = (ViewGroup) label.getParent();
            viewGroup.removeView(label);
        }
    }

    private void unSubscribe(){
        RxUtils.dispose(disposableOfUserIn);
        RxUtils.dispose(disposableOfUserOut);
        RxUtils.dispose(disposableOfMessage);
        RxUtils.dispose(disposableOfErrorMessage);
        RxUtils.dispose(disposableOfUpdateBlackboardPage);
        RxUtils.dispose(BBPageShowTimer);
        RxUtils.dispose(disposableOfClassStart);
        RxUtils.dispose(disposableOfSpeakApplyResResult);
    }

    @Override
    public void onDestroy(){
        super.onDestroy();
        unSubscribe();
        removeAllViews();
        if(bbPage != null){
            bbPage.removeAllViews();
            bbPage = null;
        }
    }
}
