package com.baijiayun.groupclassui.layer;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;
import static com.baijiayun.bjyutils.kt.UtilsKt.isEmptyList;
import static com.baijiayun.bjyutils.kt.UtilsKt.removeViewFromParent;
import static com.baijiayun.liveuibase.widgets.AwardView.TYPE_AWARD_CUSTOM;

import android.content.Context;
import android.content.pm.PackageManager;
import android.os.Build;
import android.util.AttributeSet;
import android.util.SparseArray;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.core.app.ActivityCompat;

import com.baijiayun.bjyrtcengine.Defines.BJYRtcCommon;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.model.NotifyAwardModel;
import com.baijiayun.groupclassui.model.PlayerViewActionModel;
import com.baijiayun.groupclassui.util.LayoutUtils;
import com.baijiayun.groupclassui.window.video.VideoContainerWindow;
import com.baijiayun.groupclassui.window.video.VideoWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.GroupItem;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.models.LPAnswerRacerEndModel;
import com.baijiayun.livecore.models.LPAwardUserInfo;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.livecore.models.LPMediaModel;
import com.baijiayun.livecore.models.LPPlayerViewUpdateModel;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.livecore.models.launch.LPAwardConfig;
import com.baijiayun.livecore.models.roomresponse.LPResRoomActiveUserModel;
import com.baijiayun.livecore.models.roomresponse.LPResRoomUserInModel;
import com.baijiayun.liveuibase.base.IWindow;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.widgets.AwardView;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;

/**
 * 视频列表layer
 * Created by Dujuncan on 2018/10/12.
 */
public class SeatLayer extends BaseLayerOfProfessionalClass {

    private static final String TAG = SeatLayer.class.getCanonicalName();
    private LayoutParams emptyParams;
    protected LinkedHashMap<String, VideoContainerWindow> mSpeakVideos;//正在显示的视频列表
    protected List<IMediaModel> mSpeakList;//发言列表
    protected LiveRoom mLiveRoom;
    private VideoDragLayer videoDragLayer;
    private boolean isBoardLayout = true;//默认是板书布局
    protected boolean disableOtherStudentVideo = false; // 教室内 只看老师和自己
    private HashMap<String, LPAwardUserInfo> awardRecord = new HashMap<>();
    protected int maxSpeakers;//最大上台（发言）人数
    private LPMediaModel mySelfUserActiveModel;
    private boolean isInitActiveUserComplete;
    protected boolean hasPlaceHolder;//是否有老师占位图
    protected View emptyView;//老师占位图
    private int margin_1v1 = 8;
    private int menu_size_1v1 = 20;
    protected static final int MAX_SPEAKERS_SIZE = 2;
    private Set<String> drawingAuthList = new HashSet<>();
    private List<String> pptAuthList = new ArrayList<>();
    private List<String> assistCameraList = new ArrayList<>();
    private List<String> screenShareList = new ArrayList<>();
    protected LPConstants.StudyRoomMode studyRoomMode = LPConstants.StudyRoomMode.None;
    private boolean isOnlineUserCallback;
    private float videoRatio;

    public SeatLayer(@NonNull Context context) {
        super(context);
    }

    public SeatLayer(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public SeatLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public SeatLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @Override
    protected void init() { // 进入房间后调用
        super.init();
        emptyParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        mSpeakVideos = new LinkedHashMap<>();
        mSpeakList = new ArrayList<>();
        mLiveRoom = router.getLiveRoom();
        if (showTeacherPlaceholder()) {
            //老师占位
            addEmptyView();
        }
    }

    protected void addEmptyView() {
        if (hasPlaceHolder) {
            return;
        }
        if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            emptyView = new View(getContext());
        } else {
            emptyView = View.inflate(getContext(), R.layout.bjysc_layout_video_empty, null);
        }
        int margin = UtilsKt.getDp(1);
        emptyParams.setMargins(margin, margin, margin, margin);
        addView(emptyView, 0, emptyParams);
        hasPlaceHolder = true;
    }

    protected void removeEmptyView() {
        if (hasPlaceHolder) {
            hasPlaceHolder = false;
            removeView(emptyView);
        }
    }

    protected boolean showTeacherPlaceholder() {
        return !hasPlaceHolder;
    }

    @Override
    public boolean onRoomStatusChange(boolean isRoomActive) {
        boolean isLayerInit = super.onRoomStatusChange(isRoomActive);
        if (isLayerInit) {
            if (isRoomActive) {
                LPLogger.d("wtf2", "seatLayer subscribe");
                mLiveRoom = router.getLiveRoom();
                initView();
            } else {
                //seatLayer销毁重建了，可以直接调onDestroy
                removeViewFromParent(this);
                onDestroy();
                LPLogger.d("wtf2", "seatLayer destroy");
            }
        }
        return true;
    }

    //首先判断教室类型，教室分自动上台教室、手动上台2种.
    //1.自动上台教室,2.手动上台教室。第一期只做自动上台的。

    protected void initView() {
        videoRatio = 9 / 16f;
        if (!mLiveRoom.enableFixChatPanel1v1()) {
            margin_1v1 = 8;
            menu_size_1v1 = 20;
        } else {
            margin_1v1 = 1;
            menu_size_1v1 = 0;
        }
        maxSpeakers = mLiveRoom.getMaxActiveUsers();
        LPLogger.d(TAG, "maxSpeakers=" + maxSpeakers);

        Disposable activeUsersDisposable = router.getLiveRoom().getSpeakQueueVM().getObservableOfActiveUsers()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModels -> {
                    // 刚进教室后回调一次
                    mSpeakList.clear();
                    mSpeakList.addAll(iMediaModels);
                    // 学生、助教的屏幕共享流和替换主摄像头流
                    for (IMediaModel iMediaModel : iMediaModels) {
                        if (iMediaModel.getExtraStreams() != null && iMediaModel.getUser().getType() != LPConstants.LPUserType.Teacher
                                && router.getLiveRoom().getSmallClassTemplateType() != LPConstants.SmallClassTemplateType.ONEONONE) {
                            final List<? extends IMediaModel> extraStreams = iMediaModel.getExtraStreams();
                            if (extraStreams != null && !extraStreams.isEmpty()) {
                                for (IMediaModel extMediaModel : extraStreams) {
                                    // 主屏幕分享替换原数据
                                    if (extMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainScreenShare) {
                                        iMediaModel.setVideoOn(iMediaModel.isVideoOn() || extMediaModel.isVideoOn());
                                        break;
                                    }
                                }
                            }
                        }
                    }
                    //发言列表始终会包含老师和助教.
                    //收到user_active_res时，每次重置本地维护的上台列表，防止先收到user_active_add
                    if (mySelfUserActiveModel != null) {
                        mSpeakList.add(mySelfUserActiveModel);
                    }
                    isInitActiveUserComplete = true;
                    LPLogger.d(TAG, "active users=" + iMediaModels.size() + "....speak list=" + mSpeakList.size());

                    if (mLiveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
                        LPLogger.d(TAG, "active users : teacher init...");
                        //拉自己上台
                        if (!mSpeakVideos.containsKey(router.getLiveRoom().getCurrentUser().getUserId())) {
                            router.getLiveRoom().getOnlineUserVM().requestAddActiveUser(router.getLiveRoom().getCurrentUser());
                        }
                        if (router.getLiveRoom().getMaxActiveUsers() < router.getLiveRoom().getOnlineUserVM().getActiveUserList().size()) {
                            LPMediaModel mediaModel = new LPMediaModel();
                            mediaModel.user = (LPUserModel) router.getLiveRoom().getCurrentUser();
                            mediaModel.audioOn = false;
                            mediaModel.videoOn = router.getLiveRoom().isClassStarted();
                            mediaModel.keepAlive = true;
                            mSpeakList.add(mediaModel);
                            addVideo(mediaModel, router.getLiveRoom().isClassStarted());
                        }

                        for (IMediaModel iMediaModel : mSpeakList) { // 处理已经在座位席
                            if (!iMediaModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId()) && router.getLiveRoom().getOnlineUserVM().isActiveUser(iMediaModel.getUser())) {
                                addVideo(iMediaModel, iMediaModel.isVideoOn() || iMediaModel.isAudioOn());
                            }
                        }
                    } else if (mLiveRoom.getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                        LPLogger.d(TAG, "active users : assistant init...");
                        for (IMediaModel iMediaModel : mSpeakList) { // 处理已经在座位席
                            if (!iMediaModel.getUser().getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId()) && router.getLiveRoom().getOnlineUserVM().isActiveUser(iMediaModel.getUser())) {
                                addVideo(iMediaModel, iMediaModel.isVideoOn() || iMediaModel.isAudioOn());
                            }
                        }
                    } else {
                        //学生进来，添加视频
                        LPLogger.d(TAG, "active users : student init... " + mSpeakList.size());
                        for (IMediaModel iMediaModel : mSpeakList) { // 处理已经在座位席
                            addVideo(iMediaModel, iMediaModel.isVideoOn() || iMediaModel.isAudioOn());
                        }
                    }
                    //视频窗口初始完之后处理player_view_update cache广播，还原黑板区域的视频
                    sendSyncPlayView(mLiveRoom.getMediaVM().getPlayerViewUpdate());
                    router.getLiveRoom().getOnlineUserVM().loadMoreUser();
                });

        Disposable disposableOfUserMore = router.getLiveRoom().getOnlineUserVM().getObservableOfUserMore()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserModels -> {
                    // 老师把学生、助教拉上台站位，只使用第一次回调
                    if (!isOnlineUserCallback) {
                        requestUpToSeat(iUserModels);
                        isOnlineUserCallback = true;
                    }
                });
        Disposable classStartDisposable = router.getLiveRoom().getObservableOfClassStart()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(integer -> integer == 0)
                .subscribe(integer -> {
                    //不管是老师还是学生，上课后   直接开始播放视频
                    if (router.getLiveRoom().getOnlineUserVM().isActiveUser(router.getLiveRoom().getCurrentUser()) && !mSpeakVideos.containsKey(router.getLiveRoom().getCurrentUser().getUserId())) {
                        LPMediaModel mediaModel = new LPMediaModel();
                        mediaModel.user = (LPUserModel) router.getLiveRoom().getCurrentUser();
                        mediaModel.audioOn = false;
                        mediaModel.videoOn = true;
                        mediaModel.keepAlive = true;
                        mSpeakList.add(mediaModel);
                        //本地推流
                        addVideo(mediaModel, true);
                    }
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        if (videoContainerWindow.getVideoWindow().isLocalVideo()) {
                            videoContainerWindow.getVideoWindow().publishLocalStream();
                            break;
                        }
                    }
                });

        Disposable userInDisposable = mLiveRoom.getOnlineUserVM().getObservableOfUserIn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserInModel -> {
                    if (router.getLiveRoom().getCurrentUser().getType() != LPConstants.LPUserType.Teacher) {
                        return;
                    }
                    if (iUserInModel.getUser().getType() == LPConstants.LPUserType.Assistant) {
                        return;
                    }
                    if (mSpeakVideos.size() > maxSpeakers) {
                        return;
                    }
                    //如果是老师，有学生进教室就发送"user_active_add_trigger".
                    if (!autoOnStage()) {
                        return;
                    }
                    router.getLiveRoom().getOnlineUserVM().requestAddActiveUser(iUserInModel.getUser());
                });

        Disposable userOutDisposable = mLiveRoom.getOnlineUserVM().getObservableOfUserOut()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userModel -> {
                    LPLogger.d(TAG, "user out");
                    if (mSpeakVideos.get(userModel.getUserId()) == null) {
                        return;
                    }
                    LPLogger.d(TAG, "user out" + mSpeakVideos.get(userModel.getUserId()).getMediaModel().getUser().getName());

                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        if (videoContainerWindow.getVideoWindow().getUserId().equals(userModel.getUserId())) {
                            removeVideo(videoContainerWindow.getMediaModel());
                            break;
                        }
                    }
                });

        //发言状态
        Disposable mediaStateDisposable = mLiveRoom.getSpeakQueueVM()
                .getObservableOfMediaPublish()
                .filter(iMediaModel -> {
                    if (iMediaModel.getUser().getType() == LPConstants.LPUserType.Teacher
                            || router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
                        return iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainCamera;
                    } else {
                        return iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainCamera
                                || iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainScreenShare;
                    }
                })
                .map((Function<IMediaModel, IMediaModel>) iMediaModel -> {
                    // 替换流 mediaId 取 userId
                    if (iMediaModel instanceof LPMediaModel) {
                        ((LPMediaModel) iMediaModel).mediaId = "";
                    } else if (iMediaModel instanceof LPResRoomActiveUserModel) {
                        ((LPResRoomActiveUserModel) iMediaModel).mediaId = "";
                    }
                    return iMediaModel;
                })
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    iMediaModel.setScreenShareFlag(iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainScreenShare
                            || iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.ExtScreenShare);
                    IUserModel modelUser = iMediaModel.getUser();
                    VideoContainerWindow videoContainerWindow = mSpeakVideos.get(modelUser.getUserId());
                    LPLogger.d(TAG, "media_publish: ...name=" + modelUser.getName() + "...uid=" + modelUser.getUserId() + "....video_on=" + iMediaModel.isVideoOn() + "....audio_on=" + iMediaModel.isAudioOn() + "......keep_alive=" + iMediaModel.isKeepAlive());
                    //判断是新的发言还是发言状态改变
                    if (iMediaModel.isAudioOn() || iMediaModel.isVideoOn() || iMediaModel.isKeepAlive()) {
                        if (videoContainerWindow == null) {
                            //新的发言
                            mSpeakList.add(iMediaModel);
                            addVideo(iMediaModel, iMediaModel.isAudioOn() || iMediaModel.isVideoOn());
                        } else {
                            //发言状态改变
                            if (!videoContainerWindow.getVideoWindow().getUserId().equals(mLiveRoom.getCurrentUser().getUserId())) {
                                iMediaModel.setScreenShareFlag(iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainScreenShare
                                        || iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.ExtScreenShare);
                                videoContainerWindow.setMediaModel(iMediaModel);
                                videoContainerWindow.getVideoWindow().changeVideoState(iMediaModel);
                                videoContainerWindow.getVideoWindow().setAudioOn(iMediaModel.isAudioOn());
                                videoContainerWindow.getVideoWindow().setVideoOn(iMediaModel.isVideoOn());
                            }
                        }
                    } else {
                        if (videoContainerWindow != null) {
                            removeVideo(videoContainerWindow.getMediaModel());
                        }
                    }
                });

        if (!BaseUtilsKt.isTeacher(router.getLiveRoom().getCurrentUser())) {
            compositeDisposable.add(mLiveRoom.getObservableOfDisableOtherStudentVideo()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(model -> {
                        if (!model.isFromCache()) {
                            router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(
                                    model.isDisable() ? R.string.bjysc_disable_other_student_video_tint
                                            : R.string.bjysc_enable_other_student_video_tint));
                        }
                        if (!BaseUtilsKt.isAdmin(mLiveRoom)) {
                            final boolean hideOtherStudentVideo = model.isDisable()
                                    || router.getSubjectValueByKey(EventKey.HideOtherStudentVideo, Boolean.class, false);
                            // 重新申请布局即可，在 layout 方法中回收要隐藏的布局
                            if (this.disableOtherStudentVideo == hideOtherStudentVideo) {
                                return;
                            }
                            this.disableOtherStudentVideo = hideOtherStudentVideo;
                            requestLayout();
                        }
                    }));
            compositeDisposable.add(router.getSubjectByKey(EventKey.HideOtherStudentVideo)
                    .ofType(Boolean.class)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(aBoolean -> {
                        if (!BaseUtilsKt.isAdmin(mLiveRoom)) {
                            disableOtherStudentVideo = aBoolean;
                            // 重新申请布局即可，在 layout 方法中回收要隐藏的布局
                            requestLayout();
                        }
                    }));
        }
        if (!BaseUtilsKt.isAdmin(mLiveRoom)) {
            disableOtherStudentVideo = mLiveRoom.getDisableOtherStudentVideoData().isDisable();
            requestLayout();
        }

        Disposable groupInfoDisposable = mLiveRoom.getOnlineUserVM().getObservableOfOnGroupItem()
                .subscribe(lpGroupItems -> {
                    if (lpGroupItems.size() > 0) {
                        cancelGroup();
                        setGroup(lpGroupItems);
                    } else {
                        cancelGroup();
                        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                            videoContainerWindow.getVideoWindow().setGroupItem(new GroupItem(0));
                        }
                    }
                });

        //举手监听
        Disposable speakApplyDisposable = mLiveRoom.getSpeakQueueVM().getObservableOfSpeakApply()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getContext().getString(R.string.bjysc_hans_up_from_student));
                    VideoContainerWindow videoContainerWindow = mSpeakVideos.get(iMediaModel.getUser().getUserId());
                    if (null == videoContainerWindow) {
                        return;
                    }
                    videoContainerWindow.getVideoWindow().showHandsUpIcon(true);
                });

        Disposable speakResponseDisposable = mLiveRoom.getSpeakQueueVM().getObservableOfSpeakResponse()
                // 响应抢答器邀请发言强制打开音视频信令，不在台上时才进行处理
                .mergeWith(
                        mLiveRoom.getSpeakQueueVM().getObservableOfMediaControl()
                                .filter(iMediaControlModel -> iMediaControlModel.isAudioOn() && iMediaControlModel.isVideoOn())
                )
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    LPLogger.d(TAG, "speak response: " + iMediaControlModel.isApplyAgreed());
                    if (!iMediaControlModel.isApplyAgreed()) {
                        return;
                    }
                    IUserModel iUserModel = iMediaControlModel.getUser();
                    if (iUserModel == null || !iUserModel.getUserId().equals(router.getLiveRoom().getCurrentUser().getUserId())) {
                        return;
                    }
                    VideoContainerWindow targetUserWindow = mSpeakVideos.get(iUserModel.getUserId());
                    if (targetUserWindow != null) {
                        targetUserWindow.getVideoWindow().attachAudio();
                    } else {
                        LPMediaModel mediaModel = new LPMediaModel();
                        mediaModel.user = (LPUserModel) iUserModel;
                        mediaModel.audioOn = iMediaControlModel.isAudioOn();
                        mediaModel.videoOn = iMediaControlModel.isVideoOn();
                        mediaModel.isHandUp = true;
                        //推流
                        addVideo(mediaModel, true);
                        mSpeakList.add(mediaModel);
                        if (handsAutoAttachAudio()) {
                            mLiveRoom.getRecorder().attachAudio();
                        }
                    }
                });

        Disposable speakApplyDenyDisposable = mLiveRoom.getSpeakQueueVM().getObservableOfSpeakApplyDeny()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> router.getSubjectByKey(EventKey.ReminderMessage).onNext(getResources().getString(R.string.bjysc_speak_apply_deny_limit)));

        //点赞奖励动画
        Disposable awardDisposable = mLiveRoom.getToolBoxVM().getObservableOfAward()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpInteractionAwardModel -> {
                    HashMap<String, LPAwardUserInfo> awardMap = lpInteractionAwardModel.value.getRecordAward();
                    awardRecord.putAll(awardMap);
                    if (lpInteractionAwardModel.isFromCache) {
                        for (Map.Entry<String, LPAwardUserInfo> entry : awardMap.entrySet()) {
                            IUserModel userModel = SeatLayer.this.getSpeakModelWithUserNumber(entry.getKey());
                            if (userModel != null) {
                                VideoContainerWindow videoContainerWindow = mSpeakVideos.get(userModel.getUserId());
                                if (videoContainerWindow != null) {
                                    videoContainerWindow.getVideoWindow().setAwardCount(entry.getValue().count);
                                }
                            }
                        }
                    } else if ("-1".equals(lpInteractionAwardModel.value.to)) {
                        // 台上奖励
                        for (Map.Entry<String, VideoContainerWindow> entry : mSpeakVideos.entrySet()) {
                            VideoWindow videoWindow = entry.getValue().getVideoWindow();
                            // 更新每个视频窗
                            LPAwardUserInfo awardUserInfo = awardMap.get(videoWindow.getUserModel().getNumber());
                            if (awardUserInfo != null) {
                                videoWindow.setAwardCount(awardUserInfo.count);
                            }
                            if (lpInteractionAwardModel.value.toStudents != null && lpInteractionAwardModel.value.toStudents.size() != 0) {
                                // -1但toStudents有值时隐藏动效
                                continue;
                            }
                            // 发送播放动画请求
                            router.getSubjectByKey(EventKey.ShowAward).onNext(new NotifyAwardModel(true));
                        }
                    } else {
                        IUserModel userModel = SeatLayer.this.getSpeakModelWithUserNumber(lpInteractionAwardModel.value.to);
                        if (userModel == null) {
                            return;
                        }
                        VideoContainerWindow videoContainerWindow = mSpeakVideos.get(userModel.getUserId());
                        if (awardMap.get(lpInteractionAwardModel.value.to) == null) {
                            return;
                        }
                        LPAwardUserInfo awardUserInfo = awardMap.get(lpInteractionAwardModel.value.to);
                        if (awardUserInfo == null) {
                            return;
                        }
                        int[] location = new int[2];
                        if (videoContainerWindow != null && !videoContainerWindow.isJoinedBlackboard()) {
                            videoContainerWindow.getVideoWindow().setAwardCount(awardUserInfo.count);
                            videoContainerWindow.getView().getLocationOnScreen(location);
                            location[0] += videoContainerWindow.getView().getMeasuredWidth() / 2;
                            location[1] += videoContainerWindow.getView().getMeasuredHeight() / 2;
                            if (awardMap.containsKey(lpInteractionAwardModel.value.to)) {
                                router.getSubjectByKey(EventKey.ShowAward).onNext(
                                        new NotifyAwardModel(userModel, false,
                                                location, lpInteractionAwardModel.value.pictureUrl));
                            }
                        } else {
                            if (awardMap.containsKey(lpInteractionAwardModel.value.to)) {
                                router.getSubjectByKey(EventKey.ShowAwardTransfer).onNext(
                                        new NotifyAwardModel(userModel, awardUserInfo.count,
                                                location, lpInteractionAwardModel.value.pictureUrl));
                            }
                        }
                    }
                });

        compositeDisposable.add(router.getLiveRoom().getToolBoxVM().getObservableOfGroupAward()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpGroupAwardModel -> {
                    LPUserModel userModel;
                    int[] location = new int[2];
                    //全员点赞
                    if (lpGroupAwardModel.group == 0) {
                        userModel = new LPUserModel();
                        userModel.name = "全体奖励";
                        router.getSubjectByKey(EventKey.ShowAward).onNext(new NotifyAwardModel(AwardView.TYPE_AWARD_ALL, userModel, location));
                    } else {
                        //组点赞
                        userModel = new LPUserModel();
                        userModel.name = lpGroupAwardModel.groupName;
                        router.getSubjectByKey(EventKey.GroupAward).onNext(lpGroupAwardModel);
                        router.getSubjectByKey(EventKey.ShowAward).onNext(new NotifyAwardModel(AwardView.TYPE_AWARD_GROUP, userModel, location));
                    }
                }));

        //抢答器奖励动画
        compositeDisposable.add(mLiveRoom.getToolBoxVM().getObservableOfAnswerRacerEnd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpAnswerRacerEndModel -> {
                    LPAnswerRacerEndModel answerRacerEndModel = new LPAnswerRacerEndModel(lpAnswerRacerEndModel);
                    if (answerRacerEndModel.winner != null) {
                        if (answerRacerEndModel.winner.name.length() > 3) {
                            answerRacerEndModel.winner.name = "\"" + answerRacerEndModel.winner.name.substring(0, 3) + "...\"" + getResources().getString(R.string.base_responder_success);
                        } else {
                            answerRacerEndModel.winner.name = "\"" + answerRacerEndModel.winner.name + "\"" + getResources().getString(R.string.base_responder_success);
                        }
                    }
                    if (!answerRacerEndModel.isRevoke && answerRacerEndModel.winner != null) {
                        IUserModel userModel = SeatLayer.this.getSpeakModelWithUserNumber(answerRacerEndModel.winner.getNumber());
                        int[] location = new int[2];
                        if (userModel == null) {
                            router.getSubjectByKey(EventKey.ShowAwardTransfer).onNext(new NotifyAwardModel(answerRacerEndModel.winner, false, location, LPAwardConfig.AWARD_TYPE_KEY_PRAISE));
                            return;
                        }
                        VideoContainerWindow videoContainerWindow = mSpeakVideos.get(userModel.getUserId());
                        if (videoContainerWindow != null && !videoContainerWindow.isJoinedBlackboard()) {
                            videoContainerWindow.getView().getLocationOnScreen(location);
                            location[0] += videoContainerWindow.getView().getMeasuredWidth() / 2;
                            location[1] += videoContainerWindow.getView().getMeasuredHeight() / 2;
                            router.getSubjectByKey(EventKey.ShowAward).onNext(new NotifyAwardModel(TYPE_AWARD_CUSTOM, answerRacerEndModel.winner, location));
                        } else {
                            router.getSubjectByKey(EventKey.ShowAwardTransfer).onNext(new NotifyAwardModel(answerRacerEndModel.winner, false, location, LPAwardConfig.AWARD_TYPE_KEY_PRAISE));
                        }
                    }
                }));

        Disposable micOnDisposable = router.getLiveRoom().getRecorder().getObservableOfMicOn()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isMicOn -> {
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        if (videoContainerWindow.getVideoWindow().isLocalVideo()) {
                            videoContainerWindow.getVideoWindow().setAudioOn(isMicOn);
                            break;
                        }
                    }
                });

        Disposable drawAuthDisposable = mLiveRoom.getSpeakQueueVM().getPublishSubjectOfDrawingAuth()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isAuth -> {
                    this.drawingAuthList = mLiveRoom.getSpeakQueueVM().getStudentsDrawingAuthSet();
                    //授权列表有人。对应的视频显示授权图标，不在授权列表中的视频隐藏图标。
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        videoContainerWindow.getVideoWindow().setDrawAuthOn(
                                drawingAuthList.contains(videoContainerWindow.getVideoWindow().getUserModel().getNumber()));
                    }
                });

        Disposable pptAuthDisposable = mLiveRoom.getDocListVM().getPublishSubjectOfStudentPPTAuth()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(pptAuthList -> {
                    this.pptAuthList = pptAuthList;
                    if (pptAuthList == null || pptAuthList.isEmpty()) {
                        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                            videoContainerWindow.getVideoWindow().setPPTAuthOn(false);
                        }
                        return;
                    }
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        videoContainerWindow.getVideoWindow().setPPTAuthOn(pptAuthList.contains(videoContainerWindow.getVideoWindow().getUserModel().getNumber()));
                    }
                });
        Disposable assistCameraDisposable = mLiveRoom.getMediaVM().getPublishSubjectOfStudentExtCamera()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(assistCameraList -> {
                    this.assistCameraList = assistCameraList;
                    if (assistCameraList == null || assistCameraList.isEmpty()) {
                        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                            videoContainerWindow.getVideoWindow().setAssistCameraOn(false);
                        }
                        return;
                    }
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        videoContainerWindow.getVideoWindow().setAssistCameraOn(assistCameraList.contains(videoContainerWindow.getVideoWindow().getUserModel().getNumber()));
                    }
                });

        Disposable screenShareDisposable = mLiveRoom.getMediaVM().getPublishSubjectOfStudentScreenShare()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(screenShareList -> {
                    this.screenShareList = screenShareList;
                    if (isEmptyList(screenShareList)) {
                        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                            videoContainerWindow.getVideoWindow().setScreenShareOn(false);
                        }
                        return;
                    }
                    for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                        videoContainerWindow.getVideoWindow().setScreenShareOn(screenShareList.contains(videoContainerWindow.getVideoWindow().getUserModel().getNumber()));
                    }
                });
        Disposable playerViewRemoveDisposable = router.getSubjectByKey(EventKey.PlayerViewRemove)
                .ofType(VideoWindow.class)
                .filter(blackboardVideoWindow -> null != mSpeakVideos.get(blackboardVideoWindow.getUserId()))
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(blackboardVideoWindow -> {
                    VideoContainerWindow videoContainerWindow = mSpeakVideos.get(blackboardVideoWindow.getUserId());
                    if (videoContainerWindow == null) {
                        return;
                    }
                    videoContainerWindow.getVideoWindow().setZOrderMediaOverlay(false);
                    if (isBoardLayout()) {
                        //切回座位席，拉取小流
                        videoContainerWindow.getVideoWindow().setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_LOW);
                    }
                    if (blackboardVideoWindow.getMediaSourceType() == LPConstants.MediaSourceType.MainCamera || blackboardVideoWindow.isScreenShare()) {
                        videoContainerWindow.backToSeat(blackboardVideoWindow);
                    }
                });

        //上台
        Disposable subscriptionOfUpToSeatContainer = router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserAdd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomUserInModel -> {
                    LPUserModel userModel = (LPUserModel) lpResRoomUserInModel.getUser();
                    if (!router.getLiveRoom().getOnlineUserVM().isActiveUser(userModel)) {
                        return;
                    }
                    if (userModel.status == LPConstants.LPUserState.Invisible) {
                        return;
                    }
                    if (userModel.equals(router.getLiveRoom().getCurrentUser())) {
                        mySelfUserActiveModel = new LPMediaModel();
                        mySelfUserActiveModel.user = userModel;
                        mySelfUserActiveModel.videoOn = router.getLiveRoom().isClassStarted();
                        mySelfUserActiveModel.keepAlive = true;
                        if (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
                            mySelfUserActiveModel.audioOn = router.getLiveRoom().isClassStarted();
                        } else if (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                            mySelfUserActiveModel.audioOn = false;
                        } else {
                            mySelfUserActiveModel.audioOn = router.getLiveRoom().isClassStarted() && router.getLiveRoom().getPartnerConfig().liveStudentDefaultAudioOn == 1;
                        }
                        if (!isInitActiveUserComplete) {
                            return;
                        }
                    }
                    if (mSpeakVideos.get(userModel.getUserId()) == null) {
                        if (userModel.equals(router.getLiveRoom().getCurrentUser())) {
                            mSpeakList.add(mySelfUserActiveModel);
                            addVideo(mySelfUserActiveModel, router.getLiveRoom().isClassStarted());
                        } else {
                            LPMediaModel mediaModel = new LPMediaModel();
                            mediaModel.user = userModel;
                            mediaModel.audioOn = false;
                            mediaModel.videoOn = false;
                            mediaModel.keepAlive = true;
                            mSpeakList.add(mediaModel);
                            addVideo(mediaModel, router.getLiveRoom().isClassStarted());
                        }
                    }
                    router.getLiveRoom().getOnlineUserVM().updateMediaState();
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getResources().getString(R.string.bjysc_seat_up_tips, getEncodePhoneNumber(lpResRoomUserInModel.getUser().getName())));
                });

        //下台
        /* 注意 ！！！ 下台后要收回所有权限 ！！！ */
        Disposable subscriptionOfDownFromSeatContainer = router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserRemove()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomUserInModel -> {
                    LPLogger.d(TAG, "user_active_remove: " + lpResRoomUserInModel.getUser().getName());
                    IUserModel iUserModel = lpResRoomUserInModel.getUser();
                    VideoContainerWindow videoContainerWindow = mSpeakVideos.get(lpResRoomUserInModel.getUser().getUserId());
                    if (videoContainerWindow != null) {
                        if (router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
                            //学生下台时，缓存授权的颜色
                            Map<String, String> authPaintColors = router.getLiveRoom().getAuthPaintColor();
                            Map<String, String> cachePaintColors = router.getMapValueByKey(EventKey.CachePaintColorMap, String.class, String.class);
                            if (authPaintColors == null) {
                                authPaintColors = new HashMap<>();
                            }
                            for (String color : authPaintColors.keySet()) {
                                if (authPaintColors.get(color).equals(videoContainerWindow.getVideoWindow().getUserModel().getNumber())) {
                                    cachePaintColors.put(color, videoContainerWindow.getVideoWindow().getUserModel().getNumber());
                                    router.setObjectByKey(EventKey.CachePaintColorMap, cachePaintColors);
                                }
                            }
                        }
                        videoContainerWindow.getVideoWindow().closeAV();
                        removeVideo(videoContainerWindow.getMediaModel());
                    }
                    if (iUserModel.equals(router.getLiveRoom().getCurrentUser())) {
                        mySelfUserActiveModel = null;
                    }
                    router.getSubjectByKey(EventKey.ReminderMessage).onNext(getResources().getString(R.string.bjysc_seat_down_tips, getEncodePhoneNumber(lpResRoomUserInModel.getUser().getName())));
                });

        Disposable userAddDenyDisposable = router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserAddDeny()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomUserInModel -> {
                    onActiveUserDeny(lpResRoomUserInModel);
                });

        Disposable playerViewUpdateDisposable = mLiveRoom.getMediaVM().getObservableOfPlayerViewUpdate()
                .filter(viewUpdateModel -> viewUpdateModel.action != null && !mSpeakVideos.isEmpty())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(viewUpdateModel -> {
                    if (viewUpdateModel.action == LPConstants.LPPlayerAction.ADD) {
                        VideoContainerWindow videoContainerWindow = mSpeakVideos.get(viewUpdateModel.id);
                        if (null == videoContainerWindow) {
                            return;
                        }
                        videoContainerWindow.showPlaceholder();
                        router.getSubjectByKey(EventKey.PlayerViewAdd).onNext(new PlayerViewActionModel(videoContainerWindow.getMediaModel().getUser().getNumber(), videoContainerWindow.getVideoWindow(), LPPlayerViewUpdateModel.copy(viewUpdateModel)));
                    }
                });

        Disposable videoResumeButtonDisposable = router.getSubjectByKey(EventKey.ResumeButton)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isShowResumeButton -> {
                    if (BaseUtilsKt.isTeacher(router.getLiveRoom().getCurrentUser())) {
                        VideoContainerWindow teacherWindow = mSpeakVideos.get(mLiveRoom.getCurrentUser().getUserId());
                        if (teacherWindow != null) {
                            if (isShowResumeButton) {
                                teacherWindow.showResumeButton();
                            } else {
                                teacherWindow.hideResumeButton();
                            }
                        }
                    }
                });

        compositeDisposable.add(playerViewUpdateDisposable);
        compositeDisposable.add(classStartDisposable);
        compositeDisposable.add(disposableOfUserMore);
        compositeDisposable.add(activeUsersDisposable);
        compositeDisposable.add(userInDisposable);
        compositeDisposable.add(userOutDisposable);
        compositeDisposable.add(mediaStateDisposable);
        compositeDisposable.add(groupInfoDisposable);
        compositeDisposable.add(speakApplyDisposable);
        compositeDisposable.add(speakResponseDisposable);
        compositeDisposable.add(speakApplyDenyDisposable);
        compositeDisposable.add(awardDisposable);
        compositeDisposable.add(micOnDisposable);
        compositeDisposable.add(drawAuthDisposable);
        compositeDisposable.add(playerViewRemoveDisposable);
        compositeDisposable.add(subscriptionOfUpToSeatContainer);
        compositeDisposable.add(subscriptionOfDownFromSeatContainer);
        compositeDisposable.add(userAddDenyDisposable);
        compositeDisposable.add(pptAuthDisposable);
        compositeDisposable.add(assistCameraDisposable);
        compositeDisposable.add(screenShareDisposable);
        compositeDisposable.add(videoResumeButtonDisposable);
    }

    protected boolean handsAutoAttachAudio() {
        return true;
    }

    /**
     * 人数已满
     *
     * @param lpResRoomUserInModel
     */
    protected void onActiveUserDeny(LPResRoomUserInModel lpResRoomUserInModel) {
        router.getSubjectByKey(EventKey.ReminderErrorMessage).onNext(getContext().getString(R.string.bjy_group_seat_full_tip));
    }

    private void requestUpToSeat(List<IUserModel> iUserModels) {
        if (router.getLiveRoom().getCurrentUser().getType() != LPConstants.LPUserType.Teacher) {
            // 有老师即添加老师关闭音视频占位图
            for (IUserModel user : iUserModels) {
                if (user.getType() == LPConstants.LPUserType.Teacher && !mSpeakVideos.containsKey(user.getUserId())) {
                    LPMediaModel mediaModel = new LPMediaModel();
                    mediaModel.user = (LPUserModel) user;
                    mediaModel.audioOn = false;
                    mediaModel.videoOn = false;
                    mediaModel.keepAlive = true;
                    mSpeakList.add(mediaModel);
                    addVideo(mediaModel, router.getLiveRoom().isClassStarted());
                    break;
                }
            }
            return;
        }
        if (!autoOnStage()) {
            return; // 1vN+M 班型学生不自动上台
        }
        if (mSpeakVideos.size() > maxSpeakers) {
            return;
        }
        int userCount = iUserModels.size();
        // 不能直接取这个值遍历，应该取userCount遍历
        // 当userCount = 3时，maxSpeakers = 1;助教在中间会出现学生不能自动上台问题
        // 最大台上人数为1+N
        int maxCount = Math.min(userCount, maxSpeakers) - mSpeakVideos.size() + 1;
        int count = 0;
        for (int i = 0; i < userCount; i++) {
            IUserModel iUserModel = iUserModels.get(i);
            if (iUserModel.getType() == LPConstants.LPUserType.Student &&
                    !mSpeakVideos.containsKey(iUserModel.getUserId()) && count <= maxCount) {
                router.getLiveRoom().getOnlineUserVM().requestAddActiveUser(iUserModel);
                count++;
                LPLogger.d(TAG, "requestUpToSeat: " + iUserModel.getName());
            }
        }
    }

    /**
     * 是否自动上台
     */
    protected boolean autoOnStage() {
        //1vnvm班型根据配置项决定是否自动上台，其余班型自动上台
        if (router.getLiveRoom().getMaxBackUpUsers() > 0) {
            return router.getLiveRoom().autoOnStage();
        }
        return true;
    }

    private void sendSyncPlayView(LPPlayerViewUpdateModel viewUpdateModel) {
        if (!mSpeakList.isEmpty()) { // 首次收到playerViewUpdate
            if (viewUpdateModel == null) {
                viewUpdateModel = new LPPlayerViewUpdateModel();
                viewUpdateModel.all = new HashSet<>();
            }
            Set<LPPlayerViewUpdateModel.PlayerPosition> allSet = viewUpdateModel.all;
            router.setObjectByKey(EventKey.AllPlayerViewList, allSet);

            if (viewUpdateModel.all.size() > 0) {
                for (LPPlayerViewUpdateModel.PlayerPosition playerPosition : viewUpdateModel.all) {
                    if (getIUserModel(playerPosition.id) == null) {
                        continue;
                    }
                    if (mSpeakVideos.get(playerPosition.id) == null) {
                        IMediaModel model = getIUserModel(playerPosition.id);
                        addVideo(model, false);
                    }
                    VideoContainerWindow videoContainerWindow = mSpeakVideos.get(playerPosition.id);
                    //板书布局下不还原黑板区域视频
                    if (null != videoContainerWindow && isBoardLayout) {
                        LPPlayerViewUpdateModel lpPlayerViewUpdateModel = LPPlayerViewUpdateModel.copy(viewUpdateModel);
                        lpPlayerViewUpdateModel.id = playerPosition.id;
                        videoContainerWindow.showPlaceholder();
                        router.getSubjectByKey(EventKey.PlayerViewAdd).onNext(new PlayerViewActionModel(videoContainerWindow.getMediaModel().getUser().getNumber(), videoContainerWindow.getVideoWindow(), lpPlayerViewUpdateModel));
                    }
                }
            }
        }
    }

    private IMediaModel getIUserModel(String userId) {
        for (IMediaModel iMediaModel : mSpeakList) {
            if (iMediaModel.getUser().getUserId().equals(userId)) {
                return iMediaModel;
            }
        }
        return null;
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        int selfWidth = MeasureSpec.getSize(widthMeasureSpec);
        int selfHeight = MeasureSpec.getSize(heightMeasureSpec);
        setMeasuredDimension(selfWidth, selfHeight);

        int childCount = mSpeakVideos.values().size();
        if (mLiveRoom.getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            if (hasPlaceHolder && childCount < MAX_SPEAKERS_SIZE) {
                childCount++;
            }
            int childWidth, childHeight;
            int menuSize = UtilsKt.getDp(menu_size_1v1);
            childWidth = selfWidth;
            childHeight = (int) (childWidth * videoRatio + menuSize);
            int childWidthMeasureSpec = MeasureSpec.makeMeasureSpec(childWidth, MeasureSpec.AT_MOST);
            int childHeightMeasureSpec = MeasureSpec.makeMeasureSpec(childHeight, MeasureSpec.AT_MOST);
            for (int i = 0; i < childCount; i++) {
                View childAt = getChildAt(i);
                childAt.measure(childWidthMeasureSpec, childHeightMeasureSpec);
            }
        } else {
            if (hasPlaceHolder) {
                childCount++;
            }
            int childWidth;
            int childHeight;
            if (isBoardLayout) {
                //板书布局，水平居中排列
                //学生不满6个，布局居中,宽度按6等分计算。否则按比例平分铺满
                //因为还有一个老师，所以是7
                int maxEqualWindow = 6;
                childHeight = selfHeight;
                if (childCount <= maxEqualWindow) {
                    childWidth = (int) (childHeight / videoRatio);
                } else {
                    childWidth = selfWidth / childCount;
                }
            } else {
                int showWindowCount = hasPlaceHolder ? 1 : 0;
                Object[] videoWindows = mSpeakVideos.values().toArray();
                for (Object videoWindow : videoWindows) {
                    boolean isLocalVideo = ((VideoContainerWindow) videoWindow).getVideoWindow().isLocalVideo();
                    boolean isSameGroup = isLocalVideo || ((VideoContainerWindow) videoWindow).getMediaModel().getUser().getGroup() == router.getLiveRoom().getCurrentUser().getGroup();
                    boolean hideDifferentGroup = !isSameGroup && router.getLiveRoom().getOnlineUserVM().isMediaGroup() && router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student;
                    if (!hideDifferentGroup) {
                        // 画廊布局全部显示；本地是老师或助教，全部显示；教室内未开启不看其他学生，全部显示；其他情况，仅展示本地视频和老师
                        showWindowCount++;
                    }
                }
                if (showWindowCount == 0) {
                    childWidth = childHeight = 0;
                } else {
                    childWidth = selfWidth / LayoutUtils.getMaxNumPerLine(showWindowCount);
                    childHeight = selfHeight / LayoutUtils.getLineCount(showWindowCount);
                }
            }
            int childWidthMeasureSpec = MeasureSpec.makeMeasureSpec(childWidth, MeasureSpec.EXACTLY);
            int childHeightMeasureSpec = MeasureSpec.makeMeasureSpec(childHeight, MeasureSpec.EXACTLY);
            for (int i = 0; i < childCount; i++) {
                View childAt = getChildAt(i);
                childAt.measure(childWidthMeasureSpec, childHeightMeasureSpec);
            }
        }
    }

    @Override
    protected void onLayout(boolean changed, int left, int top, int right, int bottom) {
        //使用内部维护的视频列表。按顺序取，重新队列
        Object[] videoWindows = mSpeakVideos.values().toArray();
        int childCount = videoWindows.length;
        if (mLiveRoom.getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            if (hasPlaceHolder && childCount < MAX_SPEAKERS_SIZE) {
                childCount++;
            }
            int margin = UtilsKt.getDp(margin_1v1);
            if (isBoardLayout) {
                for (int i = 0; i < childCount; i++) {
                    View childAt = getChildAt(i);
                    int childTop, childBottom;
                    childTop = top + i * childAt.getMeasuredHeight();
                    childBottom = top + (i + 1) * childAt.getMeasuredHeight();
                    if (i != 0) {
                        childTop += margin;
                        childBottom += margin;
                    }
                    childAt.layout(left, childTop, right, childBottom);
                }
            } else {
                for (int i = 0; i < childCount; i++) {
                    View childAt = getChildAt(i);
                    switch (childCount) {
                        case 1:
                            childAt.layout(left, top + (getMeasuredHeight() - childAt.getMeasuredHeight()) / 2, right, bottom);
                            break;
                        case 2:
                            childAt.layout(left + right * i / 2, top + (getMeasuredHeight() - childAt.getMeasuredHeight()) / 2 + margin, right * (i + 1) / 2, bottom + margin);
                            break;
                        default:
                            break;
                    }
                }
            }
        } else {
            int placeHolderCount = hasPlaceHolder ? 1 : 0;
            // region 根据当前 disableOtherStudentVideo 状态重构
            // childCount 实际显示的窗口个数
            childCount = 0;
            for (Object videoWindow : videoWindows) {
                boolean isTeacherVideo = ((VideoContainerWindow) videoWindow).getMediaModel().getUser().getType() == LPConstants.LPUserType.Teacher
                        || ((VideoContainerWindow) videoWindow).getMediaModel().getUser().getType() == LPConstants.LPUserType.Assistant;
                boolean isLocalVideo = ((VideoContainerWindow) videoWindow).getVideoWindow().isLocalVideo();
                boolean isSameGroup = isLocalVideo || ((VideoContainerWindow) videoWindow).getMediaModel().getUser().getGroup() == router.getLiveRoom().getCurrentUser().getGroup();
                boolean hideDifferentGroup = !isSameGroup && router.getLiveRoom().getOnlineUserVM().isMediaGroup() && router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student;
                boolean showWindow = !disableOtherStudentVideo || isTeacherVideo || isLocalVideo
                        || BaseUtilsKt.isAdmin(router.getLiveRoom()) || !isBoardLayout;
                if (showWindow && !hideDifferentGroup) {
                    // 画廊布局全部显示；本地是老师或助教，全部显示；教室内未开启不看其他学生，全部显示；其他情况，仅展示本地视频和老师
                    childCount++;
                }
            }
            childCount += placeHolderCount;
            // endregion
            int hasAddCount = 0;
            int hideWindowCount = 0;
            for (int i = 0; i < videoWindows.length + placeHolderCount; i++) {
                View childAt = getChildAt(i);
                boolean disableOtherStudentVideo = false;
                boolean hideWindow = false;
                int childLeft, childRight, childTop, childBottom;
                if (!(hasPlaceHolder && i == 0)) {
                    boolean isTeacherOrAssistantVideo =
                            ((VideoContainerWindow) videoWindows[i - placeHolderCount])
                                    .getMediaModel().getUser().getType() == LPConstants.LPUserType.Teacher ||
                                    ((VideoContainerWindow) videoWindows[i - placeHolderCount])
                                            .getMediaModel().getUser().getType() == LPConstants.LPUserType.Assistant;
                    boolean isLocalVideo = ((VideoContainerWindow) videoWindows[i - placeHolderCount])
                            .getVideoWindow().isLocalVideo();
                    if (this.disableOtherStudentVideo && !isTeacherOrAssistantVideo && !isLocalVideo) {
                        disableOtherStudentVideo = true;
                    }
                    if (router.getLiveRoom().getOnlineUserVM().isMediaGroup() && router.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student &&
                            ((VideoContainerWindow) videoWindows[i - placeHolderCount]).getMediaModel().getUser().getGroup() != router.getLiveRoom().getCurrentUser().getGroup()) {
                        hideWindow = true;
                    }
                }
                if (isBoardLayout) {
                    if (disableOtherStudentVideo || hideWindow) {
                        // 不看其他学生，layout 不分配空间
                        childLeft = childRight = 0;
                    } else {
                        childLeft = (getMeasuredWidth() - childAt.getMeasuredWidth() * (childCount)) / 2
                                + childAt.getMeasuredWidth() * hasAddCount++;
                        childRight = childLeft + childAt.getMeasuredWidth();
                    }
                    childTop = top;
                    childBottom = bottom;
                } else {
                    if (hideWindow) {
                        childLeft = childRight = 0;
                        childTop = childBottom = 0;
                        hideWindowCount++;
                    } else {
                        if (childCount == 0) {
                            childLeft = childRight = 0;
                            childTop = childBottom = 0;
                        } else {
                            // 每行最多个数
                            int maxNumPerLine = LayoutUtils.getMaxNumPerLine(childCount);
                            // 当前行号
                            int nowLine = (i - hideWindowCount) / maxNumPerLine;
                            // region 上下计算
                            // 行高
                            int itemHeight = (int) (getMeasuredHeight() / Math.ceil((double) childCount / maxNumPerLine));
                            // region 特殊处理，只有两个时，上下也需要补齐空间
                            // endregion
                            // childTop = top + 前几行的高度 + 首行偏移
                            childTop = top + nowLine * itemHeight;
                            // region 非最后一行，加一像素空隙
                            boolean isLastLine = (nowLine + 1) * maxNumPerLine >= childCount;
                            childBottom = childTop + itemHeight - (isLastLine ? 0 : 1);
                            // endregion
                            // endregion

                            // region 左右计算
                            // 每个宽度
                            int itemWidth = getMeasuredWidth() / maxNumPerLine;
                            // 当前行的个数
                            int nowLineMaxNum = Math.min(childCount - maxNumPerLine * nowLine, maxNumPerLine);
                            // 根据当前行个数，和每行最多个数计算当前行首个的左偏移
                            int nowLineLeftOffset = (maxNumPerLine - nowLineMaxNum) * itemWidth / 2;

                            // childLeft = 行左偏移 + 前几个的宽度
                            childLeft = nowLineLeftOffset + (i - hideWindowCount - maxNumPerLine * nowLine) * itemWidth;

                            // region 非当前行最后一个，加一像素空隙
                            boolean isLastItemInNowLine = (i - hideWindowCount + 1) / maxNumPerLine != nowLine;
                            childRight = childLeft + itemWidth - (isLastItemInNowLine ? 0 : 1);
                            // endregion
                            // endregion
                        }
                    }
                }
                childAt.layout(childLeft, childTop, childRight, childBottom);
            }
        }
        //每次用户发生变化时，通知视频菜单.
        router.getSubjectByKey(EventKey.RepositionVideoMenu).onNext(0);
    }

    @Override
    public void addWindow(IWindow window) {
        VideoContainerWindow videoContainerWindow = (VideoContainerWindow) window;
        if (videoContainerWindow.getVideoWindow() != null) {
            videoContainerWindow.getVideoWindow().setOnlyInBlackboard(false);
        }
        if (videoContainerWindow.getMediaModel().getUser().getType() == LPConstants.LPUserType.Teacher) {
            removeEmptyView();
            addView(window.getView(), 0);
        } else if (videoContainerWindow.getMediaModel().getUser().getType() == LPConstants.LPUserType.Assistant) {
            if (router.getLiveRoom().getOnlineUserVM().enable1v1TeacherOffStage() && !router.getLiveRoom().getOnlineUserVM().isActiveUser(router.getLiveRoom().getTeacherUser())) {
                removeEmptyView();
                addView(window.getView(), 0);
                return;
            }
            //加到学生前一个位置即可
            final int childCount = getChildCount();
            int index = -1;
            for (int i = 0; i < childCount; i++) {
                final View child = getChildAt(i);
                final Object tag = child.getTag();
                if (tag instanceof Boolean) {
                    if ((Boolean) tag) {
                        index = i;
                        break;
                    }
                }
            }
            if (index == -1) {
                addView(window.getView());
            } else {
                addView(window.getView(), index);
            }
        } else {
            final View view = window.getView();
            view.setTag(true);//标记是学生
            addView(view);
        }
    }

    private void awardUser(IUserModel userModel, String key) {
        String userNumber = userModel.getNumber();
        if (!router.getLiveRoom().isClassStarted()) {
            return;
        }
        LPAwardUserInfo userInfo = awardRecord.get(userNumber);
        if (userInfo == null) {
            Map<String, Integer> typeCount = getTypeCount();
            typeCount.put(key, 1);
            userInfo = new LPAwardUserInfo(1, typeCount);
        } else {
            userInfo.count++;
            if (userInfo.typeCount == null) {
                userInfo.typeCount = getTypeCount();
            }
            Integer keyCount = userInfo.typeCount.get(key);
            if (keyCount == null) {
                userInfo.typeCount.put(key, 1);
            } else {
                userInfo.typeCount.put(key, keyCount + 1);
            }
        }
        userInfo.name = userModel.getName();
        awardRecord.put(userNumber, userInfo);
        router.getLiveRoom().getToolBoxVM().requestAward(userNumber, key, awardRecord);
    }

    private Map<String, Integer> getTypeCount() {
        Map<String, Integer> map = new HashMap<>();
        final LPAwardConfig[] awardConfigs = router.getLiveRoom().getToolBoxVM().getAwardConfigs();
        for (LPAwardConfig lpAwardConfig : awardConfigs) {
            if (lpAwardConfig.isEnable == 1) {
                map.put(lpAwardConfig.key, 0);
            }
        }
        return map;
    }

    /**
     * 只有active_user 和 media_publish触发playVideo
     * 修改isPlay=false后本地不能推流，因此其他位置的isPlay不修改。
     * 改为修改iMediaModel的video_on和audio_on
     * <p>
     * 注：add window 视图时，map 中的顺序和 view 的视图顺序应保持同步切换，否则会导致视图显示和 debug 中的数据不一致的问题
     * 故在这里在 操作 {@link #mSpeakVideos} 时，同样需要排序
     */
    protected void addVideo(IMediaModel iMediaModel, boolean isPlay) {
        if (iMediaModel == null || mSpeakVideos.containsKey(iMediaModel.getUser().getUserId())) {
            return;
        }
        IUserModel iUserModel = iMediaModel.getUser();
        VideoContainerWindow videoContainerWindow = new VideoContainerWindow(getContext(), VideoWindow.newInstance(getContext(), router), false);
        videoContainerWindow.initMediaModel(iMediaModel);
        videoContainerWindow.bindToSeat(SeatLayer.this);

        videoContainerWindow.getVideoWindow().setAwardClickListener(this::awardUser);

        if (iUserModel.getType() == LPConstants.LPUserType.Teacher) {
            LinkedHashMap<String, VideoContainerWindow> temp = new LinkedHashMap<>(mSpeakVideos);
            mSpeakVideos.clear();
            mSpeakVideos.put(iUserModel.getUserId(), videoContainerWindow);
            mSpeakVideos.putAll(temp);
            LPLogger.d(TAG, "addVideo: type= teacher..." + iUserModel.getName() + "...userId=" + iUserModel.getUserId() + "...videoOn=" + iMediaModel.isVideoOn() + "....audioOn=" + iMediaModel.isAudioOn());
        } else {
            if (iUserModel.getType() == LPConstants.LPUserType.Assistant) {
                LinkedHashMap<String, VideoContainerWindow> temp = new LinkedHashMap<>();
                boolean hasInsert = false;
                for (VideoContainerWindow videoWindow : mSpeakVideos.values()) {
                    if (!hasInsert && videoWindow.getMediaModel().getUser().getType() == LPConstants.LPUserType.Student) {
                        temp.put(iUserModel.getUserId(), videoContainerWindow);
                        hasInsert = true;
                    }
                    temp.put(videoWindow.getMediaModel().getUser().getUserId(), videoWindow);
                }
                if (!hasInsert) {
                    temp.put(iUserModel.getUserId(), videoContainerWindow);
                }
                mSpeakVideos.clear();
                mSpeakVideos.putAll(temp);
            } else {
                mSpeakVideos.put(iUserModel.getUserId(), videoContainerWindow);
            }
            // 权限初始化
            if (drawingAuthList != null && drawingAuthList.contains(iUserModel.getNumber())) {
                videoContainerWindow.getVideoWindow().setDrawAuthOn(true);
            }

            if (pptAuthList != null && pptAuthList.contains(iUserModel.getNumber())) {
                videoContainerWindow.getVideoWindow().setPPTAuthOn(true);
            }
            if (assistCameraList != null && assistCameraList.contains(iUserModel.getNumber())) {
                videoContainerWindow.getVideoWindow().setAssistCameraOn(true);
            }
            if (screenShareList != null && screenShareList.contains(iUserModel.getNumber())) {
                videoContainerWindow.getVideoWindow().setScreenShareOn(true);
            }
            if (awardRecord.get(iUserModel.getNumber()) != null) {
                videoContainerWindow.getVideoWindow().setAwardCount(awardRecord.get(iUserModel.getNumber()).count);
            }
            LPLogger.d(TAG, "addVideo: type= student..." + iUserModel.getName() + "...userId=" + iUserModel.getUserId() + "...videoOn=" + iMediaModel.isVideoOn() + "....audioOn=" + iMediaModel.isAudioOn());
        }
        addWindow(videoContainerWindow);

        if (!isPlay) {
            if (videoContainerWindow.getVideoWindow().getEnablePullAudioOnly()) {
                videoContainerWindow.getVideoWindow().setVideoRemoteClose(VideoWindow.REMOTE_VIDEO_AUTO_CLOSE);
            }
            videoContainerWindow.getVideoWindow().setVideoOn(false);
            videoContainerWindow.getVideoWindow().setAudioOn(false);
        } else {
            if (videoContainerWindow.getVideoWindow().isLocalVideo()) {
                // 举手状态强制开音频
                mSpeakVideos.get(router.getLiveRoom().getCurrentUser().getUserId()).getVideoWindow().publishLocalStream(iMediaModel.isHandUp());
                LPLogger.d(TAG, "addVideo: has permissions,publish......");
            } else {
                videoContainerWindow.getVideoWindow().changeVideoState(iMediaModel);
            }
        }

        final SparseArray<LPGroupItem> groupMap = router.getLiveRoom().getOnlineUserVM().getGroupMap();
        LPGroupItem lpGroupItem = groupMap.get(iUserModel.getGroup());
        if (lpGroupItem == null) {
            lpGroupItem = new LPGroupItem(0);
        }
        if (lpGroupItem.groupItemModel == null) {
            lpGroupItem.groupItemModel = new GroupItem(0);
        }
        if (lpGroupItem.id != 0) {
            videoContainerWindow.getVideoWindow().setGroupColor(lpGroupItem.groupItemModel.color);
        }
        if (router.getLiveRoom().getOnlineUserVM().isMediaGroup()) {
            videoContainerWindow.getVideoWindow().setGroupItem(lpGroupItem.groupItemModel);
        }
    }

    protected void removeVideo(IMediaModel iMediaModel) {
        String uid = iMediaModel.getUser().getUserId();
        VideoContainerWindow videoContainerWindow = mSpeakVideos.get(uid);
        if (videoContainerWindow != null) {
            //更新发言列表
            mSpeakList.remove(videoContainerWindow.getMediaModel());
            mSpeakVideos.remove(uid);
            videoContainerWindow.onDestroy();
            removeWindow(videoContainerWindow);
            if (iMediaModel.getUser().getType() == LPConstants.LPUserType.Teacher && showTeacherPlaceholder()) {
                boolean addEmptyView = true;
                if (!mSpeakList.isEmpty() && router.getLiveRoom().getOnlineUserVM().enable1v1TeacherOffStage()) {
                    final IMediaModel model = mSpeakList.get(0);
                    if (model.getUser().getType() == LPConstants.LPUserType.Assistant) {
                        final VideoContainerWindow containerWindow = mSpeakVideos.get(model.getUser().getUserId());
                        if (containerWindow != null) {
                            addEmptyView = false;
                        }
                    }
                }
                if (addEmptyView) {
                    addView(emptyView, 0, emptyParams);
                    hasPlaceHolder = true;
                }
            }
        }
        //每次有用户退出时，通知视频菜单.
        router.getSubjectByKey(EventKey.HideVideoMenu).onNext(iMediaModel.getUser());
    }


    private boolean hasPermissions(String[] permissions) {
        for (String permission : permissions) {
            if (ActivityCompat.checkSelfPermission(getContext(), permission) != PackageManager.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }

    private void setGroup(List<LPGroupItem> groups) {
        // 更新分组信息
        Map<String, VideoContainerWindow> speakNums = new HashMap<>();
        for (Map.Entry<String, VideoContainerWindow> entry : mSpeakVideos.entrySet()) {
            final VideoContainerWindow videoContainerWindow = entry.getValue();
            speakNums.put(videoContainerWindow.getMediaModel().getUser().getNumber(), videoContainerWindow);
        }
        int count = 0;
        for (LPGroupItem lpGroupItem : groups) {
            count += lpGroupItem.count;
        }
        if (count == 0) {
            for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
                videoContainerWindow.getVideoWindow().setGroupItem(new GroupItem(0));
            }
            requestLayout();
            return;
        }
        for (LPGroupItem lpGroupItem : groups) {
            final List<LPUserModel> userModelList = lpGroupItem.userModelList;
            for (LPUserModel lpUserModel : userModelList) {
                if (speakNums.containsKey(lpUserModel.number)) {
                    final GroupItem groupItemModel = lpGroupItem.groupItemModel;
                    if (groupItemModel != null) {
                        final VideoContainerWindow videoContainerWindow = speakNums.remove(lpUserModel.number);
                        if (videoContainerWindow != null) {
                            videoContainerWindow.getVideoWindow().setGroupColor(groupItemModel.color);
                            if (router.getLiveRoom().getOnlineUserVM().isMediaGroup()) {
                                videoContainerWindow.getVideoWindow().setGroupItem(groupItemModel);
                            } else {
                                if (groupItemModel.id == 0) {
                                    videoContainerWindow.getVideoWindow().setGroupItem(groupItemModel);
                                }
                            }
                        }
                    }
                }
            }
        }
        // 未在分组中的数据也需要处理
        for (VideoContainerWindow videoContainerWindow : speakNums.values()) {
            if (videoContainerWindow != null) {
                videoContainerWindow.getVideoWindow().setGroupItem(new GroupItem(0));
            }
        }
        requestLayout();
    }

    private void cancelGroup() {
        //如果已经分过组，解散分组,重置布局
        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
            videoContainerWindow.getVideoWindow().clearGroupColor();
        }
    }

    public void setVideoDragLayer(VideoDragLayer videoDragLayer) {
        this.videoDragLayer = videoDragLayer;
        this.videoDragLayer.setOnVideoResetListener((uid, videoWindow) -> {
            if (mSpeakVideos.containsKey(uid)) {
                mSpeakVideos.get(uid).backToSeat(videoWindow);
            }
        });
    }

    public VideoDragLayer getDragLayer() {
        return videoDragLayer;
    }

    //切换布局模式
    public void switchLayoutMode(boolean isBoardLayout) {
        this.isBoardLayout = isBoardLayout;
        if (isBoardLayout) {
            if (router.getValueByKey(EventKey.AllPlayerViewList, Set.class) == null) {
                return;
            }
            //切回板书布局恢复黑板区域的视频
            Set<LPPlayerViewUpdateModel.PlayerPosition> playerPositionList = new HashSet<>(router.getSetByKey(EventKey.AllPlayerViewList, LPPlayerViewUpdateModel.PlayerPosition.class));
            for (LPPlayerViewUpdateModel.PlayerPosition playerPosition : playerPositionList) {
                LPPlayerViewUpdateModel model = new LPPlayerViewUpdateModel();
                model.action = LPConstants.LPPlayerAction.ADD;
                model.all = playerPositionList;
                model.id = playerPosition.id;
                model.mediaId = playerPosition.mediaId;
                if (mSpeakVideos.isEmpty()) {
                    return;
                }
                VideoContainerWindow videoContainerWindow = mSpeakVideos.get(model.id);
                if (null == videoContainerWindow) {
                    continue;
                }
                videoContainerWindow.showPlaceholder();
                router.getSubjectByKey(EventKey.PlayerViewAdd).onNext(new PlayerViewActionModel(videoContainerWindow.getMediaModel().getUser().getNumber(), videoContainerWindow.getVideoWindow(), model));
            }
        }
        for (int i = 0; i < mSpeakVideos.size(); i++) {
            View bgView = ((VideoContainerWindow) mSpeakVideos.values().toArray()[i])
                    .getVideoWindow().getView();
            if (!(bgView.getLayoutParams() instanceof FrameLayout.LayoutParams)) {
                continue;
            }
            FrameLayout.LayoutParams bgParams = (LayoutParams) bgView.getLayoutParams();
            if (isBoardLayout) {
                bgParams.setMargins(0, 0, 0, 0);
            } else {
                bgParams.setMargins(1, 1, 1, 1);
            }
            bgView.setLayoutParams(bgParams);
        }
    }

    public boolean isBoardLayout() {
        return isBoardLayout;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        clearSpeakVideo();
        if (awardRecord != null) {
            awardRecord.clear();
        }
        videoDragLayer = null;
        mLiveRoom = null;
    }

    private void clearSpeakVideo() {
        if (mSpeakVideos == null || mSpeakVideos.isEmpty()) {
            return;
        }
        for (VideoContainerWindow videoContainerWindow : mSpeakVideos.values()) {
            videoContainerWindow.onDestroy();
        }
        mSpeakVideos.clear();
    }

    public IUserModel getSpeakModelWithUserNumber(String userNumber) {
        if (router.getLiveRoom().getCurrentUser().getNumber().equals(userNumber)) {
            return router.getLiveRoom().getCurrentUser();
        }

        int count = mLiveRoom.getOnlineUserVM().getUserCount();
        for (int i = 0; i < count; i++) {
            if (userNumber.equals(mLiveRoom.getOnlineUserVM().getUser(i).getNumber())) {
                return mLiveRoom.getOnlineUserVM().getUser(i);
            }
        }
        return null;
    }
}
