package com.baijiayun.groupclassui.layer;

import android.content.Context;
import android.os.Build;
import android.util.AttributeSet;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.util.FECompatUtil;
import com.baijiayun.groupclassui.window.video.VideoContainerWindow;
import com.baijiayun.groupclassui.window.video.VideoWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPPlayerViewUpdateModel;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

/**
 * 视频拖拽动画层
 * Created by Dujuncan on 2018/9/20.
 */
public class VideoDragLayer extends BaseLayerOfProfessionalClass {
    private static final String TAG = VideoDragLayer.class.getCanonicalName();
    private float videoOriginX, videoOriginY;//长按放大后的原始坐标
    private OnVideoResetListener onVideoResetListener;
    private List<VideoContainerWindow> videoContainerWindows;
    private int seatContainerWidth;
    private int seatContainerHeight;

    public void setSeatContainerWidth(int measuredWidth) {
        this.seatContainerWidth = measuredWidth;
    }

    public void setSeatContainerHeight(int measuredHeight) {
        this.seatContainerHeight = measuredHeight;
    }

    public interface OnVideoResetListener {
        void onReset(String uid, VideoWindow videoWindow);
    }

    public VideoDragLayer(@NonNull Context context) {
        super(context);
    }

    public VideoDragLayer(@NonNull Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public VideoDragLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public VideoDragLayer(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    @Override
    protected void init() {
        super.init();
        videoContainerWindows = new ArrayList<>();
    }

    /**
     * 开始放大
     *
     * @param scale                  放大倍数
     * @param pressedContainerWindow 原窗口
     * @param startX                 按下x坐标
     * @param startY                 按下y坐标
     */
    public void startEnlarge(float scale, VideoContainerWindow pressedContainerWindow, float startX, float startY) {
        Log.d(TAG, "startEnlarge: " + startX + "..." + startY);

        videoOriginX = startX;
        videoOriginY = startY;

        View view = pressedContainerWindow.getView();
        int[] viewPositions = new int[2];
        view.getLocationOnScreen(viewPositions);
        if (pressedContainerWindow.getVideoWindow().getView().getParent() != null) {
            FrameLayout parent = (FrameLayout) pressedContainerWindow.getVideoWindow().getView().getParent();
            parent.removeView(view);
        }

        LayoutParams params = new LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        params.width = (int) (view.getMeasuredWidth() * scale);
        params.height = (int) (view.getMeasuredHeight() * scale);
        if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.DEFAULT) {
            params.leftMargin = view.getLeft() - (params.width - view.getMeasuredWidth()) / 2;
            params.topMargin = view.getTop() - (params.height - view.getMeasuredHeight()) / 2;
        } else if(router.getLiveRoom().getSmallClassTemplateType()== LPConstants.SmallClassTemplateType.ONEONONE) {
            params.leftMargin = this.getMeasuredWidth() - view.getMeasuredWidth() - (params.width - view.getMeasuredWidth()) / 2;
            params.topMargin = view.getTop() - (params.height - view.getMeasuredHeight()) / 2;
        }
        else {
            params.leftMargin = view.getLeft() - (params.width - view.getMeasuredWidth()) / 2;
            int[] containerPositions = new int[2];
            getLocationOnScreen(containerPositions);
            params.topMargin = viewPositions[1] - containerPositions[1] - (params.height - view.getMeasuredHeight());
        }
        view.setLayoutParams(params);

        VideoContainerWindow videoContainerWindow = new VideoContainerWindow(getContext(), pressedContainerWindow.getVideoWindow());
        videoContainerWindow.showScaleIcon(false);
        addWindow(videoContainerWindow, params);
        videoContainerWindows.add(videoContainerWindow);
    }

    /**
     * 拖动视频
     */
    public void scrollVideo(float currX, float currY) {
        Log.d(TAG, "draggingVideo: " + currX + "..." + currY + "...count=" + getChildCount());
        if (getChildCount() == 0) return;

        View child = getChildAt(getChildCount() - 1);
        FrameLayout.LayoutParams params = (LayoutParams) child.getLayoutParams();

        //以放大后的位置为原始位置，计算滑动距离
        params.leftMargin += (int) (currX - videoOriginX);
        params.topMargin += (int) (currY - videoOriginY);

        //判断4个边界
        if (params.leftMargin < 0) {
            params.leftMargin = 0;
        }
        if (params.topMargin < 0) {
            params.topMargin = 0;
        }
        if (params.leftMargin > getMeasuredWidth() - child.getMeasuredWidth()) {
            params.leftMargin = getMeasuredWidth() - child.getMeasuredWidth();
        }
        if (params.topMargin > getMeasuredHeight() - child.getMeasuredHeight()) {
            params.topMargin = getMeasuredHeight() - child.getMeasuredHeight();
        }

        child.setLayoutParams(params);
        videoOriginX = currX;
        videoOriginY = currY;
    }

    /**
     * 拖动结束，放大视频
     *
     * @param scale 放大倍数
     */
    public void scrollEndEnlarge(float scale) {
        if (getChildCount() == 0) return;
        VideoContainerWindow videoContainerWindow = videoContainerWindows.get(getChildCount() - 1);
        //拖动距离小于视频本身高度，还原
        boolean isReset;
        if(router.getLiveRoom().getSmallClassTemplateType()== LPConstants.SmallClassTemplateType.ONEONONE){
            //以左上为点，所以右边判断时要乘以scale，普通课是上为边界不需要乘以scale
            isReset = videoContainerWindow.getView().getRight() > this.getRight() - videoContainerWindow.getView().getMeasuredWidth() * scale;
        }else {
            isReset = videoContainerWindow.getView().getTop() < videoContainerWindow.getView().getMeasuredHeight();
        }
        if (videoContainerWindow.getView().getMeasuredWidth() == 0) {
            isReset = true;
        }
        if (isReset) {
            if (onVideoResetListener != null) {
                onVideoResetListener.onReset(videoContainerWindow.getVideoWindow().getUserId(), videoContainerWindow.getVideoWindow());
                removeWindow(videoContainerWindow);
                videoContainerWindows.remove(videoContainerWindow);
                Log.d(TAG, "scrollEndEnlarge: reset video...video count =" + getChildCount());
            }
        } else {
            removeWindow(videoContainerWindow);
            videoContainerWindows.remove(videoContainerWindow);

            Set<LPPlayerViewUpdateModel.PlayerPosition> allList = router.getSetByKey(EventKey.AllPlayerViewList, LPPlayerViewUpdateModel.PlayerPosition.class);

            LPPlayerViewUpdateModel model = new LPPlayerViewUpdateModel();
            model.action = LPConstants.LPPlayerAction.ADD;
            model.id = model.mediaId = FECompatUtil.getIOSMediaIdFromMediaId(videoContainerWindow.getVideoWindow().getMediaId());
            LPPlayerViewUpdateModel.PlayerPosition playerPosition = new LPPlayerViewUpdateModel.PlayerPosition();
            int containerWidth = getMeasuredWidth();
            int containerHeight = getMeasuredHeight();
            int videoWindowLeft = videoContainerWindow.getView().getLeft();
            int videoWindowTop = videoContainerWindow.getView().getTop();
            int videoWindowWidth = videoContainerWindow.getVideoWindow().getView().getMeasuredWidth();
            int videoWindowHeight = videoContainerWindow.getVideoWindow().getView().getMeasuredHeight();
            //边界调整，右边和底部可能越界，调整top和left
            if (videoWindowLeft + videoWindowWidth * scale > getRight()) {
                videoWindowLeft = (int) (getRight() - videoWindowWidth * scale);
            }
            if (videoWindowTop + videoWindowHeight * scale > getBottom()) {
                videoWindowTop = (int) (getBottom() - videoWindowHeight * scale);
            }
            if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
                if (seatContainerWidth != 0) {
                    containerWidth -= seatContainerWidth;
                }
            } else {
                if (seatContainerHeight != 0) {
                    containerHeight -= seatContainerHeight;
                    videoWindowTop -= seatContainerHeight;
                }
            }
            playerPosition.id = model.id;
            playerPosition.width = String.valueOf(videoWindowWidth * scale / containerWidth);
            playerPosition.height = String.valueOf(videoWindowHeight * scale / containerHeight);
            playerPosition.x = String.valueOf(videoWindowLeft * 1f / containerWidth);
            playerPosition.y = String.valueOf(videoWindowTop * 1f / containerHeight);
            if (null == allList) {
                allList = new HashSet<>();
            }
            allList.add(playerPosition);
            router.setObjectByKey(EventKey.AllPlayerViewList, allList);
            model.all = allList;
            router.getLiveRoom().getMediaVM().requestPlayerViewUpdate(model);

            Log.d(TAG, "scrollEndEnlarge: join to blackboard...video count=" + getChildCount());
        }
    }

    public void setOnVideoResetListener(OnVideoResetListener onVideoResetListener) {
        this.onVideoResetListener = onVideoResetListener;
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (videoContainerWindows != null) {
            for (VideoContainerWindow window : videoContainerWindows)
                window.onDestroy();
            videoContainerWindows.clear();
        }
    }
}
