package com.baijiayun.groupclassui.studyreport;

import android.content.Context;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.os.Bundle;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.AppCompatImageView;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.toast.ToastCompat;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.util.IntentDataHolder;
import com.baijiayun.livebase.widgets.dialog.preview.PictureWatcherFragment;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livecore.models.LPStudyReportModel;
import com.baijiayun.livecore.models.studyreport.LPBlackboardInfoModel;
import com.baijiayun.liveuibase.widgets.StudyReportDetailFragment;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;

/**
 * @author wubanglin
 */
public class StudyReportActivity extends AppCompatActivity {
    private LiveRoom liveRoom;
    private Disposable disposableOfRequestStudyReport, disposableOfGetReportStatus, disposableOfStudyReportList, disposableOfBlackboardImgList;
    private List<LPStudyReportModel> lpStudyReportModelList;
    private FrameLayout detailContainer;
    private ImageView closeIv;
    private TextView reportNameTv;
    private ImageView preStudyReportIv;
    private ImageView nextStudyReportIv;
    private TextView studyReportTitleTv;
    private RelativeLayout studyReportBottomRl;
    private TextView studyReportClassNameTv;
    private TextView studyReportDateTv;
    private TextView studyReportStatusMessageTv;
    private TextView reloadStudyReportTv;
    private AppCompatImageView closeStudyReportIv;
    private StudyReportListDialog studyReportListDialog;
    private LPStudyReportModel currStudyReport;
    private StudyReportListDialog.ReportItemClickListener reportItemClickListener;
    private SimpleDateFormat simpleDateFormat;

    private StudyReportDetailFragment studyReportDetailFragment;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.bjy_activity_loading_study_report);
        studyReportClassNameTv = findViewById(R.id.bjy_group_loading_study_report_room_name);
        studyReportDateTv = findViewById(R.id.bjy_group_loading_study_report_date);
        studyReportStatusMessageTv = findViewById(R.id.bjy_group_study_report_status_message);
        reloadStudyReportTv = findViewById(R.id.bjy_group_reload_study_report_text);
        closeStudyReportIv = findViewById(R.id.bjy_group_close_study_report_image);
        boolean isTabletDevice = (this.getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) >= Configuration.SCREENLAYOUT_SIZE_LARGE;
        if (!isTabletDevice) {
            // 设置竖屏显示
            this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
        } else {
            // 设置横屏显示
            this.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        }

        liveRoom = (LiveRoom) IntentDataHolder.getInstance().getData("liveroom");
        studyReportClassNameTv.setText(liveRoom.getRoomInfo().title);
        simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd", Locale.getDefault());
        studyReportDateTv.setText(simpleDateFormat.format(new Date()));
        requestStudyReportTaskNo();

        reloadStudyReportTv.setOnClickListener(v -> {
            requestStudyReportTaskNo();
            studyReportStatusMessageTv.setText(R.string.bjy_group_study_report_loading_words);
        });

        closeStudyReportIv.setOnClickListener(v -> {
            finish();
        });
    }

    private void requestStudyReportTaskNo() {
        disposableOfRequestStudyReport = liveRoom.requestStudyReport()
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(taskNo -> {
                    getReportStatus(taskNo);
                }, throwable -> {
                    ToastCompat.showToast(this, throwable.getMessage(), Toast.LENGTH_SHORT);
                });
    }

    private void getReportStatus(String taskNo) {
        disposableOfGetReportStatus = Observable.interval(1, TimeUnit.SECONDS)
                .flatMap((Function<Long, Observable<Integer>>) aLong -> liveRoom.getTaskStateSegment(taskNo))
                .subscribe(status -> {
                            switch (status) {
                                case -1:
                                    RxUtils.dispose(disposableOfGetReportStatus);
                                    studyReportStatusMessageTv.setText(getString(R.string.bjy_group_study_report_not_exist));
                                    reloadStudyReportTv.setVisibility(View.VISIBLE);
                                    break;
                                case 3:
                                    RxUtils.dispose(disposableOfGetReportStatus);
                                    getStudyReport();
                                    break;
                                case 4:
                                    RxUtils.dispose(disposableOfGetReportStatus);
                                    studyReportStatusMessageTv.setText(getString(R.string.bjy_group_get_study_report_fail));
                                    reloadStudyReportTv.setVisibility(View.VISIBLE);
                                    break;
                                default:
                                    break;
                            }
                        }
                );
    }

    private void getStudyReport() {
        disposableOfStudyReportList = liveRoom.getStudyReportList()
                .subscribe(lpStudyReportModelList -> {
                    if (lpStudyReportModelList.isEmpty()) {
                        studyReportStatusMessageTv.setText(R.string.bjy_group_get_study_report_fail);
                        reloadStudyReportTv.setVisibility(View.VISIBLE);
                        return;
                    }
                    this.lpStudyReportModelList = lpStudyReportModelList;
                    currStudyReport = lpStudyReportModelList.get(0);
                    initStudyReportView();
                    initListener();
                }, throwable -> {
                    ToastCompat.showToast(this, throwable.getMessage(), Toast.LENGTH_SHORT);
                });
    }

    private Context getContext() {
        return this;
    }

    private void initStudyReportView() {
        setContentView(R.layout.activity_expression_reportion);
        detailContainer = findViewById(R.id.detail_container);
        closeIv = findViewById(R.id.iv_exp_close);
        reportNameTv = findViewById(R.id.tv_exp_name);
        preStudyReportIv = findViewById(R.id.iv_exp_left);
        nextStudyReportIv = findViewById(R.id.iv_exp_right);
        studyReportTitleTv = findViewById(R.id.tv_exp_title);
        studyReportBottomRl = findViewById(R.id.rl_exp_bottom);
        reportNameTv.setText(currStudyReport.userName);
        studyReportTitleTv.setText(String.format("%s的学情报告", currStudyReport.userName));
        if (studyReportDetailFragment == null) {
            studyReportDetailFragment = StudyReportDetailFragment.Companion.newInstance(currStudyReport.reportUrl, currStudyReport.reportNo);
            studyReportDetailFragment.setIsShowToolbar(false);
            studyReportDetailFragment.setShowImageListener(new StudyReportDetailFragment.ShowBlackboardImageListener() {
                @Override
                public void requestImageAndShow(@Nullable String reportNumber) {
                    disposableOfBlackboardImgList = liveRoom.getBlackboardImgList(reportNumber)
                            .subscribe(lpStudyReportInfo -> {
                                if (lpStudyReportInfo.imgList.size() == 0) {
                                    return;
                                }
                                ArrayList<String> imgUrls = new ArrayList<>();
                                for (LPBlackboardInfoModel lpBlackboardInfoModel : lpStudyReportInfo.imgList) {
                                    imgUrls.add(lpBlackboardInfoModel.url);
                                }
                                PictureWatcherFragment pictureWatcherFragment = PictureWatcherFragment.Companion.newInstance(imgUrls, 0, true, lpStudyReportInfo.whiteboardUrl, R.style.BJYGroupDialogStyle);
                                pictureWatcherFragment.show(getSupportFragmentManager(), PictureWatcherFragment.class.getName());
                            }, throwable -> {
                                ToastCompat.showToast(getContext(), throwable.getMessage(), Toast.LENGTH_SHORT);
                            });
                }
            });
        }
        getSupportFragmentManager().beginTransaction()
                .replace(R.id.detail_container, studyReportDetailFragment)
                .commitAllowingStateLoss();
    }

    private void initListener() {
        closeIv.setOnClickListener(v -> {
            finish();
        });

        preStudyReportIv.setOnClickListener(v -> {
            if (lpStudyReportModelList.indexOf(currStudyReport) == 0) {
                ToastCompat.showToast(this, "已经是第一个学生了哦", Toast.LENGTH_SHORT);
                return;
            }
            currStudyReport = lpStudyReportModelList.get(lpStudyReportModelList.indexOf(currStudyReport) - 1);
            updateUI(currStudyReport);
        });

        nextStudyReportIv.setOnClickListener(v -> {
            if (lpStudyReportModelList.indexOf(currStudyReport) == lpStudyReportModelList.size() - 1) {
                ToastCompat.showToast(this, "已经是最后一个学生了哦", Toast.LENGTH_SHORT);
                return;
            }
            currStudyReport = lpStudyReportModelList.get(lpStudyReportModelList.indexOf(currStudyReport) + 1);
            updateUI(currStudyReport);
        });

        studyReportBottomRl.setOnClickListener(v -> {
            if (studyReportListDialog == null) {
                studyReportListDialog = new StudyReportListDialog(this, reportItemClickListener);
                studyReportListDialog.setLayout(detailContainer.getWidth(), RelativeLayout.LayoutParams.WRAP_CONTENT);
            }
            if (!studyReportListDialog.isShowing()) {
                studyReportListDialog
                        .setCurrReportItem(currStudyReport)
                        .setAllReport(lpStudyReportModelList)
                        .show();
            }
        });
        reportItemClickListener = this::updateUI;
    }

    private void updateUI(LPStudyReportModel lpStudyReportModel) {
        if (!currStudyReport.userNumber.equals(lpStudyReportModel.userNumber)) {
            currStudyReport = lpStudyReportModel;
        }
        studyReportTitleTv.setText(String.format("%s的学情报告", lpStudyReportModel.userName));
        reportNameTv.setText(lpStudyReportModel.userName);
        if (studyReportDetailFragment != null) {
            studyReportDetailFragment.updateReportUrl(lpStudyReportModel.reportUrl, lpStudyReportModel.reportNo);
        }
    }

    @Override
    public void onBackPressed() {
        if (studyReportDetailFragment != null && studyReportDetailFragment.canGoBack()) {
            studyReportDetailFragment.goBack();
        } else {
            super.onBackPressed();
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfRequestStudyReport);
        RxUtils.dispose(disposableOfGetReportStatus);
        RxUtils.dispose(disposableOfStudyReportList);
        RxUtils.dispose(disposableOfBlackboardImgList);
    }
}
