package com.baijiayun.groupclassui.user;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AbsListView;
import android.widget.BaseExpandableListAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiahulian.common.cropperv2.uikit.CircleColorView;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.util.InteractiveUtils;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.GroupItem;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.livecore.models.LPMediaModel;
import com.baijiayun.livecore.models.launch.LPAwardConfig;
import com.baijiayun.liveuibase.widgets.dialog.AwardPopupWindow;

import java.util.ArrayList;
import java.util.List;

public class ActiveGroupExpandAdapter extends BaseExpandableListAdapter {
    private final OnlineUserPresenter presenter;
    private final IUserContract listener;
    protected List<LPGroupItem> lpGroupItems;

    public ActiveGroupExpandAdapter(OnlineUserPresenter presenter, IUserContract listener) {
        this.presenter = presenter;
        this.listener = listener;
        this.lpGroupItems = new ArrayList<>();
    }

    public void setDatas(List<LPGroupItem> lpGroupItems) {
        this.lpGroupItems = lpGroupItems;
        notifyDataSetChanged();
    }

    @Override
    public int getGroupCount() {
        return lpGroupItems.size();
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        final Object group = getGroup(groupPosition);
        if (group == null) {
            return 0;
        } else {
            LPGroupItem lpGroupItem = (LPGroupItem) group;
            return lpGroupItem.userModelList == null ? 0 : lpGroupItem.userModelList.size();
        }
    }

    @Override
    public Object getGroup(int groupPosition) {
        if (groupPosition >= lpGroupItems.size())
            return null;
        return lpGroupItems.get(groupPosition);
    }

    @Override
    public Object getChild(int groupPosition, int childPosition) {
        Object group = getGroup(groupPosition);
        if (group == null) {
            return null;
        }
        LPGroupItem lpGroupItem = (LPGroupItem) group;
        if (lpGroupItem.userModelList.isEmpty() && lpGroupItem.userModelList.size() <= childPosition) {
            return null;
        }
        return lpGroupItem.userModelList.get(childPosition);
    }

    @Override
    public long getGroupId(int groupPosition) {
        final Object group = getGroup(groupPosition);
        if (group == null) {
            return 0;
        } else {
            LPGroupItem lpGroupItem = (LPGroupItem) group;
            return lpGroupItem.id;
        }
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        final Object child = getChild(groupPosition, childPosition);
        if (child == null) {
            return 0;
        } else {
            LPUserModel lpUserModel = (LPUserModel) child;
            long id = 0;
            try {
                id = Long.parseLong(lpUserModel.number);
            } catch (NumberFormatException e) {
                e.printStackTrace();
            }
            return id;
        }
    }

    @Override
    public boolean hasStableIds() {
        return true;
    }

    /**
     * 是否是台上用户(台下用户不能直接开音视频)
     */
    protected boolean isActiveGroup() {
        return true;
    }

    @SuppressLint("SetTextI18n")
    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        GroupHolder groupHolder;
        if (convertView == null) {
            groupHolder = new GroupHolder();
            convertView = LayoutInflater.from(parent.getContext()).inflate(R.layout.bjy_group_layout_group_info, parent, false);
            groupHolder.tvGroupName = convertView.findViewById(R.id.tv_group_name);
            groupHolder.tvGroupAwardCount = convertView.findViewById(R.id.tv_group_award_count);
            groupHolder.ivGroupAward = convertView.findViewById(R.id.iv_group_award);
            groupHolder.tvGroupExpand = convertView.findViewById(R.id.tv_group_expand);
            groupHolder.viewGroupColor = convertView.findViewById(R.id.view_group_color);
            groupHolder.rlLayoutGroup = convertView.findViewById(R.id.rl_layout_group);
            convertView.setTag(groupHolder);
        } else {
            groupHolder = (GroupHolder) convertView.getTag();
        }
        final Object group = getGroup(groupPosition);
        if (group != null) {
            LPGroupItem lpGroupItem = (LPGroupItem) group;
            if (lpGroupItem.id == 0 || lpGroupItem.groupItemModel == null) {
                AbsListView.LayoutParams layoutParams = new AbsListView.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 1);
                groupHolder.rlLayoutGroup.setLayoutParams(layoutParams);
                groupHolder.rlLayoutGroup.setVisibility(View.GONE);
                groupHolder.tvGroupExpand.setVisibility(View.GONE);
                groupHolder.ivGroupAward.setVisibility(View.GONE);
                groupHolder.tvGroupAwardCount.setVisibility(View.GONE);
            } else {
                groupHolder.tvGroupExpand.setVisibility(View.VISIBLE);
                groupHolder.rlLayoutGroup.setVisibility(View.VISIBLE);
                groupHolder.tvGroupExpand.setSelected(isExpanded);
                groupHolder.ivGroupAward.setVisibility(presenter.isAdmin() ? View.VISIBLE : View.GONE);
                groupHolder.tvGroupAwardCount.setVisibility(presenter.isAdmin() ? View.VISIBLE : View.GONE);
                groupHolder.tvGroupAwardCount.setText(String.valueOf(presenter.getAwardGroupCount(lpGroupItem.id)));
                RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) groupHolder.viewGroupColor.getLayoutParams();
                params.leftMargin = 0;
                AbsListView.LayoutParams layoutParams = new AbsListView.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                        parent.getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_window_user_content_height));
                groupHolder.rlLayoutGroup.setLayoutParams(layoutParams);
                final GroupItem groupItem = lpGroupItem.groupItemModel;
                groupHolder.tvGroupName.setText(groupItem.name + "(" + lpGroupItem.userModelList.size() + ")");
                groupHolder.viewGroupColor.setColor(Color.parseColor(groupItem.color));
            }
        }
        return convertView;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        GroupChildHolder groupChildHolder;
        if (convertView == null) {
            groupChildHolder = new GroupChildHolder();
            convertView = LayoutInflater.from(parent.getContext())
                    .inflate(R.layout.bjysc_item_online_user, parent, false);
            groupChildHolder.tvUserName = convertView.findViewById(R.id.tv_user_name);
            groupChildHolder.tvUserLabel = convertView.findViewById(R.id.tv_user_label);
            groupChildHolder.userContainer = convertView.findViewById(R.id.ll_user_container);
            groupChildHolder.awardContainer = convertView.findViewById(R.id.ll_award_container);
            groupChildHolder.tvAward = convertView.findViewById(R.id.tv_user_award);
            groupChildHolder.ivAwardIcon = convertView.findViewById(R.id.bjysc_online_user_award_icon);
            groupChildHolder.ivCamera = convertView.findViewById(R.id.iv_user_camera);
            groupChildHolder.ivMic = convertView.findViewById(R.id.iv_user_mic);
            groupChildHolder.ivPPTPaint = convertView.findViewById(R.id.iv_user_ppt_paint);
            groupChildHolder.ivPPTAuth = convertView.findViewById(R.id.iv_user_ppt_auth);
            groupChildHolder.ivChat = convertView.findViewById(R.id.iv_user_chat);
            groupChildHolder.ivScreenShare = convertView.findViewById(R.id.iv_user_screen_share);
            groupChildHolder.ivSpeakStauts = convertView.findViewById(R.id.iv_user_speak_status);
            groupChildHolder.ivKickout = convertView.findViewById(R.id.iv_user_kickout);
            convertView.setTag(groupChildHolder);
        } else {
            groupChildHolder = (GroupChildHolder) convertView.getTag();
        }
        final Object child = getChild(groupPosition, childPosition);
        if (child != null) {
            LPUserModel userModel = (LPUserModel) child;
            int visibility = userModel.getType() == LPConstants.LPUserType.Student ? View.VISIBLE : View.INVISIBLE;
            groupChildHolder.awardContainer.setVisibility(visibility);
            groupChildHolder.ivKickout.setVisibility(visibility);
            groupChildHolder.ivChat.setVisibility(visibility);
            int studyRoomVisible = presenter.isStudyGalleryMode() || userModel.getType() != LPConstants.LPUserType.Student ? View.INVISIBLE : View.VISIBLE;
            groupChildHolder.ivPPTPaint.setVisibility(studyRoomVisible);
            groupChildHolder.ivPPTAuth.setVisibility(studyRoomVisible);
            groupChildHolder.ivScreenShare.setVisibility(studyRoomVisible);
            if (userModel.getType() == LPConstants.LPUserType.Teacher) {
                groupChildHolder.userContainer.setVisibility(View.INVISIBLE);
                groupChildHolder.tvUserLabel.setText(presenter.getTeacherCustomLabelText());
                groupChildHolder.tvUserLabel.setBackground(ThemeDataUtil.getCustomLabelBg(
                        parent.getContext(),
                        ThemeDataUtil.getColorFromThemeConfigByAttrId(parent.getContext(), R.attr.base_theme_live_product_color)
                ));
                groupChildHolder.tvUserLabel.setVisibility(View.VISIBLE);
                if (presenter.enable1v1TeacherOffStage()) {
                    groupChildHolder.userContainer.setVisibility(View.VISIBLE);
                } else {
                    groupChildHolder.userContainer.setVisibility(View.INVISIBLE);
                }
            } else if (userModel.getType() == LPConstants.LPUserType.Assistant) {
                groupChildHolder.userContainer.setVisibility(View.VISIBLE);
                groupChildHolder.tvUserLabel.setText(presenter.getAssistantCustomLabelText());
                groupChildHolder.tvUserLabel.setBackground(ThemeDataUtil.getCustomLabelBg(
                        parent.getContext(),
                        ThemeDataUtil.getColorFromThemConfigByColorId(parent.getContext(), R.color.bjy_base_assistant_custom_label_bg_color)
                ));
                groupChildHolder.tvUserLabel.setVisibility(View.VISIBLE);
            } else {
                groupChildHolder.userContainer.setVisibility(View.VISIBLE);
                groupChildHolder.tvUserLabel.setVisibility(View.GONE);
            }
            if (!presenter.isAdmin()) {
                groupChildHolder.userContainer.setVisibility(View.GONE);
            }
            groupChildHolder.ivMic.setVisibility(isActiveGroup() ? View.VISIBLE : View.GONE);
            groupChildHolder.ivCamera.setVisibility(isActiveGroup() ? View.VISIBLE : View.GONE);
            groupChildHolder.tvUserName.setText(getEncodePhoneNumber(userModel.getName()));
            final LPMediaModel mediaModel = presenter.getMediaModel(userModel);
            boolean isVideoOn;
            boolean isAudioOn;
            if (isSelf(userModel)) {
                isAudioOn = presenter.isSelfAudioOn();
                isVideoOn = presenter.isSelfVideoOn();
            } else {
                isAudioOn = mediaModel != null && mediaModel.audioOn;
                isVideoOn = mediaModel != null && mediaModel.videoOn;
            }
            final boolean singleTutorUser = presenter.isSingleTutorUser(userModel);
            groupChildHolder.ivSpeakStauts.setEnabled(userModel.getEndType() != LPConstants.LPEndType.PC_H5 && !singleTutorUser);
            if (!isActiveGroup() || userModel.getEndType() == LPConstants.LPEndType.PC_H5) {
                groupChildHolder.ivPPTPaint.setEnabled(false);
                groupChildHolder.ivPPTAuth.setEnabled(false);
                groupChildHolder.ivScreenShare.setEnabled(false);
                groupChildHolder.tvAward.setTextColor(ContextCompat.getColor(groupChildHolder.tvAward.getContext(),R.color.base_icon_color));
            } else {
                groupChildHolder.ivPPTPaint.setEnabled(true);
                groupChildHolder.ivPPTAuth.setEnabled(true);
                groupChildHolder.ivScreenShare.setEnabled(true);
                groupChildHolder.ivMic.setEnabled(!presenter.isSelfStudyMode() && !singleTutorUser);
                groupChildHolder.ivCamera.setEnabled(!singleTutorUser);
                groupChildHolder.tvAward.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(groupChildHolder.tvAward.getContext(), R.attr.base_theme_window_main_text_color));
            }
            groupChildHolder.ivCamera.setSelected(!isVideoOn);
            groupChildHolder.ivMic.setSelected(!isAudioOn);
            groupChildHolder.ivPPTPaint.setSelected(!InteractiveUtils.hasDrawingAuth(presenter.getRouter().getLiveRoom(), userModel));
            groupChildHolder.ivPPTAuth.setSelected(!presenter.hasPPTAuth(userModel));
            groupChildHolder.ivScreenShare.setSelected(!presenter.hasScreenShare(userModel));
            groupChildHolder.ivSpeakStauts.setSelected(!presenter.isOnVideoSeat(userModel));
            groupChildHolder.ivChat.setSelected(userModel.isForbidChat);
            groupChildHolder.tvAward.setText(String.valueOf(presenter.getAwardCount(userModel.number)));
            initAward(parent.getContext(),userModel, groupChildHolder);
            groupChildHolder.awardContainer.setOnClickListener(v -> {
                if (availableAward == null) {
                    if (awardPopupWindow == null) {
                        awardPopupWindow = new AwardPopupWindow(parent.getContext(), presenter::requestAward);
                    }
                    awardPopupWindow.setUserModel(userModel);
                    awardPopupWindow.show(groupChildHolder.ivAwardIcon);
                } else {
                    presenter.requestAward(userModel, availableAward.key);
                }
            });
            groupChildHolder.ivCamera.setOnClickListener(v ->
                    presenter.controlRemoteSpeak(userModel.userId, groupChildHolder.ivCamera.isSelected(), !groupChildHolder.ivMic.isSelected()));
            groupChildHolder.ivMic.setOnClickListener(v ->
                    presenter.controlRemoteSpeak(userModel.userId, !groupChildHolder.ivCamera.isSelected(), groupChildHolder.ivMic.isSelected()));
            groupChildHolder.ivPPTAuth.setOnClickListener(v ->
                    presenter.changePPTAuth(groupChildHolder.ivPPTAuth.isSelected(), userModel.number));
            groupChildHolder.ivPPTPaint.setOnClickListener(v ->
                    presenter.changePPTPaint(groupChildHolder.ivPPTPaint.isSelected(), userModel.number));
            groupChildHolder.ivScreenShare.setOnClickListener(v ->
                    presenter.changeScreenShare(groupChildHolder.ivScreenShare.isSelected(), userModel));
            groupChildHolder.ivChat.setOnClickListener(v -> {
                if (userModel.isForbidChat) {
                    listener.unForbidChat(userModel);
                } else {
                    listener.forbidChat(userModel);
                }
            });

            groupChildHolder.ivSpeakStauts.setOnClickListener(v ->
                    listener.upOrDownSeatContainer(userModel, !presenter.isOnVideoSeat(userModel)));

            groupChildHolder.ivKickout.setOnClickListener(v ->
                    listener.kickUserOut(userModel));
        }
        return convertView;
    }

    private AwardPopupWindow awardPopupWindow;
    private LPAwardConfig availableAward = null;

    /**
     * 初始化 奖励 图标
     */
    private void initAward(Context context, LPUserModel lpUserModel,GroupChildHolder groupChildHolder) {
        LPAwardConfig[] awardConfigs = presenter.getAwardConfigs();
        if (awardConfigs == null) {
            return;
        }
        int enableAwardTypeCount = 0;
        LPAwardConfig awardConfig = null;
        for (LPAwardConfig lpAwardConfig : awardConfigs) {
            if (lpAwardConfig.isEnable == 1) {
                awardConfig = lpAwardConfig;
                enableAwardTypeCount++;
            }
        }
        if (enableAwardTypeCount <= 1 && awardConfig != null) {
            // 仅一种启用，使用对应logo
            if (!isActiveGroup() && lpUserModel.getEndType() == LPConstants.LPEndType.PC_H5) {
                groupChildHolder.awardContainer.setEnabled(false);
                groupChildHolder.ivAwardIcon.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.bjysc_user_ic_award_disable));
            } else {
                groupChildHolder.ivAwardIcon.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.bjysc_user_ic_award));
            }
            this.availableAward = awardConfig;
        } else {
            // 使用钻石
            if (!isActiveGroup() && lpUserModel.getEndType() == LPConstants.LPEndType.PC_H5) {
                groupChildHolder.awardContainer.setEnabled(false);
                groupChildHolder.ivAwardIcon.setImageResource(R.drawable.bjysc_ic_user_list_award_default_disable);
            } else {
                groupChildHolder.ivAwardIcon.setImageResource(R.drawable.bjysc_ic_user_list_award_default);
            }
            this.availableAward = null;
        }
    }

    private boolean isSelf(IUserModel userModel) {
        return TextUtils.equals(userModel.getUserId(), presenter.getCurrentUser().getUserId());
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return false;
    }

    static class GroupHolder {
        TextView tvGroupName;
        TextView tvGroupExpand;
        TextView tvGroupAwardCount;
        View ivGroupAward;
        CircleColorView viewGroupColor;
        RelativeLayout rlLayoutGroup;
    }

    static class GroupChildHolder {
        TextView tvUserName;
        TextView tvUserLabel;
        LinearLayout userContainer;
        LinearLayout awardContainer;
        TextView tvAward;
        ImageView ivAwardIcon;
        ImageView ivCamera;
        ImageView ivMic;
        ImageView ivPPTPaint;
        ImageView ivPPTAuth;
        ImageView ivChat;
        ImageView ivScreenShare;//屏幕分享
        ImageView ivSpeakStauts;
        ImageView ivKickout;
    }
}
