package com.baijiayun.groupclassui.user;

import android.text.TextUtils;

import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.base.BaseGroupPresenter;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPAwardUserInfo;
import com.baijiayun.livecore.models.LPForbidUserModel;
import com.baijiayun.livecore.models.LPGroupAwardModel;
import com.baijiayun.livecore.models.LPGroupItem;
import com.baijiayun.livecore.models.LPMediaModel;
import com.baijiayun.livecore.models.LPStudyRoomTutorModel;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livecore.models.launch.LPAwardConfig;
import com.baijiayun.livecore.viewmodels.SpeakQueueVM;
import com.baijiayun.livecore.wrapper.LPPlayer;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

import io.reactivex.BackpressureStrategy;
import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.subjects.ReplaySubject;

/**
 * Created by yongjiaming on 2018/10/11
 */
public class OnlineUserPresenter implements BaseGroupPresenter {

    private static final int FORBID_CHAT_DURATION = 3600 * 24;

    private IRouter router;
    private IUserInteractionListener listener;
    private List<IUserModel> activeUserList = new ArrayList<>();
    private List<IUserModel> unActiveUserList = new ArrayList<>();
    private List<IUserModel> blockedUserList = new ArrayList<>();
    private HashMap<String, LPAwardUserInfo> awardRecord = new HashMap<>();
    private long lastRquestAwardTime = 0L;
    private CompositeDisposable disposables;

    //台下加载更多
    private boolean isLoading = false;

    public OnlineUserPresenter(IUserInteractionListener listener) {
        this.listener = listener;
    }

    @Override
    public void setRouter(IRouter router) {
        this.router = router;
    }

    public IRouter getRouter() {
        return router;
    }

    @Override
    public void subscribe() {
        disposables = new CompositeDisposable();
        if (BaseUtilsKt.isAdmin(router.getLiveRoom())) {
            router.getLiveRoom().getChatVM().requestForbidList();
        }
        router.getLiveRoom().getOnlineUserVM().loadMoreUser();
        disposables.add(router.getLiveRoom()
                .getOnlineUserVM()
                .getObservableOfOnlineUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iUserModels -> {
                    isLoading = false;
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));

        SpeakQueueVM speakQueueVM = router.getLiveRoom().getSpeakQueueVM();

        disposables.add(speakQueueVM
                .getObservableOfSpeakApply()
                .mergeWith(speakQueueVM.getObservableOfSpeakApplyDeny())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    if (listener != null) {
                        listener.notifyHandsUpUserDataSetChange();
                    }
                }));

        disposables.add(router.getLiveRoom().getOnlineUserVM().getObservableOfOnGroupItem()
                .toFlowable(BackpressureStrategy.LATEST)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpGroupItems -> {
                    if (listener != null) {
                        listener.notifyHandsUpUserDataSetChange();
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));
        disposables.add(router.getLiveRoom().getObservableOfForbidRaiseHand()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    if (listener != null)
                        listener.notifyForbidHandUp(aBoolean);
                }));

        disposables.add(router.getLiveRoom().getChatVM().getObservableOfForbidChat()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iForbidChatModel -> {
                    updateForbidChatStatus(iForbidChatModel.getForbidUser().getUserId(), iForbidChatModel.getDuration() > 0);
                    if (listener != null) {
                        listener.notifyForbidStatus();
                    }
                }));

        disposables.add(router.getLiveRoom().getChatVM().getObservableOfForbidList()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomForbidListModel -> {
                    if (lpResRoomForbidListModel.userList == null) {
                        return;
                    }
                    for (LPForbidUserModel forbidUserModel : lpResRoomForbidListModel.userList) {
                        if (forbidUserModel.duration > 0) {
                            updateForbidChatStatus(forbidUserModel.userId, true);
                        }
                    }
                    if (listener != null) {
                        listener.notifyForbidStatus();
                    }
                }));

        disposables.add(router.getLiveRoom().getOnlineUserVM().getObservableOfBlockedUser()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpResRoomBlockedUserModel -> {
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));


        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserAdd()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userModel -> {
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));

        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getPublishSubjectOfActiveUserRemove()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(userModel -> {
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));

        disposables.add(router.getLiveRoom().getSpeakQueueVM()
                .getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (listener != null) {
                        listener.notifyHandsUpUserDataSetChange();
                    }
                }));
        disposables.add(router.getLiveRoom().getSpeakQueueVM()
                .getObservableOfMediaPublish()
                .filter(iMediaModel -> iMediaModel.getMediaSourceType() == LPConstants.MediaSourceType.MainCamera)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaModel -> {
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));

        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getObservableOfBlockedUserList()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(blockUserList -> {
                    if (listener != null) {
                        listener.notifyOnlineUserDataSetChange();
                    }
                }));
        disposables.add(router.getLiveRoom().getOnlineUserVM()
                .getObservableOfOnLineUserCount()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(onUserCount -> {
                    if (listener != null) {
                        listener.notifyOnlineUserCountChange();
                    }
                }));

        disposables.add(router.getLiveRoom().getToolBoxVM().getObservableOfAward()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpInteractionAwardModel -> {
                    final HashMap<String, LPAwardUserInfo> recordAward = lpInteractionAwardModel.value.getRecordAward();
                    awardRecord.putAll(recordAward);
                    if (listener != null) {
                        listener.notifyUserDataChange();
                    }
                }));
        disposables.add(router.getSubjectByKey(EventKey.StudyRoomSwitch)
                .ofType(LPConstants.StudyRoomMode.class)
                .subscribe(studyRoomMode -> {
                    if (listener != null) {
                        listener.notifyStudyRoomSwitch();
                        listener.notifyUserDataChange();
                    }
                }));
        final Observable<Boolean> observableOfCameraOn = router.getLiveRoom().getRecorder().getObservableOfCameraOn().toObservable();
        final Observable<Boolean> observableOfMicOn = router.getLiveRoom().getRecorder().getObservableOfMicOn().toObservable();
        final Observable<IMediaModel> observableOfMediaPublish = router.getLiveRoom().getSpeakQueueVM().getObservableOfMediaPublish().filter(mediaModel -> mediaModel.getUser().getType() != LPConstants.LPUserType.Teacher);
        final Observable<Boolean> publishSubjectOfDrawingAuth = router.getLiveRoom().getSpeakQueueVM().getPublishSubjectOfDrawingAuth();
        final ReplaySubject<List<String>> publishSubjectOfStudentPPTAuth = router.getLiveRoom().getDocListVM().getPublishSubjectOfStudentPPTAuth();
        final ReplaySubject<List<String>> publishSubjectOfStudentScreenShare = router.getLiveRoom().getMediaVM().getPublishSubjectOfStudentScreenShare();
        final Observable<LPStudyRoomTutorModel> observableOfTutorStart = router.getLiveRoom().getStudyRoomVM().getObservableOfStudyRoomTutorStart();
        final Observable<LPStudyRoomTutorModel> observableOfTutorEnd = router.getLiveRoom().getStudyRoomVM().getObservableOfStudyRoomTutorEnd();
        final Observable<LPGroupAwardModel> observableOfGroupAward = router.getLiveRoom().getToolBoxVM().getObservableOfGroupAward();
        disposables.add(Observable.mergeArray(observableOfMediaPublish, publishSubjectOfDrawingAuth,
                        publishSubjectOfStudentPPTAuth, publishSubjectOfStudentScreenShare,
                        observableOfCameraOn, observableOfMicOn, observableOfGroupAward,
                        observableOfTutorStart, observableOfTutorEnd)
                .throttleLatest(1000, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(o -> {
                    if (listener != null) {
                        listener.notifyUserDataChange();
                    }
                }));
    }

    @Override
    public void unSubscribe() {
        // do nothing
    }

    public int getAllUserCount() {
        return router.getLiveRoom().getOnlineUserVM().getAllCount();
    }

    public int getHandsUpUserCount() {
        return router.getLiveRoom().getSpeakQueueVM().getApplyList().size();
    }

    public IUserModel getUser(int position) {
        return router.getLiveRoom().getOnlineUserVM().getUser(position);
    }

    public IUserModel getUpUser(int position) {
        return activeUserList.get(position);
    }

    public void removeBlockList(String userNumber) {
        router.getLiveRoom().getOnlineUserVM().freeBlockedUser(userNumber);
    }

    public IUserModel getBlockList(int position) {
        return blockedUserList.get(position);
    }

    public int getUpUserCount() {
        activeUserList.clear();
        activeUserList.addAll(router.getLiveRoom().getOnlineUserVM().getActiveUserList());
        return activeUserList.size();
    }

    public int getDownUserCount() {
        unActiveUserList.clear();
        unActiveUserList.addAll(router.getLiveRoom().getOnlineUserVM().getUnActiveUserList());
        return isLoading ? unActiveUserList.size() + 1 : unActiveUserList.size();
    }

    public int getBlockedUserListCount() {
        blockedUserList.clear();
        blockedUserList.addAll(router.getLiveRoom().getOnlineUserVM().getBlockedUserList());
        return blockedUserList.size();
    }

    public IUserModel getDownUser(int position) {
        if (!isLoading) {
            return unActiveUserList.get(position);
        }
        IUserModel model;
        if (position == unActiveUserList.size()) {
            return null;
        } else {
            return unActiveUserList.get(position);
        }
    }

    public void updateGroupInfo(int groupId) {
        router.getLiveRoom().getOnlineUserVM().loadMoreUser(groupId);
    }

    public List<LPGroupItem> getClassifyGroupList(List<IUserModel> userModels, boolean adjustQueue) {
        final List<LPGroupItem> groupList = new ArrayList<>(router.getLiveRoom().getOnlineUserVM().getGroupList());
        List<LPGroupItem> groupItems = new ArrayList<>();
        for (LPGroupItem lpGroupItem : groupList) {
            LPGroupItem groupItem = new LPGroupItem(lpGroupItem.id);
            groupItem.groupItemModel = lpGroupItem.groupItemModel;
            groupItem.userModelList = new ArrayList<>(lpGroupItem.userModelList);
            groupItems.add(groupItem);
        }
        Set<String> userNums = new HashSet<>();
        Map<String, IUserModel> userModelMap = new LinkedHashMap<>();
        for (IUserModel userModel : userModels) {
            userNums.add(userModel.getNumber());
            if (adjustQueue) {
                userModelMap.put(userModel.getNumber(), userModel);
            }
        }
        final Iterator<LPGroupItem> groupItemIterator = groupItems.iterator();
        while (groupItemIterator.hasNext()) {
            final LPGroupItem lpGroupItem = groupItemIterator.next();
            final Iterator<LPUserModel> iterator = lpGroupItem.userModelList.iterator();
            while (iterator.hasNext()) {
                final LPUserModel userModel = iterator.next();
                if (!userNums.contains(userModel.getNumber())) {
                    iterator.remove();
                } else {
                    //userModels(台上，台下)本身的group可能是错的，这是core层数据不统一的问题
                    if (adjustQueue) {
                        final IUserModel user = userModelMap.get(userModel.getNumber());
                        if (user instanceof LPUserModel) {
                            ((LPUserModel) user).groupId = lpGroupItem.id;
                        }
                    }
                }
            }
            if (lpGroupItem.userModelList.isEmpty()) {
                groupItemIterator.remove();
            }
        }
        //调整顺序，按照userModels顺序排序
        //台上按照上台顺序排列，台下按照进教室顺序排列
        if (adjustQueue) {
            for (LPGroupItem lpGroupItem : groupItems) {
                lpGroupItem.userModelList.clear();
                //每组第一个学生
                LPUserModel studentUser = null;
                for (String key : userModelMap.keySet()) {
                    final IUserModel userModel = userModelMap.get(key);
                    if (userModel instanceof LPUserModel && lpGroupItem.id == userModel.getGroup()) {
                        if (userModel.getType() == LPConstants.LPUserType.Teacher) {
                            lpGroupItem.userModelList.add(0, (LPUserModel) userModel);
                        } else if (userModel.getType() == LPConstants.LPUserType.Assistant) {
                            if (studentUser == null) {
                                lpGroupItem.userModelList.add((LPUserModel) userModel);
                            } else {
                                int index = lpGroupItem.userModelList.indexOf(studentUser);
                                lpGroupItem.userModelList.add(index, (LPUserModel) userModel);
                            }
                        } else {
                            if (studentUser == null) {
                                studentUser = (LPUserModel) userModel;
                            }
                            lpGroupItem.userModelList.add((LPUserModel) userModel);
                        }
                    }
                }
            }
        }
        return groupItems;
    }

    public List<LPGroupItem> getActiveGroupList() {
        return getClassifyGroupList(activeUserList, true);
    }

    public List<LPGroupItem> getUnActiveGroupList() {
        return getClassifyGroupList(unActiveUserList, false);
    }


    public LPGroupItem getGroupItem(int groupId) {
        final List<LPGroupItem> groupList = router.getLiveRoom().getOnlineUserVM().getGroupList();
        int size = groupList.size();
        for (int i = 0; i < size; i++) {
            final LPGroupItem lpGroupItem = groupList.get(i);
            if (lpGroupItem.id == groupId) {
                return lpGroupItem;
            }
        }
        return null;
    }

    public IUserModel getHandsUpUser(int position) {
        return router.getLiveRoom().getSpeakQueueVM().getApplyList().get(position);
    }

    public void agreeSpeakApply(IUserModel requestUser) {
        List<IUserModel> curList = router.getLiveRoom().getOnlineUserVM().getActiveUserList();
        if (curList != null) {
            boolean isActive = false;
            IUserModel tea = null;
            for (IUserModel iUserModel : curList) {
                if (iUserModel.getType() == LPConstants.LPUserType.Teacher) {
                    tea = iUserModel;
                }
                if (iUserModel.getUserId().equals(requestUser.getUserId())) {
                    isActive = true;
                }
            }
            // 过滤老师，在遍历外删除防止出现异常
            curList.remove(tea);
            if (curList.size() >= router.getLiveRoom().getMaxActiveUsers()) {
                if (isActive) {
                    router.getLiveRoom().getSpeakQueueVM().agreeSpeakApply(requestUser.getUserId());
                    return;
                }
                ReplaceUserWithStatus replaceUserWithStatus = new ReplaceUserWithStatus(requestUser);
                replaceUserWithStatus.isHandsUp = true;
                router.getPublishSubjectByKey(EventCode.UserWhenFullActiveAdd).onNext(replaceUserWithStatus);
                if (listener != null)
                    listener.notifyCurrentSeatFull();
                return;
            }
        }
        router.getLiveRoom().getSpeakQueueVM().agreeSpeakApply(requestUser.getUserId());
    }


    public void disagreeSpeakApply(String userId) {
        router.getLiveRoom().getSpeakQueueVM().disagreeSpeakApply(userId);
    }

    public void forbidChat(IUserModel forbidUser) {
        router.getLiveRoom().getChatVM().forbidChat(forbidUser, FORBID_CHAT_DURATION);
    }

    public void unForbidChat(IUserModel unForbidUser) {
        router.getLiveRoom().getChatVM().forbidChat(unForbidUser, -1);
    }

    public void updateForbidChatStatus(String userId, boolean isForbid) {
        LPUserModel userModel = (LPUserModel) router.getLiveRoom().getOnlineUserVM().getUserById(userId);
        if (userModel != null) {
            userModel.isForbidChat = isForbid;
        }
        for (IUserModel iUserModel : router.getLiveRoom().getOnlineUserVM().getActiveUserList()) {
            if (iUserModel.getUserId().equals(userId)) {
                ((LPUserModel) iUserModel).isForbidChat = isForbid;
                break;
            }
        }
    }

    public boolean isAdmin() {
        return BaseUtilsKt.isAdmin(router.getLiveRoom());
    }

    public boolean isMediaGroup() {
        return router.getLiveRoom().getOnlineUserVM().isMediaGroup();
    }

    public IUserModel getCurrentUser() {
        return router.getLiveRoom().getCurrentUser();
    }

    public void kickOutUser(IUserModel userModel) {
        router.getSubjectByKey(EventKey.KickOutConfirm).onNext(userModel);
    }

    public void requestForbidRaiseHand(boolean isForbid) {
        router.getLiveRoom().requestForbidRaiseHand(isForbid);
    }

    private boolean containStudentOrAssist() {
        List<IUserModel> activeUserList = router.getLiveRoom().getOnlineUserVM().getActiveUserList();
        for (IUserModel iUserModel : activeUserList) {
            if (iUserModel.getType() == LPConstants.LPUserType.Student || iUserModel.getType() == LPConstants.LPUserType.Assistant) {
                return true;
            }
        }
        return false;
    }

    public boolean enable1v1TeacherOffStage() {
        return router.getLiveRoom().getOnlineUserVM().enable1v1TeacherOffStage();
    }

    public void upOrDownFromSeatContainer(IUserModel userModel, boolean isShowOnSeatContainer) {
        if (isShowOnSeatContainer) {
            // 上台
            List<IUserModel> curList = router.getLiveRoom().getOnlineUserVM().getActiveUserList();
            // 新list避免影响对activeList
            List<IUserModel> curNewList = new ArrayList<>();
            if (curList != null) {
                for (IUserModel iUserModel : curList) {
                    if (iUserModel.getType() != LPConstants.LPUserType.Teacher || enable1v1TeacherOffStage()) {
                        curNewList.add(iUserModel);
                    }
                }
                int count = router.getLiveRoom().getMaxActiveUsers();
                if (enable1v1TeacherOffStage()) {
                    count = count + 1;
                }
                if (curNewList.size() >= count) {
                    router.getPublishSubjectByKey(EventCode.UserWhenFullActiveAdd).onNext(new ReplaceUserWithStatus(userModel));
                    if (listener != null)
                        listener.notifyCurrentSeatFull();
                    return;
                }
            }
            if (router.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE
                    && containStudentOrAssist() && !enable1v1TeacherOffStage()) {
                router.getPublishSubjectByKey(EventCode.UserWhenFullActiveAdd).onNext(new ReplaceUserWithStatus(userModel));
                if (listener != null) {
                    listener.notifyCurrentSeatFull();
                    return;
                }
            }
            LPUserModel lpUserModel = (LPUserModel) userModel;
            router.getLiveRoom().getOnlineUserVM().requestAddActiveUser(lpUserModel);
        } else {
            // 下台
            router.getLiveRoom().getOnlineUserVM().requestRemoveActiveUser(userModel);
        }
    }

    public void getForbidRaiseHandUp() {
        if (listener != null)
            listener.notifyForbidHandUp(router.getLiveRoom().getForbidRaiseHandStatus());
    }


    //是否上台
    public boolean isOnVideoSeat(IUserModel userModel) {
        return router.getLiveRoom().getOnlineUserVM().isActiveUser(userModel);
    }

    public LPMediaModel getMediaModel(IUserModel userModel) {
        final LPPlayer player = router.getLiveRoom().getPlayer();
        if (player != null) {
            return player.getChmUserMediaModel().get(userModel.getUserId());
        }
        return null;
    }

    public boolean isSelfVideoOn() {
        return router.getLiveRoom().getRecorder().isVideoAttached();
    }

    public boolean isSelfAudioOn() {
        return router.getLiveRoom().getRecorder().isAudioAttached();
    }

    public void controlRemoteSpeak(String userId, boolean isVideoOn, boolean isAudioOn) {
        if (userId.equals(router.getLiveRoom().getCurrentUser().getUserId())) {
            router.getPublishSubjectByKey(EventCode.AttachVideo).onNext(isVideoOn);
            router.getPublishSubjectByKey(EventCode.AttachAudio).onNext(isAudioOn);
        } else {
            router.getLiveRoom().getSpeakQueueVM().controlRemoteUser(userId, isVideoOn, isAudioOn);
        }
    }

    /**
     * 自习室，画廊布局模式
     *
     * @return
     */
    public boolean isStudyGalleryMode() {
        return router.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout();
    }

    public boolean isSelfStudyMode() {
        return router.getLiveRoom().getStudyRoomVM().getStudyRoomMode() == LPConstants.StudyRoomMode.SelfStudy;
    }

    public boolean hasPPTAuth(IUserModel userModel) {
        if (userModel.getType() == LPConstants.LPUserType.Teacher || userModel.getType() == LPConstants.LPUserType.Assistant) {
            return true;
        }
        final List<String> authList = router.getLiveRoom().getDocListVM().getStudentPPTAuthList();
        return authList != null && authList.contains(userModel.getNumber());
    }

    public boolean isSingleTutorUser(IUserModel userModel) {
        final List<LPStudyRoomTutorModel> roomTutorModels = router.getLiveRoom().getStudyRoomVM().getStudyRoomCurrentTutorGroup();
        for (LPStudyRoomTutorModel lpStudyRoomTutorModel : roomTutorModels) {
            if (TextUtils.equals(lpStudyRoomTutorModel.from, userModel.getUserId()) || TextUtils.equals(lpStudyRoomTutorModel.to, userModel.getUserId())) {
                return true;
            }
        }
        return false;
    }

    public boolean hasScreenShare(IUserModel userModel) {
        if (userModel.getType() == LPConstants.LPUserType.Teacher || userModel.getType() == LPConstants.LPUserType.Assistant) {
            return true;
        }
        final List<String> screenShareList = router.getLiveRoom().getMediaVM().getStudentScreenShareList();
        return screenShareList != null && screenShareList.contains(userModel.getNumber());
    }

    public void changePPTPaint(boolean grandPPT, String userNumber) {
        router.getLiveRoom().getSpeakQueueVM().requestStudentDrawingAuthChange(grandPPT, userNumber);
    }

    public void changePPTAuth(boolean grandPPT, String userNumber) {
        router.getLiveRoom().getDocListVM().requestStudentPPTAuthChange(grandPPT, userNumber);
    }

    public void changeScreenShare(boolean grandPPT, IUserModel userModel) {
        router.getLiveRoom().getMediaVM().requestStudentScreenShareChange(grandPPT, userModel);
    }

    public void requestAward(IUserModel userModel, String key) {
        if (!router.getLiveRoom().isClassStarted()) {
            return;
        }
        String userNumber = userModel.getNumber();
        long currentTime = System.currentTimeMillis();
        boolean canAward = BaseUtilsKt.isAdmin(router.getLiveRoom());
        if (canAward && currentTime - lastRquestAwardTime > 3000) {
            LPAwardUserInfo userInfo = awardRecord.get(userNumber);
            if (userInfo == null) {
                Map<String, Integer> typeCount = getTypeCount();
                typeCount.put(key, 1);
                userInfo = new LPAwardUserInfo(1, typeCount);
            } else {
                userInfo.count++;
                if (userInfo.typeCount == null) {
                    userInfo.typeCount = getTypeCount();
                }
                Integer keyCount = userInfo.typeCount.get(key);
                if (keyCount == null) {
                    userInfo.typeCount.put(key, 1);
                } else {
                    userInfo.typeCount.put(key, keyCount + 1);
                }
            }
            userInfo.name = userModel.getName();
            awardRecord.put(userNumber, userInfo);
            router.getLiveRoom().getToolBoxVM().requestAward(userNumber, key, awardRecord);
            lastRquestAwardTime = currentTime;
        }
    }

    private Map<String, Integer> getTypeCount() {
        Map<String, Integer> map = new HashMap<>();
        final LPAwardConfig[] awardConfigs = router.getLiveRoom().getToolBoxVM().getAwardConfigs();
        for (LPAwardConfig lpAwardConfig : awardConfigs) {
            if (lpAwardConfig.isEnable == 1) {
                map.put(lpAwardConfig.key, 0);
            }
        }
        return map;
    }

    public int getAwardGroupCount(int group) {
        final Integer count = router.getLiveRoom().getToolBoxVM().getAwardGroupMap().get(group);
        return count == null ? 0 : count;
    }

    public int getAwardCount(String userNumber) {
        final LPAwardUserInfo lpAwardUserInfo = awardRecord.get(userNumber);
        return lpAwardUserInfo == null ? 0 : lpAwardUserInfo.count;
    }

    public LPAwardConfig[] getAwardConfigs() {
        return router.getLiveRoom().getToolBoxVM().getAwardConfigs();
    }

    public void closeWindow() {
        router.getSubjectByKey(EventKey.DisplayUserList).onNext(UserStatus.Close);
    }

    public void showBlockedUserFreeAllUI() {
        if (listener != null) {
            listener.showBlockedUserFreeAllUI();
        }
    }

    public void setBlockedUserFreeAll() {
        router.getLiveRoom().getOnlineUserVM().freeAllBlockedUser();
    }

    public boolean isLoading() {
        return isLoading;
    }

    public void loadMore() {
        isLoading = true;
        router.getLiveRoom().getOnlineUserVM().loadMoreUser();
    }

    public String getTeacherCustomLabelText() {
        return TextUtils.isEmpty(router.getLiveRoom().getCustomizeTeacherLabel()) ? "主讲" : router.getLiveRoom().getCustomizeTeacherLabel();
    }

    public String getAssistantCustomLabelText() {
        return TextUtils.isEmpty(router.getLiveRoom().getCustomizeAssistantLabel()) ? "助教" : router.getLiveRoom().getCustomizeAssistantLabel();
    }

    @Override
    public void destroy() {
        RxUtils.dispose(disposables);
    }
}
