package com.baijiayun.groupclassui.user

import android.content.Context
import android.text.TextUtils
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.LinearLayout
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.baijiayun.groupclassui.R
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.livebase.context.LPConstants
import com.baijiayun.livecore.models.LPMediaModel
import com.baijiayun.livecore.models.imodels.IMediaModel
import com.baijiayun.livebase.models.imodels.IUserModel
import com.baijiayun.livecore.wrapper.LPPlayer
import kotlin.math.min

/**
 * 触发此adapter的条件为：当台上人数已满且有人想要上台时
 */
class ReplaceListRecyclerViewAdapter(var activeUserListCur: MutableList<IUserModel>, val context: Context, val presenter: ReplacePresenter)
    : RecyclerView.Adapter<ReplaceListRecyclerViewAdapter.ViewHolder>() {

    private var activeUserListWithStatus: MutableList<IMediaModel>? = null
    private var selectedUserPosition = -1
    private var selectedUserOldPosition = -1

    init {
        // speakList
        filterTeacherFromSpeakUserList()
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder =
            ViewHolder(LayoutInflater.from(parent.context).inflate(R.layout.bjy_group_item_replace_user, parent, false))

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val positionSelected = position
        if (positionSelected == selectedUserPosition) {
            holder.replaceIv.setImageResource(R.drawable.bjy_group_replace_shape_rb_bg_selected)
        } else {
            holder.replaceIv.setImageResource(R.drawable.bjy_group_replace_shape_rb_bg_normal)
        }
        activeUserListCur.let {
            holder.itemView.setOnClickListener {
                selectedUserOldPosition = selectedUserPosition
                holder.replaceIv.setImageResource(R.drawable.bjy_group_replace_shape_rb_bg_selected)
                selectedUserPosition = positionSelected
                notifyItemChanged(positionSelected)
                // need
                notifyItemChanged(selectedUserOldPosition)
            }
            if (it.isEmpty()) {
                holder.replaceUserNameTv.text = context.resources.getString(R.string.bjy_group_activeuser_dialog_vacancy)
                holder.replaceIconContainer.visibility = View.GONE
                holder.replaceUserLabelTv.visibility = View.GONE
            } else {
                if (positionSelected <= it.size - 1) {
                    val iUserModel = it[positionSelected]
                    holder.replaceUserNameTv.text = iUserModel.name
                    holder.replaceIconContainer.visibility = View.VISIBLE
                    // activeUserListWithStatus.size <= activeUserListCur.size
                    if (activeUserListWithStatus?.size == it.size) {
                        // 全员都有发言，但是此处activeUserListWithStatus的内容可能不对而导致助教状态不了
                        val mediaModel = getMediaModelFromSpeakList(iUserModel.userId)
                        if (mediaModel != null) {
                            // 此处远程的video、audio状态与selected状态相反
                            holder.replaceUserCameraIv.isSelected = !mediaModel.isVideoOn
                            holder.replaceUserMicIv.isSelected = !mediaModel.isAudioOn
                        } else {
                            if (isSelf(iUserModel)) {
                                // 助教
                                holder.replaceUserCameraIv.isSelected = !presenter.isSelfVideoOn
                                holder.replaceUserMicIv.isSelected = !presenter.isSelfAudioOn
                            } else {
                                holder.replaceUserCameraIv.isSelected = true
                                holder.replaceUserMicIv.isSelected = true
                            }
                        }
                    } else {
                        // 刚打开window时或有空位上台时
                        var isVideoOn = false
                        var isAudioOn = false
                        if (isSelf(iUserModel)) {
                            // 选中的人刚好是自己时，只有助教才可能选到自己
                            isAudioOn = presenter.isSelfAudioOn
                            isVideoOn = presenter.isSelfVideoOn
                        } else {
                            val mediaModel = getMediaModel(iUserModel)
                            if (mediaModel != null) {
                                // 此处远程的video、audio状态与selected状态相反
                                isVideoOn = mediaModel.videoOn
                                isAudioOn = mediaModel.audioOn
                            }
                        }
                        holder.replaceUserCameraIv.isSelected = !isVideoOn
                        holder.replaceUserMicIv.isSelected = !isAudioOn
                    }
                    if (iUserModel.type == LPConstants.LPUserType.Assistant) {
                        // 当item为助教时，不可选
//                        holder.itemView.isClickable = false
//                        holder.replaceCb.isSelected = false
                        holder.replaceUserLabelTv.text = presenter.assistantCustomLabelText
                        holder.replaceUserLabelTv.background = ThemeDataUtil.getCustomLabelBg(
                            context,
                            ThemeDataUtil.getColorFromThemConfigByColorId(
                                context,
                                R.color.bjy_base_assistant_custom_label_bg_color
                            )
                        )
                        holder.replaceUserLabelTv.visibility = View.VISIBLE
                    } else {
                        holder.replaceUserLabelTv.visibility = View.GONE
                    }
                } else {
                    // 空位
                    holder.replaceUserNameTv.text = context.resources.getString(R.string.bjy_group_activeuser_dialog_vacancy)
                    holder.replaceIconContainer.visibility = View.GONE
                    holder.replaceUserLabelTv.visibility = View.GONE
                }
            }
        }
    }

    fun getSelectedUserPosition(): Int {
        return selectedUserPosition
    }

    private fun isSelf(userModel: IUserModel): Boolean {
        return TextUtils.equals(userModel.userId, presenter.router.liveRoom.currentUser.userId)
    }

    fun notifyCurrentActiveUserChange(activeUserListCur: MutableList<IUserModel>) {
        this.activeUserListCur = activeUserListCur
        filterTeacherFromSpeakUserList()
        notifyDataSetChanged()
    }

    private fun filterTeacherFromSpeakUserList() {
        // 过滤发言的老师，且因为用不到该list的下标，所以不需要重排序
        val speakLists = presenter.router.liveRoom.speakQueueVM.activeUserList
        val speakUsers = ArrayList<IMediaModel>()
        speakLists?.let {
            for (iUserModel in it) {
                if (iUserModel.user.type != LPConstants.LPUserType.Teacher) {
                    speakUsers.add(iUserModel)
                }
            }
        }
        activeUserListWithStatus = speakUsers
    }

    private fun getMediaModelFromSpeakList(useId: String): IMediaModel? {
        for (mediaModel in activeUserListWithStatus!!) {
            if (useId == mediaModel.user.userId) {
                return mediaModel
            }
        }
        return null
    }

    private fun getMediaModel(userModel: IUserModel): LPMediaModel? {
        val player: LPPlayer = presenter.router.liveRoom.player
        return player.chmUserMediaModel[userModel.userId]
    }

    // 此时count应为最大上台人数
    override fun getItemCount(): Int = min(
        if (presenter.router.liveRoom.onlineUserVM.enable1v1TeacherOffStage()) {
            presenter.router.liveRoom.maxActiveUsers + 1
        } else {
            presenter.router.liveRoom.maxActiveUsers
        }, activeUserListCur.size
    )

    class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        var replaceIv: ImageView = view.findViewById(R.id.replace_iv)
        var replaceUserNameTv: TextView = view.findViewById(R.id.replace_user_name_tv)
        var replaceUserLabelTv: TextView = view.findViewById(R.id.replace_user_label_tv)
        var replaceIconContainer: LinearLayout = view.findViewById(R.id.replace_icon_container)
        var replaceUserCameraIv: ImageView = view.findViewById(R.id.replace_user_camera_iv)
        var replaceUserMicIv: ImageView = view.findViewById(R.id.replace_user_mic_iv)
    }
}