package com.baijiayun.groupclassui.util;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.view.View;

import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.context.LiveRoom;
import com.baijiayun.livebase.models.imodels.IUserModel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;
import java.util.Locale;
import java.util.Set;

/**
 * Created by yongjiaming on 2018/10/16
 */
public class InteractiveUtils {

    //24小时内有效
    public static String formatTime(int timeInSeconds) {
        if (timeInSeconds < 10) {
            return String.valueOf(timeInSeconds);
        } else if (timeInSeconds < 60) {
            return "00:" + timeInSeconds;
        } else if (timeInSeconds < 60 * 60) {
            return ((timeInSeconds / 60) < 10) ? "0" + timeInSeconds / 60 + ":" + timeInSeconds % 60 : timeInSeconds / 60 + ":" + timeInSeconds % 60;
        } else {
            return timeInSeconds / 3600 + ":" + timeInSeconds % 3600 / 60 + ":" + timeInSeconds % 3600 % 60;
        }
    }

    /**
     * 将 long 时间格式化 仅支持正数，负数会被转成正数
     * @param time 时间 - 秒级
     * @return "HH:mm:ss" 格式的字符串
     */
    public static String formatTimeByLong(long time) {
        time = Math.abs(time);
        long sec = time % 60;
        long min = time % (60 * 60) / 60;
        long hou = time / (60 * 60);
        return fillZeroToSize_2(hou) + ":"
                + fillZeroToSize_2(min) + ":"
                + fillZeroToSize_2(sec);
    }

    private static String fillZeroToSize_2(long value) {
        if (value < 10) {
            return "0" + value;
        } else {
            return String.valueOf(value);
        }
    }

    public static void saveViewCapture(View view, String dir) {
        Bitmap bitmap = Bitmap.createBitmap(view.getWidth(), view.getHeight(), Bitmap.Config.RGB_565);
        Canvas canvas = new Canvas(bitmap);
        view.draw(canvas);
        File f = new File(dir, "bjy_capture_" + System.currentTimeMillis() + ".png");
        try {
            FileOutputStream out = new FileOutputStream(f);
            bitmap.compress(Bitmap.CompressFormat.PNG, 80, out);
            out.flush();
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static boolean hasDrawingAuth(LiveRoom liveRoom, IUserModel userModel) {
        if (userModel == null) {
            return false;
        }
        boolean hasDrawingAuth;
        if (userModel.getType() == LPConstants.LPUserType.Teacher || userModel.getType() == LPConstants.LPUserType.Assistant) {
            hasDrawingAuth = true;
        } else {
            Set<String> list = liveRoom.getSpeakQueueVM().getStudentsDrawingAuthSet();
            hasDrawingAuth = list != null && list.contains(userModel.getNumber());
        }
        return hasDrawingAuth;
    }

    public static boolean hasPPTAuth(LiveRoom liveRoom, IUserModel userModel) {
        if (liveRoom == null) {
            return false;
        }
        List<String> userNumbers = liveRoom.getDocListVM().getPublishSubjectOfStudentPPTAuth().getValue();
        return liveRoom.isTeacherOrAssistant() || (userNumbers != null && userNumbers.contains(userModel.getNumber()));
    }

    public static boolean hasScreenShareAuth(LiveRoom liveRoom, IUserModel userModel) {
        if (liveRoom == null) {
            return false;
        }
        List<String> userNumbers = liveRoom.getMediaVM().getStudentScreenShareList();
        return liveRoom.isTeacherOrAssistant() || (userNumbers != null && userNumbers.contains(userModel.getNumber()));
    }

    public static String formatDuration(int duration) {
        int _duration = Math.abs(duration);
        int hour = _duration / 3600;
        int temp = _duration % 3600;

        int minutes = temp / 60;
        temp = temp % 60;
        int seconds = temp % 60;

        String res;
        if (hour > 0) {
            res = String.format(Locale.ENGLISH,"%02d:%02d:%02d", hour, minutes, seconds);
        } else {
            res = String.format(Locale.ENGLISH,"%02d:%02d", minutes, seconds);
        }

        return duration >= 0 ? res : "- " + res;
    }

    public static String getEndTypeName(LPConstants.LPEndType endType) {
        switch (endType) {
            case iOS:
                return "苹果手机";
            case PC_H5:
                return "网页";
            case PC_HTML:
                return "电脑网页";
            case PC_Client:
                return "电脑客户端";
            case Android:
                return "Android手机";
            case PC_MAC_Client:
                return "Mac客户端";
        }
        return "电脑";
    }
}
