package com.baijiayun.groupclassui.widget;


import android.content.Context;
import android.util.AttributeSet;
import android.util.DisplayMetrics;
import android.view.MotionEvent;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

/**
 * 可拖动的 RelativeLayout
 * panzq
 * 20190523
 */
public class DragRelativeLayout extends RelativeLayout {

    private final int DRAG_DISTANCE_PX = 50;//活动屏蔽事件像素距离

    private int lastX, lastY, screenWidth, screenHeight, width, height;
    private int left, top, right, bottom;

    private int downX, downY;

    private boolean isDragEnable = true;
    private boolean error = true;

    private boolean isDown = false;
    private OnUpdateLayoutParamsListener onUpdateLayoutParamsListener;

    //用于控制屏幕计算时去除顶部与两边边距
    private int fWidth, fHeight;


    public DragRelativeLayout(Context context) {
        this(context, null);
    }

    public DragRelativeLayout(Context context, AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public DragRelativeLayout(Context context, AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        DisplayMetrics dm = getResources().getDisplayMetrics();
        screenWidth = dm.widthPixels;
        screenHeight = dm.heightPixels;
    }

    @Override
    protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
        super.onMeasure(widthMeasureSpec, heightMeasureSpec);
        width = getMeasuredWidth();
        height = getMeasuredHeight();
    }

    //定位
    @Override
    protected void onLayout(boolean changed, int l, int t, int r, int b) {
        super.onLayout(changed, l, t, r, b);
        //可以在这里确定这个viewGroup的：宽 = r-l.高 = b - t
    }

    public void setDragEnable(boolean isEnable){
        this.isDragEnable = isEnable;
    }

    private final int moveJudgeUnitHold = 50;
    private final int moveJudgeUnitMoving = 1;
    private int moveJudgeUnit = moveJudgeUnitHold;

    //拦截touch事件
    @Override
    public boolean onInterceptTouchEvent(MotionEvent ev) {

        if (!isDragEnable)
            return super.onInterceptTouchEvent(ev);

        int action = ev.getAction();
        switch (action) {
            case MotionEvent.ACTION_DOWN:
                lastX = (int) ev.getRawX();//设定移动的初始位置相对位置
                lastY = (int) ev.getRawY();

                downX = lastX;
                downY = lastY;

                if (!isDown) {
                    isDown = true;
                    if (onUpdateLayoutParamsListener != null){
                        onUpdateLayoutParamsListener.onActionDown();
                    }
                }

                if (error) {
                    error = false;
                    FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(this.width, FrameLayout.LayoutParams.WRAP_CONTENT);
                    layoutParams.leftMargin = getLeft();
                    layoutParams.topMargin = getTop();
                    layoutParams.rightMargin = 0;
                    layoutParams.bottomMargin = 0;
                    this.setLayoutParams(layoutParams);
                }
                break;
            case MotionEvent.ACTION_MOVE://移动
                //event.getRawX()事件点距离屏幕左上角的距离
                int dx = (int) ev.getRawX() - lastX;
                int dy = (int) ev.getRawY() - lastY;

                if (fWidth == 0) {
                    fWidth = screenWidth;
                }
                if (fHeight == 0) {
                    fHeight = screenHeight;
                }

                if (Math.abs(dx) >= moveJudgeUnit || Math.abs(dy) >= moveJudgeUnit) {
                    moveJudgeUnit = moveJudgeUnitMoving;
                    int left = this.getLeft() + dx;
                    int top = this.getTop() + dy;
                    int right = this.getRight() + dx;
                    int bottom = this.getBottom() + dy;
                    if (left < 0) { //最左边
                        left = 0;
                        right = left + this.getWidth();
                    }
                    if (right > fWidth) { //最右边
                        right = fWidth;
                        left = right - this.getWidth();
                    }
                    if (top < 0) {  //最上边
                        top = 0;
                        bottom = top + this.getHeight();
                    }
                    if (bottom > fHeight) {//最下边
                        bottom = fHeight;
                        top = bottom - this.getHeight();
                    }
                    this.layout(left, top, right, bottom);//设置控件的新位置
                    this.left = left;
                    this.top = top;
                    this.right = right;
                    this.bottom = bottom;

                    lastX = (int) ev.getRawX();//再次将滑动其实位置定位
                    lastY = (int) ev.getRawY();
                }
                break;
            case MotionEvent.ACTION_UP:
                //保存移动后的位置状态
                isDown =false;
                moveJudgeUnit = moveJudgeUnitHold;
                if (onUpdateLayoutParamsListener != null) {
                    onUpdateLayoutParamsListener.onUpdateLayoutParams(this.left, this.top, this.right, this.bottom);
                }

                if (Math.abs(lastX - downX) >= DRAG_DISTANCE_PX
                        || Math.abs(lastY - downY) >= DRAG_DISTANCE_PX) {
                    FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(this.width, FrameLayout.LayoutParams.WRAP_CONTENT);
                    layoutParams.leftMargin = this.left;
                    layoutParams.topMargin = this.top;
                    layoutParams.rightMargin = 0;
                    layoutParams.bottomMargin = 0;
                    this.setLayoutParams(layoutParams);
                    return true;
                }
                break;
            default:
                break;
        }
        return super.onInterceptTouchEvent(ev);
    }

    public void setDragParams(int width, int height) {
        this.fWidth = width;
        this.fHeight = height;
    }

    public void setOnUpdateLayoutParamsListener(OnUpdateLayoutParamsListener listener) {
        this.onUpdateLayoutParamsListener = listener;
    }

    public interface OnUpdateLayoutParamsListener{

        void onUpdateLayoutParams(int left, int top, int right, int bottom);

        void onActionDown();
    }
}
