package com.baijiayun.groupclassui.widget;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.RectF;
import android.media.AudioManager;
import android.os.Build;
import androidx.annotation.ColorInt;
import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;
import androidx.core.content.ContextCompat;
import android.util.AttributeSet;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;

import com.baijiayun.groupclassui.R;
import com.baijiayun.livebase.utils.ThemeDataUtil;

/**
 * Created by Shubo on 2018/10/19.
 */
public class ToggleSpeakerVolumeView extends View {

    private int max, min, current;

    @ColorInt
    private int bgColor, curColor;

    private RectF rect;

    private Paint paint;

    private AudioManager mAudioManager;

    private OnTouchSpeakVolumeChangeListener listener;

    public ToggleSpeakerVolumeView(Context context) {
        this(context, null);
    }

    public ToggleSpeakerVolumeView(Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public ToggleSpeakerVolumeView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public ToggleSpeakerVolumeView(Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        init();
    }

    public void setListener(OnTouchSpeakVolumeChangeListener listener) {
        this.listener = listener;
    }

    private void init() {
        setLayerType(LAYER_TYPE_HARDWARE, paint);

        bgColor = ContextCompat.getColor(getContext(), R.color.bjysc_half_black);
        curColor = ThemeDataUtil.getColorFromThemeConfigByAttrId(getContext(), R.attr.base_theme_live_product_color);
        mAudioManager = (AudioManager) getContext().getSystemService(Context.AUDIO_SERVICE);
        if (mAudioManager != null) {
            max = mAudioManager.getStreamMaxVolume(AudioManager.STREAM_VOICE_CALL);
            min = 0;
            current = mAudioManager.getStreamVolume(AudioManager.STREAM_VOICE_CALL);
        }
        rect = new RectF();

        rect.top = 0;
        rect.left = 0;

        paint = new Paint();
        paint.setAntiAlias(true);
        paint.setStyle(Paint.Style.FILL_AND_STROKE);
    }

    @Override
    protected void onDraw(Canvas canvas) {
        super.onDraw(canvas);

        int width = getWidth();
        int height = getHeight();

        rect.right = width;
        rect.bottom = height;

        paint.setColor(bgColor);
        canvas.drawRoundRect(rect, height / 2, height / 2, paint);

        rect.right = ((float) current) / max * width;

        paint.setColor(curColor);
        canvas.drawRoundRect(rect, height / 2, height / 2, paint);

        if (current > min && current < max) {
            canvas.drawRect(height / 2, 0, rect.right, height, paint);
        }
    }

    @SuppressLint("ClickableViewAccessibility")
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        switch (event.getAction()) {
            case MotionEvent.ACTION_MOVE:
                float x = event.getX();
                int newCur = Math.max(min, Math.min(max, (int) (x / getWidth() * max)));
                if (current != newCur) {
                    current = newCur;
                    invalidate();
                    if (mAudioManager != null) {
                        mAudioManager.setStreamVolume(AudioManager.STREAM_VOICE_CALL, current, 0);
                    }
                    if (listener != null) {
                        listener.onVolumeChange(current, min, max);
                    }
                }
                break;
            default:
                break;
        }
        return true;
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (mAudioManager == null) return super.onKeyDown(keyCode, event);
        switch (keyCode) {
            case KeyEvent.KEYCODE_VOLUME_UP:
                mAudioManager.adjustStreamVolume(
                        AudioManager.STREAM_VOICE_CALL,
                        AudioManager.ADJUST_RAISE, 0);
                current++;
                invalidate();
                return true;
            case KeyEvent.KEYCODE_VOLUME_DOWN:
                mAudioManager.adjustStreamVolume(
                        AudioManager.STREAM_VOICE_CALL,
                        AudioManager.ADJUST_LOWER, 0);
                current--;
                invalidate();
                return true;
            default:
                break;
        }
        return super.onKeyDown(keyCode, event);
    }

    @FunctionalInterface
    public interface OnTouchSpeakVolumeChangeListener {

        void onVolumeChange(int volumeLevel, int min, int max);
    }
}
