package com.baijiayun.groupclassui.window;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.livecore.models.LPPlayerViewUpdateModel;
import com.baijiayun.liveuibase.widgets.DragScaleRelativeLayout;

/**
 *
 * @author Shubo
 * @date 2018/9/13
 */
public abstract class BaseTitledWindow extends InteractiveBaseWindow {

    /**
     * 最小的一个边的尺寸 dp 值
     */
    protected static int DEFAULT_MIN_SIZE = 100;
    /**
     * 最小的一个边的尺寸
     */
    protected int minSize;
    /**
     * 另一个边 / 最小的边 的比例 - 大于 0 表示 宽 > 高； 小于 0 表示 高 > 宽
     */
    protected float otherSizeRatio = 2.0f;

    protected TextView tvTitle;
    protected ImageView ivClose, ivMaxFull, ivMaxSync, ivScale;
    /**
     *  顶栏最小化按钮
     */
    protected ImageView minIv;
    protected FrameLayout rlTitleContainer, rlContentContainer, rlFooterContainer, contentOuterContainer;
    protected DragScaleRelativeLayout dragScaleRelativeLayout;
    protected ViewGroup.LayoutParams layoutParamsSmall;
    private ViewGroup.LayoutParams lastLayoutParams;
    protected boolean isMaxInSync, isMaxInFull, isReceiveOrder,isLastMaxInSync;
    /**
     * 是否最小化
     */
    protected boolean isMinInSync;
    /**
     *  pptWindow的图标暂时不替换
     */
    protected boolean isPPTWindow;
    /**
     * 允许缩放 ，默认为true
     */
    protected boolean canScale = true;
    protected Drawable fullDrawable,fullResetDrawable,maxDrawable, maxResetDrawable;

    protected BaseTitledWindow(Context context) {
        super(context);
    }

    @Override
    protected View onCreateView(Context context) {
        minSize = UtilsKt.getDp(DEFAULT_MIN_SIZE);
        setNeedShowShadow(true);

        dragScaleRelativeLayout = (DragScaleRelativeLayout) LayoutInflater.from(context).inflate(R.layout.bjy_base_window_titled, null);
        initViews();

        onCreateTopView(context);
        onCreateContentView(context);
        onCreateFooterView(context);
        initTopTitleView();
        initDrawableIds();
        initDrawables();

        return dragScaleRelativeLayout;
    }

    protected void initTopTitleView() {
        tvTitle = dragScaleRelativeLayout.findViewById(R.id.window_titled_title);
        ivClose = dragScaleRelativeLayout.findViewById(R.id.window_titled_close);
        ivMaxFull = dragScaleRelativeLayout.findViewById(R.id.window_titled_max_in_full_layer);
        ivMaxSync = dragScaleRelativeLayout.findViewById(R.id.window_titled_max_in_sync_layer);

        ivMaxFull.setOnClickListener(l -> {
            isReceiveOrder = false;
            if (isMaxInFull) {
                BaseTitledWindow.this.resetInFull();
            } else {
                BaseTitledWindow.this.maxInFull();
            }
        });
        ivMaxSync.setOnClickListener(l -> {
            isReceiveOrder = false;
            if (isMaxInSync) {
                BaseTitledWindow.this.resetInSync();
            } else {
                BaseTitledWindow.this.maxInSync();
            }
        });
    }

    private void initViews() {
        ivScale = dragScaleRelativeLayout.findViewById(R.id.window_titled_scale_icon);
        rlTitleContainer = dragScaleRelativeLayout.findViewById(R.id.window_titled_top_container);
        rlFooterContainer = dragScaleRelativeLayout.findViewById(R.id.window_titled_footer_container);
        rlContentContainer = dragScaleRelativeLayout.findViewById(R.id.window_titled_content);
        contentOuterContainer = dragScaleRelativeLayout.findViewById(R.id.window_titled_content_container);

        // 设置拖动事件范围 宽度为右侧按钮宽 高度为titleContainer高
        dragScaleRelativeLayout.setDragThreshold(
                UtilsKt.getDp(128),
                UtilsKt.getDp(40));

        if (!isAdmin()) {
            ivScale.setVisibility(View.GONE);
        }
    }

    protected int maxDrawableId;
    protected int maxResetDrawableId;
    protected int fullDrawableId;
    protected int fullResetDrawableId;

    protected void initDrawableIds() {
        maxDrawableId = R.drawable.ic_max_in_sync_layer;
        maxResetDrawableId = R.drawable.ic_max_reset_to_sync_layer;
        fullDrawableId = R.drawable.ic_full_to_full_layer;
        fullResetDrawableId = R.drawable.ic_full_reset_to_sync_layer;
    }

    private void initDrawables() {
        fullDrawable = ContextCompat.getDrawable(context, fullDrawableId);
        fullResetDrawable = ContextCompat.getDrawable(context, fullResetDrawableId);
        maxDrawable = ContextCompat.getDrawable(context, maxDrawableId);
        maxResetDrawable = ContextCompat.getDrawable(context, maxResetDrawableId);
    }

    /**
     * 最大化到黑板区
     */
    public void maxInSync() {
        if (view == null) {
            return;
        }
        ViewGroup viewGroup = getParentViewGroup();
        if (viewGroup == null || isMaxInSync) {
            return;
        }
        isMaxInSync = true;
        dragScaleRelativeLayout.bringToFront();
        if (!isMaxInFull && !isPPTWindow) {
            layoutParamsSmall = dragScaleRelativeLayout.getLayoutParams();
        }
        isMaxInFull = false;
        maximize();
        iRouter.getSubjectByKey(EventKey.SetMaximumWindowOn).onNext(this);
        ivMaxSync.setImageDrawable(maxResetDrawable);
    }

    /**
     * 退出最大化状态
     */
    public void resetInSync() {
        if (view == null) {
            return;
        }
        ViewGroup viewGroup = getParentViewGroup();
        if (viewGroup == null) {
            return;
        }
        ivMaxSync.setImageDrawable(maxDrawable);
        dragScaleRelativeLayout.setLayoutParams(layoutParamsSmall);
        iRouter.getSubjectByKey(EventKey.SetMaximumWindowOff).onNext(this);
        isMaxInSync = false;
    }

    /**
     * 全屏
     */
    public void maxInFull() {
        if (view == null || isMaxInFull) {
            return;
        }
        isMaxInFull = true;
        if (!isMaxInSync && !isPPTWindow) {
            layoutParamsSmall = dragScaleRelativeLayout.getLayoutParams();
        }
        lastLayoutParams = dragScaleRelativeLayout.getLayoutParams();
        isLastMaxInSync = isMaxInSync;
        isMaxInSync = false;
        ivMaxSync.setVisibility(View.GONE);
        ivMaxFull.setImageDrawable(fullResetDrawable);
        iRouter.getSubjectByKey(EventKey.SetFullScreenWindowOn).onNext(this);
    }

    /**
     * 退出全屏状态
     */
    public void resetInFull() {
        if (view == null) {
            return;
        }
        isMaxInFull = false;
        isMaxInSync = isLastMaxInSync && !isPPTWindow;
        // 根据进入之前的状态设置icon
        ivMaxSync.setImageDrawable(isMaxInSync ? maxResetDrawable : maxDrawable);
        ivMaxSync.setVisibility(View.VISIBLE);
        ivMaxFull.setImageDrawable(fullDrawable);
        dragScaleRelativeLayout.setLayoutParams(lastLayoutParams == null ? layoutParamsSmall : lastLayoutParams);
        iRouter.getSubjectByKey(EventKey.SetFullScreenWindowOff).onNext(this);
    }

    /**
     * 黑板区最小化
     */
    public void minInSync() {
        contentOuterContainer.setVisibility(View.GONE);
        rlFooterContainer.setVisibility(View.GONE);
        ivScale.setVisibility(View.GONE);
        minIv.setVisibility(View.GONE);
        ivClose.setVisibility(View.GONE);

        ivMaxSync.setImageDrawable(ContextCompat.getDrawable(context, R.drawable.bjy_base_window_ic_reset));
        dragScaleRelativeLayout.setCanScaleOrDrag(false);
        if (!isMaxInFull) {
            // 全屏最小化时不需要重新设置lastLayoutParams
            lastLayoutParams = dragScaleRelativeLayout.getLayoutParams();
        }
        iRouter.getSubjectByKey(EventKey.MinimizeWindowOn).onNext(this);
        isMinInSync = true;
    }

    /**
     * 从最小化恢复到最小化之前的状态，除全屏
     */
    public void resetMinInSync() {
        if (isMinInSync) {
            contentOuterContainer.setVisibility(View.VISIBLE);
            rlFooterContainer.setVisibility(View.VISIBLE);
            minIv.setVisibility(View.VISIBLE);
            ivScale.setVisibility(View.VISIBLE);
            ivClose.setVisibility(View.VISIBLE);
        }
        // 主要更改layoutParams
        iRouter.getSubjectByKey(EventKey.MinimizeWindowOff).onNext(this);
        ivMaxSync.setImageDrawable(maxDrawable);
        if (isMaxInSync) {
            maxInSync();
            ivMaxSync.setImageDrawable(maxResetDrawable);
        } else if (isMaxInFull) {
            // 最小化恢复至进入全屏之前的状态：normal or max
            resetInFull();
        }
        isMinInSync = false;
    }

    public ViewGroup.LayoutParams getLayoutParamsSmall() {
        return layoutParamsSmall;
    }

    public ViewGroup.LayoutParams getLastLayoutParams() {
        return lastLayoutParams;
    }

    /**
     * 是否收到信令改动，防止老师助教端重复发送信令 !!
     */
    public void setReceiveOrder(boolean isReceiveOrder) {
        this.isReceiveOrder = isReceiveOrder;
    }

    public boolean isReceiveOrder() {
        return isReceiveOrder;
    }

    @Override
    public void moveTo(int x, int y) {
        if (isMaxInFull || isMaxInSync) {
            return;
        }
        super.moveTo(x, y);
    }

    @Override
    public void scaleTo(int width, int height) {
        if (isMaxInFull || isMaxInSync) {
            return;
        }
        super.scaleTo(width, height);
    }

    public boolean isMaxInSync() {
        return isMaxInSync;
    }

    public boolean isMaxInFull() {
        return isMaxInFull;
    }

    public boolean isMinInSync() {
        return isMinInSync;
    }

    protected abstract void onCreateContentView(Context context);
    //默认空实现，没有底栏
    protected void onCreateFooterView(Context context){

    }

    protected void onCreateTopView(Context context){
        LayoutInflater.from(context).inflate(R.layout.layout_window_top, rlTitleContainer);
    }

    //PPT样式顶栏才可调用
    public BaseTitledWindow title(String title) {
        tvTitle.setText(title);
        return this;
    }

    public BaseTitledWindow showClose(boolean isShow) {
        ivClose.setVisibility(isShow ? View.VISIBLE : View.GONE);
        return this;
    }

    public void showTopContainer(boolean isShow) {
        rlTitleContainer.setVisibility(isShow ? View.VISIBLE : View.GONE);
    }

    public void showScaleIcon(boolean isShow) {
        ivScale.setVisibility(isShow ? View.VISIBLE : View.GONE);
    }

    protected void setCanScale(boolean canScale) {
        this.canScale = canScale;
        showScaleIcon(canScale);
    }

    protected void hideContentMask() {
        $.id(R.id.window_titled_content_layer).invisible();
    }

    public BaseTitledWindow showFull(boolean isShow) {
        ivMaxFull.setVisibility(isShow ? View.VISIBLE : View.GONE);
        return this;
    }

    public BaseTitledWindow showSync(boolean isShow) {
        ivMaxSync.setVisibility(isShow ? View.VISIBLE : View.GONE);
        return this;
    }

    int downWidth, downHeight, downX, downY, measuredParentWidth, measuredParentHeight;

    boolean scaling = false;

    @Override
    protected boolean onTouchEvent(View v, MotionEvent event) {
        if (gestureDetector.onTouchEvent(event)) {
            return true;
        }
        if (isMinInSync) {
            return true;
        }
        if (event.getAction() == MotionEvent.ACTION_DOWN
                && dragScaleRelativeLayout.isDownActionInScaleThreshold(event)
                && !isMaxInSync && !isMaxInFull && canScale) {
            scaling = true;
        }
        if (!scaling || !allowTouch) {
            return super.onTouchEvent(v, event);
        }
        handleScaleTouchEvent(v, event);
        return true;
    }

    protected void handleScaleTouchEvent(View v, MotionEvent event) {
        int x = (int) event.getX();
        int y = (int) event.getY();
        switch (event.getAction()) {
            case MotionEvent.ACTION_DOWN:
                downX = x;
                downY = y;
                ViewGroup.LayoutParams layoutParams = dragScaleRelativeLayout.getLayoutParams();
                downWidth = layoutParams.width;
                downHeight = layoutParams.height;
                if (v.getParent() != null && v.getParent() instanceof ViewGroup) {
                    ViewGroup viewGroup = (ViewGroup) v.getParent();
                    measuredParentHeight = viewGroup.getMeasuredHeight();
                    measuredParentWidth = viewGroup.getMeasuredWidth();
                }
                break;
            case MotionEvent.ACTION_MOVE:
                int offX = x - downX;
                int offY = y - downY;

                // 缩放不超出边界
                if (v.getLeft() + downWidth + offX >= measuredParentWidth) {
                    offX = measuredParentWidth - downWidth - v.getLeft();
                }
                if (v.getTop() + downHeight + offY >= measuredParentHeight) {
                    offY = measuredParentHeight - downHeight - v.getTop();
                }

                // 缩放不小于最小值
                int height, width;
                if (otherSizeRatio > 0) {
                    height = Math.max(downHeight + offY, minSize);
                    width = Math.max(downWidth + offX, ((int) (minSize * otherSizeRatio)));
                } else {
                    height = Math.max(downHeight + offY, ((int) (minSize * - otherSizeRatio)));
                    width = Math.max(downWidth + offX, minSize);
                }
                scaleTo(width, height);
                if (onWindowRepositionListener != null) {
                    onWindowRepositionListener.OnWindowScale(width, height);
                }
                break;
            case MotionEvent.ACTION_UP:
                scaling = false;
                break;
            default:
                break;
        }
    }

    protected LPPlayerViewUpdateModel.PlayerPosition getPlayerPosition() {
        LPPlayerViewUpdateModel.PlayerPosition playerPosition = new LPPlayerViewUpdateModel.PlayerPosition();
        playerPosition.x = String.valueOf(1f * getView().getLeft() / getParentViewGroup().getMeasuredWidth());
        playerPosition.y = String.valueOf(1f * getView().getTop() / getParentViewGroup().getMeasuredHeight());
        playerPosition.width = String.valueOf(1f * getView().getMeasuredWidth() / getParentViewGroup().getMeasuredWidth());
        playerPosition.height = String.valueOf(1f * getView().getMeasuredHeight() / getParentViewGroup().getMeasuredHeight());

        if (isMaxInFull()) {
            playerPosition.full = 1;
        } else if (isMaxInSync()) {
            playerPosition.max = 1;
        }

        return playerPosition;
    }
}


