package com.baijiayun.groupclassui.window;

import android.content.Context;
import android.view.View;

import androidx.annotation.UiThread;

import com.baijiayun.groupclassui.base.BaseGroupPresenter;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.global.RouterListener;
import com.baijiayun.liveuibase.base.BaseWindow;
import com.baijiayun.liveuibase.base.ILayer;

/**
 *
 * @author Shubo
 * @date 2018/9/7
 */
public abstract class InteractiveBaseWindow extends BaseWindow implements IRoomStatusListener {

    protected IRouter iRouter;
    protected BaseGroupPresenter basePresenter;
    protected boolean isWindowInit;
    //延时回调
    private boolean laterCallback;
    private boolean isRoomActive;

    private View.OnAttachStateChangeListener onAttachStateChangeListener;

    @UiThread
    public InteractiveBaseWindow(Context context) {
        super(context);
        onAttachStateChangeListener = new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                if (context instanceof RouterListener) {
                    ((RouterListener) context).addRoomStatusListener(InteractiveBaseWindow.this);
                }
                isWindowInit = true;
                if (laterCallback) {
                    onRoomStatusChange(isRoomActive);
                    laterCallback = false;
                }
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
                if(context instanceof RouterListener){
                    ((RouterListener) context).removeRoomStatusListener(InteractiveBaseWindow.this);
                }
            }
        };
        view.addOnAttachStateChangeListener(onAttachStateChangeListener);
    }

    @Override
    protected void doOnCreateView(Context context) {
        super.doOnCreateView(context);
        if (context instanceof RouterListener) {
            iRouter = ((RouterListener) context).getRouter();
        }
    }

    protected void setBasePresenter(BaseGroupPresenter presenter) {
        basePresenter = presenter;
        presenter.setRouter(iRouter);
        basePresenter.subscribe();
    }

    @Override
    protected boolean isAdmin() {
        return iRouter != null && (iRouter.getLiveRoom().isTeacherOrAssistant() || iRouter.getLiveRoom().isGroupTeacherOrAssistant());
    }

    @Override
    protected boolean canBringToForeground() {
        return isAdmin();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if (basePresenter != null) {
            basePresenter.destroy();
            basePresenter = null;
        }
        if (view != null) {
            view.removeOnAttachStateChangeListener(onAttachStateChangeListener);
        }
        isWindowInit = false;
    }

    @Override
    protected void unSubscribe() {
        super.unSubscribe();
        if (basePresenter != null) {
            basePresenter.destroy();
            basePresenter = null;
        }
    }

    public String getId() {
        return "";
    }

    /**
     * 房间状态更新回调
     *
     * @param isActive true,进房间成功  false,退出房间
     */
    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        if (isWindowInit) {
            if (isActive) {

            } else {
                unSubscribe();
            }
        } else {
            laterCallback = true;
        }
        this.isRoomActive = isActive;
        return isWindowInit;
    }

    @Override
    public void moveTo(ILayer layer) {
        super.moveTo(layer);
    }

    @Override
    public void moveTo(int x, int y, int z) {
        super.moveTo(x, y, z);
    }
}
