package com.baijiayun.groupclassui.window.bottommenu;

import android.animation.Animator;
import android.animation.AnimatorSet;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.PopupWindow;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.dialog.HandsUpRemindPopupWindow;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.user.UserStatus;
import com.baijiayun.groupclassui.widget.ToggleSpeakerVolumeView;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.models.roomresponse.LPResCloudRecordModel;
import com.baijiayun.liveuibase.toolbox.zxyb.WritingBoardWrapper;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.utils.ShadowUtil;
import com.baijiayun.liveuibase.utils.ToastUtil;
import com.baijiayun.liveuibase.widgets.common.DotProgressView;
import com.baijiayun.liveuibase.widgets.common.ProgressCircleView;
import com.zxyb.zxybbaselib.ble.entity.BleDevice;

import org.jetbrains.annotations.Nullable;

import java.lang.ref.WeakReference;
import java.util.concurrent.TimeUnit;

import io.reactivex.android.schedulers.AndroidSchedulers;

/**
 * Created by Shubo on 2018/9/21.
 * 1v1左侧菜单栏
 */
public class LeftMenuWindow extends InteractiveBaseWindow implements BottomMenuContract.View {

    private BottomMenuContract.Presenter presenter;
    private AnimatorSet animatorSet;

    private boolean isChatDialogShowing = false, isFloatChatWindowShowing;
    private ImageView ivMic, ivCamera;
    private boolean isFirstCallRecordStatus = true;
    private Context context;
    private boolean eyeCareIsEnable = false;
    private HandsUpRemindPopupWindow handsUpRemindPopupWindow;
    private WritingBoardWrapper writingboardWrapper;

    public LeftMenuWindow(Context context) {
        super(context);
        this.context = context;
        animatorSet = new AnimatorSet();
        animatorSet.addListener(new AnimationSetListener($.id(R.id.ll_bottom_container).view()));

        FrameLayout.LayoutParams lp = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        lp.gravity = Gravity.START | Gravity.TOP;
        this.view.setLayoutParams(lp);

        initView();
        initDrawable();
    }

    private void initDrawable() {
        $.id(R.id.ll_bottom).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeWidth(UtilsKt.getDp(1))
                .strokeColor(ContextCompat.getColor(context, R.color.bjysc_bg_stroke_10))
                .build());
    }

    private void initView() {
        presenter = new BottomMenuPresenter(this, context);
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            showStudentView();
        }
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            showTeacherView();
        }
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
            showAssistantView();
        }
        $.id(R.id.chat_container).visibility(iRouter.getLiveRoom().enableFixChatPanel1v1() ? View.GONE : View.VISIBLE);
        if (iRouter.getLiveRoom().getZXYBVM().enableUseHandWritingBoard()
                && (iRouter.getLiveRoom().getZXYBVM().getConnectType()!= LPConstants.ZXYBConnectType.TYPE_BLE
                        || !iRouter.getLiveRoom().getZXYBVM().getRecentBleDevices().isEmpty())) {
            $.id(R.id.window_bottom_writingboard_container).visible();
        }
        writingboardWrapper = new WritingBoardWrapper((AppCompatActivity) context, iRouter.getLiveRoom(), $.id(R.id.window_bottom_writingboard).view());
        WritingBoardWrapper.Config config = new WritingBoardWrapper.Config();
        config.setBleConnectResId(R.drawable.bjy_group_ic_ble_connected);
        config.setBleDisconnectResId(R.drawable.bjy_group_ic_ble_disconnected);
        config.setBleSleepResId(R.drawable.bjy_group_ic_ble_sleep);
        config.setUsbConnectResId(R.drawable.bjy_group_ic_usb_connected);
        config.setUsbDisconnectResId(R.drawable.bjy_group_ic_usb_disconnected);
        config.setUsbSleepResId(R.drawable.bjy_group_ic_usb_sleep);
        writingboardWrapper.setConfig(config);
        writingboardWrapper.setDeviceScanCallback(new WritingBoardWrapper.DeviceScanCallback() {
            @Override
            public void onStartScan() {
                $.id(R.id.window_bottom_writingboard_progress).visible();
                DotProgressView dotProgressView = $.id(R.id.window_bottom_writingboard_progress).view();
                dotProgressView.invalidate();
            }

            @Override
            public void onScanDevice(@Nullable BleDevice bleDevice) {
                // do nothing
            }
            
            @Override
            public void onStopScan() {
                $.id(R.id.window_bottom_writingboard_progress).gone();
            }
        });
        writingboardWrapper.setConnectStateCallback(connectState -> {
            if (connectState == LPConstants.ZXYBConnectStatus.CONNECT_STATUS_SUCCESS) {
                $.id(R.id.window_bottom_writingboard_container).visible();
            }
            if (connectState == LPConstants.ZXYBConnectStatus.CONNECT_STATUS_CONNECTING) {
                $.id(R.id.window_bottom_writingboard_progress).visible();
                DotProgressView dotProgressView = $.id(R.id.window_bottom_writingboard_progress).view();
                dotProgressView.invalidate();
            } else {
                $.id(R.id.window_bottom_writingboard_progress).gone();
            }
        });
        ivMic = $.id(R.id.iv_mic).view();
        ivCamera = $.id(R.id.iv_camera).view();
        ivMic.setSelected(true);
        ivCamera.setSelected(true);
        if (LPConstants.LPUserType.Assistant == iRouter.getLiveRoom().getCurrentUser().getType()) {
            iRouter.getLiveRoom().getRecorder().detachAVideo();
        } else {
            if (iRouter.getLiveRoom().isClassStarted()) {
                iRouter.getLiveRoom().getRecorder().attachAVideo();
            }
        }

        $.id(R.id.iv_userlist).clicked(v -> presenter.navigateToUserList());
        $.id(R.id.iv_chat).clicked(v -> {
            $.id(R.id.red_point_iv).invisible();
            presenter.navigateToChat();
        });
        $.id(R.id.iv_homework).clicked(v -> presenter.navigateToHomework());
        eyeCareIsEnable = iRouter.getSubjectValueByKey(EventKey.DisplayEyeCare,Boolean.class);

        ((ImageView) $.id(R.id.ct_eye_care).view()).setImageDrawable(ContextCompat.getDrawable(view.getContext(),eyeCareIsEnable ? R.drawable.icon_group_eye_care_checked
                : R.drawable.icon_group_eye_care_normal));
        $.id(R.id.ct_eye_care).clicked(v -> {
            presenter.navigateToEyeCare(!eyeCareIsEnable);
            eyeCareIsEnable = !eyeCareIsEnable;
            ((ImageView) $.id(R.id.ct_eye_care).view()).setImageDrawable(
                    ContextCompat.getDrawable(view.getContext(),eyeCareIsEnable ? R.drawable.icon_group_eye_care_checked
                            : R.drawable.icon_group_eye_care_normal));
        });
        $.id(R.id.container_status_bar_close).clicked(v -> presenter.exit());
        $.id(R.id.iv_speaker).clicked((View v) -> {
            ToggleSpeakerVolumeView volumeView = new ToggleSpeakerVolumeView(getView().getContext());
            volumeView.setListener(this::showSpeakerIcon);
            int iconSize = getView().getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_left_menu_width);
            PopupWindow popupWindow = new PopupWindow(volumeView, UtilsKt.getDp(148), UtilsKt.getDp(20));
            popupWindow.setOutsideTouchable(true);
            popupWindow.setFocusable(true);
            popupWindow.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            popupWindow.showAsDropDown($.id(R.id.iv_speaker).view(), iconSize, -(iconSize + popupWindow.getHeight()) / 2);
        });

        $.id(R.id.iv_courseware).clicked(v -> iRouter.getSubjectByKey(EventKey.CoursewareManageEnable).onNext(true));
        $.id(R.id.activity_toolbar_box_image_1).clicked(v -> iRouter.getSubjectByKey(EventKey.ToolBox).onNext(v));

        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
            //学生有举手按钮，上移80dp
            RelativeLayout.LayoutParams layoutParams = (RelativeLayout.LayoutParams) $.id(R.id.ll_media_container).view().getLayoutParams();
            layoutParams.bottomMargin = UtilsKt.getDp(80);
        }
        if (iRouter.getLiveRoom().getPartnerConfig().disableRecordStatus()) {
            $.id(R.id.iv_record).gone();
        }
    }


    private void subscribe() {
        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.DisPlayBleDialog).ofType(Boolean.class)
                .filter(aBoolean -> aBoolean)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    if (writingboardWrapper != null) {
                        writingboardWrapper.showBleDialogWithPermission();
                    }
                }));
        compositeDisposable.add($.id(R.id.iv_record).clicks().throttleFirst(1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(o -> presenter.changeRecordStatus()));

        compositeDisposable.add($.id(R.id.fl_speak_wrapper).clicks().throttleFirst(500, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(c -> presenter.speakApply()));

        compositeDisposable.add($.id(R.id.iv_mic).clicks().throttleFirst(500, TimeUnit.MILLISECONDS)
                .subscribe(o -> {
                    if (!iRouter.getLiveRoom().getOnlineUserVM().isActiveUser(iRouter.getLiveRoom().getCurrentUser())) {
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(getString(iRouter.getLiveRoom().isClassStarted() ? R.string.bjysc_microphone_not_up_seat : R.string.bjysc_microphone_class_not_not_start));
                        return;
                    }
                    iRouter.getPublishSubjectByKey(EventCode.AttachAudio).onNext(!iRouter.getLiveRoom().getRecorder().isAudioAttached());
                }));

        compositeDisposable.add($.id(R.id.iv_camera).clicks().throttleFirst(500, TimeUnit.MILLISECONDS)
                .subscribe(o -> {
                    if (!iRouter.getLiveRoom().getOnlineUserVM().isActiveUser(iRouter.getLiveRoom().getCurrentUser())) {
                        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(getString(iRouter.getLiveRoom().isClassStarted() ? R.string.bjysc_camera_not_up_seat : R.string.bjysc_camera_class_not_start));
                        return;
                    }
                    iRouter.getPublishSubjectByKey(EventCode.AttachVideo).onNext(!iRouter.getLiveRoom().getRecorder().isVideoAttached());
                }));

    }

    private void showSpeakerIcon(int volumeLevel, int min, int max) {
        int percent = (max - min) / 3 + 1;
        if (volumeLevel == min) {
            $.id(R.id.iv_speaker).image(R.drawable.ic_interactive_speaker_normal_0);
        } else if (volumeLevel <= percent) {
            $.id(R.id.iv_speaker).image(R.drawable.ic_interactive_speaker_normal_1);
        } else if (volumeLevel <= percent * 2) {
            $.id(R.id.iv_speaker).image(R.drawable.ic_interactive_speaker_normal_2);
        } else {
            $.id(R.id.iv_speaker).image(R.drawable.ic_interactive_speaker_normal_3);
        }
    }

    @Override
    protected View onCreateView(Context context) {
        return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_menu_left, null);
    }

    @Override
    public void showStudentView() {
        $.id(R.id.iv_courseware).gone();
        $.id(R.id.activity_toolbar_box_image).gone();
        $.id(R.id.activity_toolbar_box_image_1).gone();
        $.id(R.id.iv_record).gone();
        if (!iRouter.getLiveRoom().getPartnerConfig().enableStudentShowUserList) {
            $.id(R.id.iv_userlist).gone();
            $.id(R.id.fl_user).gone();
        }
        $.id(R.id.fl_speak_wrapper).visible();
        if (iRouter.getLiveRoom().getPartnerConfig().enableUseHomeWork == 1) {
            $.id(R.id.iv_homework).visible();
        }
    }

    @Override
    public void showTeacherView() {
        $.id(R.id.fl_speak_wrapper).gone();
    }

    @Override
    public void showAssistantView() {
        $.id(R.id.fl_speak_wrapper).gone();
        $.id(R.id.activity_toolbar_box_image).gone();
        $.id(R.id.activity_toolbar_box_image_1).gone();
    }

    @Override
    public void showRecordingStatus(LPResCloudRecordModel recordStatus) {
        if (!isFirstCallRecordStatus) {
            String needShow = getNeedShowMessage(recordStatus);
            if (!TextUtils.isEmpty(needShow)) {
                new ToastUtil(context).setText(needShow).show();
            }
        }
        if (isFirstCallRecordStatus && recordStatus.status == LPConstants.CloudRecordStatus.Recording.getStatus()) {
            isFirstCallRecordStatus = false;
        }
        //在CloudRecordControlPanel未初始化时，若云端改变了录制状态，需要暂存，并在初始化时显示
        if (recordStatus.status == LPConstants.CloudRecordStatus.Recording.getStatus()) {
            ((ImageView)$.id(R.id.iv_record).view()).setImageDrawable(
                    ContextCompat.getDrawable(context, R.drawable.ic_interactive_cloud_record_recording));
        } else {
            ((ImageView)$.id(R.id.iv_record).view()).setImageDrawable(
                    ContextCompat.getDrawable(context, R.drawable.ic_interactive_cloud_record_normal));
        }
    }

    @Override
    public void showForbidHandsUpStatus(boolean isForbidden) {
        if (!isAdmin() && !isForbidden) {
            showSpeakApplyNormal();
        } else {
            showSpeakApplyDisable();
        }
    }

    @Override
    public void showUserListWindowShowingStatus(boolean isShowing) {
        Drawable drawable;
        if (isShowing) {
            drawable = ContextCompat.getDrawable(view.getContext(), R.drawable.ic_interactive_user_list_on);
        } else {
            drawable = ContextCompat.getDrawable(view.getContext(), R.drawable.ic_interactive_user_list_off);
        }
        ImageView imageView = $.id(R.id.iv_userlist).view();
        imageView.setImageDrawable(drawable);
    }

    @Override
    public void showChatWindowShowingStatus(boolean isShowing) {
        isChatDialogShowing = isShowing;
        Drawable drawable;
        if (view == null || view.getContext() == null) {
            return;
        }
        if (isShowing) {
            showChatWindowRedPoint(false);
            drawable = ContextCompat.getDrawable(view.getContext(), R.drawable.ic_interactive_chat_on);
        } else {
            drawable = ContextCompat.getDrawable(view.getContext(), R.drawable.bjy_group_ic_chat_off);
        }
        ImageView imageView = $.id(R.id.iv_chat).view();
        imageView.setImageDrawable(drawable);
    }

    @Override
    public void showFloatChatWindowShowingStatus(boolean isShowing) {
        isFloatChatWindowShowing = isShowing;
    }

    @Override
    public void showGalleryModel(boolean isOn) {
        // do nothing
    }

    @Override
    public void showStudyGalleryModel(LPConstants.StudyRoomMode studyRoomMode) {
        // do nothing
    }

    @Override
    public void showChatWindowRedPoint(boolean isVisible) {
        $.id(R.id.red_point_iv).visibility(isVisible && !isChatDialogShowing && !isFloatChatWindowShowing ? View.VISIBLE : View.GONE);
    }

    @Override
    public void showShortTermClassAlreadyHasPlayback() {
        iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(getString(R.string.bjysc_record_short_term));
    }

    @Override
    public void showCloudRecordHint() {
        //延时1s弹框，避免loading页面还未消失就弹框
        getView().postDelayed(() -> {
            if (!presenter.showCloudRecord()) {
                return;
            }
            String hint = getView().getContext().getString(R.string.bjysc_record_on) + "\n"
                    + getView().getContext().getString(R.string.bjysc_record_hint);
            TextView textView = new TextView(getView().getContext());
            textView.setText(hint);
            textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(view.getContext(), R.attr.base_theme_bottom_menu_text_color));
            textView.setLineSpacing(UtilsKt.getDp(10), 1.0f);
            ShadowUtil.ShadowGroup shadowGroup = ShadowUtil.initShadowView(context, textView);
            if (shadowGroup == null) {
                return;
            }
            ShadowUtil.setViewBoundShadow(shadowGroup.getShadowContainer());

            textView.setPadding(
                    UtilsKt.getDp(10),
                    UtilsKt.getDp(10),
                    UtilsKt.getDp(10),
                    UtilsKt.getDp(10));
            textView.setGravity(Gravity.CENTER_VERTICAL);
            textView.setBackground(new DrawableBuilder()
                    .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                    .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                    .build());
            PopupWindow window = new PopupWindow(shadowGroup.getContentView(),
                    UtilsKt.getDp(376),
                    UtilsKt.getDp(116));
            window.setOutsideTouchable(true);
            window.setFocusable(true);
            window.setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            int iconSize = getView().getContext().getResources().getDimensionPixelSize(R.dimen.bjysc_left_menu_width);
            window.showAsDropDown($.id(R.id.iv_record).view(), iconSize,
                    -(iconSize + window.getHeight()) / 2);
        }, 1000);
    }

    @Override
    public void showCloudRecordRestartDialog() {
        new CommonDialog(getView().getContext(), CommonDialog.ChoiceMode.Double_Blue)
                .setMainDisplayText(getString(R.string.bjysc_record_restart_hint))
                .setPositive(getString(R.string.bjysc_record_continue), ((dialog, which) -> presenter.continueCloudRecord()))
                .setNegative(getString(R.string.bjysc_record_new), ((dialog, which) -> presenter.startNewCloudRecord())).show();
    }

    @Override
    public void showMicStatus(boolean isMicOn) {
        if (ivMic != null) {
            ivMic.setSelected(!isMicOn);
        }
    }

    @Override
    public void showCameraStatus(boolean isCameraOn) {
        if (ivCamera != null) {
            ivCamera.setSelected(!isCameraOn);
        }
    }

    @Override
    public void showUserListStatus(boolean isShow) {
        $.id(R.id.window_bottom_menu_user_list).visibility(isShow ? View.VISIBLE : View.GONE);
    }

    @Override
    public void showSpeakApplyCountDown(int countDownTime, int total) {
        $.id(R.id.view_hand_countdown).visible();
        ((ProgressCircleView) $.id(R.id.view_hand_countdown).view()).setRatio(countDownTime / (float) total);
        $.id(R.id.iv_speak_apply).view().setEnabled(true);
        $.id(R.id.iv_speak_apply).view().setSelected(true);
    }

    @Override
    public void showSpeakApplyNormal() {
        $.id(R.id.iv_speak_apply).view().setEnabled(true);
        $.id(R.id.iv_speak_apply).view().setSelected(false);
        $.id(R.id.view_hand_countdown).invisible();
    }

    @Override
    public void showSpeakApplyDisable() {
        $.id(R.id.iv_speak_apply).view().setEnabled(false);
        $.id(R.id.iv_speak_apply).view().setSelected(false);
        $.id(R.id.view_hand_countdown).invisible();
    }

    @Override
    public void showHandsUpRemind(int count, String name) {
        View users = $.id(R.id.iv_userlist).view();
        if (DisplayUtils.isPad(getView().getContext())) {
            $.id(R.id.iv_user_tip).text(String.valueOf(count));
        }
        $.id(R.id.iv_user_tip).visible();
        if (handsUpRemindPopupWindow == null) {
            handsUpRemindPopupWindow = new HandsUpRemindPopupWindow.Builder(getView().getContext())
                    .build()
                    .setHandsUpCount(count, name)
                    .getPopupWindow();
        } else {
            handsUpRemindPopupWindow.setRemindText(count, name);
        }

        int offsetX, offsetY;
        handsUpRemindPopupWindow.getContentView().measure(View.MeasureSpec.UNSPECIFIED, View.MeasureSpec.UNSPECIFIED);
        int windowY = handsUpRemindPopupWindow.getContentView().getMeasuredHeight() / 2;
        int userY = -users.getMeasuredHeight() / 2;
        offsetX = users.getMeasuredWidth() - UtilsKt.getDp(4);
        //让 popup window 居中
        offsetY = userY - windowY;
        handsUpRemindPopupWindow.getContentView().setOnClickListener(v -> {
            iRouter.getSubjectByKey(EventKey.DisplayUserList).onNext(UserStatus.HandsupList);
            $.id(R.id.iv_user_tip).invisible();
            if (handsUpRemindPopupWindow != null && handsUpRemindPopupWindow.isShowing()) {
                handsUpRemindPopupWindow.dismiss();
            }
        });
        handsUpRemindPopupWindow.showAsDropDown(users, offsetX, offsetY);
    }

    @Override
    public void hideHandsUpCount() {
        $.id(R.id.iv_user_tip).invisible();
        if (handsUpRemindPopupWindow != null && handsUpRemindPopupWindow.isShowing()) {
            handsUpRemindPopupWindow.dismiss();
        }
    }

    @Override
    public void showCloudRecordButton(boolean cloudRecord) {
        $.id(R.id.iv_record).view().setVisibility(cloudRecord?View.VISIBLE:View.GONE);
    }

    @Override
    public void showMuteAllStateChange(boolean mute) {
        if (mute) {
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.bjysc_mute_all_tip));
        } else {
            iRouter.getSubjectByKey(EventKey.ReminderMessage).onNext(context.getString(R.string.bjysc_unmute_all_tip));
        }
    }

    @Override
    public void showBonusPointsAdd(Integer integer) {
        // do nothing
    }

    @Override
    public void setPresenter(BottomMenuContract.Presenter presenter) {
        // do nothing
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if (isWindowInit && isActive) {
            setBasePresenter(presenter);
            subscribe();
        }
        return true;
    }


    @Override
    public void onDestroy() {
        if (animatorSet != null) {
            animatorSet.removeAllListeners();
            animatorSet.cancel();
            animatorSet = null;
        }
        if (writingboardWrapper != null) {
            writingboardWrapper.onDestroy();
        }
        super.onDestroy();
    }

    @Override
    protected boolean onTouchEvent(View v, MotionEvent event) {
        return false;
    }

    private static class AnimationSetListener implements Animator.AnimatorListener {

        private WeakReference<View> weakReferenceView;
        private boolean needToChangeVisibility = true;

        AnimationSetListener(View view) {
            this.weakReferenceView = new WeakReference<>(view);
        }

        @Override
        public void onAnimationStart(Animator animation) {
            if (weakReferenceView.get() == null) {
                return;
            }
            View view = weakReferenceView.get();
            if (view.getVisibility() == View.GONE) {
                view.setVisibility(View.VISIBLE);
                needToChangeVisibility = false;
            }
        }

        @Override
        public void onAnimationEnd(Animator animation) {
            if (weakReferenceView.get() == null) {
                return;
            }
            View view = weakReferenceView.get();
            if (view.getVisibility() == View.VISIBLE && needToChangeVisibility) {
                view.setVisibility(View.GONE);
            }
            needToChangeVisibility = true;
        }

        @Override
        public void onAnimationCancel(Animator animation) {
            needToChangeVisibility = true;
        }

        @Override
        public void onAnimationRepeat(Animator animation) {
            // do nothing
        }
    }

    private String getNeedShowMessage(LPResCloudRecordModel nowStatus) {
        return !isAdmin() ? "" : (
                (
                        nowStatus.operator == LPConstants.LPUserType.Teacher && !BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) ? "老师" :
                                nowStatus.operator == LPConstants.LPUserType.Assistant && BaseUtilsKt.isTeacher(iRouter.getLiveRoom().getCurrentUser()) ? "助教" : ""
                )
                        + "已" + (nowStatus.status == LPConstants.CloudRecordStatus.Recording.getStatus() ? "开启" : "停止") + "云端录制"
        );
    }
}
