package com.baijiayun.groupclassui.window.group;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.text.InputFilter;
import android.util.SparseArray;
import android.widget.EditText;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.viewpager.widget.ViewPager;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.widgets.SimpleTextWatcher;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.base.BaseRecyclerAdapter;
import com.baijiayun.groupclassui.base.BaseViewHolder;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.widget.CircleIndicator;
import com.baijiayun.livebase.models.imodels.IUserModel;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import io.reactivex.disposables.Disposable;

/**
 * 已分组adapter
 * Created by Dujuncan on 2018/09/30.
 */
public class GroupedUserAdapter extends BaseRecyclerAdapter<GroupedBean> {

    //随机颜色值
    private String[] colors = {"#F44336", "#E91E63", "#D500F9", "#3D5AFE", "#775FCF"
            , "#03A9F4", "#00BCD4", "#51B155", "#8BC34A", "#50E3C2"
            , "#FFEB3B", "#FFC107", "#FF9800", "#795548", "#A47564"};
    private SparseArray<String> groupColorSparseArray;//缓存分组颜色，防止复用
    private SparseArray<String> groupNameSparseArray;//缓存组名，防止复用
    private OnEventListener onEventListener;
    private IRouter iRouter;
    private List<IUserModel> ungroupSelectedUsers;//未分组列表选中用户
    private List<IUserModel> groupedSelectedUsers;//分组内列表选中用户
    private List<GroupManagePagerAdapter> pagerAdapters;
    private Disposable disposable;
    private int maxLen = 24;//输入字符最大长度
    //长度限制
    private InputFilter lengthFilter = (source, start, end, dest, dstart, dend) -> {
        int dindex = 0;
        int count = 0;
        while (count <= maxLen && dindex < dest.length()) {
            char c = dest.charAt(dindex++);
            if (c < 128) {
                count = count + 1;
            } else {
                count = count + 2;
            }
        }
        if (count > maxLen) {
            return dest.subSequence(0, dindex - 1);
        }
        int sindex = 0;
        while (count <= maxLen && sindex < source.length()) {
            char c = source.charAt(sindex++);
            if (c < 128) {
                count = count + 1;
            } else {
                count = count + 2;
            }
        }
        if (count > maxLen) {
            sindex--;
        }
        return source.subSequence(0, sindex);
    };
    //特殊字符限制
    private InputFilter charFilter = (source, start, end, dest, dstart, dend) -> {
        String specialChar = "[`~!@#$%^&*()+=|{}':;',\\\\[\\\\].<>/?~！@#￥%……&*（）——+|{}【】‘；：”“’。，、？]";
        Pattern pattern = Pattern.compile(specialChar);
        Matcher matcher = pattern.matcher(source);
        if (matcher.find()) {
            return "";
        } else {
            return source;
        }
    };

    GroupedUserAdapter(Context context, IRouter iRouter) {
        super(context, R.layout.item_group_manage_window_grouped);
        this.iRouter = iRouter;
        pagerAdapters = new ArrayList<>();
        groupColorSparseArray = new SparseArray<>();
        groupNameSparseArray = new SparseArray<>();
        disposable = iRouter.getSubjectByKey(EventKey.UngroupSelectedList)
                .ofType(List.class)
                .subscribe(list -> {
                    ungroupSelectedUsers = list;
                });
    }

    @Override
    protected void convert(BaseViewHolder holder, GroupedBean groupedBean) {
        setBgBorderColor(holder);
        setInputFilter(holder);
        GroupManagePagerAdapter groupManagePagerAdapter = createViewPagerAndAdapter(holder, groupedBean);
        listenButton(holder, groupManagePagerAdapter);
        pagerAdapters.add(groupManagePagerAdapter);
    }

    @Override
    public void removeData(int position) {
        super.removeData(position);
        pagerAdapters.remove(position);
    }

    private void setBgBorderColor(BaseViewHolder holder) {
        ConstraintLayout clBorder = holder.getView(R.id.group_manage_window_item_cl_border);
        GradientDrawable drawable = new GradientDrawable();
        drawable.setCornerRadius(UtilsKt.getDp(4));
        drawable.setColor(Color.TRANSPARENT);
        int position = holder.getAdapterPosition();
        if (groupColorSparseArray.get(position) == null) {
            groupColorSparseArray.put(position, generateRandomColor());
        }
        drawable.setStroke(UtilsKt.getDp(2), Color.parseColor(groupColorSparseArray.get(position)));
        clBorder.setBackgroundDrawable(drawable);
    }

    private void setInputFilter(BaseViewHolder holder) {
        int position = holder.getAdapterPosition();

        EditText editText = holder.getView(R.id.group_manage_window_item_et);
        editText.setFilters(new InputFilter[]{charFilter, lengthFilter});
        editText.addTextChangedListener(new SimpleTextWatcher() {
            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
                groupNameSparseArray.put(position, s.toString());
            }
        });
        if (groupNameSparseArray.get(position) != null) {
            editText.setText(groupNameSparseArray.get(position));
        }
    }

    @NonNull
    private GroupManagePagerAdapter createViewPagerAndAdapter(BaseViewHolder holder, GroupedBean groupedBean) {
        ViewPager viewPager = holder.getView(R.id.group_manage_window_item_vp);
        GroupManagePagerAdapter groupManagePagerAdapter = new GroupManagePagerAdapter(mContext, groupedBean.getGroupUsers());
        groupManagePagerAdapter.setOnSelectedListener(selectUsers -> {
            //更新组内选中用户集合，更新按钮状态
            groupedSelectedUsers = selectUsers;
            holder.setText(R.id.group_manage_window_item_btn, selectUsers.size() > 0 ? R.string.bjysc_group_manage_disjoint : R.string.bjysc_group_manage_join_group);
        });
        groupManagePagerAdapter.bindToViewPager(viewPager);
        viewPager.setOffscreenPageLimit(2);
        CircleIndicator indicator = holder.getView(R.id.group_manage_window_item_indicator);
        indicator.setViewPager(viewPager);
        return groupManagePagerAdapter;
    }

    private void listenButton(BaseViewHolder holder, GroupManagePagerAdapter groupManagePagerAdapter) {
        holder.getView(R.id.group_manage_window_item_iv_close)
                .setOnClickListener(v -> {
                    //关闭分组
                    groupColorSparseArray.removeAt(holder.getAdapterPosition());
                    if (onEventListener != null) {
                        onEventListener.closeGroup(holder.getAdapterPosition(), groupManagePagerAdapter.getGroupUsers());
                    }
                });
        holder.click(R.id.group_manage_window_item_btn, v -> {
            if (holder.getText(R.id.group_manage_window_item_btn).equals(mContext.getString(R.string.bjysc_group_manage_join_group))) {
                //加入分组
                if (ungroupSelectedUsers != null && ungroupSelectedUsers.size() > 0) {
                    groupManagePagerAdapter.addData(ungroupSelectedUsers);
                    iRouter.getSubjectByKey(EventKey.JoiningGroupList)
                            .onNext(ungroupSelectedUsers);
                }
            } else if (groupedSelectedUsers != null) {
                //移出分组
                if (onEventListener != null) {
                    onEventListener.onUserExitGroup(groupedSelectedUsers);
                }
                groupManagePagerAdapter.removeData(groupedSelectedUsers);
                holder.setText(R.id.group_manage_window_item_btn, R.string.bjysc_group_manage_join_group);
            }
        });
    }

    @Override
    protected boolean isEnableListener() {
        return false;
    }

    public void updateViewPager(int position, IUserModel groupUser) {
        pagerAdapters.get(position).addData(groupUser);
    }

    private String generateRandomColor() {
        if (getItemCount() == 15) {
            return colors[0];
        }
        Random random = new Random();
        int index = random.nextInt(colors.length);
        if (groupColorSparseArray.indexOfValue(colors[index]) == -1) {
            return colors[index];
        } else {
            //生成的颜色重复，递归
            return generateRandomColor();
        }
    }

    //获取指定位置的背景值
    public String getBgColor(int position) {
        return groupColorSparseArray.get(position);
    }

    //获取指定位置的组名
    public String getGroupName(int position) {
        return groupNameSparseArray.get(position) != null ? groupNameSparseArray.get(position) : "";
    }

    interface OnEventListener {
        void closeGroup(int position, List<IUserModel> groupUsers);

        void onUserExitGroup(List<IUserModel> list);
    }

    public void setOnEventListener(OnEventListener onEventListener) {
        this.onEventListener = onEventListener;
    }

    public void destroy() {
        if (disposable != null && !disposable.isDisposed()) {
            disposable.dispose();
        }
        iRouter = null;
        if (groupColorSparseArray != null) {
            groupColorSparseArray.clear();
        }
        if (ungroupSelectedUsers != null) {
            ungroupSelectedUsers.clear();
        }
        if (groupedSelectedUsers != null) {
            groupedSelectedUsers.clear();
        }
        if (pagerAdapters != null) {
            pagerAdapters.clear();
        }
    }
}
