package com.baijiayun.groupclassui.window.ppt;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.util.AttributeSet;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.layer.BaseLayerOfProfessionalClass;
import com.baijiayun.liveuibase.toolbox.browser.Tuple;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * Created by pengwenjian on 2021/4/23
 * Describe:
 */
public class PPTTabLayer extends BaseLayerOfProfessionalClass {
    private Drawable tabSelectedDrawable;
    private final Context context;
    private static final int TAB_WIDTH = 200;
    protected Drawable fullDrawable, fullResetDrawable, maxResetDrawable;
    private ImageView ivMaxFull, ivMaxSync;

    public PPTTabLayer(@NotNull Context context) {
        this(context, null);
    }

    public PPTTabLayer(@NotNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public PPTTabLayer(@NotNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        this.context = context;
    }

    public PPTTabLayer(@NotNull Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
        this.context = context;
    }

    private void initDrawables() {
        fullDrawable = ContextCompat.getDrawable(context, R.drawable.ic_title_full_to_full_layer);
        fullResetDrawable = ContextCompat.getDrawable(context, R.drawable.ic_title_full_reset_to_sync_layer);
        maxResetDrawable = ContextCompat.getDrawable(context, R.drawable.ic_title_max_reset_to_sync_layer);
    }

    /**
     *
     * @return
     */
    @Override
    protected boolean isDestroyRouter() {
        return false;
    }

    public void refreshTabs() {
        if (!BaseUtilsKt.isAdmin(router.getLiveRoom())) {
            return;
        }
        View rootView = View.inflate(context, R.layout.bjy_group_layout_ppt_tab, null);
        ivMaxFull = rootView.findViewById(R.id.window_titled_max_in_full_layer);
        ivMaxSync = rootView.findViewById(R.id.window_titled_max_in_sync_layer);
        LinearLayout titleContainer = rootView.findViewById(R.id.window_titled_title_container);
        removeAllViews();
        titleContainer.removeAllViews();
        ConcurrentLinkedQueue<PPTWindow> cacheWindows = router.getQueueByKey(EventKey.PPTWindows, PPTWindow.class);
        if (cacheWindows == null || cacheWindows.isEmpty()) {
            setVisibility(GONE);
            return;
        }
        final PPTWindow fullPPTWindow = router.getValueByKey(EventKey.FullScreenPPTWindow, PPTWindow.class);
        final PPTWindow maxPPTWindow = router.getValueByKey(EventKey.MaxScreenPPTWindow, PPTWindow.class);
        if (fullPPTWindow == null && maxPPTWindow == null) {
            for (PPTWindow pptWindow : cacheWindows) {
                pptWindow.showTitle();
            }
            setVisibility(GONE);
            return;
        }

        List<PPTWindow> pptWindows = new ArrayList<>(cacheWindows);
        if (pptWindows.size() == 1) {
            final PPTWindow pptWindow = pptWindows.get(0);
            if (pptWindow != null) {
                pptWindow.showTitle();
            }
            setVisibility(GONE);
            return;
        }
        for (PPTWindow pptWindow : pptWindows) {
            pptWindow.hideTitle();
        }
        setVisibility(View.VISIBLE);
        if (tabSelectedDrawable == null) {
            tabSelectedDrawable = new DrawableBuilder()
                    .solidColor(0x66313847)
                    .cornerRadii(UtilsKt.getDp(6), UtilsKt.getDp(6), 0, 0)
                    .build();
        }
        int count = pptWindows.size();
        final int fullScreenWidth = router.getValueByKey(EventKey.FullScreenWidth, Integer.class, 0);
        final int measuredWidth = fullScreenWidth - UtilsKt.getDp(100);
        int destWidth = UtilsKt.getDp(TAB_WIDTH);
        if (measuredWidth != 0 && destWidth * count > measuredWidth) {
            destWidth = measuredWidth / count;
        }
        int index = 0;
        View lastDiv = null;
        final PPTWindow selectedPPTWindow = fullPPTWindow != null ? fullPPTWindow : maxPPTWindow;
        refreshBtn(fullPPTWindow != null);
        for (PPTWindow pptWindow : pptWindows) {
            View tab = View.inflate(context, R.layout.bjy_group_item_ppt_title, null);
            if (index == count - 1) {
                tab.findViewById(R.id.view_div).setVisibility(View.INVISIBLE);
            }
            if (pptWindow == selectedPPTWindow && (selectedPPTWindow.isMaxInSync() || selectedPPTWindow.isMaxInFull())) {
                tab.findViewById(R.id.view_div).setVisibility(View.INVISIBLE);
                if (lastDiv != null) {
                    lastDiv.setVisibility(View.INVISIBLE);
                }
                tab.setBackground(tabSelectedDrawable);
            } else {
                tab.setBackground(null);
            }
            ((TextView) tab.findViewById(R.id.tv_title)).setText(pptWindow.getDocName());
            tab.findViewById(R.id.iv_close).setOnClickListener(v -> {
                if (pptWindow.isMaxInSync() || pptWindow.isMaxInFull()) {
                    if (pptWindows.isEmpty() || pptWindows.size() <= 1) {
                        return;
                    }
                    List<PPTWindow> tempWindows = new ArrayList<>(pptWindows);
                    if (tempWindows.remove(pptWindow)) {
                        final PPTWindow destPPTWindow = tempWindows.get(tempWindows.size() - 1);
                        if (destPPTWindow == pptWindow) {
                            return;
                        }
                        pptWindow.switchTab(destPPTWindow);
                    }
                }
                router.getSubjectByKey(EventKey.ClosePPTWindow).onNext(new Tuple(pptWindow.getId()));
            });
            tab.setOnClickListener(v -> selectedPPTWindow.switchTab(pptWindow));
            titleContainer.addView(tab, destWidth, LinearLayout.LayoutParams.MATCH_PARENT);
            lastDiv = tab.findViewById(R.id.view_div);
            index++;
        }
        FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, UtilsKt.getDp(30));
        layoutParams.gravity = Gravity.TOP;
        addView(rootView, layoutParams);
        ivMaxFull.setOnClickListener(v -> {
            selectedPPTWindow.setReceiveOrder(false);
            if (selectedPPTWindow.isMaxInFull()) {
                selectedPPTWindow.resetInFull();
            } else {
                selectedPPTWindow.maxInFull();
            }
        });
        ivMaxSync.setOnClickListener(v -> {
            selectedPPTWindow.setReceiveOrder(false);
            selectedPPTWindow.resetInSync();
        });
    }

    public void refreshBtn(boolean isFull) {
        if (!BaseUtilsKt.isAdmin(router.getLiveRoom())) {
            return;
        }
        if (fullDrawable == null) {
            initDrawables();
        }
        if (isFull) {
            if (ivMaxSync != null) {
                ivMaxSync.setVisibility(INVISIBLE);
                ivMaxFull.setImageDrawable(fullResetDrawable);
            }
        } else {
            if (ivMaxSync != null) {
                ivMaxSync.setVisibility(VISIBLE);
                ivMaxSync.setImageDrawable(maxResetDrawable);
                ivMaxFull.setImageDrawable(fullDrawable);
            }
        }
    }

    public void show(boolean isShow) {
        ConcurrentLinkedQueue<PPTWindow> cacheWindows = router.getQueueByKey(EventKey.PPTWindows, PPTWindow.class);
        if (cacheWindows == null || cacheWindows.isEmpty()) {
            setVisibility(GONE);
            return;
        }
        final PPTWindow fullPPTWindow = router.getValueByKey(EventKey.FullScreenPPTWindow, PPTWindow.class);
        final PPTWindow maxPPTWindow = router.getValueByKey(EventKey.MaxScreenPPTWindow, PPTWindow.class);
        if (fullPPTWindow == null && maxPPTWindow == null) {
            setVisibility(GONE);
            return;
        }
        setVisibility(isShow ? VISIBLE : GONE);
    }
}
