package com.baijiayun.groupclassui.window.status

import android.annotation.SuppressLint
import android.content.Context
import android.os.Build
import android.view.View
import androidx.appcompat.widget.AppCompatImageView
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.livebase.context.LPConstants.MediaNetworkQuality
import com.baijiayun.liveuibase.utils.NetworkUtils
import com.baijiayun.liveuibase.widgets.popupwindow.BaseAutoArrangePopupWindow
import kotlinx.android.synthetic.main.bjy_group_network_window.view.*
import java.util.*

/**
 * Created by pengwenjian on 2021/3/29
 * Describe:
 */
class NetworkPopuwindow(context: Context?, router: IRouter) : BaseAutoArrangePopupWindow(context),
    StatusBarContract.View {
    private val container by lazy {
        View.inflate(context, R.layout.bjy_group_network_window, null)
    }
    private val presenter by lazy {
        StatusBarPresenter(this, context)
    }

    init {
        initView(router)
    }

    @SuppressLint("SetTextI18n")
    private fun initView(router: IRouter) {
        presenter.setRouter(router)
        presenter.subscribe()
        setDirectionMode(HORIZONTAL)
        container?.background = DrawableBuilder()
            .cornerRadius(context.resources.getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
            .strokeColor(ContextCompat.getColor(context, R.color.base_bg_stroke_10))
            .strokeWidth(1)
            .solidColor(
                ThemeDataUtil.getColorFromThemeConfigByAttrId(
                    context,
                    R.attr.base_theme_brand_container_color
                )
            )
            .build()
        container.window_network_device_type_value.text = "${Build.BRAND} ${Build.MODEL}"
        container.window_network_device_ip_value.text = NetworkUtils.getIPAddress(context)
        createView(container, false)
    }

    fun onDestroy() {
        presenter.destroy()
    }

    override fun initViewDataHashMap() {
        // do nothing
    }

    override fun show(anchor: View) {
        showWithViewOfDirection(anchor, RIGHT_BOTTOM)
    }

    @SuppressLint("SetTextI18n")
    override fun showDownLinkLossRate(
        downLossRate: Double,
        downLinkNetStatus: MediaNetworkQuality
    ) {
        if (downLossRate <= -1.0) {
            container.window_network_loss_rate_down_link_value.text = "- -"
            container.window_network_loss_rate_down_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        } else {
            container.window_network_loss_rate_down_link_value.text =
                "${String.format(Locale.CHINA, "%.2f", downLossRate)}%"
            updateNetworkStatusIcon(container.window_network_loss_rate_down_link_icon,downLinkNetStatus);
        }
    }

    override fun showResetStatusBarText() {
        // do nothing
    }

    @SuppressLint("SetTextI18n")
    override fun showUpLinkLossRate(upLossRate: Double, upLinkNetStatus: MediaNetworkQuality) {
        if (upLossRate <= -1.0) {
            container.window_network_loss_rate_up_link_value.text = "- -"
            container.window_network_loss_rate_up_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        } else {
            container.window_network_loss_rate_up_link_value.text =
                "${String.format(Locale.CHINA, "%.2f", upLossRate)}%"
            updateNetworkStatusIcon(container.window_network_loss_rate_up_link_icon,upLinkNetStatus)
        }
    }

    override fun showUpVideoBitrate(
        videoBitrate: Double,
        videoNetWorkQuality: MediaNetworkQuality
    ) {
        if (videoBitrate <= -1.0) {
            container.window_network_video_up_link_value.text = "- -"
            container.window_network_loss_rate_up_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        }else {
            container.window_network_video_up_link_value.text = "${String.format(Locale.CHINA, "%.2f", videoBitrate)}kbps"
            updateNetworkStatusIcon(container.window_network_video_up_link_icon,videoNetWorkQuality)
        }
    }

    override fun showUpAudioBitrate(
        audioBitrate: Double,
        audioNetWorkQuality: MediaNetworkQuality
    ) {
        if (audioBitrate <= -1.0) {
            container.window_network_audio_up_link_value.text = "- -"
            container.window_network_audio_up_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        }else {
            container.window_network_audio_up_link_value.text = "${String.format(Locale.CHINA, "%.2f", audioBitrate)}kbps"
            updateNetworkStatusIcon(container.window_network_audio_up_link_icon,audioNetWorkQuality)
        }
    }

    override fun showDownVideoBitrate(
        videoBitrate: Double,
        videoNetWorkQuality: MediaNetworkQuality
    ) {
        if (videoBitrate <= -1.0) {
            container.window_network_video_down_link_value.text = "- -"
            container.window_network_video_down_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        }else {
            container.window_network_video_down_link_value.text = "${String.format(Locale.CHINA, "%.2f", videoBitrate)}kbps"
            updateNetworkStatusIcon(container.window_network_video_down_link_icon,videoNetWorkQuality)
        }
    }

    override fun showDownAudioBitrate(
        audioBitrate: Double,
        audioNetWorkQuality: MediaNetworkQuality
    ) {
        if (audioBitrate <= -1.0) {
            container.window_network_audio_down_link_value.text = "- -"
            container.window_network_audio_down_link_icon.setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown))
        }else {
            container.window_network_audio_down_link_value.text = "${String.format(Locale.CHINA, "%.2f", audioBitrate)}kbps"
            updateNetworkStatusIcon(container.window_network_audio_down_link_icon,audioNetWorkQuality)
        }
    }

    override fun updateWorstNetworkQuality(
        quality1: MediaNetworkQuality?,
        quality2: MediaNetworkQuality?
    ) {
        // do nothing
    }

    private fun updateNetworkStatusIcon(
        netWorkIcon: AppCompatImageView,
        netStatus: MediaNetworkQuality
    ) {
        when(netStatus){
            MediaNetworkQuality.EXCELLENT->{
                netWorkIcon.setColorFilter(ContextCompat.getColor(context,R.color.bjysc_net_unknown))
            }
            MediaNetworkQuality.GOOD->{
                netWorkIcon.setColorFilter(ContextCompat.getColor(context,R.color.bjysc_net_unknown))
            }
            MediaNetworkQuality.BAD->{
                netWorkIcon.setColorFilter(ContextCompat.getColor(context,R.color.bjysc_net_unknown))
            }
            MediaNetworkQuality.TERRIBLE->{
                netWorkIcon.setColorFilter(ContextCompat.getColor(context,R.color.bjysc_net_terrible))
            }
        }
    }

    override fun showClassName(className: String) {
        // do nothing
    }

    override fun showClassTime(classTime: String) {
        // do nothing
    }

    override fun setPresenter(presenter: StatusBarContract.Presenter) {
        // do nothing
    }
}