package com.baijiayun.groupclassui.window.status;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;

/**
 * Created by Shubo on 2018/9/20.
 */
public class StatusBarWindow extends InteractiveBaseWindow implements StatusBarContract.View {

    private StatusBarContract.Presenter presenter;
    private NetworkPopuwindow networkPopuwindow;

    public StatusBarWindow(Context context) {
        super(context);
        presenter = new StatusBarPresenter(this, context);
        $.id(R.id.container_status_bar_setting).clicked(v -> presenter.setting());
        $.id(R.id.container_status_bar_close).clicked(v -> presenter.exit());
        $.id(R.id.container_status_bar_help).clicked(v -> presenter.help());
        $.id(R.id.container_status_bar_class_signal).clicked(v -> showNetworkStatePanel());
        $.id(R.id.container_status_bar_network_status).clicked(v -> showNetworkStatePanel());
        $.id(R.id.container_status_bar_report).clicked(v -> {
            presenter.report();
        });
        showResetStatusBarText();
        presenter.setRouter(iRouter);
        if (iRouter.getLiveRoom().isAudition()) {
            $.id(R.id.container_status_bar_setting).gone();
            $.id(R.id.container_status_bar_report).gone();
            $.id(R.id.container_status_bar_help).gone();
        }
    }

    private void showNetworkStatePanel() {
        if (networkPopuwindow == null) {
            networkPopuwindow = new NetworkPopuwindow(context, iRouter);
        }
        networkPopuwindow.show($.id(R.id.container_status_bar_class_signal).view());
    }

    @Override
    protected View onCreateView(Context context) {
        return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_status_bar, null);
    }

    @Override
    public void showDownLinkLossRate(double downLossRate, LPConstants.MediaNetworkQuality downLinkNetStatus) {
        // do nothing
    }

    @Override
    public void showUpLinkLossRate(double upLossRate, LPConstants.MediaNetworkQuality upLinkNetStatus) {
        // do nothing
    }

    @Override
    public void showUpVideoBitrate(double videoBitrate, LPConstants.MediaNetworkQuality videoNetWorkQuality) {
        // do nothing
    }

    @Override
    public void showUpAudioBitrate(double audioBitrate, LPConstants.MediaNetworkQuality audioNetWorkQuality) {
        // do nothing
    }

    @Override
    public void showDownVideoBitrate(double videoBitrate, LPConstants.MediaNetworkQuality videoNetWorkQuality) {
        // do nothing
    }

    @Override
    public void showDownAudioBitrate(double audioBitrate, LPConstants.MediaNetworkQuality audioNetWorkQuality) {
        // do nothing
    }

    @Override
    public void updateWorstNetworkQuality(LPConstants.MediaNetworkQuality quality1, LPConstants.MediaNetworkQuality quality2) {
        if(quality1==null&&quality2==null){
            showResetStatusBarText();
            return;
        }
        if(quality1==null){
            updateNetworkStatusLabel(quality2);
            return;
        }
        if(quality2==null){
            updateNetworkStatusLabel(quality1);
            return;
        }
        if(quality1.getQuality()>quality2.getQuality()){
            updateNetworkStatusLabel(quality1);
        }else{
            updateNetworkStatusLabel(quality2);
        }
    }


    @Override
    public void showClassName(String className) {
        $.id(R.id.container_status_bar_class_name).text(className);
    }

    @Override
    public void showClassTime(String classTime) {
        TextView tvClassTime = $.id(R.id.container_status_bar_class_time).view();
        tvClassTime.setText(classTime);
        tvClassTime.setTextColor(classTime.startsWith(context.getString(R.string.bjysc_status_bar_distance_from_class))
                || classTime.startsWith(context.getString(R.string.bjysc_status_bar_overtime))
                ? ContextCompat.getColor(context, R.color.bjysc_warning_color)
                : ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));
    }

    @Override
    public void setPresenter(StatusBarContract.Presenter presenter) {
        // do nothing
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if(isWindowInit){
            if(isActive){
                if (presenter != null) {
                    presenter.subscribe();
                }
            } else{
                if(presenter != null){
                    presenter.unSubscribe();
                    showResetStatusBarText();
                }
            }
        }
        return true;
    }

    @Override
    public void showResetStatusBarText() {
        ((TextView) $.id(R.id.container_status_bar_network_status).text("--").view()).setTextColor(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown));
        showClassTime(context.getString(R.string.bjysc_status_bar_class_time));
        ((ImageView)$.id(R.id.container_status_bar_class_signal).view()).setColorFilter(
                ContextCompat.getColor(context,R.color.bjysc_net_unknown));
    }

    private void updateNetworkStatusLabel(LPConstants.MediaNetworkQuality quality) {
        switch (quality) {
            case EXCELLENT:
                $.id(R.id.container_status_bar_network_status)
                        .textColor(R.color.bjysc_net_good)
                        .text(R.string.bjysc_net_good);
                ((ImageView)$.id(R.id.container_status_bar_class_signal).view()).setColorFilter(
                        ContextCompat.getColor(context,R.color.bjysc_net_good));
                break;
            case GOOD:
                $.id(R.id.container_status_bar_network_status)
                        .textColor(R.color.bjysc_net_normal)
                        .text(R.string.bjysc_net_normal);
                ((ImageView)$.id(R.id.container_status_bar_class_signal).view()).setColorFilter(
                        ContextCompat.getColor(context,R.color.bjysc_net_normal));
                break;
            case BAD:
                $.id(R.id.container_status_bar_network_status)
                        .textColor(R.color.bjysc_net_bad)
                        .text(R.string.bjysc_net_bad);
                ((ImageView)$.id(R.id.container_status_bar_class_signal).view()).setColorFilter(
                        ContextCompat.getColor(context,R.color.bjysc_net_bad));
                break;
            case TERRIBLE:
                $.id(R.id.container_status_bar_network_status)
                        .textColor(R.color.bjysc_net_terrible)
                        .text(R.string.bjysc_net_terrible);
                ((ImageView)$.id(R.id.container_status_bar_class_signal).view()).setColorFilter(
                        ContextCompat.getColor(context,R.color.bjysc_net_terrible));
                break;
            default:
                break;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        presenter.destroy();
    }
}
