package com.baijiayun.groupclassui.window.study

import android.content.Context
import android.view.Gravity
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.screen.DisplayUtils.setRoundCorner
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.liveuibase.widgets.popupwindow.BaseAutoArrangePopupWindow

/**
 * Created by pengwenjian on 2021/03/10
 * Describe:
 */
class QuestionNewWindow(context: Context, val router: IRouter) : BaseAutoArrangePopupWindow(context) {
    var itemClickListener: OnItemClickListener? = null
    private var viewWidth = 0

    init {
        val container = LinearLayout(context)
        container.orientation = LinearLayout.VERTICAL
        setDirectionMode(HORIZONTAL)
        container.background = DrawableBuilder()
                .cornerRadius(context.resources.getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeColor(ContextCompat.getColor(context, R.color.base_bg_stroke_10))
                .strokeWidth(1)
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                .build()
        setRoundCorner(container, context.resources.getDimension(R.dimen.bjy_base_common_bg_radius))
        addViews(container, context.getString(R.string.bjysc_study_room_question_new_msg))
                .run {
                    setOnClickListener {
                        itemClickListener?.onItemClick(0)
                        dismiss()
                    }
                    viewWidth = measuredWidth
                }
        createView(container, false)
    }

    private fun addViews(container: LinearLayout, text: String?): View {
        val textView = TextView(context)
        textView.setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context,R.attr.base_theme_window_main_text_color))
        val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT)
        params.gravity = Gravity.CENTER
        val topPadding = 8.dp
        textView.setPadding(2 * topPadding, topPadding, 2 * topPadding, topPadding)
        textView.text = text
        container.addView(textView, params)
        return textView
    }

    override fun initViewDataHashMap() {
        // do nothing
    }

    override fun show(anchor: View?) {
        initCustomOffset(-viewWidth / 2, 0)
        showWithViewOfDirection(anchor, LEFT_TOP)
    }

    interface OnItemClickListener {
        fun onItemClick(position: Int)
    }
}