package com.baijiayun.groupclassui.window.study;

import android.content.Context;
import android.view.View;
import android.widget.LinearLayout;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.groupclassui.window.multiupload.MultiUploadWindow;
import com.baijiayun.livecore.models.LPStudyRoomQuestionModel;

import java.util.List;

import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

/**
 * Created by pengwenjian on 2021/3/9
 * Describe:
 */
public class QuestionRecordWindow extends InteractiveBaseWindow {
    private Disposable disposableOfQuestionList;
    private Disposable disposableOfTutorAnswer;

    public QuestionRecordWindow(Context context) {
        super(context);
        initView();
    }

    private void initView() {
        DisplayUtils.setRoundCorner(view, context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius));
        $.id(R.id.iv_close).clicked(v -> iRouter.getSubjectByKey(EventKey.DisplayStudyQuestion).onNext(DisplayStudyQuestion.Question_Record_Close));
        View emptyView = View.inflate(context, R.layout.bjysc_layout_question_record_empty, null);
        LinearLayout container = $.id(R.id.ll_container).view();
        container.addView(emptyView, LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT);
        initQuestionRecordViews();

    }

    private void initQuestionRecordViews() {
        LinearLayout container = $.id(R.id.ll_container).view();
        RxUtils.dispose(disposableOfQuestionList);
        disposableOfQuestionList = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfQuestionAndAnswerList()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpStudyRoomQuestionModels -> !lpStudyRoomQuestionModels.isEmpty())
                .subscribe(lpStudyRoomQuestionModels ->
                        addQuestionRecordViews(container, lpStudyRoomQuestionModels), Throwable::printStackTrace);

        disposableOfTutorAnswer = iRouter.getLiveRoom().getStudyRoomVM().getObservableOfTutorAnswer()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpStudyRoomTutorAnswerModel -> {
                    if (lpStudyRoomTutorAnswerModel.answerType == 1) {
                        initQuestionRecordViews();
                    }
                });
    }

    private void addQuestionRecordViews(LinearLayout container, List<LPStudyRoomQuestionModel> lpStudyRoomQuestionModels) {
        container.removeAllViews();
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
        layoutParams.leftMargin = layoutParams.rightMargin = UtilsKt.getDp(10);
        layoutParams.topMargin = layoutParams.bottomMargin = UtilsKt.getDp(8);
        for (int i = 0; i < lpStudyRoomQuestionModels.size(); i++) {
            LinearLayout linearLayout = new LinearLayout(context);
            linearLayout.setOrientation(LinearLayout.VERTICAL);
            final LPStudyRoomQuestionModel questionModel = lpStudyRoomQuestionModels.get(i);
            MultiUploadWindow questionWindow = new MultiUploadWindow(context);
            questionWindow.setUploaded(true);
            questionWindow.setTitle(context.getString(R.string.bjysc_study_room_question_time, questionModel.getCreateTime()));
            questionWindow.setContent(questionModel.getQuestionContent());
            questionWindow.setRemoteImgPaths(questionModel.getQuestionPic());
            questionWindow.showTitle(true);

            MultiUploadWindow answerWindow = new MultiUploadWindow(context);
            answerWindow.setUploaded(true);
            answerWindow.setTitle(context.getString(R.string.bjysc_study_room_question_answer, questionModel.getAnswerAssistantName()));
            answerWindow.setContent(questionModel.getAnswerContent());
            answerWindow.setRemoteImgPaths(questionModel.getAnswerPic());
            answerWindow.showTitle(true);
            if (i == 0) {
                answerWindow.showTitleIcon(true);
            }
            linearLayout.addView(questionWindow.getView(), LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            linearLayout.addView(answerWindow.getView(), LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            DisplayUtils.setRoundCorner(linearLayout, context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius));
            container.addView(linearLayout, layoutParams);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfQuestionList);
        RxUtils.dispose(disposableOfTutorAnswer);
    }

    @Override
    protected View onCreateView(Context context) {
        return View.inflate(context, R.layout.bjysc_window_question_record, null);
    }
}
