package com.baijiayun.groupclassui.window.study

import android.content.Context
import android.content.res.ColorStateList
import android.os.Build
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import androidx.annotation.DrawableRes
import androidx.core.content.ContextCompat
import com.baijiayun.bjyutils.drawable.DrawableBuilder
import com.baijiayun.bjyutils.kt.dp
import com.baijiayun.bjyutils.screen.DisplayUtils.setRoundCorner
import com.baijiayun.groupclassui.R
import com.baijiayun.groupclassui.global.IRouter
import com.baijiayun.livebase.context.LPConstants
import com.baijiayun.livebase.utils.ThemeDataUtil
import com.baijiayun.liveuibase.widgets.popupwindow.BaseAutoArrangePopupWindow
import kotlinx.android.synthetic.main.bjysc_window_study_mode.view.ivMode
import kotlinx.android.synthetic.main.bjysc_window_study_mode.view.tvMode

/**
 * Created by pengwenjian on 2020/12/11
 * Describe:
 */
class StudyModeWindow(context: Context, val router: IRouter) :BaseAutoArrangePopupWindow(context){
    var itemClickListener: OnItemClickListener? = null
    init {
        val container = LinearLayout(context)
        container.orientation = LinearLayout.VERTICAL
        setDirectionMode(HORIZONTAL)
        container.background = DrawableBuilder()
                .cornerRadius(context.resources.getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeColor(ContextCompat.getColor(context,R.color.base_bg_stroke_10))
                .strokeWidth(1)
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context,R.attr.base_theme_brand_container_color))
                .build()
        setRoundCorner(container,context.resources.getDimension(R.dimen.bjy_base_common_bg_radius))
        if (router.liveRoom.studyRoomVM.enableTutorMode()) {
            addViews(container,context.getString(R.string.bjysc_study_room_mode_tutor),R.drawable.bjysc_ic_mode_tutor)
                    .run {
                        setOnClickListener {
                            if (router.liveRoom.studyRoomVM.studyRoomMode != LPConstants.StudyRoomMode.Tutor) {
                                itemClickListener?.onItemClick(LPConstants.StudyRoomMode.Tutor)
                            }
                            dismiss()
                        }
                        if (router.liveRoom.studyRoomVM.studyRoomMode == LPConstants.StudyRoomMode.Tutor) {
                            val textColor = ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_positive_text_color)
                            ivMode.imageTintList = ColorStateList.valueOf(textColor)
                            tvMode.setTextColor(textColor)
                            setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context,R.attr.base_theme_live_product_color))
                        }
                    }
        }

        if (router.liveRoom.studyRoomVM.enableDiscussMode()) {
            addViews(container,context.getString(R.string.bjysc_study_room_mode_discuss),R.drawable.bjysc_ic_mode_discuss)
                    .run {
                        setOnClickListener {
                            if (router.liveRoom.studyRoomVM.studyRoomMode != LPConstants.StudyRoomMode.Discuss) {
                                itemClickListener?.onItemClick(LPConstants.StudyRoomMode.Discuss)
                            }
                            dismiss()
                        }
                        if (router.liveRoom.studyRoomVM.studyRoomMode == LPConstants.StudyRoomMode.Discuss) {
                            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                                val textColor = ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_positive_text_color)
                                ivMode.imageTintList = ColorStateList.valueOf(textColor)
                                tvMode.setTextColor(textColor)
                            }
                            setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context,R.attr.base_theme_live_product_color))
                        }
                    }
        }
        addViews(container,context.getString(R.string.bjysc_study_room_mode_study),R.drawable.bjysc_ic_mode_study)
                .run {
                    setOnClickListener {
                        if (router.liveRoom.studyRoomVM.studyRoomMode != LPConstants.StudyRoomMode.SelfStudy) {
                            itemClickListener?.onItemClick(LPConstants.StudyRoomMode.SelfStudy)
                        }
                        dismiss()
                    }
                    if (router.liveRoom.studyRoomVM.studyRoomMode == LPConstants.StudyRoomMode.SelfStudy) {
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                            val textColor = ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_dialog_positive_text_color)
                            ivMode.imageTintList = ColorStateList.valueOf(textColor)
                            tvMode.setTextColor(textColor)
                        }
                        setBackgroundColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context,R.attr.base_theme_live_product_color))
                    }
                }

        createView(container,false)
    }

    private fun addViews(container: LinearLayout, text: String, @DrawableRes resId: Int, isAddDiv: Boolean = true):View {
        val itemView = View.inflate(context, R.layout.bjysc_window_study_mode, null)
        itemView.ivMode.setImageResource(resId)
        itemView.tvMode.text = text
        container.addView(itemView)
        if (isAddDiv) {
            val div = TextView(context)
            div.setBackgroundColor(ContextCompat.getColor(context, R.color.base_divider_line_other))
            val params = LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, 1.dp)
            container.addView(div, params)
        }
        return itemView
    }

    override fun initViewDataHashMap() {
        // do nothing
    }

    override fun show(anchor: View?) {
        showWithViewOfDirection(anchor, LEFT_TOP)
    }
    interface OnItemClickListener{
        fun onItemClick(studyRoomMode: LPConstants.StudyRoomMode)
    }
}