package com.baijiayun.groupclassui.window.toolbar;

import android.content.Context;

import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.liveuibase.widgets.toolbar.BaseGraphMenuWindow;
import com.baijiayun.liveuibase.widgets.toolbar.ColorSelectData;
import com.baijiayun.liveuibase.widgets.toolbar.WidthSelectData;

import io.reactivex.disposables.Disposable;

public class GraphMenuWindow extends BaseGraphMenuWindow {
    private Disposable disposableOfColorAllChange;
    private IRouter iRouter;
    private OnShapeChangeModel defaultModel;

    public GraphMenuWindow(Context context, IRouter iRouter) {
        super(context);
        this.iRouter = iRouter;
        if (defaultModel != null) {
            onChangeShape(defaultModel);
        }
    }

    /**
     * 初始化颜色选中
     */
    @Override
    protected boolean initColorsSelect() {
        boolean rst = super.initColorsSelect();
        if (rst) {
            if (disposableOfColorAllChange != null) {
                disposableOfColorAllChange.dispose();
            }
            disposableOfColorAllChange = iRouter.getSubjectByKey(EventKey.DrawColorAllChange)
                    .ofType(String.class)
                    .subscribe(this::handleColorAllChangeRst);
        }
        return rst;
    }

    @Override
    protected void initDefaultDrawable(OnShapeChangeModel initShapeModel) {
        defaultModel = initShapeModel;
    }

    @Override
    protected void onChangeShape(OnShapeChangeModel onShapeChangeModel) {
        iRouter.getSubjectByKey(EventKey.GraphDrawEnable).onNext(onShapeChangeModel);
    }

    @Override
    protected void onChangeWidth(WidthSelectData widthSelectData) {
        iRouter.getSubjectByKey(EventKey.CustomGraphWidthChange).onNext(widthSelectData);
    }

    @Override
    protected void onChangeColor(ColorSelectData colorSelectData) {
        iRouter.setObjectByKey(EventKey.IsChangedPaintColor,true);
        iRouter.getSubjectByKey(EventKey.DrawColorChange).onNext(colorSelectData);
    }

    public void onDestroy() {
        if (disposableOfColorAllChange != null) {
            disposableOfColorAllChange.dispose();
            disposableOfColorAllChange = null;
        }
    }
}
