package com.baijiayun.groupclassui.window.toolbar;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.base.BaseGroupPresenter;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.widget.DragRelativeLayout;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.liveuibase.base.BaseUIConstant;
import com.baijiayun.liveuibase.widgets.common.CheckImageView;
import com.baijiayun.liveuibase.widgets.toolbar.BaseGraphMenuWindow;
import com.baijiayun.liveuibase.widgets.toolbar.ColorSelectData;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.reactivex.android.schedulers.AndroidSchedulers;

public class ToolbarWindow extends ToolbarBaseWindow implements ToolbarContract.ToolbarView {

    final static int TOOL_WINDOW_TOP_MARGIN_BIG = 40;
    final static int TOOL_WINDOW_TOP_MARGIN_SMALL = 4;
    private boolean isFullScreen;
    private ToolbarContract.ToolbarPresenter presenter;
    private CheckImageView eraseIv;
    //是否框选删除
    private boolean isSelectEraseMode = false;

    @Override
    public void setPresenter(BaseGroupPresenter presenter) {
        // do nothing
    }

    public FrameLayout.LayoutParams getParams() {
        if (view == null)
            return null;
        return (FrameLayout.LayoutParams) view.getLayoutParams();
    }

    public ToolbarWindow(Context context, boolean isFullScreen) {
        super(context);
        this.isFullScreen = isFullScreen;
        presenter = new ToolbarPresenter(this);
        presenter.setRouter(iRouter);
        initDrawable();

        // 判断学生老师端工具栏
        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student || isFullScreen) {
            $.id(R.id.activity_toolbar_split_line).view().setVisibility(View.GONE);
            $.id(R.id.activity_toolbar_courseware_image).view().setVisibility(View.GONE);
            $.id(R.id.toolbox_container).view().setVisibility(View.GONE);
        }

        if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student
                || iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Visitor) {
            //学生or游客不显示激光笔
            $.id(R.id.activity_toolbar_laser_image).visibility(View.GONE);
        }

        if (isFullScreen
                && (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher
                || iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant)) {
            $.id(R.id.activity_toolbar_laser_image).visibility(View.GONE);
        }

        eraseIv = $.id(R.id.activity_toolbar_eraser_image).view();

        $.id(R.id.toolbar_ppt_auth_img).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.toolbar_ppt_auth_img).view()).isChecked();
            iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(isChecked);
            notifyItemSelected(R.id.toolbar_ppt_auth_img, isChecked);
        });

        //默认显示橡皮
        setEraseMode(false);

        $.id(R.id.activity_toolbar_selector_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_selector_image).view()).isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            } else {
                presenter.selectMode(true);
            }
            notifyItemSelected(R.id.activity_toolbar_selector_image, isChecked);
        });

        $.id(R.id.activity_toolbar_brush_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_brush_image).view()).isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            } else {
                List<Object> clickData = new ArrayList<>();
                clickData.add(DrawLineMode.LINE_MODE);
                clickData.add($.id(R.id.activity_toolbar_brush_image).view());
                presenter.lineDrawMode(clickData);
            }
            notifyItemSelected(R.id.activity_toolbar_brush_image, isChecked);
        });


        $.id(R.id.activity_toolbar_marker_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_marker_image).view()).isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            } else {
                List<Object> clickData = new ArrayList<>();
                clickData.add(DrawLineMode.MARKER_MODE);
                clickData.add($.id(R.id.activity_toolbar_marker_image).view());
                presenter.lineDrawMode(clickData);
            }
            notifyItemSelected(R.id.activity_toolbar_marker_image, isChecked);
        });

        $.id(R.id.activity_toolbar_graph_image).clicked(v -> {
            CheckImageView checkImageView = $.id(R.id.activity_toolbar_graph_image).view();
            boolean isChecked = checkImageView.isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            } else {
                v.setTag(null);
                iRouter.getSubjectByKey(EventKey.GraphMenuEnable).onNext(v);
            }
            notifyItemSelected(R.id.activity_toolbar_graph_image, isChecked);
        });

        $.id(R.id.activity_toolbar_laser_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_laser_image).view()).isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            }
            notifyItemSelected(R.id.activity_toolbar_laser_image, isChecked);
        });

        ((CheckImageView) $.id(R.id.activity_toolbar_laser_image).view()).setOnCheckedChangeListener(isChecked -> iRouter.getSubjectByKey(EventKey.LaserDrawEnable).onNext(isChecked));

        $.id(R.id.activity_toolbar_word_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_word_image).view()).isChecked();
            if (isChecked) {
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
            } else {
                iRouter.getSubjectByKey(EventKey.DrawTextEnable).onNext($.id(R.id.activity_toolbar_word_image).view());
            }
            notifyItemSelected(R.id.activity_toolbar_word_image, isChecked);
        });

        $.id(R.id.activity_toolbar_eraser_image).clicked(v -> {
                    boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_eraser_image).view()).isChecked();
                    if (isSelectEraseMode) {
                        iRouter.getSubjectByKey(EventKey.GraphSelectEraseMode).onNext(true);
                        isSelectEraseMode = false;
                        setEraseMode(false);
                    } else {
                        iRouter.getSubjectByKey(EventKey.GraphEraseMode).onNext(!isChecked);
                        setEraseMode(false);
                        eraseIv.setChecked(true);
                        notifyItemSelected(R.id.activity_toolbar_eraser_image, isChecked);
                    }
                }
        );

        $.id(R.id.activity_toolbar_eraser_all)
                .visibility(isAdmin() ? View.VISIBLE : View.GONE)
                .clicked(v -> iRouter.getSubjectByKey(EventKey.GraphEraseAll).onNext(true));

        $.id(R.id.activity_toolbar_courseware_image).clicked(v ->
                iRouter.getSubjectByKey(EventKey.CoursewareManageEnable).onNext(true));
        $.id(R.id.activity_toolbar_box_image).clicked(v -> {
            boolean isChecked = ((CheckImageView) $.id(R.id.activity_toolbar_box_image).view()).isChecked();
            iRouter.getSubjectByKey(EventKey.ToolBox).onNext(v);
            notifyItemSelected(R.id.activity_toolbar_box_image, isChecked);
        });
        $.id(R.id.activity_toolbar_roll_call_tip).background(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .oval()
                .build());
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE
                && iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Teacher) {
            $.id(R.id.activity_toolbar_box_image).gone();
        }
    }

    @Override
    protected View onCreateView(Context context) {
        view = new DragRelativeLayout(context);
        LayoutInflater.from(context).inflate(R.layout.bjy_group_activity_brush_toolbar, (ViewGroup) view);
        FrameLayout.LayoutParams layoutParams = new FrameLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.DEFAULT) {
            layoutParams.gravity = Gravity.END;
        } else {
            layoutParams.gravity = Gravity.END | Gravity.CENTER_VERTICAL;
            view.findViewById(R.id.activity_toolbar_courseware_image).setVisibility(View.GONE);
            view.findViewById(R.id.activity_toolbar_split_line).setVisibility(View.GONE);
            view.findViewById(R.id.toolbox_container).setVisibility(View.GONE);
        }
        //自习室不要工具箱
        if (iRouter.getLiveRoom().getStudyRoomVM().isStudyRoom()) {
            view.findViewById(R.id.toolbox_container).setVisibility(View.GONE);
        }
        layoutParams.rightMargin = UtilsKt.getDp(4);
        // 统一偏移距黑板顶部 TOOL_WINDOW_TOP_MARGIN_STUDENT
        if (isAdmin()) {
            layoutParams.topMargin = UtilsKt.getDp(TOOL_WINDOW_TOP_MARGIN_SMALL);
        } else {
            layoutParams.topMargin = UtilsKt.getDp(TOOL_WINDOW_TOP_MARGIN_BIG);
        }
        view.setLayoutParams(layoutParams);
        return view;
    }

    private void initDrawable() {
        view.findViewById(R.id.bjysc_toolbar_root_container).setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_brand_container_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .strokeWidth(UtilsKt.getDp(1))
                .strokeColor(ContextCompat.getColor(context, R.color.bjysc_bg_stroke_10))
                .build());
    }

    @Override
    public boolean onRoomStatusChange(boolean isActive) {
        boolean isWindowInit = super.onRoomStatusChange(isActive);
        if (isWindowInit) {
            if (isActive) {
                subscribe();
            } else {
                resetToolbar();
                iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(false);
                compositeDisposable.clear();
            }
        }
        return true;
    }

    private GradientDrawable getDrawable(int color) {
        GradientDrawable drawable = new GradientDrawable();
        drawable.setCornerRadius(3);
        drawable.setColor(color);
        return drawable;
    }

    private void subscribe() {
        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.DrawColorChange)
                .ofType(ColorSelectData.class)
                .subscribe(colorSelectData -> {
                    if (colorSelectData == null) {
                        return;
                    }
                    preColorCache.put(colorSelectData.getSelectSrc(), colorSelectData.getSelectColor());
                    updatePreColor(colorSelectData.getSelectSrc(), colorSelectData.getSelectColor());
                }));

        Object color = iRouter.getValueByKey(EventKey.DrawColorAllChange, Object.class);
        if (color instanceof Integer) {
            initToolbarItemBackgroundColor((Integer) color);
        } else {
            initToolbarItemBackgroundColor(Color.parseColor("#1795FF"));
        }
        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.DrawColorAllChange)
                .ofType(String.class)
                .subscribe(colorString -> {
                    initToolbarItemBackgroundColor(Color.parseColor(colorString));
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.GraphDrawEnable)
                .ofType(BaseGraphMenuWindow.OnShapeChangeModel.class)
                .subscribe(model -> {
                    view.post(() -> {
                        CheckImageView checkImageView = $.id(R.id.activity_toolbar_graph_image).view();
                        checkImageView.setCheckedDrawable(model.getSelectDrawable());
                        checkImageView.setUnCheckedDrawable(model.getNormalDrawable());
                        updateChecked(R.id.activity_toolbar_graph_image, model.isInitDrawable());
                        lastImageId = R.id.activity_toolbar_graph_image;
                    });
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.EditTextShape)
                .ofType(String.class)
                .subscribe(s -> {
                    iRouter.getSubjectByKey(EventKey.DrawTextEnable).onNext(true);
                    notifyItemSelected(R.id.activity_toolbar_word_image, false);
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.ResetToolbarStatus)
                .filter(o -> o instanceof Boolean && (Boolean) o)
                .subscribe(o -> {
                    resetToolbar();
                    // 重置避免后续新建 ToolbarWindow 重新走这里
                    iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext("");
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.LaserShowState)
                .ofType(Boolean.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isShow -> {
                    //激光笔是否显示
                    if (isFullScreen)
                        return;

                    boolean isReset = lastImageId == R.id.activity_toolbar_laser_image;

                    if (isShow) {
                        $.id(R.id.activity_toolbar_laser_image).visibility(View.VISIBLE);
                        ((CheckImageView) $.id(R.id.activity_toolbar_laser_image).view()).setChecked(false);
                    } else {
                        $.id(R.id.activity_toolbar_laser_image).visibility(View.GONE);
                        if (isReset) {
                            iRouter.getSubjectByKey(EventKey.ResetToolbarStatus).onNext(true);
                        }
                    }
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.Action2ShapeSelected)
                .observeOn(AndroidSchedulers.mainThread())
                .ofType(Boolean.class)
                .subscribe(aBoolean -> {
                    isSelectEraseMode = aBoolean;
                    setEraseMode(isSelectEraseMode);
                })
        );

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.ToolbarItemSelected)
                .observeOn(AndroidSchedulers.mainThread())
                .ofType(HashMap.class)
                .subscribe(hashMap -> {
                    Map.Entry<Integer, Boolean> entry = (Map.Entry<Integer, Boolean>) hashMap.entrySet().iterator().next();
                    updateChecked(entry.getKey(), entry.getValue());
                })
        );
        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.ShowRollCallDot)
                .observeOn(AndroidSchedulers.mainThread())
                .ofType(Boolean.class)
                .filter(aBoolean -> isAdmin())
                .subscribe(aBoolean -> {
                    $.id(R.id.activity_toolbar_roll_call_tip).visibility(aBoolean && $.id(R.id.toolbox_container).view().getVisibility() == View.VISIBLE ? View.VISIBLE : View.GONE);
                }));

        compositeDisposable.add(iRouter.getSubjectByKey(EventKey.ZXYBSoftKey)
                .ofType(Integer.class)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(this::doSoftKeyCallBack));

    }
    // 重置上次选中按钮状态
    private void resetToolbar() {
        if (lastImageId != -1) {
            if ($.id(lastImageId).view() instanceof CheckImageView) {
                ((CheckImageView) $.id(lastImageId).view()).setChecked(false);
            } else {
                $.id(lastImageId).view().setSelected(false);
            }
            $.id(lastImageId).view().setBackground(null);
        }
        // 同时置空画笔工具栏item选中事件
        iRouter.getSubjectByKey(EventKey.ToolbarItemSelected).onNext(false);
    }

    private void doSoftKeyCallBack(int keyCode) {
        switch (keyCode) {
            case 101:
                //鼠标
                break;
            case 102:
                //上一页
                break;
            case 103:
                //下一页
                break;
            case 104:
                //选择
                boolean isSelectChecked = ((CheckImageView) $.id(R.id.activity_toolbar_selector_image).view()).isChecked();
                if (!isSelectChecked) {
                    presenter.selectMode(true);
                    notifyItemSelected(R.id.activity_toolbar_selector_image, false);
                }
                break;
            case 105:
                //画笔
                boolean isBrushChecked = ((CheckImageView) $.id(R.id.activity_toolbar_brush_image).view()).isChecked();
                if (!isBrushChecked) {
                    List<Object> clickData = new ArrayList<>();
                    clickData.add(DrawLineMode.LINE_MODE);
                    clickData.add($.id(R.id.activity_toolbar_brush_image).view());
                    //多加一个让弹窗不弹出
                    clickData.add("");
                    presenter.lineDrawMode(clickData);
                    notifyItemSelected(R.id.activity_toolbar_brush_image, false);
                }
                break;
            case 106:
                //马克笔
                boolean isMarkChecked = ((CheckImageView) $.id(R.id.activity_toolbar_marker_image).view()).isChecked();
                if (!isMarkChecked) {
                    List<Object> clickData = new ArrayList<>();
                    clickData.add(DrawLineMode.MARKER_MODE);
                    clickData.add($.id(R.id.activity_toolbar_marker_image).view());
                    clickData.add("");
                    presenter.lineDrawMode(clickData);
                    notifyItemSelected(R.id.activity_toolbar_marker_image, false);
                }
                break;
            case 107:
                //箭头
                CheckImageView checkImageView = $.id(R.id.activity_toolbar_graph_image).view();
                boolean isGrapImageChecked = checkImageView.isChecked();
                if (!isGrapImageChecked) {
                    checkImageView.setTag(false);
                    iRouter.getSubjectByKey(EventKey.GraphMenuEnable).onNext(checkImageView);
                    notifyItemSelected(R.id.activity_toolbar_graph_image, false);
                }
                break;
            case 108:
                //橡皮
                if (!eraseIv.isChecked()) {
                    eraseIv.performClick();
                }
                break;
            case 109:
                //颜色
                break;
            case 110:
                //粗细
                break;
            case 111:
                //F1
                break;
            case 112:
                //F2
                break;
            default:
                break;
        }
    }

    /**
     * 设置画笔删除模式
     *
     * @param isSelectEraseMode true 框选删除、 false 橡皮擦触摸删除
     */
    @SuppressLint("UseCompatLoadingForDrawables")
    private void setEraseMode(boolean isSelectEraseMode) {
        if (isSelectEraseMode) {
            eraseIv.setCheckedDrawable(context.getResources().getDrawable(R.drawable.base_ic_toolbar_delete_select));
            eraseIv.setUnCheckedDrawable(context.getResources().getDrawable(R.drawable.base_ic_toolbar_delete_normal));
        } else {
            eraseIv.setCheckedDrawable(context.getResources().getDrawable(R.drawable.base_ic_toolbar_erase_select));
            eraseIv.setUnCheckedDrawable(context.getResources().getDrawable(R.drawable.base_ic_toolbar_erase_normal));
        }
        eraseIv.setChecked(eraseIv.isChecked());
    }

    /**
     * 更新选中状态
     *
     * @param checkId   view id
     * @param isChecked 当前选中状态
     */
    private void updateChecked(int checkId, boolean isChecked) {
        if ($.id(checkId).view() instanceof CheckImageView) {
            ((CheckImageView) $.id(checkId).view()).setChecked(!isChecked);
        } else {
            $.id(checkId).view().setSelected(!isChecked);
        }
        // BaseGraphMenuWindow 传过来的 model 里自带了一个背景阴影，所以不需要了
        boolean needBg = !isChecked && checkId != R.id.activity_toolbar_graph_image;
        $.id(checkId).view().setBackground(needBg ? ContextCompat.getDrawable(context, R.drawable.tool_bar_check_layer_bg) : null);

        if (lastImageId != checkId && $.id(lastImageId).view() != null) {
            if ($.id(lastImageId).view() instanceof CheckImageView && lastImageId != checkId) {
                ((CheckImageView) $.id(lastImageId).view()).setChecked(false);
            } else {
                $.id(lastImageId).view().setSelected(!$.id(lastImageId).view().isSelected());
            }
        }
        if (lastImageId != -1 && lastImageId != checkId) {
            $.id(lastImageId).view().setBackground(null);
        }
        lastImageId = isChecked ? -1 : checkId;
    }

    /**
     * 改为EventKey通知，全屏和非全屏两个toolbarWindow需要同步更新选中状态。。。
     */
    private void notifyItemSelected(int checkId, boolean isChecked) {
        HashMap<Integer, Boolean> map = new HashMap<>();
        map.put(checkId, isChecked);
        iRouter.getSubjectByKey(EventKey.ToolbarItemSelected).onNext(map);
    }

    /**
     * 设置item边缘色块颜色
     */
    private void initToolbarItemBackgroundColor(int color) {
        $.id(R.id.activity_toolbar_word_color).view().setBackground(getDrawable(color));
        $.id(R.id.activity_toolbar_brush_color).view().setBackground(getDrawable(color));
        $.id(R.id.activity_toolbar_graph_color).view().setBackground(getDrawable(color));
        $.id(R.id.activity_toolbar_marker_color).view().setBackground(getDrawable(color));

        for (Map.Entry<BaseUIConstant.SelectSrc, Integer> entry : preColorCache.entrySet()) {
            // 检查缓存颜色
            updatePreColor(entry.getKey(), entry.getValue());
        }
    }

    /**
     * 预览色块颜色缓存 - full screen 下的 toolbar 需要取到黑板下的 toolbar 色值，来进行初始化同步
     */
    public static Map<BaseUIConstant.SelectSrc, Integer> preColorCache = new HashMap<>();

    /**
     * 更新预览色块
     */
    private void updatePreColor(BaseUIConstant.SelectSrc selectSrc, int color) {
        Drawable selectDrawable = getDrawable(color);
        switch (selectSrc) {
            case Text:
                $.id(R.id.activity_toolbar_word_color).view().setBackground(selectDrawable);
                break;
            case Brush:
                $.id(R.id.activity_toolbar_brush_color).view().setBackground(selectDrawable);
                break;
            case Graph:
                $.id(R.id.activity_toolbar_graph_color).view().setBackground(selectDrawable);
                break;
            case Marker:
                $.id(R.id.activity_toolbar_marker_color).view().setBackground(selectDrawable);
                break;
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        compositeDisposable.clear();
        preColorCache.clear();
    }

    public void setOnUpdateLayoutParamsListener(DragRelativeLayout.OnUpdateLayoutParamsListener listener) {
        ((DragRelativeLayout) view).setOnUpdateLayoutParamsListener(listener);
    }

    public void setDragParam(int width, int height) {
        ((DragRelativeLayout) view).setDragParams(width, height);
    }

    public void updateView(boolean showPPTAuth, boolean showBrush, boolean showToolBox) {
        $.id(R.id.toolbar_ppt_auth).visibility(showPPTAuth ? View.VISIBLE : View.GONE);
        $.id(R.id.brush_container).visibility(showBrush ? View.VISIBLE : View.GONE);
        $.id(R.id.toolbox_container).visibility(showToolBox ? View.VISIBLE : View.GONE);
    }
}
