package com.baijiayun.groupclassui.window.toolbox.smallblackboard;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.model.SbbStatusModel;

import java.util.List;

/**
 * Created by yongjiaming on 2019/3/23
 */
public class SbbCommitStatusAdapter extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private List<SbbStatusModel> sbbStatusModelList;

    private static final int VIEW_TYPE_SBB = 0;
    private static final int VIEW_TYPE_LOADING = 1;

    public boolean isLoading;

    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        if (viewType == VIEW_TYPE_SBB) {
            return new SbbCommitStatusViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_sbb_commit_status, null));
        } else if (viewType == VIEW_TYPE_LOADING) {
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_snippet_loadmore, parent, false);
            return new SbbLoadingViewHolder(view);
        }
        return null;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder viewHolder, int position) {
        if (position >= sbbStatusModelList.size()) {
            return;
        }
        if(viewHolder instanceof SbbCommitStatusViewHolder){
            SbbCommitStatusViewHolder holder = (SbbCommitStatusViewHolder) viewHolder;
            SbbStatusModel statusModel = sbbStatusModelList.get(position);
            switch (statusModel.sbbStatus) {
                case NORMAL:
                    holder.checkIv.setVisibility(View.GONE);
                    holder.redDotIv.setVisibility(View.GONE);
                    break;
                case COMMIT:
                    holder.checkIv.setVisibility(View.VISIBLE);
                    holder.redDotIv.setVisibility(View.VISIBLE);
                    break;
                case READ:
                    holder.checkIv.setVisibility(View.VISIBLE);
                    holder.redDotIv.setVisibility(View.GONE);
                    break;
                default:
                    break;
            }
            holder.commitStudentTv.setText(sbbStatusModelList.get(position).user.getName());
        } else{
            SbbLoadingViewHolder loadingViewHolder = (SbbLoadingViewHolder) viewHolder;
            loadingViewHolder.progressBar.setIndeterminate(true);
        }
    }

    @Override
    public int getItemCount() {
        return sbbStatusModelList.size();
    }

    @Override
    public int getItemViewType(int position) {
        return isLoading ? VIEW_TYPE_LOADING : VIEW_TYPE_SBB;
    }

    public void setData(List<SbbStatusModel> models) {
        sbbStatusModelList = models;
    }

    static class SbbCommitStatusViewHolder extends RecyclerView.ViewHolder {

        ImageView checkIv;
        TextView commitStudentTv;
        ImageView redDotIv;


        public SbbCommitStatusViewHolder(View itemView) {
            super(itemView);
            checkIv = itemView.findViewById(R.id.sbb_commit_item_check_iv);
            commitStudentTv = itemView.findViewById(R.id.sbb_commit_student_name);
            redDotIv = itemView.findViewById(R.id.sbb_red_dot);
        }
    }

    static class SbbLoadingViewHolder extends RecyclerView.ViewHolder {
        ProgressBar progressBar;

        public SbbLoadingViewHolder(@NonNull View itemView) {
            super(itemView);
            progressBar = itemView.findViewById(R.id.item_online_user_progress);
        }
    }
}


