package com.baijiayun.groupclassui.window.toolbox.smallblackboard;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.graphics.Color;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.BaseExpandableListAdapter;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.appcompat.content.res.AppCompatResources;

import com.baijiahulian.common.cropperv2.uikit.CircleColorView;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.model.SbbStatusModel;
import com.baijiayun.livebase.models.LPUserModel;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by yongjiaming on 2019-11-24
 * Describe:
 */
public class SbbExpandableListViewAdapter extends BaseExpandableListAdapter {


    private List<SbbStatusModel> sbbActiveStatusModelList = new ArrayList<>();
    private List<SbbStatusModel> sbbUnActiveStatusModelList = new ArrayList<>();

    @Override
    public int getGroupCount() {
        return 2;
    }

    @Override
    public int getChildrenCount(int groupPosition) {
        if(groupPosition == 0){
            return sbbActiveStatusModelList.size();
        }
        return sbbUnActiveStatusModelList.size();
    }

    @Override
    public Object getGroup(int groupPosition) {
        return groupPosition;
    }

    @Override
    public Object getChild(int groupPosition, int childPosition) {
       if(groupPosition == 0){
           return sbbActiveStatusModelList.get(childPosition);
       }
       return sbbUnActiveStatusModelList.get(childPosition);
    }

    @Override
    public long getGroupId(int groupPosition) {
        return groupPosition;
    }

    @Override
    public long getChildId(int groupPosition, int childPosition) {
        return Long.parseLong(((SbbStatusModel)getChild(groupPosition, childPosition)).user.getUserId());
    }

    @Override
    public boolean hasStableIds() {
        return true;
    }

    @Override
    public View getGroupView(int groupPosition, boolean isExpanded, View convertView, ViewGroup parent) {
        GroupViewHolder groupViewHolder;
        if(convertView == null){
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_sbb_group_title, null);
            groupViewHolder = new GroupViewHolder();
            groupViewHolder.groupTextView = view.findViewById(R.id.sbb_group_title_tv);
            groupViewHolder.itemStateIv = view.findViewById(R.id.sbb_group_state_iv);
            convertView = view;
            convertView.setTag(groupViewHolder);
        } else{
            groupViewHolder = (GroupViewHolder) convertView.getTag();
        }
        if (isExpanded) {
            groupViewHolder.itemStateIv.setImageDrawable(AppCompatResources.getDrawable(parent.getContext(),R.drawable.iv_lp_ui_down));
        } else {
            groupViewHolder.itemStateIv.setImageDrawable(AppCompatResources.getDrawable(parent.getContext(),R.drawable.iv_lp_ui_group_close));
        }
        if(groupPosition == 0){
            groupViewHolder.groupTextView.setText(parent.getContext().getResources().getString(R.string.sbb_active_group, sbbActiveStatusModelList.size()));
        } else{
            groupViewHolder.groupTextView.setText(parent.getContext().getResources().getString(R.string.sbb_unactive_group, sbbUnActiveStatusModelList.size()));
        }
        return convertView;
    }

    @Override
    public View getChildView(int groupPosition, int childPosition, boolean isLastChild, View convertView, ViewGroup parent) {
        SbbCommitStatusViewHolder holder;
        if(convertView == null){
            View view = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_sbb_commit_status, null);
            holder = new SbbCommitStatusViewHolder();
            holder.checkIv = view.findViewById(R.id.sbb_commit_item_check_iv);
            holder.commitStudentTv = view.findViewById(R.id.sbb_commit_student_name);
            holder.redDotIv = view.findViewById(R.id.sbb_red_dot);
            holder.circleColorView = view.findViewById(R.id.view_group_color);
            convertView = view;
            convertView.setTag(holder);
        } else{
            holder = (SbbCommitStatusViewHolder) convertView.getTag();
        }
        List<SbbStatusModel> statusModelList = groupPosition == 0 ? sbbActiveStatusModelList : sbbUnActiveStatusModelList;
        SbbStatusModel statusModel = statusModelList.get(childPosition);
        if(statusModel != null){
            LPUserModel lpUserModel = (LPUserModel) statusModel.user;
            if (lpUserModel.groupId != 0 && lpUserModel.groupItem != null) {
                holder.circleColorView.setVisibility(View.VISIBLE);
                holder.circleColorView.setColor(Color.parseColor(lpUserModel.groupItem.color));
            } else {
                holder.circleColorView.setVisibility(View.GONE);
            }
            switch (statusModel.sbbStatus) {
                case NORMAL:
                    holder.checkIv.setVisibility(View.GONE);
                    holder.redDotIv.setVisibility(View.GONE);
                    break;
                case COMMIT:
                    holder.checkIv.setVisibility(View.VISIBLE);
                    holder.redDotIv.setVisibility(View.VISIBLE);
                    break;
                case READ:
                    holder.checkIv.setVisibility(View.VISIBLE);
                    holder.redDotIv.setVisibility(View.GONE);
                    break;
                default:
                    break;
            }
        }
        holder.commitStudentTv.setText(getEncodePhoneNumber(statusModelList.get(childPosition).user.getName()));
        return convertView;
    }

    @Override
    public boolean isChildSelectable(int groupPosition, int childPosition) {
        return true;
    }


    static class GroupViewHolder{
        TextView groupTextView;
        ImageView itemStateIv;
    }

    static class SbbCommitStatusViewHolder {

        ImageView checkIv;
        TextView commitStudentTv;
        ImageView redDotIv;
        CircleColorView circleColorView;
    }

    public void setData(List<SbbStatusModel> activeList, List<SbbStatusModel> unActiveList){
        sbbActiveStatusModelList = activeList;
        sbbUnActiveStatusModelList = unActiveList;
    }

    public int getActiveGroupCount(){
        return sbbActiveStatusModelList.size();
    }
}
