package com.baijiayun.groupclassui.window.toolbox.smallblackboard

import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.ImageView
import android.widget.ProgressBar
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.baijiayun.groupclassui.R
import com.baijiayun.livebase.models.imodels.IUserModel
import java.util.ArrayList

/**
 * Created by pengwenjian on 2021/11/22
 * Describe:
 */
class SbbStudentAdapter(val studentList: MutableList<IUserModel>, val isSelectPage: Boolean = false) : RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    companion object{
        private const val VIEW_TYPE_USER = 0
        private const val VIEW_TYPE_LOADING = 1
    }
    var onSelectListener: OnSelectListener? = null
    var isLoading = false
    private var selectMap = HashMap<IUserModel, Boolean>()

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return if (viewType == VIEW_TYPE_USER) {
            val view = LayoutInflater.from(parent.context)
                .inflate(R.layout.bjy_group_item_sbb_student, parent, false)
            SbbStudentViewHolder(view)
        } else {
            val view = LayoutInflater.from(parent.context)
                .inflate(R.layout.bjy_group_item_online_user_loadmore, parent, false)
            SbbLoadingViewHolder(view)
        }
    }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        when (holder) {
            is SbbStudentViewHolder -> {
                val userModel = studentList[position]
                holder.nameTv.text = userModel.name
                holder.selectIv.setImageResource(
                    if (isSelectPage) R.drawable.bjy_group_ic_sbb_close else {
                        R.drawable.bjy_group_sel_sbb_student_select
                    }
                )
                holder.selectIv.isSelected = selectMap[userModel] ?: false
                holder.selectIv.setOnClickListener {
                    selectMap[userModel] = !(selectMap[userModel] ?: false)
                    if (isSelectPage) {
                        studentList.remove(userModel)
                        notifyItemRemoved(position)
                    } else {
                        notifyItemChanged(position)
                    }
                    onSelectListener?.select(userModel, selectMap[userModel] ?: false)
                }
            }
            else -> {
                (holder as SbbLoadingViewHolder).progressBar.isIndeterminate = true
            }
        }
    }

    override fun getItemViewType(position: Int): Int {
        if (isLoading && position == itemCount - 1) {
            return VIEW_TYPE_LOADING
        }
        return VIEW_TYPE_USER
    }

    // 刷新列表，人数可能变化
    fun setData(userModels: List<IUserModel>) {
        studentList.clear()
        studentList.addAll(userModels)
        val removeList = ArrayList<IUserModel>()
        for ((k, _) in selectMap) {
            if (!studentList.contains(k)) {
                removeList.add(k)
                selectMap[k] = false
            }
        }
        onSelectListener?.unSelect(removeList)
        notifyDataSetChanged()
    }

    fun notifyItemInserted(userModel: IUserModel) {
        studentList.add(userModel)
        notifyItemInserted(itemCount)
    }

    fun notifyItemRemoved(userModel: IUserModel) {
        val index = studentList.indexOf(userModel)
        if (index != -1) {
            studentList.remove(userModel)
            selectMap.remove(userModel)
            notifyItemRemoved(index)
        }
    }

    fun notifyItemRemoved(userModels: List<IUserModel>) {
        for (user in userModels) {
            selectMap.remove(user)
        }
        studentList.removeAll(userModels)
        notifyDataSetChanged()
    }

    fun notifyItemChanged(userModel: IUserModel) {
        for ((index,user) in studentList.withIndex()) {
            if (user == userModel) {
                selectMap[user] = false
                notifyItemChanged(index)
                break
            }
        }
    }

    fun setSelectList(selectList: List<IUserModel>) {
        if (isSelectPage) {
            this.studentList.clear()
            this.studentList.addAll(selectList)
        } else {
            for (user in selectList) {
                this.selectMap[user] = true
            }
        }
        notifyDataSetChanged()
    }

    fun clear() {
        if (isSelectPage) {
            studentList.clear()
        } else {
            selectMap.clear()
        }
        notifyDataSetChanged()
    }

    override fun getItemCount() = studentList.size

    interface OnSelectListener{
        fun select(userModel: IUserModel, selected: Boolean)
        fun unSelect(users: List<IUserModel>)
    }
    class SbbStudentViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        val nameTv = itemView.findViewById<TextView>(R.id.sbb_name_tv)!!
        val selectIv = itemView.findViewById<ImageView>(R.id.sbb_select_iv)!!
    }
    class SbbLoadingViewHolder(itemView: View) : RecyclerView.ViewHolder(itemView) {
        var progressBar: ProgressBar = itemView.findViewById(R.id.item_online_user_progress)
    }
}