package com.baijiayun.groupclassui.window.toolbox.timer;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.media.SoundPool;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPTimerModel;

import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.Disposable;

public class StudentTimerWindow extends InteractiveBaseWindow {
    private Disposable disposableOfTimerStart, disposableOfInterval;
    private long remainSeconds;
    private long timeDuration;
    private Drawable normalDrawable;
    private Drawable warningDrawable;
    private final TextView tvTimer;
    private SoundPool soundPool;

    public StudentTimerWindow(Context context, LPTimerModel lpTimerModel) {
        super(context);
        timeDuration = remainSeconds = lpTimerModel.value.duration;
        tvTimer = $.id(R.id.student_timer_main_text).view();
        initDrawable();
        init();
    }

    private void initDrawable() {
        // 普通计时时的背景色
        normalDrawable = ThemeDataUtil.getCommonWindowBg(context);
        // 倒计时一分钟内的红色警示背景色
        warningDrawable = new DrawableBuilder()
                .solidColor(ContextCompat.getColor(context, R.color.bjysc_warning_color))
                .cornerRadius(context.getResources().getDimensionPixelSize(R.dimen.bjy_base_common_bg_radius))
                .build();

        tvTimer.setBackground(normalDrawable);
    }

    @Override
    protected View onCreateView(Context context) {
        setNeedShowShadow(true);
        return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_timer_student, null);
    }

    private void init() {
        setAllowTouch(true);
        setTimer(remainSeconds);
        showViewState(remainSeconds != 60);
        if(remainSeconds > 0) {
            startInterval();
        }
        disposableOfTimerStart = iRouter.getLiveRoom().getToolBoxVM().getObservableOfTimerStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpTimerModel -> {
                    timeDuration = remainSeconds = lpTimerModel.value.duration;
                    setTimer(remainSeconds);
                    showViewState(remainSeconds != 60);
                    startInterval();
                });
    }

    private void startInterval() {
        RxUtils.dispose(disposableOfInterval);
        disposableOfInterval = Observable.interval(1, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread()).subscribe(aLong -> {
                    if (remainSeconds > 0) {
                        remainSeconds--;
                        setTimer(remainSeconds);
                        if (remainSeconds <= 60 && timeDuration >= 60) {
                            showViewState(false);
                        }
                    } else {
                        RxUtils.dispose(disposableOfInterval);
                        if(soundPool != null) {
                            soundPool.release();
                        }
                        soundPool = UtilsKt.playAudio(context, R.raw.timer_end);
                        showViewState(false);
                    }
                });
    }

    private void setTimer(long remainSeconds) {
        long min = remainSeconds / 60;
        long second = remainSeconds % 60;
        String minStr = min > 9 ? String.valueOf(min) : "0" + min;
        String secStr = second > 9 ? String.valueOf(second) : "0" + second;
        tvTimer.setText(String.format("%s : %s", minStr, secStr));
    }

    private void showViewState(boolean isNormal) {
        tvTimer.setBackground(isNormal ? normalDrawable : warningDrawable);
        tvTimer.setTextColor(isNormal
                ? ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color)
                : ContextCompat.getColor(context, R.color.bjysc_live_white));
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        RxUtils.dispose(disposableOfTimerStart);
        RxUtils.dispose(disposableOfInterval);
        if(soundPool != null) {
            soundPool.release();
        }
    }
}
