package com.baijiayun.groupclassui.window.toolbox.timer;

import android.content.Context;
import android.media.SoundPool;
import android.text.TextUtils;
import android.text.TextWatcher;
import android.view.View;
import android.widget.EditText;
import android.widget.TextView;

import androidx.core.content.ContextCompat;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.bjyutils.widgets.SimpleTextWatcher;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.window.BaseTitledWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livecore.models.LPTimerModel;

import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;


public class TeacherTimerWindow extends BaseTitledWindow {
    private CompositeDisposable disposables;
    private Disposable timeDisposable;
    private boolean isPublish;
    private long remainSeconds;
    private long timeDuration;
    private EditText etMinHigh;
    private EditText etMinLow;
    private EditText etSecondHigh;
    private EditText etSecondLow;
    private TextView tvPublish;

    private long[] lastTimerSetting;
    private SoundPool soundPool;

    public TeacherTimerWindow(Context context) {
        super(context);
        init(context);
    }

    public TeacherTimerWindow(Context context, LPTimerModel lpTimerModel) {
        super(context);
        this.timeDuration = lpTimerModel.value.duration;
        init(context);
    }

    private void init(Context context) {
        title(context.getResources().getString(R.string.bjysc_tool_box_timer))
                .showClose(true).showFull(false).showSync(false).showScaleIcon(false);
        disposables = new CompositeDisposable();
        tvPublish = $.id(R.id.tv_publish).view();
        tvPublish.setSelected(true);
        etMinHigh = $.id(R.id.et_min_high).view();
        etMinLow = $.id(R.id.et_min_low).view();
        etSecondHigh = $.id(R.id.et_second_high).view();
        etSecondLow = $.id(R.id.et_second_low).view();
        etMinLow.setText(String.valueOf(5));
        etMinHigh.addTextChangedListener(textWatcher);
        etMinLow.addTextChangedListener(textWatcher);
        etSecondHigh.addTextChangedListener(textWatcher);
        etSecondLow.addTextChangedListener(textWatcher);
        ((TextView)$.id(R.id.tv_risk).view()).setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));

        disposables.add(iRouter.getLiveRoom().getToolBoxVM().getObservableOfTimerStart()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpTimerModel -> {
                    if (isAdmin() && !isSelf(lpTimerModel.userId)) {
                        timeDuration = remainSeconds = lpTimerModel.value.duration;
                        publish(timeDuration);
                    }
                }));
        disposables.add(iRouter.getLiveRoom().getToolBoxVM().getObservableOfTimerRevoke()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(lpDataUserModel -> {
                    if (isAdmin() && !isSelf(lpDataUserModel.userId)) {
                        revoke();
                    }
                }));

        ivClose.setOnClickListener(v -> {
            sendRequestComponentDestroy();
            sendRequestTimerRevoke();
            closeSelf();
        });

        disposables.add($.id(R.id.tv_publish).clicks().throttleFirst(500, TimeUnit.MILLISECONDS)
                .observeOn(AndroidSchedulers.mainThread()).subscribe(o -> {
                    if (!tvPublish.isEnabled()) {
                        showToastMessage("请填入时间，时间不能为空");
                        return;
                    }
                    if (tvPublish.isSelected()) {
                        saveTimerSetting();
                        timeDuration = getTimerSeconds();
                        publish(timeDuration);
                        sendRequestTimerStart(remainSeconds);
                    } else {
                        sendRequestTimerRevoke();
                        revoke();
                    }
                }));
        if (timeDuration > 0) {
            publish(timeDuration);
        }
    }

    /**
     * 保存上次计时器设置的时间
     */
    private void saveTimerSetting() {
        lastTimerSetting = new long[] {
                Long.parseLong(etMinHigh.getText().toString()),
                Long.parseLong(etMinLow.getText().toString()),
                Long.parseLong(etSecondHigh.getText().toString()) * 10,
                Long.parseLong(etSecondLow.getText().toString())
        };
    }

    public long[] getLastTimerSetting() {
        return lastTimerSetting;
    }

    public void recoverTimerSetting(long[] lastTimerSetting) {
        if (lastTimerSetting == null || lastTimerSetting.length != 4) {
            return;
        }
        etMinHigh.setText(String.valueOf(lastTimerSetting[0]));
        etMinLow.setText(String.valueOf(lastTimerSetting[1]));
        etSecondHigh.setText(String.valueOf(lastTimerSetting[2]));
        etSecondLow.setText(String.valueOf(lastTimerSetting[3]));
    }

    private boolean isSelf(String userId) {
        return TextUtils.equals(iRouter.getLiveRoom().getCurrentUser().getUserId(), userId);
    }

    private void closeSelf() {
        iRouter.getSubjectByKey(EventKey.TeacherTimerWindow).onNext(false);
    }

    private void publish(long duration) {
        remainSeconds = timeDuration = duration;
        showViewState(remainSeconds != 60);
        isPublish = true;
        setTimer(timeDuration);
        showEditable(false);
        tvPublish.setSelected(false);
        tvPublish.setText(getString(R.string.bjysc_revoke));
        startTime();
    }

    private void revoke() {
        RxUtils.dispose(timeDisposable);
        isPublish = false;
        setTimer(timeDuration);
        showEditable(true);
        showViewState(true);
        tvPublish.setSelected(true);
        tvPublish.setText(getString(R.string.bjysc_timer_publish));
    }

    private void startTime() {
        RxUtils.dispose(timeDisposable);
        timeDisposable = Observable.interval(1, 1, TimeUnit.SECONDS)
                .observeOn(AndroidSchedulers.mainThread()).subscribe(aLong -> {
                    if (remainSeconds == 0) {
                        if(soundPool != null) {
                            soundPool.release();
                        }
                        soundPool = UtilsKt.playAudio(context, R.raw.timer_end);
                        RxUtils.dispose(timeDisposable);
                        showViewState(false);
                    }
                    if (remainSeconds > 0) {
                        remainSeconds--;
                        setTimer(remainSeconds);
                        if (remainSeconds <= 60 && timeDuration >= 60) {
                            showViewState(false);
                        }
                    }
                });
    }

    private boolean isLegal() {
        return getTimerSeconds() > 0;
    }

    private void setTimer(long remainSeconds) {
        long min = remainSeconds / 60;
        long second = remainSeconds % 60;
        etMinHigh.setText(String.valueOf(min / 10));
        etMinLow.setText(String.valueOf(min % 10));
        etSecondHigh.setText(String.valueOf(second / 10));
        etSecondLow.setText(String.valueOf(second % 10));
    }

    private void showEditable(boolean canEditable) {
        etMinHigh.setFocusable(canEditable);
        etMinLow.setFocusable(canEditable);
        etSecondHigh.setFocusable(canEditable);
        etSecondLow.setFocusable(canEditable);
        etMinHigh.setFocusableInTouchMode(canEditable);
        etMinLow.setFocusableInTouchMode(canEditable);
        etSecondHigh.setFocusableInTouchMode(canEditable);
        etSecondLow.setFocusableInTouchMode(canEditable);
        etMinHigh.setCursorVisible(canEditable);
        etMinLow.setCursorVisible(canEditable);
        etSecondHigh.setCursorVisible(canEditable);
        etSecondLow.setCursorVisible(canEditable);
    }

    private void showViewState(boolean isWarning) {
        etMinHigh.setEnabled(isWarning);
        etMinLow.setEnabled(isWarning);
        etSecondHigh.setEnabled(isWarning);
        etSecondLow.setEnabled(isWarning);
        if(isWarning) {
            ((TextView)$.id(R.id.tv_risk).view()).setTextColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_window_main_text_color));
        } else {
            ((TextView)$.id(R.id.tv_risk).view()).setTextColor(ContextCompat.getColor(context, R.color.bjysc_warning_color));
        }
    }

    private void sendRequestTimerStart(long seconds) {
        LPTimerModel timerModel = new LPTimerModel();
        LPTimerModel.ValueBean valueBean = new LPTimerModel.ValueBean();
        valueBean.duration = seconds;
        timerModel.value = valueBean;
        iRouter.getLiveRoom().getToolBoxVM().requestTimerStart(timerModel);
    }

    private void sendRequestTimerRevoke() {
        iRouter.getLiveRoom().getToolBoxVM().requestTimerRevoke();
    }

    private void sendRequestComponentDestroy() {
        iRouter.getLiveRoom().getToolBoxVM().requestComponentDestroy(LPConstants.ComponentType.Timer);
    }

    private long getTimerSeconds() {
        long min = 0, second = 0;
        long seconds;
        try {
            min = Long.parseLong(etMinHigh.getText().toString()) * 10 +
                    Long.parseLong(etMinLow.getText().toString());
            second = Long.parseLong(etSecondHigh.getText().toString()) * 10 +
                    Long.parseLong(etSecondLow.getText().toString());
        } catch (NumberFormatException e) {
            e.printStackTrace();
        }
        seconds = min * 60 + second;
        return seconds;
    }

    private TextWatcher textWatcher = new SimpleTextWatcher() {
        @Override
        public void onTextChanged(CharSequence s, int start, int before, int count) {
            if (isPublish) {
                return;
            }
            if (TextUtils.isEmpty(s)) {
                tvPublish.setSelected(false);
                tvPublish.setEnabled(false);
            } else {
                boolean isEmpty = TextUtils.isEmpty(etMinHigh.getText()) || TextUtils.isEmpty(etMinLow.getText())
                        || TextUtils.isEmpty(etSecondHigh.getText()) || !isLegal();
                tvPublish.setSelected(!isEmpty);
                tvPublish.setEnabled(!isEmpty);
            }
        }
    };

    @Override
    public void onDestroy() {
        if (disposables != null) {
            disposables.dispose();
        }
        RxUtils.dispose(timeDisposable);
        etMinHigh.removeTextChangedListener(textWatcher);
        etMinLow.removeTextChangedListener(textWatcher);
        etSecondHigh.removeTextChangedListener(textWatcher);
        etSecondLow.removeTextChangedListener(textWatcher);
        super.onDestroy();
        if(soundPool != null) {
            soundPool.release();
            soundPool = null;
        }
    }


    @Override
    protected void onCreateContentView(Context context) {
        View.inflate(context, R.layout.bjy_group_window_timer_teacher_content, rlContentContainer);
    }
}
