package com.baijiayun.groupclassui.window.video;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Color;
import android.util.Log;
import android.util.TypedValue;
import android.view.GestureDetector;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.view.GestureDetectorCompat;

import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.screen.DisplayUtils;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.layer.SeatLayer;
import com.baijiayun.groupclassui.util.FECompatUtil;
import com.baijiayun.groupclassui.window.BaseTitledWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livecore.models.LPPlayerViewUpdateModel;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.liveuibase.utils.ShadowUtil;

import java.lang.ref.WeakReference;

/**
 * 视频层包裹容器
 * Created by Dujuncan on 2018/12/7
 *
 * 新建一个VideoContainerWindow,专门用来放VideoWindow.
 * VideoWindow以后只负责视频层和图片状态层.
 * 初始化时判断是本地视频还是远程流视频，显示对应的视频层view.
 * 进入黑板时，将座位席上的VideoWindow传给黑板中的新VideoContainerWindow.
 * VideoWindow负责监听一些状态的变化。
 */
public class VideoContainerWindow extends BaseTitledWindow {
    private final String TAG = VideoWindow.class.getCanonicalName();
    private FrameLayout flContainer;
    private VideoWindow videoWindow;
    private IMediaModel iMediaModel;

    private WeakReference<SeatLayer> seatLayerWeakReference;
    // 是否已进入黑板
    private boolean isJoinBlackboard;
    // 按住放大比例
    public static final float PRESS_SCALE_RATIO = 1.2f;
    // 进入黑板的比例PRESS_SCALE_RATIO*JOIN_BLACKBOARD_RATIO
    public static final float JOIN_BLACKBOARD_RATIO = 1.7f;
    public static final float ONE2ONE_PRESS_SCALE_RATIO = 1.1f;
    public static final float ONE2ONE_JOIN_SCALE_RATIO_PAD = 1.3f;
    private float startScale;
    private float endScale;

    //座位席初始化,由container内部自己创建VideoWindow.
    //座位席上的逻辑
    //拖入黑板时初始化,由外部传入VideoWindow.
    //进入黑板的逻辑
    public VideoContainerWindow(Context context, @NonNull VideoWindow videoWindow) {
        this(context, videoWindow, true);
    }

    public VideoContainerWindow(Context context, @NonNull VideoWindow videoWindow, boolean isJoinBlackboard) {
        super(context);
        this.videoWindow = videoWindow;
        this.isJoinBlackboard = isJoinBlackboard;
        showTopContainer(false);
        if (isJoinBlackboard) {
            showScaleIcon(isAdmin());
            videoWindow.setJoinBlackboard(true);
            boundView = ShadowUtil.setViewBoundShadow(view, UtilsKt.getDp(8),
                    Color.parseColor("#000F0F0F"), Color.parseColor("#0C0F0F0F"),
                    ShadowUtil.Create());
        } else {
            showScaleIcon(false);
        }
        initView();
        initListener();
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) $.id(R.id.window_video_container_to_seat_placeholder).view().getLayoutParams();
            layoutParams.matchConstraintPercentWidth = 0.5f;
            layoutParams.matchConstraintPercentHeight = 0.5f;
            startScale = ONE2ONE_PRESS_SCALE_RATIO;
            endScale = ONE2ONE_PRESS_SCALE_RATIO;
            if (DisplayUtils.isPad(context)) {
                endScale = ONE2ONE_JOIN_SCALE_RATIO_PAD;
            }
            ((TextView)$.id(R.id.window_video_resume_button).view()).setTextSize(TypedValue.COMPLEX_UNIT_SP,12);
        } else {
            startScale = PRESS_SCALE_RATIO;
            endScale = JOIN_BLACKBOARD_RATIO;
        }
    }

    @Override
    protected void onCreateContentView(Context context) {
        setNeedShowShadow(false);
        dragScaleRelativeLayout.setForceNotConsume(true);
        LayoutInflater.from(context).inflate(R.layout.bjy_group_window_video_container, rlContentContainer);
    }

    @Override
    public void maxInSync() {
        super.maxInSync();
        ivMaxSync.setVisibility(View.GONE);
        dragScaleRelativeLayout.setVisibility(View.VISIBLE);
        showBoundView(false);
    }

    @Override
    public void resetInSync() {
        super.resetInSync();
        ivMaxSync.setVisibility(View.GONE);
        dragScaleRelativeLayout.setVisibility(View.VISIBLE);
        showBoundView(true);
    }

    @Override
    public void maxInFull() {
        super.maxInFull();
        ivMaxFull.setVisibility(View.GONE);
        dragScaleRelativeLayout.setVisibility(View.VISIBLE);
        showBoundView(false);
    }

    @Override
    public void resetInFull() {
        super.resetInFull();
        ivMaxFull.setVisibility(View.GONE);
        dragScaleRelativeLayout.setVisibility(View.VISIBLE);
        if (!isMaxInSync) {
            showBoundView(true);
        }
    }

    @Override
    public void minInSync() {
        dragScaleRelativeLayout.setVisibility(View.GONE);
        isMinInSync = true;
        if (isMaxInFull) {
            iRouter.getSubjectByKey(EventKey.SetFullScreenWindowOff).onNext(this);
        }
    }

    private void showBoundView(boolean show) {
        if (boundView != null && boundView.getShadowView() != null) {
            boundView.getShadowView().setVisibility(show ? View.VISIBLE : View.GONE);
        }
    }

    @Override
    public void resetMinInSync() {
        dragScaleRelativeLayout.setVisibility(View.VISIBLE);
        isMinInSync = false;
    }

    private void initView() {
        if (videoWindow.getView() == null) {
            return;
        }
        flContainer = $.id(R.id.window_video_container_fl).view();
        if (videoWindow.getView() != null && videoWindow.getView().getParent() != null) {
            ((ViewGroup) videoWindow.getView().getParent()).removeAllViews();
        }
        flContainer.addView(videoWindow.getView(), new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
    }

    @SuppressLint("ClickableViewAccessibility")
    private void initListener() {
        setOnSingleTapListener(e -> {
            if (!isJoinBlackboard) {
                return;
            }
            if (videoWindow != null) {
                if (videoWindow.isShowMenu() && isJoinedBlackboard() && videoWindow.getView() != null) {
                    videoWindow.showMenu((int) e.getRawX() + UtilsKt.getDp(40), (int) e.getRawY() + UtilsKt.getDp(2));
                }
            }
            if (!isAdmin()) {
                return;
            }
            iRouter.getSubjectByKey(EventKey.StickVideoWindow).onNext(getPlayerPosition());
        });

        setOnWindowRepositionListener(new OnWindowRepositionListener() {
            @Override
            public void onWindowMove(int x, int y) {
                iRouter.getSubjectByKey(EventKey.MoveVideoWindow).onNext(getPlayerPosition());
            }

            @Override
            public void OnWindowScale(int newWidth, int newHeight) {
                iRouter.getSubjectByKey(EventKey.MoveVideoWindow).onNext(getPlayerPosition());
            }
        });

        ivClose.setOnClickListener(v -> {
            if (!isAdmin()) {
                return;
            }
            iRouter.getSubjectByKey(EventKey.CloseVideoWindow).onNext(videoWindow.getMediaId());
        });

        $.id(R.id.window_video_resume_button).view().setOnClickListener(v ->  {
            iRouter.getSubjectByKey(EventKey.RemoveAllBlackboardVideoWindow).onNext(false);
        });

        $.id(R.id.window_video_container_back_to_seat).clicked(v -> {
            // 老师将自己的视频移回座位
            if (!isAdmin()) {
                return;
            }
            iRouter.getSubjectByKey(EventKey.CloseVideoWindow).onNext(videoWindow.getMediaId());
        });

        if (!isJoinBlackboard) {
            GestureListener gestureListener = new GestureListener(this);
            GestureDetectorCompat gestureDetector = new GestureDetectorCompat(getView().getContext(), gestureListener);
            getView().setOnTouchListener((v, event) -> {
                if (seatLayerWeakReference == null) {
                    return true;
                }
                SeatLayer seatLayer = seatLayerWeakReference.get();
                switch (event.getAction()) {
                    case MotionEvent.ACTION_MOVE:
                        if (!gestureListener.isLongPress) {
                            return true;
                        }
                        if (seatLayer.getDragLayer() != null) {
                            seatLayer.getDragLayer().scrollVideo(event.getRawX(), event.getRawY());
                        }
                        break;
                    case MotionEvent.ACTION_UP:
                        if (!gestureListener.isLongPress) {
                            break;
                        }
                        gestureListener.isLongPress = false;
                        if (seatLayer.getDragLayer() != null) {
                            seatLayer.getDragLayer().scrollEndEnlarge(endScale);
                        }
                        break;
                    default:
                        break;
                }
                if (seatLayer.isBoardLayout()) {
                    gestureDetector.onTouchEvent(event);
                }
                return true;
            });
        }
    }

    public VideoWindow getVideoWindow() {
        return videoWindow;
    }

    //返回座位席
    public void backToSeat(VideoWindow videoWindow) {
        if (videoWindow.getView() == null) {
            return;
        }
        this.videoWindow = videoWindow;
        if (videoWindow.getView().getParent() != null) {
            ((ViewGroup) videoWindow.getView().getParent()).removeAllViews();
        }
        flContainer.addView(videoWindow.getView(), new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        videoWindow.setJoinBlackboard(false);
        if (isAdmin()) {
            $.id(R.id.window_video_container_back_to_seat).visibility(View.GONE);
            $.id(R.id.window_video_container_to_seat_placeholder).visibility(View.GONE);
        }
        $.id(R.id.window_video_container_seat_name).gone();
    }

    public void showPlaceholder() {
        if (iMediaModel.getUser().getUserId().equals(iRouter.getLiveRoom().getCurrentUser().getUserId()) && iMediaModel.getUser().getType() == LPConstants.LPUserType.Teacher) {
            $.id(R.id.window_video_container_back_to_seat).visibility(View.VISIBLE);
        } else {
            $.id(R.id.window_video_container_to_seat_placeholder).visibility(View.VISIBLE);
            $.id(R.id.window_video_container_seat_name).visible();
        }
    }

    public void initMediaModel(IMediaModel iMediaModel) {
        videoWindow.setMediaModel(iMediaModel);
        this.iMediaModel = iMediaModel;
        $.id(R.id.window_video_container_seat_name).text(context.getString(R.string.bjysc_seat_placeholder_tip, getEncodePhoneNumber(iMediaModel.getUser().getName())));
    }

    public void setMediaModel(IMediaModel iMediaModel) {
        this.iMediaModel = iMediaModel;
    }

    public IMediaModel getMediaModel() {
        return iMediaModel;
    }

    public void bindToSeat(SeatLayer seatLayer) {
        seatLayerWeakReference = new WeakReference<>(seatLayer);
    }

    public boolean isJoinedBlackboard() {
        return videoWindow != null && videoWindow.isJoinBlackboard();
    }

    public void showResumeButton() {
        if($.id(R.id.window_video_resume_button).view().getVisibility() != View.VISIBLE) {
            $.id(R.id.window_video_resume_button).view().setVisibility(View.VISIBLE);
        }
    }

    public void hideResumeButton() {
        if($.id(R.id.window_video_resume_button).view().getVisibility() != View.GONE) {
            $.id(R.id.window_video_resume_button).view().setVisibility(View.GONE);
        }
    }

    @Override
    public void onDestroy() {
        if (videoWindow != null) {
            videoWindow.onDestroy();
        }
        videoWindow = null;
        super.onDestroy();
    }

    private class GestureListener extends GestureDetector.SimpleOnGestureListener {
        WeakReference<VideoContainerWindow> containerWindowWeakReference;
        boolean isLongPress = false;

        GestureListener(VideoContainerWindow videoContainerWindow) {
            containerWindowWeakReference = new WeakReference<>(videoContainerWindow);
        }

        @Override
        public boolean onSingleTapConfirmed(MotionEvent e) {
            if (videoWindow != null) {
                if (videoWindow.isShowMenu() && !isJoinedBlackboard()) {
                    videoWindow.showMenu((int) e.getRawX()
                            + UtilsKt.getDp(40)
                            , (int) e.getRawY() + UtilsKt.getDp(2));
                }
            }
            return true;
        }

        @Override
        public void onLongPress(MotionEvent e) {
            super.onLongPress(e);
            Log.d(TAG, "onLongPress: start....");
            if (videoWindow != null && videoWindow.isJoinBlackboard()) {
                return;
            }
            // 助教也不允许拖动
            if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student
                    || iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                return;
            }
            //长按放大
            isLongPress = true;
            SeatLayer seatLayer = seatLayerWeakReference.get();
            if (seatLayer.getDragLayer() != null) {
                VideoContainerWindow videoContainerWindow = containerWindowWeakReference.get();
                seatLayer.getDragLayer().startEnlarge(startScale, videoContainerWindow, e.getRawX(), e.getRawY());
            }
        }
    }

    @Override
    protected LPPlayerViewUpdateModel.PlayerPosition getPlayerPosition() {
        LPPlayerViewUpdateModel.PlayerPosition playerPosition = super.getPlayerPosition();
        playerPosition.id = FECompatUtil.getIOSMediaIdFromMediaId(videoWindow.getMediaId());
        return playerPosition;
    }
}
