package com.baijiayun.groupclassui.window.video;

import static com.baijiayun.bjyutils.kt.UtilsKt.getEncodePhoneNumber;

import android.Manifest;
import android.animation.ValueAnimator;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;
import android.view.GestureDetector;
import android.view.LayoutInflater;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewStub;
import android.view.animation.LinearInterpolator;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.IntDef;
import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.view.GestureDetectorCompat;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.OnLifecycleEvent;

import com.baijiahulian.common.permission.AppPermissions;
import com.baijiahulian.common.permission.PermissionTipsView;
import com.baijiayun.bjyrtcengine.Defines.BJYRtcCommon;
import com.baijiayun.bjyutils.drawable.DrawableBuilder;
import com.baijiayun.bjyutils.kt.UtilsKt;
import com.baijiayun.bjyutils.log.LPLogger;
import com.baijiayun.bjyutils.reactivex.RxUtils;
import com.baijiayun.glide.Glide;
import com.baijiayun.glide.request.target.SimpleTarget;
import com.baijiayun.glide.request.target.Target;
import com.baijiayun.glide.request.transition.Transition;
import com.baijiayun.groupclassui.BuildConfig;
import com.baijiayun.groupclassui.R;
import com.baijiayun.groupclassui.global.EventCode;
import com.baijiayun.groupclassui.global.EventKey;
import com.baijiayun.groupclassui.global.IRouter;
import com.baijiayun.groupclassui.model.VideoMenuControlModel;
import com.baijiayun.groupclassui.window.InteractiveBaseWindow;
import com.baijiayun.livebase.context.LPConstants;
import com.baijiayun.livebase.models.GroupItem;
import com.baijiayun.livebase.models.LPUserModel;
import com.baijiayun.livebase.models.imodels.IUserModel;
import com.baijiayun.livebase.utils.LimitedQueue;
import com.baijiayun.livebase.utils.ThemeDataUtil;
import com.baijiayun.livebase.widgets.dialog.CommonDialog;
import com.baijiayun.livecore.models.imodels.IMediaModel;
import com.baijiayun.livecore.models.launch.LPAwardConfig;
import com.baijiayun.livecore.viewmodels.debug.IDebugLink;
import com.baijiayun.livecore.wrapper.LPPlayer;
import com.baijiayun.livecore.wrapper.LPRecorder;
import com.baijiayun.livecore.wrapper.impl.LPCameraView;
import com.baijiayun.livecore.wrapper.impl.LPVideoView;
import com.baijiayun.livecore.wrapper.listener.LPPlayerListener;
import com.baijiayun.liveuibase.utils.BaseUtilsKt;
import com.baijiayun.liveuibase.utils.PermissionTipBuilderFactory;
import com.baijiayun.liveuibase.widgets.dialog.AwardPopupWindow;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.ref.WeakReference;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

import io.reactivex.Observable;
import io.reactivex.ObservableSource;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.disposables.CompositeDisposable;
import io.reactivex.disposables.Disposable;
import io.reactivex.functions.Function;
import io.reactivex.schedulers.Schedulers;
import kotlin.Pair;


/**
 * 用户视频区域
 * Created by Dujuncan on 2018/9/23.
 * 画廊布局、1v1模板、黑板区域拉大流，坐席区域拉小流
 */
public class VideoWindow extends InteractiveBaseWindow implements Cloneable {
    private final String TAG = VideoWindow.class.getCanonicalName();
    protected LPPlayer mPlayer;
    protected LPRecorder mRecorder;
    protected AppCompatImageView ivPlaceholder, ivPlaceholderOnlyAudio, ivMenu, ivSpeak, ivSignal, ivHandsUp, ivDrawAuth, ivPPTAuth;
    protected LPCameraView cameraView;
    protected LPVideoView videoView;
    private View llHandsUpControl;
    private LinearLayout awardContainer;
    private ImageView awardIcon;
    private TextView awardCount;
    protected ViewGroup placeHolderContainer;
    private ViewGroup clBottom;
    private TextView resolutionTest;
    private ViewStub screenShareStub;
    private View screenShareContainer;
    private ImageView screenShareIv;

    protected IMediaModel mediaModel;//用户信息
    protected CompositeDisposable disposables;
    private boolean isJoinBlackboard;//是否已进入黑板
    private boolean isLocalVideo;//是本地视频还是远程流
    private boolean isOverlay = false;//默认置顶
    protected boolean isAudioOn = false;//音频是否打开
    protected boolean isVideoOn = false;//视频是否打开,默认进教室视频就是开启状态
    private boolean isDrawAuthOn = false; // 是否已授权画笔
    private boolean isPPTAuthOn = false; // 是否授权PPT
    private boolean isAssistCameraOn = false; // 是否授权辅助摄像头
    private boolean isScreenShareOn = false; // 是否授权屏幕分享
    private boolean isShowMenu;//是否显示menu
    private String videoCloseBackGroundUrl;
    public int currentX, currentY;//点击位置在videoView中的位置
    private LimitedQueue<Double> upLossRateQueue, downLinkLossRateQueue;
    // 本地上行丢包率
    private double recorderAverageUpLinkLossRate;
    private List<Integer> packetLossRateLevel;
    private LoadingListener loadingListener;
    private Disposable timer;
    private boolean isActive;
    private int videoFit;
    // 切后台之前的音视频状态
    protected boolean isAudioOnLast, isVideoOnLast;
    public static final int REMOTE_VIDEO_AUTO_OPEN = 0;
    // 配置只看自己和老师
    public static final int REMOTE_VIDEO_AUTO_CLOSE = 1;
    // 手动打开视频
    public static final int REMOTE_VIDEO_HAND_OPEN = 2;
    // 手动关闭视频
    public static final int REMOTE_VIDEO_HAND_CLOSE = 3;
    // 音视频都关闭
    public static final int REMOTE_VIDEO_AUDIO_CLOSE = 4;
    // 远端视频状态，对应只看老师，手动关闭-打开等状态
    protected @RemoteVideoStatus
    int remoteVideoStatus;
    private boolean enablePullAudioOnly = false;
    protected LPConstants.MediaState videoState = LPConstants.MediaState.Normal, audioState = LPConstants.MediaState.Normal;
    private boolean disablePullStream;
    private boolean showWindow = true;
    private GroupItem groupItemModel;
    /**
     * 只显示在黑板区的视频
     */
    private boolean isOnlyInBlackboard = true;


    @Retention(RetentionPolicy.SOURCE)
    @IntDef({REMOTE_VIDEO_AUTO_OPEN, REMOTE_VIDEO_AUTO_CLOSE, REMOTE_VIDEO_HAND_OPEN, REMOTE_VIDEO_HAND_CLOSE, REMOTE_VIDEO_AUDIO_CLOSE})
    public @interface RemoteVideoStatus {
    }

    //占位图target
    private Target<Drawable> simpleTarget;
    private BJYRtcCommon.DualStreamType defaultRemoteStreamType = BJYRtcCommon.DualStreamType.VIDEO_STREAM_LOW;

    // 自习室和普通小班课所创建的window不同
    public static VideoWindow newInstance(Context context, IRouter router) {
        if (router.getLiveRoom().getStudyRoomVM().isStudyRoom()) {
            return new StudyVideoWindow(context);
        } else {
            return new VideoWindow(context);
        }
    }

    protected VideoWindow(Context context) {
        super(context);
        mPlayer = iRouter.getLiveRoom().getPlayer();
        mRecorder = iRouter.getLiveRoom().getRecorder();
        view.setOnTouchListener(null);
        cameraView = $.id(R.id.window_video_camera_view).view();
        cameraView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
        videoView = $.id(R.id.window_video_video_view).view();
        videoView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
        placeHolderContainer = $.id(R.id.window_video_placerHolder_container).view();
        ivMenu = $.id(R.id.window_video_menu).view();
        ivPlaceholder = $.id(R.id.iv_video_placeholder).view();
        ivPlaceholderOnlyAudio = $.id(R.id.iv_video_placeholder_only_audio).view();
        ivSpeak = $.id(R.id.window_video_mic).view();
        ivSignal = $.id(R.id.window_video_signal).view();
        ivHandsUp = $.id(R.id.window_video_hands_up).view();
        llHandsUpControl = $.id(R.id.window_video_hands_up_control).view();
        ivDrawAuth = $.id(R.id.window_video_draw_auth).view();
        ivPPTAuth = $.id(R.id.window_video_ppt_auth).view();
        awardContainer = $.id(R.id.window_video_awards).view();
        awardIcon = $.id(R.id.window_video_awards_icon).view();
        awardCount = $.id(R.id.window_video_awards_count).view();
        clBottom = $.id(R.id.cl_bottom).view();
        resolutionTest = $.id(R.id.tv_test_resolution).view();
        screenShareStub = $.id(R.id.screen_share_stub).view();
        resolutionTest.setVisibility(BuildConfig.LP_SDK_RELEASE ? View.GONE : View.VISIBLE);
        changePlaceholderState(false);
        ivSpeak.setImageLevel(0);
        ivSignal.setImageLevel(0);
        isActive = true;
        $.id(R.id.window_video_agree).view().setBackground(new DrawableBuilder()
                .solidColor(ThemeDataUtil.getColorFromThemeConfigByAttrId(context, R.attr.base_theme_live_product_color))
                .cornerRadius(UtilsKt.getDp(2))
                .build());
        initListener();
        subscribe();
    }

    @SuppressLint("InflateParams")
    @Override
    protected View onCreateView(Context context) {
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE && !iRouter.getLiveRoom().enableFixChatPanel1v1()) {
            return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_video_one, null);
        }
        return LayoutInflater.from(context).inflate(R.layout.bjy_group_window_video, null);
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_RESUME)
    protected void onResume() {
        isActive = true;
        if (!isLocalVideo() || mRecorder == null) {
            return;
        }
        timer = Observable.timer(300, TimeUnit.MILLISECONDS)
                .filter(aLong -> mRecorder != null)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aLong -> {
                    if (isVideoOnLast && isAudioOnLast) {
                        attachAVideo();
                    } else if (isVideoOnLast) {
                        attachVideo();
                    } else if (isAudioOnLast) {
                        attachAudio();
                    }
                    isAudioOnLast = false;
                    isVideoOnLast = false;
                }, throwable -> {
                });
    }

    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    protected void onStop() {
        isActive = false;
        if (!isLocalVideo() || mRecorder == null) {
            return;
        }
        // 正在屏幕分享不停止推流
        if (mRecorder.isScreenSharing()) {
            return;
        }
        LPLogger.d(TAG, "onStop() 停止推流");

        if (mRecorder.isVideoAttached()) {
            isVideoOnLast = true;
        }
        if (mRecorder.isAudioAttached()) {
            isAudioOnLast = true;
        }
        mRecorder.stopPublishing();
    }


    public int[] getWindowPosition() {
        int[] locations = new int[2];
        if (this.getView() != null) {
            this.getView().getLocationOnScreen(locations);
        }
        return locations;
    }

    void showMenu(int currX, int currY) {
        if (ivMenu.isSelected()) {
            //菜单展示中，关闭
            iRouter.getSubjectByKey(EventKey.ShowVideoMenu)
                    .onNext(new VideoMenuControlModel());
            setMenuState(false);
        } else {
            //显示菜单
            this.currentX = currX - getWindowPosition()[0];
            this.currentY = currY - getWindowPosition()[1];
            iRouter.getSubjectByKey(EventKey.ShowVideoMenu)
                    .onNext(new VideoMenuControlModel(VideoWindow.this, currX, currY, true));
            setMenuState(true);
        }
    }

    /**
     * 这里这些可以抽成接口
     * 子类都实现=。=
     *
     * @return
     */
    public boolean isGalleryLayout() {
        return iRouter.getLiveRoom().getCurrentRoomLayout() == LPConstants.RoomLayoutMode.GALLERY;
    }

    public boolean isTouchAble() {
        return iRouter.getLiveRoom().getCurrentRoomLayout() == LPConstants.RoomLayoutMode.GALLERY;
    }

    public boolean isSingleTutorHelping() {
        return false;
    }


    public boolean isSelfSingleTutorHelping() {
        return false;
    }

    /**
     * 单人辅导是否显示菜单
     *
     * @return
     */
    public boolean isShowSingleTutorMenu() {
        //组内看组内，组外看组外
        return (isSelfSingleTutorHelping() && isSingleTutorHelping()) ||
                (!isSelfSingleTutorHelping() && !isSingleTutorHelping());
    }

    @SuppressLint("ClickableViewAccessibility")
    protected void initListener() {
        //设置画廊布局的点击事件
        GestureDetectorCompat gestureDetector = new GestureDetectorCompat(getView().getContext(), new GestureDetector.SimpleOnGestureListener() {
            @Override
            public boolean onSingleTapConfirmed(MotionEvent e) {
                if (isShowMenu()) {
                    showMenu((int) e.getRawX() + UtilsKt.getDp(40),
                            (int) e.getRawY() + UtilsKt.getDp(2));
                }
                return super.onSingleTapConfirmed(e);
            }
        });
        getView().setOnTouchListener(((v, event) -> {
            gestureDetector.onTouchEvent(event);
            return isTouchAble();
        }));
        $.id(R.id.window_video_hands_up).clicked(v -> {
            llHandsUpControl.setVisibility(View.VISIBLE);
            ivHandsUp.setVisibility(View.GONE);
        }).id(R.id.window_video_agree
        ).clicked(v -> controlHansUp(getUserId(), true)).id(R.id.window_video_disagree
        ).clicked(v -> controlHansUp(getUserId(), false)).id(R.id.window_video_mic
        ).clicked(v -> {
            if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                return;
            }
            if (isLocalVideo()) {
                if (isAudioOn() && mRecorder.isAudioAttached()) {
                    iRouter.getLiveRoom().getRecorder().detachAudio();
                } else {
                    iRouter.getLiveRoom().getRecorder().attachAudio();
                }
            } else {
                if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant && getUserModel().getType() == LPConstants.LPUserType.Teacher) {
                    return;
                }
                iRouter.getLiveRoom().getSpeakQueueVM().controlRemoteUser(getUserId(), isVideoOn(), !isAudioOn());
            }
        });
    }

    protected void subscribe() {
        packetLossRateLevel = iRouter.getLiveRoom().getPartnerConfig().packetLossRate.packetLossRateLevel;

        int packetLossDuration = iRouter.getLiveRoom().getPartnerConfig().packetLossDuration;
        upLossRateQueue = new LimitedQueue<>(packetLossDuration);
        downLinkLossRateQueue = new LimitedQueue<>(packetLossDuration);

        disposables = new CompositeDisposable();
        disposables.add(iRouter.getLiveRoom().getObservableOfClassStart()
                .filter(integer -> enablePullAudioOnly)
                .subscribe(integer -> remoteVideoStatus = REMOTE_VIDEO_AUTO_CLOSE));
        //是否同意发言
        disposables.add(iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfSpeakResponse()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (!mediaModel.getUser().getUserId().equals(iMediaControlModel.getUser().getUserId())) {
                        return;
                    }
                    showHandsUpIcon(false);
                    showHandsUpControl(false);
                    //开启音频
                    changeMicLevel(LPConstants.VolumeLevel.NONE);
                }));

        //Debug信令
        disposables.add(((IDebugLink) mRecorder)
                .getObservableDebugStateUI()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(isShowLocalVideo -> {
                    if (isShowLocalVideo) {
                        mRecorder.attachVideo();
                    } else {
                        mRecorder.detachVideo();
                    }
                }));

        disposables.add(iRouter.getPublishSubjectByKey(EventCode.AttachVideo)
                .ofType(Boolean.class)
                .filter(o -> isLocalVideo)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    if (aBoolean) {
                        attachVideo();
                    } else {
                        detachVideo();
                    }
                }));

        disposables.add(iRouter.getPublishSubjectByKey(EventCode.AttachAudio)
                .ofType(Boolean.class)
                .filter(o -> isLocalVideo)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    if (aBoolean && canAttachAudio()) {
                        attachAudio();
                    } else {
                        detachAudio();
                    }
                }));

        disposables.add(iRouter.getLiveRoom().getSpeakQueueVM().getObservableOfMediaControl()
                .filter(iMediaControlModel -> isLocalVideo && isActive)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(iMediaControlModel -> {
                    if (iMediaControlModel.isVideoOn() && !mRecorder.isVideoAttached()) {
                        attachVideo();
                    } else if (!iMediaControlModel.isVideoOn() && mRecorder.isVideoAttached()) {
                        if (mRecorder.isScreenSharing()) {
                            mRecorder.stopScreenCapture();
                        } 
                        detachVideo();
                    }
                    if (iMediaControlModel.isAudioOn() && !mRecorder.isAudioAttached()) {
                        attachAudio();
                    } else if (!iMediaControlModel.isAudioOn() && mRecorder.isAudioAttached()) {
                        detachAudio();
                    }
                }));
        disposables.add(iRouter.getSubjectByKey(EventKey.ScreenShare)
                .ofType(Boolean.class)
                .filter(aBoolean -> isLocalVideo)
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(aBoolean -> {
                    // 开启屏幕共享把自己的窗口切到黑板区且放大
                    if (aBoolean) {
                        if (screenShareContainer == null) {
                            screenShareContainer = screenShareStub.inflate();
                            screenShareIv = screenShareContainer.findViewById(R.id.iv);
                            TextView screenShareTv = screenShareContainer.findViewById(R.id.tv);
                            screenShareTv.setTextSize(10);
                        }
                        screenShareContainer.setVisibility(View.VISIBLE);
                        ConstraintLayout.LayoutParams layoutParams = (ConstraintLayout.LayoutParams) screenShareIv.getLayoutParams();
                        if (view.getWidth() > view.getHeight()) {
                            layoutParams.matchConstraintPercentWidth = 1f;
                            layoutParams.matchConstraintPercentHeight = 0.5f;
                        } else {
                            layoutParams.matchConstraintPercentWidth = 0.5f;
                            layoutParams.matchConstraintPercentHeight = 1f;
                        }
                        iRouter.getSubjectByKey(EventKey.VideoWindowToBlackboardAndMax).onNext(this);
                    } else {
                        if (screenShareContainer != null) {
                            screenShareContainer.setVisibility(View.GONE);
                        }
                        // 关闭屏幕共享把窗口缩小切坐席区
                        iRouter.getSubjectByKey(EventKey.CloseVideoWindow).onNext(getMediaId());
                    }
                    if (!aBoolean && !UtilsKt.isAppForeground(context)) {
                        onStop();
                        isVideoOnLast = true;
                        iRouter.getLiveRoom().getOnlineUserVM().updateMediaState();
                    }
                }));
    }

    public void attachAVideo() {
        disposables.add(
                BaseUtilsKt.getBluetoothObservable((Activity) context, iRouter.getLiveRoom().getRoomInfo().webRTCType)
                        .flatMap((Function<Boolean, ObservableSource<Boolean>>) aBoolean -> AppPermissions.newPermissions((Activity) context)
                                .request(PermissionTipBuilderFactory.createPermissionTipsView(context, Manifest.permission.CAMERA), Manifest.permission.CAMERA))
                        .flatMap((Function<Boolean, ObservableSource<Pair<Boolean, Boolean>>>) cameraPermitted -> AppPermissions.newPermissions((Activity) context)
                                .request(PermissionTipBuilderFactory.createPermissionTipsView(context, Manifest.permission.RECORD_AUDIO), Manifest.permission.RECORD_AUDIO)
                                .flatMap((Function<Boolean, ObservableSource<Pair<Boolean, Boolean>>>) audioPermitted -> Observable.just(new Pair<>(cameraPermitted, audioPermitted))))
                        .subscribe(pair -> {
                            if (pair.getFirst() && pair.getSecond()) {
                                if (cameraView != null) {
                                    cameraView.setVisibility(View.VISIBLE);
                                    mRecorder.setPreview(cameraView);
                                }
                                mRecorder.publish();
                                mRecorder.attachAVideo();
                                isVideoOn = true;
                                isAudioOn = true;
                            } else if (pair.getFirst()) {
                                mRecorder.publish();
                                if (cameraView != null) {
                                    cameraView.setVisibility(View.VISIBLE);
                                    mRecorder.setPreview(cameraView);
                                }
                                mRecorder.attachVideo();
                                isVideoOn = true;
                            } else if (pair.getSecond()) {
                                mRecorder.publish();
                                mRecorder.attachAudio();
                                isAudioOn = true;
                            } else {
                                showPermissionNotGrantedDialog(context.getResources().getString(R.string.live_no_media_permission));
                            }
                            iRouter.getLiveRoom().getOnlineUserVM().updateMediaState();
                        }));
    }

    public void attachVideo() {
        PermissionTipsView permissionTipsView = PermissionTipBuilderFactory.createPermissionTipsView(context, Manifest.permission.CAMERA);
        disposables.add(
                BaseUtilsKt.getBluetoothObservable((Activity) context, iRouter.getLiveRoom().getRoomInfo().webRTCType)
                        .flatMap((Function<Boolean, ObservableSource<Boolean>>) aBoolean -> AppPermissions.newPermissions((Activity) context)
                                .request(permissionTipsView, Manifest.permission.CAMERA))
                        .subscribe(aBoolean -> {
                            if (aBoolean) {
                                mRecorder.publish();
                                if (cameraView != null) {
                                    cameraView.setVisibility(View.VISIBLE);
                                    mRecorder.setPreview(cameraView);
                                }
                                mRecorder.attachVideo();
                                isVideoOn = true;
                            } else {
                                showPermissionNotGrantedDialog(context.getResources().getStringArray(R.array.live_no_permission)[1]);
                            }
                            iRouter.getLiveRoom().getOnlineUserVM().updateMediaState();
                        }));
    }

    public void detachVideo() {
        mRecorder.detachVideo();
        if (!mRecorder.isAudioAttached()) {
            mRecorder.stopPublishing();
        }
    }

    public void attachAudio() {
        String[] permissions = new String[]{Manifest.permission.RECORD_AUDIO};
        PermissionTipsView permissionTipsView = PermissionTipBuilderFactory.createPermissionTipsView(context, Manifest.permission.RECORD_AUDIO);
        disposables.add(
                BaseUtilsKt.getBluetoothObservable((Activity) context, iRouter.getLiveRoom().getRoomInfo().webRTCType)
                        .flatMap((Function<Boolean, ObservableSource<Boolean>>) aBoolean -> AppPermissions.newPermissions((Activity) context)
                                .request(permissionTipsView, permissions))
                        .subscribe(aBoolean -> {
                            if (aBoolean) {
                                mRecorder.publish();
                                mRecorder.attachAudio();
                                isAudioOn = true;
                            } else {
                                showPermissionNotGrantedDialog(context.getResources().getString(R.string.live_no_media_permission));
                            }
                            iRouter.getLiveRoom().getOnlineUserVM().updateMediaState();
                        }));
    }

    public void detachAudio() {
        mRecorder.detachAudio();
        if (!mRecorder.isVideoAttached()) {
            mRecorder.stopPublishing();
        }
    }

    public void detachAVideo() {
        mRecorder.detachAVideo();
        mRecorder.stopPublishing();
    }

    protected boolean canAttachAudio() {
        return true;
    }

    /**
     * 获取media_state初始值
     */
    private void initMediaState(IMediaModel iMediaModel) {
        final IUserModel userModel = iMediaModel.getUser();
        if (userModel instanceof LPUserModel) {
            LPUserModel lpUserModel = (LPUserModel) userModel;
            if (groupItemModel != null) {
                lpUserModel.groupId = groupItemModel.id;
            }
            if (lpUserModel.audioState != null) {
                audioState = lpUserModel.audioState;
            }
            if (lpUserModel.videoState != null) {
                videoState = lpUserModel.videoState;
            }
        }
    }

    public void setMediaModel(IMediaModel iMediaModel) {
        this.mediaModel = iMediaModel;
        initMediaState(iMediaModel);
        ivPlaceholderOnlyAudio.setImageResource(isAudioOn ? R.drawable.ic_only_audio_normal : R.drawable.ic_only_audio_none);
        IUserModel iUserModel = iMediaModel.getUser();
        if (iUserModel instanceof LPUserModel) {
            videoCloseBackGroundUrl = ((LPUserModel) iUserModel).cameraCover;
        }
        setName(getEncodePhoneNumber(iMediaModel.getUser().getName()));
        isLocalVideo = iUserModel.getUserId().equals(iRouter.getLiveRoom().getCurrentUser().getUserId());
        //非自己窗口根据mp更改状态，自己窗口根据 getObservableOfCameraOn  getObservableOfMicOn更改
        if (!isLocalVideo) {
            setAudioOn(iMediaModel.isAudioOn());
            setVideoOn(iMediaModel.isVideoOn());

            if (isGalleryLayout()) {
                setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_HIGH);
            }
            // 老师、助教看别人肯定显示点赞
            if (BaseUtilsKt.isAdmin(iRouter.getLiveRoom()) && iUserModel.getType() == LPConstants.LPUserType.Student) {
                setAwardTvVisibility(View.VISIBLE);
            }
        } else {
            disposables.add(iRouter.getLiveRoom().getRecorder().getObservableOfCameraOn()
                    .filter(aBoolean -> iRouter.getLiveRoom() != null)
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(videoOn -> {
                        setVideoOn(videoOn);
                        if (videoView.getVisibility() == View.VISIBLE) {
                            videoView.setVisibility(View.GONE);
                        }
                        changePlaceHolder();
                    }));

            disposables.add(iRouter.getLiveRoom().getRecorder().getObservableOfMicOn()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(this::setAudioOn));
        }
        if (!BuildConfig.LP_SDK_RELEASE) {
            disposables.add(mPlayer.getObservableOfVideoSizeChange()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(lpVideoSizeModel -> {
                        if (lpVideoSizeModel.userId.equals(iUserModel.getUserId())) {
                            setResolutionText(lpVideoSizeModel.width, lpVideoSizeModel.height);
                        }
                    }));
        }

        disposables.add(iRouter.getLiveRoom().getOnlineUserVM().getObservableOfUserUpdate()
                .observeOn(AndroidSchedulers.mainThread())
                .filter(lpUserModel -> TextUtils.equals(lpUserModel.userId, iMediaModel.getUser().getUserId()) &&
                        !TextUtils.equals(lpUserModel.userId, iRouter.getLiveRoom().getCurrentUser().getUserId()))
                .subscribe(lpUserModel -> {
                    if (lpUserModel.audioState != null) {
                        audioState = lpUserModel.audioState;
                    }
                    if (lpUserModel.videoState != null) {
                        videoState = lpUserModel.videoState;
                    }
                    videoCloseBackGroundUrl = lpUserModel.cameraCover;
                    changePlaceholderState(ivPlaceholder.getVisibility() != View.VISIBLE);
                }));

        Disposable upPacketLossRateDisposable = iRouter.getLiveRoom().getRecorder().getObservableOfUpPacketLossRate()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(localStreamStats -> {
                    double lossRate = isVideoOn() ? localStreamStats.videoPacketsLostRateSent : localStreamStats.audioPacketsLostRateSent;
                    upLossRateQueue.add(lossRate);
                    double average = upLossRateQueue.getAverage();
                    //本地采集视频流才提示，否则只统计推流的上行丢包率
                    recorderAverageUpLinkLossRate = average;
                });

        /**
         * ATTENTION 主讲丢包广播发送太频繁，已去掉。声网暂无丢包回调
         * 丢包逻辑如下
         * 统计平均10s的丢包情况。时间间隔和丢包等级均走配置
         * 如果拉流的丢包率大于自己上行丢包的2倍，认为是拉流端的丢包过高导致，这个时候需在拉流学员对应的窗口显示文案提示
         * 关于两倍的逻辑，解释如下
         * 学生A的下行丢包=学生A的上行丢包（现在没这个数据，所以拿学生B的上行丢包等价了） + 学生B拉学生A产生的下行丢包
         */
        Disposable disposableOfDownLinkLossRate = iRouter.getLiveRoom().getPlayer().getObservableOfDownLinkLossRate()
                .filter(stringMap -> !isLocalVideo)
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(stats -> {
                    if (getUserId().equals(stats.uid)) {
                        downLinkLossRateQueue.add(isVideoOn() ? stats.receivedVideoLostRate : stats.receivedAudioLossRate);

                        double average = downLinkLossRateQueue.getAverage();
                        if (getUserModel().equals(iRouter.getLiveRoom().getPresenterUser())) {
                            handlerNetworkUI(average, false);
                        } else {
                            if (average > recorderAverageUpLinkLossRate * 2) {
                                handlerNetworkUI(average, false);
                            } else {
                                iRouter.getSubjectByKey(EventKey.NetWorkLossRateNotify).onNext(average);
                            }
                        }
                    }
                });

        //更新推流窗口网络显示
        Disposable disposableOfLocalWindow = iRouter.getSubjectByKey(EventKey.NetWorkLossRateNotify)
                .ofType(Double.class)
                .filter(aDouble -> isLocalVideo)
                .subscribe(aDouble -> handlerNetworkUI(aDouble, false));

        disposables.add(iRouter.getLiveRoom().getObservableOfRoomLayoutSwitch()
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(layoutMode -> {
                    if (layoutMode == LPConstants.RoomLayoutMode.BOARD) {
                        setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_LOW);
                    } else {
                        setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_HIGH);
                    }
                    setJoinBlackboard(isJoinBlackboard);
                }));
        disposables.add(iRouter.getSubjectByKey(EventKey.StudyRoomSwitch)
                .ofType(LPConstants.StudyRoomMode.class)
                .subscribe(studyRoomMode -> {
                    if (studyRoomMode.isGalleryLayout()) {
                        setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_HIGH);
                    } else {
                        setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_LOW);
                    }
                }));

        // 只看老师和学生
        boolean isStudent = iMediaModel.getUser().getType() != LPConstants.LPUserType.Teacher
                && iMediaModel.getUser().getType() != LPConstants.LPUserType.Assistant;
        if (!isLocalVideo && isStudent && !BaseUtilsKt.isAdmin(iRouter.getLiveRoom())) {
            // 禅道#15522
            disposables.add(iRouter.getLiveRoom().getObservableOfDisableOtherStudentVideo()
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(model -> {
                        boolean hideOtherStudentVideo = initEnablePullAudioOnly(model.isDisable()
                                || iRouter.getSubjectValueByKey(EventKey.HideOtherStudentVideo, Boolean.class, false));
                        // enablePullAudioOnly值未改变则不用再次改变状态
                        if (hideOtherStudentVideo == enablePullAudioOnly) {
                            return;
                        }
                        enablePullAudioOnly = hideOtherStudentVideo;
                        // 只看老师则,不在黑板区状态变为REMOTE_VIDEO_AUTO_CLOSE,在黑板区不改变状态
                        // 取消只看老师,不管在不在黑板区都变为REMOTE_VIDEO_AUTO_OPEN
                        if (enablePullAudioOnly) {
                            if (!isJoinBlackboard) {
                                remoteVideoStatus = REMOTE_VIDEO_AUTO_CLOSE;
                            }
                        } else {
                            remoteVideoStatus = REMOTE_VIDEO_AUTO_OPEN;
                        }
                        if (!disablePullStream) {
                            changeVideoState(this.mediaModel, true);
                            setVideoOn(isVideoOn());
                            setAudioOn(isAudioOn());
                        } else {
                            remoteVideoStatus = REMOTE_VIDEO_AUDIO_CLOSE;
                        }
                    }));
            disposables.add(iRouter.getSubjectByKey(EventKey.HideOtherStudentVideo)
                    .ofType(Boolean.class)
                    .filter(aBoolean -> !iRouter.getLiveRoom().getDisableOtherStudentVideoData().isDisable())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(aBoolean -> {
                        // 学生主动开启只看老师
                        enablePullAudioOnly = initEnablePullAudioOnly(aBoolean);
                        if (enablePullAudioOnly) {
                            if (!isJoinBlackboard) {
                                remoteVideoStatus = REMOTE_VIDEO_AUTO_CLOSE;
                            }
                        } else {
                            remoteVideoStatus = REMOTE_VIDEO_AUTO_OPEN;
                        }
                        if (!disablePullStream) {
                            changeVideoState(this.mediaModel, true);
                            setVideoOn(isVideoOn());
                            setAudioOn(isAudioOn());
                        } else {
                            remoteVideoStatus = REMOTE_VIDEO_AUDIO_CLOSE;
                        }
                    }));
            enablePullAudioOnly = initEnablePullAudioOnly(iRouter.getLiveRoom().getDisableOtherStudentVideoData().isDisable());
            if (enablePullAudioOnly) {
                if (!isJoinBlackboard) {
                    remoteVideoStatus = REMOTE_VIDEO_AUTO_CLOSE;
                }
            } else {
                remoteVideoStatus = REMOTE_VIDEO_AUTO_OPEN;
            }
            setJoinBlackboard(isJoinBlackboard);
        }

        disposables.add(upPacketLossRateDisposable);
        disposables.add(disposableOfDownLinkLossRate);
        disposables.add(disposableOfLocalWindow);
        // attention 1v1默认拉大流，setRemoteVideoStreamType要在subscribe()之后
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            setRemoteVideoStreamType(BJYRtcCommon.DualStreamType.VIDEO_STREAM_HIGH);
        }
    }


    /**
     * 更新视频窗口网络状态显示
     *
     * @param lossRate      平均丢包率
     * @param showStrongTip 是否显示强提示
     */
    private void handlerNetworkUI(double lossRate, boolean showStrongTip) {
        if (lossRate <= packetLossRateLevel.get(0)) {
            $.id(R.id.window_video_network_tips).text("");
            ivSignal.setImageLevel(0);
        } else if (lossRate <= packetLossRateLevel.get(1)) {
            $.id(R.id.window_video_network_tips).text(getView().getContext().getString(R.string.bjysc_network_tips_level_1))
                    .textColor(R.color.bjysc_low_network_tips_middle);
            ivSignal.setImageLevel(1);
        } else if (lossRate <= packetLossRateLevel.get(2)) {
            $.id(R.id.window_video_network_tips).text(getView().getContext().getString(R.string.bjysc_network_tips_level_2))
                    .textColor(R.color.bjysc_low_network_tips_middle);
            ivSignal.setImageLevel(1);
        } else if (lossRate <= packetLossRateLevel.get(3) || lossRate <= packetLossRateLevel.get(4)) {
            $.id(R.id.window_video_network_tips).text(getView().getContext().getString(R.string.bjysc_network_tips_level_3))
                    .textColor(R.color.bjysc_low_network_tips_terrible);
            ivSignal.setImageLevel(2);
        } else {
            if (showStrongTip) {
                iRouter.getSubjectByKey(EventKey.NetworkTipsDialog).onNext(getView().getContext().getString(R.string.bjysc_network_tips_level_5));
            } else {
                $.id(R.id.window_video_network_tips).text(getView().getContext().getString(R.string.bjysc_network_tips_level_3))
                        .textColor(R.color.bjysc_low_network_tips_terrible);
                ivSignal.setImageLevel(2);
            }
        }
    }

    public IUserModel getUserModel() {
        return mediaModel.getUser();
    }

    public void publishLocalStream() {
        publishLocalStream(false);
    }

    public void publishLocalStream(boolean forceAudioOn) {
        videoView.setVisibility(View.GONE);
        if (iRouter.getLiveRoom().getSmallClassTemplateType() == LPConstants.SmallClassTemplateType.ONEONONE) {
            if (getUserModel().getType() == LPConstants.LPUserType.Assistant) {
                if (iRouter.getLiveRoom().getOnlineUserVM().enable1v1TeacherOffStage()) {
                    attachAVideo();
                }
            } else {
                if (isAppForeground()) {
                    attachAVideo();
                } else {
                    isVideoOnLast = true;
                    isAudioOnLast = true;
                }
            }
        } else {
            if (getUserModel().getType() == LPConstants.LPUserType.Teacher) {
                //老师同时推音视频
                if (isAppForeground()) {
                    attachAVideo();
                } else {
                    isAudioOnLast = true;
                    isVideoOnLast = true;
                }
            } else {
                // 学生默认只推视频
                if (isAppForeground()) {
                    if (iRouter.getLiveRoom().isClassStarted() && (iRouter.getLiveRoom().getPartnerConfig().liveStudentDefaultAudioOn == 1) || forceAudioOn) {
                        attachAVideo();
                    } else {
                        attachVideo();
                    }
                } else {
                    isVideoOnLast = true;
                    isAudioOnLast = iRouter.getLiveRoom().isClassStarted() && (iRouter.getLiveRoom().getPartnerConfig().liveStudentDefaultAudioOn == 1);
                }
            }
        }
    }

    protected boolean isAppForeground() {
        return UtilsKt.isAppForeground(view.getContext());
    }

    /**
     * 视频状态切换(因为BJY的engine，不支持自动播放流,必须手动)
     */
    public void changeVideoState(IMediaModel iMediaModel) {
        changeVideoState(iMediaModel, false);
    }

    private void changeVideoState(IMediaModel iMediaModel, boolean isFromEnablePullOnlyAudioSignal) {
        this.mediaModel = iMediaModel;
        initMediaState(iMediaModel);
        if (iMediaModel.getUser() instanceof LPUserModel) {
            videoCloseBackGroundUrl = ((LPUserModel) iMediaModel.getUser()).cameraCover;
            // 调整Fit/Fill 模式
            final int aspectRatio = ((LPUserModel) iMediaModel.getUser()).videoFit;
            if (this.videoFit != aspectRatio) {
                if (aspectRatio == 0) {
                    videoView.setAspectRatio(LPConstants.LPAspectRatio.Fill);
                } else {
                    videoView.setAspectRatio(LPConstants.LPAspectRatio.Fit);
                }
                stopStreaming(mediaModel.getMediaId());
                this.videoFit = aspectRatio;
            }
        }
        setRemoteVideoStatus(remoteVideoStatus);
        //标记音视频都关闭了，直接return
        if (isRemoteAVideoClose()) {
            stopStreaming(getMediaId());
            return;
        }
        boolean isReplacedByCamera = false;
        final IUserModel replacedUser = iRouter.getLiveRoom().getSpeakQueueVM().getReplacedUser();
        if (iRouter.getLiveRoom().getSpeakQueueVM().hasAsCameraUser()) {
            isReplacedByCamera = getMediaId().equals(replacedUser.getUserId());
        }
        // b-t 底层老师切分辨率黑屏，暂时去掉 isVideoPlaying() 的判断
        if (isReplacedByCamera || iMediaModel.isVideoOn()) {
            // 未订阅视频需要主动调playVideo()传入videoView
            stopStreaming(getMediaId());
            streamVideo(getMediaId());
        } else if (iMediaModel.isAudioOn()) {
            // 仅当未subscribe()才主动调
            stopStreaming(getMediaId());
            streamAudio(getMediaId());
        }
        if (iMediaModel.isVideoOn()) {
            // 窗口已关闭视频
            if (isRemoteVideoClose()) {
                if (!iMediaModel.isAudioOn()) {
                    stopStreaming(getMediaId());
                } else {
                    mPlayer.muteRemoteVideo(getMediaId(), true);
                }
            } else {
                if (isFromEnablePullOnlyAudioSignal) {
                    mPlayer.muteRemoteVideo(getMediaId(), false);
                } else {
                    mPlayer.muteRemoteAudio(getMediaId(), !iMediaModel.isAudioOn());
                }
            }
        } else {
            if (!iMediaModel.isAudioOn()) {
                stopStreaming(getMediaId());
            }
        }
    }

    private boolean initEnablePullAudioOnly(boolean disableOtherStudentVideo) {
        return iRouter != null && mediaModel != null && disableOtherStudentVideo
                && mediaModel.getUser().getType() == LPConstants.LPUserType.Student && !isLocalVideo;
    }

    public boolean getEnablePullAudioOnly() {
        return enablePullAudioOnly;
    }

    public void setName(String name) {
        $.id(R.id.window_video_name).text(name);
    }

    public boolean isJoinBlackboard() {
        return isJoinBlackboard;
    }

    public void setJoinBlackboard(boolean joinBlackboard) {
        isJoinBlackboard = joinBlackboard;
        //摄像头打开且只看老师才改变窗口状态，且拖到黑板区即打开摄像头
        if (enablePullAudioOnly && isVideoOn && videoView != null) {
            videoView.post(() -> {
                remoteVideoStatus = isJoinBlackboard || isGalleryLayout() ? REMOTE_VIDEO_AUTO_OPEN : REMOTE_VIDEO_AUTO_CLOSE;
                changeVideoState(mediaModel);
            });
        }
    }

    public void setZOrderMediaOverlay(boolean isMediaOverlay) {
        if (isDestroyed()) {
            return;
        }
        isOverlay = isMediaOverlay;
        if (getUserModel().getUserId().equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
            cameraView.setZOrderMediaOverlay(isMediaOverlay);
        } else {
            videoView.setZOrderMediaOverlay(isMediaOverlay);
        }
    }

    protected void streamVideo(String mediaId) {
        showLoading();
        if (mPlayer != null) {
            mPlayer.playVideo(mediaId, videoView);
        }
    }

    protected void streamAudio(String mediaId) {
        if (mPlayer != null) {
            mPlayer.playAudio(mediaId);
        }
    }

    protected void stopStreaming(String mediaId) {
        hideLoading();
        if (mPlayer != null) {
            mPlayer.playAVClose(mediaId);
        }
    }

    /**
     * 主动关闭音视频
     */
    public void closeAV() {
        if (isLocalVideo()) {
            if (mRecorder != null) {
                mRecorder.stopPublishing();
            }
        } else {
            stopStreaming(getMediaId());
        }
    }

    /**
     * 麦克风声音大小
     *
     * @param volumeLevel 0.没声音，1.1格，2.2格，3.3格
     */
    private void changeMicLevel(LPConstants.VolumeLevel volumeLevel) {
        if (!isAudioOn()) {
            return;
        }
        int level = 0;
        switch (volumeLevel) {
            case NONE:
                level = 1;
                break;
            case LOW:
                level = 2;
                break;
            case MIDDLE:
                level = 3;
                break;
            case HIGH:
                level = 4;
                break;
        }
        ivSpeak.setImageLevel(level);
    }

    private void controlHansUp(String uid, boolean isAgreed) {
        if (isAgreed) {
            iRouter.getLiveRoom().getSpeakQueueVM().agreeSpeakApply(uid);
        } else {
            iRouter.getLiveRoom().getSpeakQueueVM().disagreeSpeakApply(uid);
        }
        llHandsUpControl.setVisibility(View.GONE);
    }

    /**
     * 设置分组颜色
     *
     * @param color 16进制颜色值
     */
    public void setGroupColor(String color) {
        if (mediaModel != null && mediaModel.getMediaSourceType() != LPConstants.MediaSourceType.MainCamera) {
            return;
        }
        View view = $.id(R.id.window_video_view_group).view();
        view.setVisibility(View.VISIBLE);
        view.setBackgroundColor(Color.parseColor(color));
    }

    public boolean disablePullStream() {
        return disablePullStream;
    }

    public boolean isShowWindow() {
        return showWindow;
    }

    public void setGroupItem(GroupItem groupItemModel) {
        this.groupItemModel = groupItemModel;
        final IUserModel user = mediaModel.getUser();
        if (user instanceof LPUserModel) {
            ((LPUserModel) user).groupId = groupItemModel.id;
        }
        if (isLocalVideo()) {
            return;
        }
        boolean isDifferentGroup = iRouter.getLiveRoom().getOnlineUserVM().isMediaGroup() && iRouter.getLiveRoom().getCurrentUser().getGroup() != groupItemModel.id;
        // 非主摄、辅摄流，都直接拉流，辅摄流不显示占位
        if (isDifferentGroup) {
            if (getMediaSourceType() == LPConstants.MediaSourceType.ExtCamera) {
                showWindow = false;
            } else if (getMediaSourceType() != LPConstants.MediaSourceType.MainCamera) {
                showWindow = true;
                return;
            }
        }
        if (isDifferentGroup) {
            disablePullStream = true;
            remoteVideoStatus = REMOTE_VIDEO_AUDIO_CLOSE;
            stopStreaming(getMediaId());
            if (videoView != null) {
                videoView.setVisibility(View.GONE);
            }
            placeHolderContainer.setVisibility(View.VISIBLE);
            ivPlaceholderOnlyAudio.setVisibility(View.GONE);
            ivPlaceholder.setVisibility(View.GONE);
            ivSpeak.setImageLevel(isAudioOn ? 1 : 0);
            if (groupItemModel.id == 0) {
                $.id(R.id.tv_video_group).text(getString(R.string.bjy_group_not_group));
            } else {
                $.id(R.id.tv_video_group).text(groupItemModel.name);
            }
            $.id(R.id.tv_video_group).visibility(View.VISIBLE);
            if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                showWindow = false;
            }
            if (!showWindow) {
                iRouter.getSubjectByKey(EventKey.PlayerViewHide).onNext(this);
            }
        } else {
            // 防止非音视频分组走进来
            if (disablePullStream) {
                disablePullStream = false;
                if (enablePullAudioOnly) {
                    if (!isJoinBlackboard) {
                        remoteVideoStatus = REMOTE_VIDEO_AUTO_CLOSE;
                    }
                } else {
                    remoteVideoStatus = REMOTE_VIDEO_AUTO_OPEN;
                }
                if (videoView != null) {
                    videoView.setVisibility(View.VISIBLE);
                }
                changeVideoState(this.mediaModel);
                setVideoOn(isVideoOn());
                setAudioOn(isAudioOn());
                $.id(R.id.tv_video_group).visibility(View.GONE);
                if (!showWindow) {
                    showWindow = true;
                    iRouter.getSubjectByKey(EventKey.PlayerViewHide).onNext(this);
                }
            }
        }
    }

    /**
     * 设置摄像头占位图状态
     *
     * @param checked image checked value
     */
    @SuppressLint("UseCompatLoadingForDrawables")
    protected void changePlaceholderState(boolean checked) {
        if (!checked && !TextUtils.isEmpty(videoCloseBackGroundUrl)) {
            //摄像头关闭态，占位图非空
            simpleTarget = Glide.with(context).load(videoCloseBackGroundUrl)
                    .into(new SimpleTarget<Drawable>() {
                        @Override
                        public void onResourceReady(@NonNull Drawable resource, @Nullable Transition<? super Drawable> transition) {
                            setPlaceholderResource(true, resource);
                        }

                        @Override
                        public void onLoadFailed(@Nullable Drawable errorDrawable) {
                            setPlaceholderResource(false,
                                    view.getContext().getResources().getDrawable(R.drawable.base_ic_video_camera_mute));
                            $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_camera_mute);
                            super.onLoadFailed(errorDrawable);
                        }
                    });
        } else {
            setPlaceholderResource(false,
                    view.getContext().getResources().getDrawable(R.drawable.base_ic_video_camera_mute));
            $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_camera_mute);
        }
        if (mediaModel != null) {
            if (videoState != LPConstants.MediaState.Normal) {
                if (videoState == LPConstants.MediaState.Unavailable) {
                    setPlaceholderResource(false,
                            view.getContext().getResources().getDrawable(R.drawable.base_ic_video_camera_error));
                    $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_canot_use);
                } else if (videoState == LPConstants.MediaState.Backstage) {
                    setPlaceholderResource(false,
                            view.getContext().getResources().getDrawable(R.drawable.base_ic_video_backstage));
                    $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_backstage);
                    ivPlaceholder.setVisibility(isVideoOn ? View.INVISIBLE : View.VISIBLE);
                    placeHolderContainer.setVisibility(isVideoOn ? View.INVISIBLE : View.VISIBLE);
                    ivPlaceholderOnlyAudio.setVisibility(View.GONE);
                } else {
                    setPlaceholderResource(false,
                            view.getContext().getResources().getDrawable(R.drawable.base_ic_video_occupied));
                    $.id(R.id.tv_video_placeholder).text(R.string.bjysc_study_room_study_status_canot_use);
                }
            }
            if (!isAudioOn) {
                if (audioState == LPConstants.MediaState.Unavailable) {
                    ivSpeak.setImageResource(R.drawable.bjysc_video_ic_mic_error);
                } else {
                    ivSpeak.setImageResource(R.drawable.level_ic_video_mic);
                    ivSpeak.setImageLevel(0);
                }
            }
            if (disablePullStream) {
                if (groupItemModel != null && groupItemModel.id != 0) {
                    $.id(R.id.tv_video_group).text(groupItemModel.name);
                } else {
                    $.id(R.id.tv_video_group).text(getString(R.string.bjy_group_not_group));
                }
                placeHolderContainer.setVisibility(View.VISIBLE);
                ivPlaceholderOnlyAudio.setVisibility(View.GONE);
                ivPlaceholder.setVisibility(View.GONE);
                ivSpeak.setImageLevel(isAudioOn ? 1 : 0);
            }
        }
    }


    private ConstraintLayout.LayoutParams videoCloseImageOriginParams;

    protected void setPlaceholderResource(boolean isCustom, Drawable videoCloseDrawable) {
        if (iRouter == null) {
            return;
        }
        if (isCustom && videoCloseImageOriginParams == null) {
            videoCloseImageOriginParams = (ConstraintLayout.LayoutParams) ivPlaceholder.getLayoutParams();
            ConstraintLayout.LayoutParams newImageParams = new ConstraintLayout.LayoutParams(videoCloseImageOriginParams);
            newImageParams.width = ConstraintLayout.LayoutParams.MATCH_PARENT;
            newImageParams.height = ConstraintLayout.LayoutParams.MATCH_PARENT;
            newImageParams.bottomMargin = 0;
            ivPlaceholder.setLayoutParams(newImageParams);
            ivPlaceholder.setScaleType(ImageView.ScaleType.CENTER_CROP);
        } else if (!isCustom && videoCloseImageOriginParams != null) {
            ivPlaceholder.setLayoutParams(videoCloseImageOriginParams);
            ivPlaceholder.setScaleType(ImageView.ScaleType.FIT_CENTER);
            videoCloseImageOriginParams = null;
        }
        ivPlaceholder.setImageDrawable(videoCloseDrawable);
    }

    /**
     * 清空分组颜色
     */
    public void clearGroupColor() {
        $.id(R.id.window_video_view_group).view().setVisibility(View.GONE);
    }

    public String getName() {
        TextView tvName = $.id(R.id.window_video_name).view();
        return tvName.getText().toString();
    }

    public String getUserId() {
        return null != mediaModel ? mediaModel.getUser().getUserId() : "";
    }

    public String getMediaId() {
        return null != mediaModel ? mediaModel.getMediaId() : "";
    }

    public boolean isOverlay() {
        return isOverlay;
    }

    public void showMenu(boolean isShowMenu) {
        this.isShowMenu = isShowMenu;
        ivMenu.setVisibility(View.GONE);
    }

    public boolean isShowMenu() {
        return isShowMenu && !isOnlyInBlackboard && isShowSingleTutorMenu();
    }

    public void setOnlyInBlackboard(boolean isOnlyInBlackboard) {
        this.isOnlyInBlackboard = isOnlyInBlackboard;
    }

    public void showHandsUpIcon(boolean isShow) {
        ivHandsUp.setVisibility(isShow && !iRouter.getLiveRoom().getStudyRoomVM().getStudyRoomMode().isGalleryLayout() ? View.VISIBLE : View.GONE);
    }

    public void showHandsUpControl(boolean isShow) {
        llHandsUpControl.setVisibility(isShow ? View.VISIBLE : View.GONE);
    }

    public void showTutorHelpIcon(boolean isShow) {
        // do nothing
    }

    public void showTutorHandleIcon(boolean isShow) {
        // do nothing
    }

    public boolean isLocalVideo() {
        return isLocalVideo;
    }

    public void setMenuState(boolean isPressed) {
        ivMenu.setSelected(isPressed);
    }

    public boolean isVideoOn() {
        if (isLocalVideo) {
            return mRecorder.isVideoAttached();
        } else {
            return isVideoOn;
        }
    }

    public boolean isAudioOn() {
        if (isLocalVideo) {
            return mRecorder.isAudioAttached();
        } else {
            return isAudioOn;
        }
    }

    public boolean isDrawAuthOn() {
        return isDrawAuthOn;
    }

    public void setDrawAuthOn(boolean authOn) {
        isDrawAuthOn = authOn;
        ivDrawAuth.setVisibility(authOn ? View.VISIBLE : View.GONE);
    }


    public void setPPTAuthOn(boolean pptAuthOn) {
        isPPTAuthOn = pptAuthOn;
        ivPPTAuth.setVisibility(pptAuthOn ? View.VISIBLE : View.GONE);
    }

    public boolean isPPTAuthOn() {
        return isPPTAuthOn;
    }

    public boolean isAssistCameraOn() {
        return isAssistCameraOn;
    }

    public void setAssistCameraOn(boolean assistCameraOn) {
        isAssistCameraOn = assistCameraOn;
    }

    public boolean isScreenShareOn() {
        return isScreenShareOn;
    }


    public void setScreenShareOn(boolean screenShareOn) {
        isScreenShareOn = screenShareOn;
    }

    public void setAudioOn(boolean audioOn) {
        isAudioOn = audioOn;
        ivSpeak.setImageLevel(isAudioOn ? 1 : 0);
        ivPlaceholderOnlyAudio.setImageResource(isAudioOn ? R.drawable.ic_only_audio_normal : R.drawable.ic_only_audio_none);
        if (getUserId().equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
            showMenu(true);
        } else {
            if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                //学生看他人的视频菜单，只有音视频都有时，才显示
                showMenu(isVideoOn());
            } else if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                showMenu(getUserModel().getType() != LPConstants.LPUserType.Teacher || (isAudioOn() && isVideoOn()));
            } else {
                //老师看他人的视频菜单，肯定显示
                showMenu(true);
            }
        }
    }

    public void setVideoOn(boolean videoOn) {
        isVideoOn = videoOn;
        if (getUserId().equals(iRouter.getLiveRoom().getCurrentUser().getUserId())) {
            showMenu(true);
            changePlaceHolder();
        } else {
            videoView.setVisibility(isVideoOn && !isRemoteVideoClose() ? View.VISIBLE : View.GONE);
            changePlaceHolder();
            if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Student) {
                //学生看他人的视频菜单，只有音视频都有时，才显示
                showMenu(isVideoOn());
            } else if (iRouter.getLiveRoom().getCurrentUser().getType() == LPConstants.LPUserType.Assistant) {
                showMenu(getUserModel().getType() != LPConstants.LPUserType.Teacher || (isAudioOn() && isVideoOn()));
            } else {
                //老师看他人的视频菜单，肯定显示
                showMenu(true);
            }
        }
    }

    private void changePlaceHolder() {
        ivPlaceholder.setVisibility(isVideoOn && !disablePullStream ? View.INVISIBLE : View.VISIBLE);
        changePlaceholderState(isVideoOn);
        if (isVideoOn) {
            if (isRemoteVideoClose()) {
                hideLoading();
                placeHolderContainer.setVisibility(View.VISIBLE);
                //自己和该窗口都不在辅导中
                if (!isRemoteVideoClose()) {
                    ivPlaceholderOnlyAudio.setVisibility(View.VISIBLE);
                }
            } else {
                placeHolderContainer.setVisibility(View.GONE);
            }
        } else {
            placeHolderContainer.setVisibility(View.VISIBLE);
            ivPlaceholderOnlyAudio.setVisibility(View.GONE);
        }
    }

    public String getSelfTutorId() {
        return iRouter.getSubjectValueByKey(EventKey.StudyStatusTutor, String.class, "");
    }

    public boolean isRemoteVideoClose() {
        return remoteVideoStatus != REMOTE_VIDEO_AUTO_OPEN && remoteVideoStatus != REMOTE_VIDEO_HAND_OPEN;
    }

    //音视频都关闭
    public boolean isRemoteAVideoClose() {
        return remoteVideoStatus == REMOTE_VIDEO_AUDIO_CLOSE;
    }

    public void setVideoRemoteClose(@RemoteVideoStatus int remoteVideoStatus) {
        this.remoteVideoStatus = remoteVideoStatus;
    }

    public void setRemoteVideoStatus(@RemoteVideoStatus int remoteVideoStatus) {
        this.remoteVideoStatus = remoteVideoStatus;
        // 远端视频未开则 占位图肯定存在
        placeHolderContainer.setVisibility(isRemoteVideoClose() || !isVideoOn ? View.VISIBLE : View.GONE);
        ivPlaceholderOnlyAudio.setVisibility(isRemoteVideoClose() && !isRemoteAVideoClose() && !disablePullStream ? View.VISIBLE : View.GONE);
        videoView.setVisibility(isRemoteVideoClose() ? View.GONE : View.VISIBLE);
        if (cameraView.getVisibility() == View.VISIBLE) {
            cameraView.setVisibility(View.GONE);
        }
    }

    public void muteRemoteVideo(boolean mute) {
        if (mPlayer != null) {
            // 开视频并且当前未拉流，重新调playVideo()拉流
            if (!mute && !mPlayer.isVideoPlaying(getMediaId())) {
                stopStreaming(getMediaId());
                streamVideo(getMediaId());
            } else {
                mPlayer.muteRemoteVideo(getMediaId(), mute);
            }
        }
    }

    int[] getMenuPosition() {
        return new int[]{currentX + getWindowPosition()[0], currentY + getWindowPosition()[1]};
    }

    // region 奖励相关
    public void setAwardTvVisibility(int visibility) {
        awardContainer.setVisibility(visibility);
    }

    public void setAwardCount(int count) {
        if (count > 0) {
            awardContainer.setVisibility(View.VISIBLE);
        }
        awardCount.setText(String.valueOf(count));
    }

    private AwardPopupWindow awardPopupWindow;
    private LPAwardConfig availableAward = null;

    private void initAward() {
        LPAwardConfig[] awardConfigs = iRouter.getLiveRoom().getToolBoxVM().getAwardConfigs();
        if (awardConfigs == null) {
            return;
        }
        int enableAwardTypeCount = 0;
        LPAwardConfig availableAward = null;
        for (LPAwardConfig lpAwardConfig : awardConfigs) {
            if (lpAwardConfig.isEnable == 1) {
                availableAward = lpAwardConfig;
                enableAwardTypeCount++;
            }
        }
        if (enableAwardTypeCount <= 1 && availableAward != null) {
            // 仅一种启用，使用对应logo
            Glide.with(awardIcon).load(availableAward.logoUrl).into(awardIcon);
            // 保存 当前 LPAwardConfig
            this.availableAward = availableAward;
        } else {
            // 使用钻石
            awardIcon.setImageResource(R.drawable.base_ic_video_award_default);
            this.availableAward = null;
        }
    }

    public void setAwardClickListener(IVideoWindowCallback callback) {
        initAward();
        $.id(R.id.window_video_awards).clicked(v -> {
            if (availableAward == null) {
                // 多种，弹窗
                if (awardPopupWindow == null) {
                    awardPopupWindow = new AwardPopupWindow(context, getUserModel(), callback::onAwardTvClicked);
                }
                if (awardPopupWindow.isShowing()) {
                    awardPopupWindow.dismiss();
                } else {
                    awardPopupWindow.setUserModel(getUserModel());
                    awardPopupWindow.show($.id(R.id.window_video_awards).view());
                }
            } else {
                // 一种，使用对应方式进行点赞
                if (!BaseUtilsKt.isAdmin(iRouter.getLiveRoom())) {
                    return;
                }
                callback.onAwardTvClicked(getUserModel(), availableAward.key);
            }
        });
    }

    public interface IVideoWindowCallback {
        void onAwardTvClicked(IUserModel userModel, String awardKey);
    }
    // endregion

    public LPConstants.MediaSourceType getMediaSourceType() {
        return mediaModel.getMediaSourceType();
    }

    public boolean isScreenShare() {
        return mediaModel.isScreenShare();
    }

    public void setBottomVisibility(int visibility) {
        clBottom.setVisibility(visibility);
    }

    // region 加载中 视图及动画
    private int[] loadingImgGroup = new int[]{
            R.id.item_speak_speaker_loading_img1,
            R.id.item_speak_speaker_loading_img2,
            R.id.item_speak_speaker_loading_img3,
            R.id.item_speak_speaker_loading_img4,
            R.id.item_speak_speaker_loading_img5
    };
    private ValueAnimator animator;

    private void showLoading() {
        if (loadingListener == null) {
            loadingListener = new LoadingListener(this);
            mPlayer.addPlayerListener(loadingListener);
        }
        $.id(R.id.item_speak_speaker_loading_container).visible();

        if (animator == null) {
            animator = ValueAnimator.ofFloat(0.5f, 1f);
            animator.setDuration(500);
            animator.setInterpolator(new LinearInterpolator());
            animator.setRepeatCount(ValueAnimator.INFINITE);
            animator.setRepeatMode(ValueAnimator.REVERSE);
        }
        animator.addUpdateListener(animation -> {
            float currentScale = (float) animation.getAnimatedValue();
            for (int value : loadingImgGroup) {
                if (currentScale >= 0.5f && currentScale <= 1.5f) {
                    $.id(value).view().setScaleY(currentScale <= 1 ? currentScale : 2 - currentScale);
                } else {
                    currentScale -= 1;
                    $.id(value).view().setScaleY(currentScale);
                }
                $.id(value).view().requestLayout();
                currentScale += 0.25f;
            }
        });
        animator.start();
    }

    private void hideLoading() {
        // fixme VideoContainerWindow.OnDestroy() 主动调了 VideoWindow.onDestroy()，VideoWindow() 自身 lifecycle 回调又调了一次
        if ($ == null) {
            return;
        }
        $.id(R.id.item_speak_speaker_loading_container).gone();
        if (animator != null) {
            animator.removeAllUpdateListeners();
            animator.cancel();
        }
        for (int loadingImgId : loadingImgGroup) {
            $.id(loadingImgId).view().clearAnimation();
        }
    }
    // endregion

    @Override
    protected void unSubscribe() {
        super.unSubscribe();
        if (animator != null) {
            animator.removeAllUpdateListeners();
            animator.cancel();
        }
    }

    @Override
    public void onDestroy() {
        disposables.clear();
        if (animator != null) {
            animator.removeAllUpdateListeners();
            animator.cancel();
        }
        RxUtils.dispose(timer);
        closeAV();
        if (mPlayer != null) {
            mPlayer.removePlayerListener(loadingListener);
        }
        if (awardPopupWindow != null) {
            awardPopupWindow.dismiss();
            awardPopupWindow.onDestroy();
            awardPopupWindow = null;
            availableAward = null;
        }
        mPlayer = null;
        mRecorder = null;
        upLossRateQueue.clear();
        downLinkLossRateQueue.clear();
        if (simpleTarget != null && context instanceof Activity) {
            if (!((Activity) context).isDestroyed()) {
                Glide.with(context).clear(simpleTarget);
            }
        }
        super.onDestroy();
    }

    public boolean isDestroyed() {
        return !isWindowInit;
    }

    public class LoadingListener implements LPPlayerListener {

        private WeakReference<VideoWindow> videoWindowWeakReference;

        LoadingListener(VideoWindow videoWindow) {
            videoWindowWeakReference = new WeakReference<>(videoWindow);
        }

        @Override
        public void onReadyToPlay(String mediaId) {
            if (Objects.equals(mediaId, mediaModel.getMediaId())) {
                LPLogger.d(TAG, "onReadyToPlay mediaId=" + mediaId);
            }
            hideLoading(mediaId);
            setRemoteVideoStreamType(defaultRemoteStreamType);
        }

        @Override
        public void onPlayAudioSuccess(String mediaId) {
            // do nothing
        }

        @Override
        public void onPlayVideoSuccess(String mediaId) {
            // do nothing
        }

        @Override
        public void onPlayClose(String mediaId) {
            // do nothing
        }

        private void hideLoading(String mediaId) {
            final VideoWindow videoWindow = videoWindowWeakReference.get();
            if (videoWindow == null || videoWindow.getView() == null || !TextUtils.equals(mediaId, videoWindow.getMediaId())) {
                return;
            }
            LPLogger.d(TAG, "hideLoading mediaId=" + mediaId);
            videoWindow.hideLoading();
        }
    }

    public void setRemoteVideoStreamType(BJYRtcCommon.DualStreamType streamType) {
        if (isLocalVideo()) {
            return;
        }
        mPlayer.setRemoteVideoStreamType(getMediaId(), streamType);
        defaultRemoteStreamType = streamType;
    }

    public void setResolutionText(int width, int height) {
        resolutionTest.setText(width + "*" + height);
    }

    public void setAspectRatio(LPConstants.LPAspectRatio aspectRatio) {
        cameraView.setAspectRatio(aspectRatio);
        videoView.setAspectRatio(aspectRatio);
    }

    private void showPermissionNotGrantedDialog(String tip) {
        new CommonDialog(context, CommonDialog.ChoiceMode.Single_Blue)
                .setIconDrawableId(R.drawable.ic_warning)
                .setMainDisplayText(tip)
                .setPositive(context.getResources().getText(R.string.bjysc_confirm).toString(), ((dialog, which) -> {
                    dialog.dismiss();
                })).show();
    }
}
